package com.project.survey.adapter.base;

import android.content.Context;
import android.view.ContextMenu;
import android.view.LayoutInflater;
import android.view.MenuItem;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import java.util.List;

/**
 * Created by YanZhenjie on 2017/10/3.
 */
public abstract class BaseAdapter<VH extends RecyclerView.ViewHolder, T> extends RecyclerView.Adapter<VH> {
    private IItemClickListener<T> mItemClickListener;
    protected IAdapterContextMenu<T> adapterContextMenu;

    protected List<T> mDataList;
    protected Context mContext;

    private final LayoutInflater mInflater;

    public BaseAdapter(Context context) {
        this.mContext = context;
        this.mInflater = LayoutInflater.from(context);
    }

    public LayoutInflater getInflater() {
        return mInflater;
    }


    @Override
    public int getItemCount() {
        return mDataList == null ? 0 : mDataList.size();
    }

    public T getBean(int position) {
        return mDataList.get(position);
    }

    @Override
    public void onBindViewHolder(@NonNull VH holder, int position) {
        holder.itemView.setOnClickListener(v -> {
            if (mItemClickListener != null) {
                mItemClickListener.clickItem(mDataList.get(holder.getBindingAdapterPosition()));
            }
        });

        holder.itemView.setOnCreateContextMenuListener((menu, v, menuInfo) ->
                createMenu(menu, mDataList.get(holder.getBindingAdapterPosition())));

        onBindViewHolderDo(holder, position);
    }

    protected abstract void onBindViewHolderDo(VH holder, int position);


    protected void createMenu(ContextMenu menu, T bean) {
        if (adapterContextMenu == null) return;
        menu.clear();
        for (int index = 0; index < adapterContextMenu.menuItemSize(); index++) {
            menu.add(0, index, 0, adapterContextMenu.menuItemName(index));
            MenuItem menuItem = menu.getItem(index);
            if (menuItem != null) {
                menuItem.setOnMenuItemClickListener(item ->
                        adapterContextMenu.onMenuItemClick(bean, item));
            }
        }
    }

    public void setAdapterContextMenu(IAdapterContextMenu<T> adapterContextMenu) {
        this.adapterContextMenu = adapterContextMenu;
    }

    public void setOnItemClickListener(IItemClickListener<T> mItemClickListener) {
        this.mItemClickListener = mItemClickListener;
    }

    public void refreshData(List<T> dataList) {
        this.mDataList = dataList;
        notifyDataSetChanged();
    }

}
