package com.project.survey.dialog;

import android.content.Context;
import android.graphics.Color;
import android.graphics.drawable.ColorDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.PopupWindow;

import androidx.annotation.DrawableRes;
import androidx.annotation.NonNull;

import com.bingce.AppChannel;
import com.project.survey.R;
import com.project.survey.databinding.LayoutBubblePopwindowBinding;

import java.util.Collections;
import java.util.List;


public class BubblePopWindow extends PopupWindow {

    private OnHelpDocClickListener onHelpDocClickListener;
    private OnTiltSwitchClickListener onTiltSwitchClickListener;
    private OnRedrawClickListener onRedrawClickListener;

    private LayoutBubblePopwindowBinding binding;

    public BubblePopWindow(Context context, boolean isCADCustomView) {
        this(context, isCADCustomView, Collections.emptyList());
    }

    public BubblePopWindow(Context context, List<CustomMenuItem> menuItems) {
        this(context, false, menuItems);
    }

    public BubblePopWindow(Context context, boolean isCADCustomView, List<CustomMenuItem> menuItems) {
        super(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        //这里要注意设置setOutsideTouchable之前要设置 setBackgroundDrawable()
        //否则点击外部无法关闭pop
        setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
        setOutsideTouchable(true);
        //获取焦点设置为true 这样再次点击菜单时 就隐藏菜单
        setFocusable(true);

        binding = LayoutBubblePopwindowBinding.inflate(LayoutInflater.from(context));
        setContentView(binding.getRoot());

//        binding.llSoftwareSetup.setOnClickListener(v -> {
//            dismiss();
////            SoftwareSetupActivity.start(context);
//        });

        binding.llHelpDocument.setOnClickListener(v -> {
            dismiss();
            onHelpDocClickListener.onHelpClick();
        });

        binding.llTiltSwitch.setOnClickListener(v -> {
            dismiss();
            onTiltSwitchClickListener.onTiltSwitchClick();
        });

        if (isCADCustomView) {
            binding.llCadRedraw.setVisibility(View.VISIBLE);
        } else {
            binding.llCadRedraw.setVisibility(View.GONE);
        }

        binding.llCadRedraw.setOnClickListener(v -> {
            dismiss();
            onRedrawClickListener.onCadRedrawClicked();
        });

        if (AppChannel.customChannel == AppChannel.CUSTOM_FJD ||
                AppChannel.customChannel == AppChannel.CUSTOM_COMNAV) {
            binding.llHelpDocument.setVisibility(View.GONE);
        }

        if (menuItems != null) {
            if (!menuItems.isEmpty()) {
                binding.llMenu1.setVisibility(View.VISIBLE);
                binding.ivMenu1.setImageResource(menuItems.get(0).iconRes);
                binding.tvMenu1.setText(menuItems.get(0).title);
                binding.llMenu1.setOnClickListener(v -> {
                    dismiss();
                    menuItems.get(0).action.run();
                });
            }
            if (menuItems.size() > 1) {
                binding.llMenu2.setVisibility(View.VISIBLE);
                binding.ivMenu2.setImageResource(menuItems.get(1).iconRes);
                binding.tvMenu2.setText(menuItems.get(1).title);
                binding.llMenu2.setOnClickListener(v -> {
                    dismiss();
                    menuItems.get(1).action.run();
                });
            }
            if (menuItems.size() > 2) {
                binding.llMenu3.setVisibility(View.VISIBLE);
                binding.ivMenu3.setImageResource(menuItems.get(2).iconRes);
                binding.tvMenu3.setText(menuItems.get(2).title);
                binding.llMenu3.setOnClickListener(v -> {
                    dismiss();
                    menuItems.get(2).action.run();
                });
            }
            if (menuItems.size() > 3) {
                binding.llMenu4.setVisibility(View.VISIBLE);
                binding.ivMenu4.setImageResource(menuItems.get(3).iconRes);
                binding.tvMenu4.setText(menuItems.get(3).title);
                binding.llMenu4.setOnClickListener(v -> {
                    dismiss();
                    menuItems.get(3).action.run();
                });
            }
            if (menuItems.size() > 4) {
                binding.llMenu5.setVisibility(View.VISIBLE);
                binding.ivMenu5.setImageResource(menuItems.get(4).iconRes);
                binding.tvMenu5.setText(menuItems.get(4).title);
                binding.llMenu5.setOnClickListener(v -> {
                    dismiss();
                    menuItems.get(4).action.run();
                });
            }
            if (menuItems.size() > 5) {
                binding.llMenu6.setVisibility(View.VISIBLE);
                binding.ivMenu6.setImageResource(menuItems.get(5).iconRes);
                binding.tvMenu6.setText(menuItems.get(5).title);
                binding.llMenu6.setOnClickListener(v -> {
                    dismiss();
                    menuItems.get(5).action.run();
                });
            }
            if (menuItems.size() > 6) {
                binding.llMenu7.setVisibility(View.VISIBLE);
                binding.ivMenu7.setImageResource(menuItems.get(6).iconRes);
                binding.tvMenu7.setText(menuItems.get(6).title);
                binding.llMenu7.setOnClickListener(v -> {
                    dismiss();
                    menuItems.get(6).action.run();
                });
            }
        }

        //设置窗口进入和退出的动画
        setAnimationStyle(R.style.popWindowStyle);
    }

    public interface OnHelpDocClickListener {
        void onHelpClick();
    }

    public void setOnHelpDocClickListener(OnHelpDocClickListener onHelpDocClickListener) {
        this.onHelpDocClickListener = onHelpDocClickListener;
    }

    public void setHideHelpDoc(boolean isHide) {
        binding.llHelpDocument.setVisibility(isHide ? View.GONE : View.VISIBLE);
    }

    public interface OnTiltSwitchClickListener {
        void onTiltSwitchClick();
    }

    public void setOnTiltSwitchClickListener(OnTiltSwitchClickListener onTiltSwitchClickListener) {
        this.onTiltSwitchClickListener = onTiltSwitchClickListener;
    }

    public void setHideTiltSwitch(boolean isHide) {
        binding.llTiltSwitch.setVisibility(isHide ? View.GONE : View.VISIBLE);
    }

    public interface OnRedrawClickListener {
        void onCadRedrawClicked();
    }

    public void setOnRedrawClickListener(OnRedrawClickListener onRedrawClickListener) {
        this.onRedrawClickListener = onRedrawClickListener;
    }

    public static class CustomMenuItem {
        public final @DrawableRes int iconRes;
        public final String title;
        public final Runnable action;


        public CustomMenuItem(@DrawableRes int iconRes, @NonNull String title, @NonNull Runnable action) {
            this.iconRes = iconRes;
            this.title = title;
            this.action = action;
        }
    }
}