package com.project.survey.dialog;

import android.app.Dialog;
import android.content.Context;
import android.text.InputType;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.bingce.ui.CleanableEditText;
import com.project.survey.R;
import com.project.survey.util.CommonUtils;


public class CustomInputListDialog extends Dialog {

    public CustomInputListDialog(Context context, int themeResId) {
        super(context, themeResId);
    }

    /* Builder */
    public static class Builder {
        private TextView tvTitle,btnCancel,btnConfirm;
        private CleanableEditText editText01, editText02, editText03;
        private View mLayout;
        private View.OnClickListener mButtonCancelClickListener;
        private setOnConfirmClickListener mButtonConfirmClickListener;
        private int inputFormat;
        public static int INPUT_TYPE_TEXT = 0;
        public static int INPUT_TYPE_DECIMAL = 1;
        public static int INPUT_TYPE_INTEGER = 2;
        public static int INPUT_TYPE_PM_DECIMAL = 3;
        public static int INPUT_TYPE_PM_INTEGER = 4;
        private CustomInputListDialog mDialog;

        public Builder(Context context) {
            mDialog = new CustomInputListDialog(context, R.style.gif_dialog);
            LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            // 加载布局文件
            mLayout = inflater.inflate(R.layout.layout_edit_input_list_dialog, null, false);
            // 添加布局文件到 Dialog
            mDialog.addContentView(mLayout, new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));

            tvTitle = mLayout.findViewById(R.id.tv_title);
            editText01 = mLayout.findViewById(R.id.editText01);
            editText02 = mLayout.findViewById(R.id.editText02);
            editText03 = mLayout.findViewById(R.id.editText03);
            btnCancel = mLayout.findViewById(R.id.tv_cancel);
            btnConfirm = mLayout.findViewById(R.id.tv_confirm);
        }

        public Builder setInputFormat(int inputFormat){
            this.inputFormat = inputFormat;
            return this;
        }


        /**
         * 设置 Dialog 标题
         */
        public Builder setTitle(String title) {
            tvTitle.setText(title);
            tvTitle.setVisibility(View.VISIBLE);
            return this;
        }

        /**
         * 设置 默认值
         * @param editTextContent
         * @return
         */
        public Builder setInputText01(String editTextContent) {
            editText01.setText(editTextContent);
            return this;
        }

        public Builder setInputText01(double editTextContent) {
            editText01.setText(String.valueOf(editTextContent));
            return this;
        }

        /**
         * 设置 hint
         */
        public Builder setInputHint01(String editHint) {
            editText01.setHint(editHint);
            return this;
        }

        /**
         * 设置 默认值
         * @param editTextContent
         * @return
         */
        public Builder setInputText02(String editTextContent) {
            editText02.setText(editTextContent);
            return this;
        }

        public Builder setInputText02(double editTextContent) {
            editText02.setText(String.valueOf(editTextContent));
            return this;
        }

        /**
         * 设置 hint
         */
        public Builder setInputHint02(String editHint) {
            editText02.setHint(editHint);
            return this;
        }

        /**
         * 设置 默认值
         * @param editTextContent
         * @return
         */
        public Builder setInputText03(String editTextContent) {
            editText03.setText(editTextContent);
            return this;
        }

        public Builder setInputText03(double editTextContent) {
            editText03.setText(String.valueOf(editTextContent));
            return this;
        }

        /**
         * 设置 hint
         */
        public Builder setInputHint03(String editHint) {
            editText03.setHint(editHint);
            return this;
        }

        /**
         * 设置取消按钮文字和监听
         */
        public Builder setButtonCancel(String text) {
            btnCancel.setText(text);
            return this;
        }

        public Builder setButtonCancel(View.OnClickListener listener) {
            mButtonCancelClickListener = listener;
            return this;
        }

        /**
         * 设置确认按钮文字和监听
         */
        public Builder setButtonConfirm(setOnConfirmClickListener listener) {
            mButtonConfirmClickListener = listener;
            return this;
        }

        public CustomInputListDialog create() {

            switch (inputFormat){
                case 1: //正小数限制
                    editText01.setInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
                    editText02.setInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
                    editText03.setInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
                    break;
                case 2: //正整数限制
                    editText01.setInputType(InputType.TYPE_CLASS_NUMBER);
                    editText02.setInputType(InputType.TYPE_CLASS_NUMBER);
                    editText03.setInputType(InputType.TYPE_CLASS_NUMBER);
                    break;
                case 3: //正负小数限制
                    editText01.setInputType(InputType.TYPE_NUMBER_FLAG_SIGNED | InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
                    editText02.setInputType(InputType.TYPE_NUMBER_FLAG_SIGNED | InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
                    editText03.setInputType(InputType.TYPE_NUMBER_FLAG_SIGNED | InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
                    break;
                case 4: //正负整数限制
                    editText01.setInputType(InputType.TYPE_NUMBER_FLAG_SIGNED | InputType.TYPE_CLASS_NUMBER);
                    editText02.setInputType(InputType.TYPE_NUMBER_FLAG_SIGNED | InputType.TYPE_CLASS_NUMBER);
                    editText03.setInputType(InputType.TYPE_NUMBER_FLAG_SIGNED | InputType.TYPE_CLASS_NUMBER);
                    break;
            }

            btnCancel.setOnClickListener(view -> {
                CommonUtils.hideSoftInput();
                mDialog.dismiss();
                if (mButtonCancelClickListener != null) {
                    mButtonCancelClickListener.onClick(view);
                }
            });

            btnConfirm.setOnClickListener(view -> {
                String inputText01 = editText01.getText().toString();
                String inputText02 = editText02.getText().toString();
                String inputText03 = editText03.getText().toString();
                if (!TextUtils.isEmpty(inputText01) && !TextUtils.isEmpty(inputText02) && !TextUtils.isEmpty(inputText03)) {
                    CommonUtils.hideSoftInput();
                    mDialog.dismiss();
                }
                if (mButtonConfirmClickListener != null) {
                    mButtonConfirmClickListener.OnConfirmClick(inputText01, inputText02, inputText03);
                }
            });

            mDialog.setContentView(mLayout);
            mDialog.setCancelable(true);
            mDialog.setCanceledOnTouchOutside(false);
            return mDialog;
        }
    }

    public interface setOnConfirmClickListener{
        void OnConfirmClick(String editText01, String editText02, String editText03);
    }
}
