package com.project.survey.dialog;

import static androidx.recyclerview.widget.LinearLayoutManager.VERTICAL;

import android.app.Dialog;
import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.LinearLayout;
import android.widget.TextView;

import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.project.survey.R;
import com.project.survey.adapter.CustomRecycleTtemStyleDialogAdapter;
import com.project.survey.model.TextStyleBean;

import java.util.List;

public class CustomRecycleStyleDialog extends Dialog {

    public CustomRecycleStyleDialog(Context context) {
        super(context);
    }

    public CustomRecycleStyleDialog(Context context, int theme) {
        super(context, theme);
    }

    public static class Builder {
        private Context context; // 上下文对象
        private String title; // 对话框标题
        private List<TextStyleBean> list;
        private String preferenceKey;
        private int selectedIndex;
        private static OnItemClickListener onItemClickListener;

        public Builder(Context context) {
            this.context = context;
        }

        /**
         * Set the Dialog title from resource
         *
         * @param title
         * @return
         */
        public Builder setTitle(int title) {
            this.title = (String) context.getText(title);
            return this;
        }

        /**
         * Set the Dialog title from String
         *
         * @param title
         * @return
         */
        public Builder setTitle(String title) {
            this.title = title;
            return this;
        }

        public Builder setItemClickListener(OnItemClickListener onItemClickListener){
            this.onItemClickListener = onItemClickListener;
            return this;
        }

        public Builder setListData(List<TextStyleBean> list, String preferenceKey){
            this.list = list;
            this.preferenceKey = preferenceKey;
            this.selectedIndex = 0;
            return this;
        }

        public Builder setListData(List<TextStyleBean> list, String preferenceKey, int selectedIndex){
            this.list = list;
            this.preferenceKey = preferenceKey;
            this.selectedIndex = selectedIndex;
            return this;
        }

        public CustomRecycleStyleDialog create() {
            LayoutInflater inflater = (LayoutInflater) context
                    .getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            // instantiate the dialog with the custom Theme
            final CustomRecycleStyleDialog dialog = new CustomRecycleStyleDialog(context,
                    R.style.gif_dialog);
            View layout = inflater.inflate(R.layout.layout_recycle_style_dialog, null);
            dialog.addContentView(layout, new LinearLayout.LayoutParams(
                    LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT));

            RecyclerView recyclerView = layout.findViewById(R.id.recyclerView);
            if (list != null && list.size() > 0){
                CustomRecycleTtemStyleDialogAdapter adapter = new CustomRecycleTtemStyleDialogAdapter(context,list,selectedIndex);
                LinearLayoutManager linearLayoutManager = new LinearLayoutManager(context);
                linearLayoutManager.setOrientation(VERTICAL);
                recyclerView.setLayoutManager(linearLayoutManager);
                recyclerView.setAdapter(adapter);
                adapter.setOnClickListener(position -> {
                    onItemClickListener.OnItemClick(preferenceKey,position,list.get(position));
                    dialog.dismiss();
                });
            }

            TextView tv_title = layout.findViewById(R.id.tv_title);
            if (title != null || !TextUtils.isEmpty(title)){
                tv_title.setText(title);
//                tv_title.getPaint().setFakeBoldText(true);
                tv_title.setVisibility(View.VISIBLE);
            }else {
                tv_title.setVisibility(View.GONE);
            }

            TextView tv_cancel = layout.findViewById(R.id.tv_cancel);
            tv_cancel.setOnClickListener(v -> dialog.dismiss());

            dialog.setContentView(layout);
            return dialog;
        }
    }

    public interface OnItemClickListener{
        void OnItemClick(String preferenceKey, int position, TextStyleBean textStyleBean);
    }
}