package com.project.survey.dialog;

import android.app.Dialog;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import com.project.survey.R;


public class CustomStyleDialog extends Dialog {

    private CustomStyleDialog(Context context, int themeResId) {
        super(context, themeResId);
    }

    private CustomStyleDialog(Context context, boolean cancelable, OnCancelListener cancelListener) {
        super(context, cancelable, cancelListener);
    }

    /* Builder */
    public static class Builder {
        private TextView tvTitle, tvContent;
        private TextView btnSample,btnCancel, btnConfirm;

        private View mLayout;
        private View.OnClickListener mButtonSampleClickListener;
        private View.OnClickListener mButtonCancelClickListener;
        private View.OnClickListener mButtonConfirmClickListener;

        private CustomStyleDialog mDialog;

        public Builder(Context context) {
            mDialog = new CustomStyleDialog(context, R.style.gif_dialog);
            LayoutInflater inflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
            // 加载布局文件
            mLayout = inflater.inflate(R.layout.layout_content_style_dialog, null, false);
            // 添加布局文件到 Dialog
            mDialog.addContentView(mLayout, new ViewGroup.LayoutParams(ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.WRAP_CONTENT));

            tvTitle = mLayout.findViewById(R.id.tv_title);
            tvContent = mLayout.findViewById(R.id.tv_content);
            btnSample = mLayout.findViewById(R.id.btn_look_sample);
            btnCancel = mLayout.findViewById(R.id.btn_cancel);
            btnConfirm = mLayout.findViewById(R.id.btn_confirm);
        }

        /**
         * 设置 Dialog 标题
         */
        public Builder setTitle(String title) {
            tvTitle.setText(title);
            tvTitle.setVisibility(View.VISIBLE);
            return this;
        }

        /**
         * 设置 Dialog内容
         */
        public Builder setContent(String content) {
            tvContent.setText(content);
            return this;
        }

        public Builder setButtonSample(String text){
            btnSample.setText(text);
            return this;
        }

        public Builder setButtonSample(View.OnClickListener listener) {
            mButtonSampleClickListener = listener;
            return this;
        }

        public Builder setButtonSample(String text, View.OnClickListener listener) {
            btnSample.setText(text);
            mButtonSampleClickListener = listener;
            return this;
        }

        /**
         * 设置取消按钮文字和监听
         */
        public Builder setButtonCancel(String text) {
            btnCancel.setText(text);
            return this;
        }

        public Builder setButtonCancel(View.OnClickListener listener) {
            mButtonCancelClickListener = listener;
            return this;
        }

        public Builder setButtonCancel(String text, View.OnClickListener listener) {
            btnCancel.setText(text);
            mButtonCancelClickListener = listener;
            return this;
        }

        /**
         * 设置确认按钮文字和监听
         */
        public Builder setButtonConfirm(View.OnClickListener listener) {
            mButtonConfirmClickListener = listener;
            return this;
        }

        public Builder setButtonConfirm(String text, View.OnClickListener listener) {
            btnConfirm.setText(text);
            mButtonConfirmClickListener = listener;
            return this;
        }

        public CustomStyleDialog create() {

            btnSample.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    mDialog.dismiss();
                    if (mButtonSampleClickListener != null) {
                        mButtonSampleClickListener.onClick(view);
                    }
                }
            });

            btnCancel.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    mDialog.dismiss();
                    if (mButtonCancelClickListener != null) {
                        mButtonCancelClickListener.onClick(view);
                    }
                }
            });

            btnConfirm.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    mDialog.dismiss();
                    if (mButtonConfirmClickListener != null) {
                        mButtonConfirmClickListener.onClick(view);
                    }
                }
            });

            mDialog.setContentView(mLayout);
            mDialog.setCancelable(true);
            mDialog.setCanceledOnTouchOutside(false);
            return mDialog;
        }
    }
}
