package com.project.survey.extend

import android.view.View
import android.widget.EditText
import com.project.survey.util.KeyboardUtils
import com.scwang.smart.refresh.layout.SmartRefreshLayout


/**
 * 防止重复点击
 */
var lastTime = 0L
fun View.setOnClickNoRepeatListener(interval: Long = 300, onClick: (View) -> Unit) {
    setOnClickListener {
        val currentTime = System.currentTimeMillis()
        if (lastTime != 0L && (currentTime - lastTime < interval)) {
            return@setOnClickListener
        }
        lastTime = currentTime
        onClick(it)
    }
}

fun View.isVisibleOrInvisible(visible: Boolean){
    this.visibility = if (visible) View.VISIBLE else View.INVISIBLE
}

fun View.isVisibleOrGone(visible: Boolean){
    this.visibility = if (visible) View.VISIBLE else View.GONE
}

fun SmartRefreshLayout.smartDismiss() {
    finishRefresh(0)
    finishLoadMore(0)
}

fun EditText.showSoftInput(){
    KeyboardUtils.showSoftInput(this)
}

fun EditText.hideSoftInput(){
    KeyboardUtils.hideSoftInput(this)
}