package com.project.survey.logic.viewmodel

import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import blankj.utilcode.util.LogUtils
import com.project.survey.extend.toast
import com.project.survey.logic.event.LiveBus
import com.project.survey.logic.event.Message
import com.project.survey.logic.repository.GlobalRepository
import com.project.survey.network.ERROR
import com.project.survey.ui.login.LoginActivity
import com.project.survey.util.ActivityNavUtil
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import retrofit2.HttpException

open class BaseViewModel : ViewModel() {

    val repo: GlobalRepository = GlobalRepository()

    val msgEvent by lazy { LiveBus.getInstance().with(Message::class.java) }

    val errorResponse = MutableLiveData<String?>()

    fun launch(block: suspend CoroutineScope.() -> Unit) {
        viewModelScope.launch {
            withContext(Dispatchers.IO) {
                try {
                    block()
                } catch (e: Exception) {
                    if (e is HttpException) {
                        if (e.code() == ERROR.AUTH_ERROR.code) { //token失效
                            toast("token失效，请重新登录")
                            delay(2000L)
                            LoginActivity.start()
                        } else {
                            toast("${e.code()};${e.message}")
                        }
                    }
                    errorResponse.postValue(e.message)
                }
            }
        }
    }
}