package com.project.survey.logic.viewmodel

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import com.project.survey.constants.SPConstants
import com.project.survey.logic.bean.ListResult
import com.project.survey.model.ControlNetCGBean
import com.project.survey.model.ControlNetworkBean
import com.project.survey.model.ControlNetworkInfoBean
import com.project.survey.model.FirstControlNetworkBean
import com.project.survey.model.FirstControlNetworkVersionBean
import com.project.survey.model.ProjectBean
import com.project.survey.model.SpecialControlNetworkVersionBean
import com.project.survey.network.RetrofitClient
import com.project.survey.util.SPUtils
import retrofit2.http.Query

class ControlNetViewModel : BaseViewModel() {

    val api = RetrofitClient.createApiService()


    val controlNetworkListResponse: LiveData<ListResult<ControlNetworkBean>>
        get() = _controlNetworkListResponse
    private val _controlNetworkListResponse = MutableLiveData<ListResult<ControlNetworkBean>>()

    /**
     * 获取专用控制网列表
     */
    fun fetchSpecialControlNetworkList(
        controlNetWorkName: String?,
        version: String?,
        pageNum: Int = 1
    ) {
        launch {
            val res = api.fetchSpecialControlNetworkList(
                controlNetWorkName = controlNetWorkName,
                version = version,
                pageNum = pageNum
            )
            if (res.success) {
                _controlNetworkListResponse.postValue(res.data)
            }
        }
    }


    val controlNetworkInfoResponse: LiveData<ListResult<ControlNetworkInfoBean>>
        get() = _controlNetworkInfoResponse
    private val _controlNetworkInfoResponse = MutableLiveData<ListResult<ControlNetworkInfoBean>>()

    /**
     *  获取专用控制网信息
     */
    fun fetchSpecialControlNetworkInfo(
        controlNetworkName: String,
        pointName: String?,
        pageNum: Int = 1
    ) {
        launch {
            val res = api.fetchSpecialControlNetworkInfo(
                controlNetWorkName = controlNetworkName,
                pointName = pointName,
                pageNum = pageNum
            )
            if (res.success) {
                _controlNetworkInfoResponse.postValue(res.data)
            }
        }
    }

    val controlNetCGListResponse: LiveData<ListResult<FirstControlNetworkBean>>
        get() = _controlNetCGListResponse
    private val _controlNetCGListResponse = MutableLiveData<ListResult<FirstControlNetworkBean>>()

//    /**
//     * 获取专用控制网成果列表
//     */
//    fun fetchSpecialControlNetCGList(j_query: String, page: Int = 1, start: Int = 0) {
//        launch {
//            val res = api.fetchSpecialControlNetCGList(
//                tableCode = "BD_KZWZBCG",
//                funcCode = "BD_KZWZBCG_ZY",
//                j_query = j_query,
//                page = page,
//                start = start
//            )
//            if (res.success) {
//                _controlNetCGListResponse.postValue(res.data)
//            }
//        }
//    }

//    /**
//     * 获取首级控制网列表
//     */
//    fun fetchFirstControlNetList(j_query: String, page: Int = 1, start: Int = 0) {
//        launch {
//            val res = api.fetchSpecialControlNetCGList(
//                tableCode = "BD_KZWZBCG",
//                funcCode = "BD_KZWZBCG",
//                j_query = j_query,
//                page = page,
//                start = start
//            )
//            if (res.success) {
//                _controlNetCGListResponse.postValue(res.data)
//            }
//        }
//    }

    /**
     * 获取首级控制网列表
     */
    fun fetchFistControlNetworkList(version: String?, pageNum: Int) {
        launch {
            val res = api.fetchFistControlNetworkList(version = version, pageNum = pageNum)
            if (res.success) {
                _controlNetCGListResponse.postValue(res.data)
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    val firstControlNetworkVersionResponse: LiveData<List<FirstControlNetworkVersionBean>>
        get() = _firstControlNetworkVersionResponse
    private val _firstControlNetworkVersionResponse =
        MutableLiveData<List<FirstControlNetworkVersionBean>>()

    fun fetchFirstControlNetworkVersion() {
        launch {
            val res = api.fetchFirstControlNetworkVersion()
            if (res.success) {
                _firstControlNetworkVersionResponse.postValue(res.data)
            }
        }
    }

    val specialControlNetworkVersionResponse: LiveData<List<SpecialControlNetworkVersionBean>>
        get() = _specialControlNetworkVersionResponse
    private val _specialControlNetworkVersionResponse =
        MutableLiveData<List<SpecialControlNetworkVersionBean>>()

    fun fetchSpecialControlNetworkVersion() {
        launch {
            val res = api.fetchSpecialControlNetworkVersion()
            if (res.success) {
                _specialControlNetworkVersionResponse.postValue(res.data)
            }
        }
    }

}