package com.project.survey.logic.viewmodel

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import blankj.utilcode.util.ActivityUtils
import com.bingce.data.base.user.UserConfig
import com.bingce.data.base.user.UserConfigConstants
import com.bingce.data.database.AppDataBase
import com.bingce.log.BCLog
import com.google.gson.Gson
import com.project.survey.constants.EventConstants
import com.project.survey.constants.SPConstants
import com.project.survey.extend.toast
import com.project.survey.model.LoginBean
import com.project.survey.logic.event.Message
import com.project.survey.network.RetrofitClient
import com.project.survey.ui.login.LoginActivity
import com.project.survey.ui.login.LoginInputCodeActivity
import com.project.survey.ui.project.ProjectListActivity
import com.project.survey.util.SPUtils

class LoginViewModel : BaseViewModel() {

    val api = RetrofitClient.createApiService()

//    val externalPersonSMSCodeResponse: LiveData<Any>
//        get() = _externalPersonSMSCodeResponse
//    private val _externalPersonSMSCodeResponse = MutableLiveData<Any>()

    /**
     * 外部用户获取验证码
     * @param isOutLogin: false内部用户，true外部用户
     */
    fun fetchLoginSMSCode(isOutLogin: Boolean, account: String, password: String) {
        launch {
            val res = if (isOutLogin)
                api.fetchExternalPersonSMSCode(account, password)
            else
                api.fetchInternalPersonSMSCode(account, password)
            if (res.success) {
                LoginInputCodeActivity.start(isOutLogin, account, password, res.data ?: "")
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

    val loginResponse: LiveData<LoginBean>
        get() = _loginResponse
    private val _loginResponse = MutableLiveData<LoginBean>()

    /**
     * 用户登录
     */
    fun externalPersonLogin(isOutLogin: Boolean, account: String, password: String, code: String) {
        launch {
            val res = if (isOutLogin)
                api.externalPersonLogin(account, password, code)
            else
                api.internalPersonLogin(account, password, code)
            if (res.success) {
//                _externalPersonSMSCodeResponse.postValue(res.data)
                if (res.data == null || res.data is String) {
                    errorResponse.postValue(res.message)
                    return@launch
                }
                try {
                    val gson = Gson()
                    val loginBean = gson.fromJson(res.data.toString(), LoginBean::class.java)
                    SPUtils.put(SPConstants.ACCOUNT, account)
                    SPUtils.put(SPConstants.TOKEN, loginBean.token)
                    SPUtils.put(SPConstants.PERSON_ID, loginBean.personId)
                    SPUtils.put(SPConstants.MOBILE_PHONE, loginBean.mobilePhone)
                    SPUtils.put(SPConstants.PERSON_NAME, loginBean.name)


                    var userConfig = AppDataBase.getInstance().userConfigDao()
                        .findByLeanCloudUserId(loginBean.personId)
                    if (userConfig == null) { //创建第一个用户配置--使用默认数据目录
                        userConfig = UserConfig.newItem(
                            loginBean.personId,
                            UserConfigConstants.VALUE_DEFAULT_DB_FOLDER
                        )
                        AppDataBase.getInstance().userConfigDao().save(userConfig)
                    }
                    // 登录成功，进入工程列表页面
                    ProjectListActivity.start()
                    ActivityUtils.finishActivity(LoginActivity::class.java)
                } catch (e: Exception) {
                    errorResponse.postValue(res.message)
                    return@launch
                }
            } else {
                errorResponse.postValue(res.message)
            }
        }
    }

//    /**
//     * 登录
//     */
//    fun login(account: String, password: String) {
//        launch {
//            val res = api.login(account, password)
//            if (res.success) {
//                _loginResponse.postValue(res.data)
//                SPUtils.put(SPConstants.TOKEN, res.data.token)
//                SPUtils.put(SPConstants.ACCOUNT, account)
//                SPUtils.put(SPConstants.MOBILE_PHONE, res.data.mobilePhone)
//                msgEvent.postValue(Message(EventConstants.LOGIN_STATUS))
//            }
//
////            _loginResponse.postValue("res.data")
////            SPUtils.put(SPConstants.TOKEN, "res.data")
////            SPUtils.put(SPConstants.LOGIN, true)
////            SPUtils.put(SPConstants.ACCOUNT, account)
////            msgEvent.postValue(Message(EventConstants.LOGIN_STATUS))
//        }
//    }

//    val sendSMSResponse: LiveData<Any>
//        get() = _sendSMSResponse
//    private val _sendSMSResponse = MutableLiveData<Any>()
//
//    /**
//     * 发送验证码
//     */
//    fun sendSMS(mobile: String) {
//        launch {
//            val res = api.sendSMS(mobile)
//            if (res.success) {
//                _sendSMSResponse.postValue(res.data)
//            }
//        }
//    }
//
//    val checkVerifyCodeResponse: LiveData<Any>
//        get() = _checkVerifyCodeResponse
//    private val _checkVerifyCodeResponse = MutableLiveData<Any>()
//
//    /**
//     * 验证验证码
//     */
//    fun checkVerifyCode(mobile: String, verifycode: String) {
//        launch {
//            val res = api.checkVerifyCode(mobile, verifycode)
//            if (res.success) {
//                _checkVerifyCodeResponse.postValue(res.data)
//            }
//        }
//    }
}