package com.project.survey.model

import androidx.annotation.Keep
import com.bingce.data.surveyor.surveydata.pointstake.PointStakeRecord
import com.bingce.device.Device
import com.bingce.device.enums.DeviceTypeEnum

@Keep
data class LoftingPointRecordEntry(
    val settingOutPointId: String, //放样点ID，对应 LoftingPointBean 里的 BD_ZYLBMX_ID
    val antennaHight: String, //天线高 "11"
    val remark: String, //备注    "点1"
    val x: String,  //"111"
    val y: String,  //"11"
    val z: String,  //"11"
    val longitude: String, //经度
    val latitude: String, //纬度
    val hrms: String,
    val vrms: String,
    val pdop: String,
    val delay: String, //延迟 "xx"
    val jzt: String, //解状态  "xx"
    val satellite: String, //结算卫星   "xx"
) {
    /**
     * 将放样记录对象 转为 上传需要的对象
     */
    constructor(record: PointStakeRecord) : this(
        record.pointId,
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) "${record.deviceInfoData.rtkStatusData?.poleHeight}" else "${record.deviceInfoData.tsStatusData?.hr}",
        record.remarks ?: "",
        "${record.measuredX}",
        "${record.measuredY}",
        "${record.measuredZ}",
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) "${record.deviceInfoData.rtkStatusData?.longitude}" else "",
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) "${record.deviceInfoData.rtkStatusData?.latitude}" else "",
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) "${record.deviceInfoData.rtkStatusData?.hrms}" else "",
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) "${record.deviceInfoData.rtkStatusData?.vrms}" else "",
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) "${record.deviceInfoData.rtkStatusData?.pdop}" else "",
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) "${record.deviceInfoData.rtkStatusData?.diffAge}" else "",
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) record.deviceInfoData.rtkStatusData?.solutionState ?: "" else "",
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) "${record.deviceInfoData.rtkStatusData?.satSolutionNum}" else ""
    )
}
