package com.project.survey.network

import com.project.survey.constants.SPConstants
import com.project.survey.logic.bean.ListResult
import com.project.survey.model.ApprovalResponse
import com.project.survey.model.ControlNetworkBean
import com.project.survey.model.ControlNetCGBean
import com.project.survey.model.ControlNetworkInfoBean
import com.project.survey.model.CoordinateSystemData
import com.project.survey.model.FirstControlNetworkBean
import com.project.survey.model.FirstControlNetworkVersionBean
import com.project.survey.model.InspectionReportBean
import com.project.survey.model.LoftingBean
import com.project.survey.model.LoftingPointListBean
import com.project.survey.model.LoftingPointRecordEntry
import com.project.survey.model.LoginBean
import com.project.survey.model.MeasureBean
import com.project.survey.model.PointEntry
import com.project.survey.model.ProjectBean
import com.project.survey.model.ProjectData
import com.project.survey.model.SpecialControlNetworkVersionBean
import com.project.survey.util.SPUtils
import okhttp3.RequestBody
import retrofit2.http.Body
import retrofit2.http.Field
import retrofit2.http.FormUrlEncoded
import retrofit2.http.GET
import retrofit2.http.Header
import retrofit2.http.Headers
import retrofit2.http.POST
import retrofit2.http.Query

interface Api {

    /**
     * 内部登录获取验证码
     */
    @GET("je/personInfo/internalSendSms")
    suspend fun fetchInternalPersonSMSCode(
        @Query("account") account: String,
        @Query("password") password: String
    ): HttpResult<String?>

    /**
     * 内部用户登录
     */
    @GET("je/personInfo/internalEntry")
    suspend fun internalPersonLogin(
        @Query("account") account: String,
        @Query("password") password: String,
        @Query("code") code: String
    ): HttpResult<Any?>


    /**
     * 外部用户获取验证码
     */
    @GET("je/personInfo/sendSms")
    suspend fun fetchExternalPersonSMSCode(
        @Query("account") account: String,
        @Query("password") password: String
    ): HttpResult<String?>

    /**
     * 外部用户登录
     */
    @GET("je/personInfo/externalEntry")
    suspend fun externalPersonLogin(
        @Query("account") account: String,
        @Query("password") password: String,
        @Query("code") code: String
    ): HttpResult<Any?>

    /**
     * 获取用户可访问的所有项目的信息
     */
    @GET("je/project/getAllProjectIDByPersonId")
    suspend fun fetchProjectList(
        @Query("personId") personId: String
    ): HttpResult<List<ProjectData>>

    /**
     * 获取流程审批列表
     */
    @Headers("usePD:false")
    @FormUrlEncoded
    @POST("je/workflow/currentUserTask/getTask")
    suspend fun fetchWorkTask(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Field("type") type: String,
        @Field("userName") userName: String?,
        @Field("end") end: Int?,
        @Field("page") page: Int,
        @Field("limit") limit: Int = 30,
    ): HttpResult<ApprovalResponse>

    /**
     * 加载功能权限
     */
    @Headers("usePD:false")
    @GET("je/rbac/permission/loadFuncPerm")
    suspend fun loadFuncPerm(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Query("funcCode") funcCode: String
    ): HttpResult<Any>

    /**
     * 专用控制网版本列表
     */
    @GET("je/controlnetwork/queryControlNetworkVersion")
    suspend fun fetchSpecialControlNetworkVersion(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID)
    ): HttpResult<List<SpecialControlNetworkVersionBean>>

    /**
     * 首级控制网版本列表
     */
    @GET("je/controlnetwork/queryPrimaryControlNetworkVersion")
    suspend fun fetchFirstControlNetworkVersion(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID)
    ): HttpResult<List<FirstControlNetworkVersionBean>>

    /**
     * 获取专用控制网列表
     */
    @GET("je/controlnetwork/queryControlNetwork")
    suspend fun fetchSpecialControlNetworkList(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Query("controlNetWorkName") controlNetWorkName: String?,
        @Query("version") version: String?,
        @Query("pageNum") pageNum: Int,
        @Query("pageSize") pageSize: Int = 30
    ): HttpResult<ListResult<ControlNetworkBean>>
//    @FormUrlEncoded
//    @POST("je/cloud/beiDou/load")
//    suspend fun fetchSpecialControlNetList(
//        @Header("projectId") projectId: String,
//        @Field("tableCode") tableCode: String,
//        @Field("funcCode") funcCode: String,
//        @Field("j_query") j_query: String,
//        @Field("page") page: Int,
//        @Field("limit") limit: Int = 30,
//        @Field("start") start: Int = 0
//    ): HttpResult<ListResult<ControlNetBean>>

    /**
     * 根据条件查询控制网信息
     */
    @GET("je/controlnetwork/queryControlNetworkInfoByName")
    suspend fun fetchSpecialControlNetworkInfo(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Query("controlNetWorkName") controlNetWorkName: String, //当前控制网name
        @Query("pointName") pointName: String?, //关键词，搜索用
        @Query("pageNum") pageNum: Int,
        @Query("pageSize") pageSize: Int = 30
    ): HttpResult<ListResult<ControlNetworkInfoBean>>

    /**
     * 获取首级控制网列表
     */
    @GET("je/controlnetwork/queryPrimaryControlNetworkInfo")
    suspend fun fetchFistControlNetworkList(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Query("version") version: String?,
        @Query("pageNum") pageNum: Int,
        @Query("pageSize") pageSize: Int = 30
    ): HttpResult<ListResult<FirstControlNetworkBean>>

    /**
     * 获取首级控制网列表
     */
    @FormUrlEncoded
    @POST("je/common/load")
    suspend fun fetchSpecialControlNetCGList(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Field("tableCode") tableCode: String,
        @Field("funcCode") funcCode: String,
        @Field("j_query") j_query: String = "",
        @Field("page") page: Int,
        @Field("limit") limit: Int = 30,
        @Field("start") start: Int = 0
    ): HttpResult<ListResult<ControlNetCGBean>>


    /**
     * 获取测量工作：测量计划与工程计量
     * @param tableCode 测量计划传BD_CLJH，工程计量传：BD_CLGZ_GCJL
     * @param funcCode 测量计划传BD_CLJH，工程计量传：BD_CLGZ_GCJL_ZB(*******)
     * @param j_qurey  固定参数
     */
    @FormUrlEncoded
    @POST("je/cloud/beiDou/load")
    suspend fun fetchMeasureDataList(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Field("tableCode") tableCode: String,
        @Field("funcCode") funcCode: String,
        @Field("j_query") j_query: String,
        @Field("page") page: Int,
        @Field("limit") limit: Int = 1000,
        @Field("start") start: Int = 0
    ): HttpResult<ListResult<MeasureBean>>

    /**
     * 修改测量工作状态
     * @param workId    测量工作ID
     * @param workType    测量计划传BD_CLJH，工程计量传：BD_CLGZ_GCJL
     * @param status    执行中、已结束（对应按钮：进入测量、结束测量）
     */
    @GET("je/measureWork/modifyWorkStatus")
    suspend fun updateMeasureWorkStatus(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Query("workId") workId: String,
        @Query("workType") workType: String,
        @Query("status") status: String
    ): HttpResult<String>

    /**
     * 测点记录录入
     */
    @POST("je/measureWork/pointsEntry")
    suspend fun uploadPointEntry(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Body requestBody: List<PointEntry>
    ): HttpResult<String>

    /**
     * 生成报验单
     */
    @GET("je/measureWork/createInspectionReport")
    suspend fun createInspectionReport(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Query("workId") workId: String
    ): HttpResult<InspectionReportBean>

    /**
     * 放样任务查询
     */
    @FormUrlEncoded
    @POST("je/cloud/beiDou/load")
    suspend fun fetchLoftingList(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Field("tableCode") tableCode: String = "BD_ZYLB",
        @Field("funcCode") funcCode: String = "BD_ZYLB",
        @Field("j_qurey") j_qurey: String
    ): HttpResult<ListResult<LoftingBean>>

    /**
     * 根据放样作业ID获取放样点
     */
    @GET("je/measureWork/qureySettingOutPointsByWorkId")
    suspend fun fetchLoftingPointList(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Query("workId") workId: String
    ): HttpResult<List<LoftingPointListBean>>

    /**
     * 上传放样测点记录
     */
    @POST("je/measureWork/pointsEntryForSettingOutPoint")
    suspend fun uploadLoftingPointRecord(
        @Header("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Body requestBody: List<LoftingPointRecordEntry>
    ): HttpResult<String>

    /**
     * 插入坐标参数
     */
    @FormUrlEncoded
    @POST("je/measureWork/insertCoordinateParameter")
    suspend fun uploadCoordinateSystem(
        @Field("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID),
        @Field("coordinateParameter") coordinateParameter: String,
        @Field("name") name: String
    ): HttpResult<String>

    /**
     * 查询坐标参数
     */
    @FormUrlEncoded
    @POST("je/measureWork/findCoordinateParameter")
    suspend fun downloadCoordinateSystem(
        @Field("projectId") projectId: String = SPUtils.getString(SPConstants.PROJECT_ID)
    ): HttpResult<List<CoordinateSystemData>>

    /**
     * 修改坐标参数
     */
    @FormUrlEncoded
    @POST("je/measureWork/updateCoordinateParameter")
    suspend fun updateCoordinateSystem(
        @Field("id") id: String,
        @Field("coordinateParameter") coordinateParameter: String,
        @Field("name") name: String
    ): HttpResult<Any>
}