package com.project.survey.ui

import android.content.Intent
import android.view.KeyEvent
import androidx.activity.viewModels
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentPagerAdapter
import androidx.viewpager.widget.ViewPager
import blankj.utilcode.util.LogUtils
import blankj.utilcode.util.NetworkUtils
import com.bingce.device.Device
import com.bingce.ui.TabEntity
import com.bingce.utils.SdkUtils
import com.flyco.tablayout.listener.OnTabSelectListener
import com.project.survey.BuildConfig
import com.project.survey.R
import com.project.survey.databinding.ActivityMainBinding
import com.project.survey.extend.toast
import com.project.survey.logic.viewmodel.MainViewModel
import com.project.survey.logic.viewmodel.ProjectViewModel
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.ui.home.HomeFragment
import com.project.survey.ui.home.InstrumentFragment
import com.project.survey.ui.home.MeFragment
import com.project.survey.util.ActivityNavUtil

/**
 * 主页
 */
class MainActivity : BaseBindingActivity<ActivityMainBinding>() {

    companion object {
        val RC_OPEN_BT = 1330

        fun start() {
            ActivityNavUtil.startActivity<MainActivity> {}
        }
    }

    override fun getBinding(): ActivityMainBinding = ActivityMainBinding.inflate(layoutInflater)

    private val viewModel: MainViewModel by viewModels()
    private val projectViewModel: ProjectViewModel by viewModels()

    override fun initView() {
        viewModel.verifySdk()
        if (BuildConfig.DEBUG) {
            val text = "soresult1：[v=" + SdkUtils.isVipValidate() +
                    ",c=" + SdkUtils.isRegCodeValidate() +
                    ",i=" + SdkUtils.isIntlCodeValidate() +
                    ",a=" + SdkUtils.availableTime() +
                    ",trialTimes=" + SdkUtils.remainingTrialTimes()
//            toastLong(text) LogUtils.d(text)
        }

        initTabFragment()
    }

    override fun initData() {
        // 检查网络连接，获取之前未上传的测点数据，进行上传
        if (NetworkUtils.isConnected()) {
            projectViewModel.uploadPointEntry()
            projectViewModel.uploadLoftingPointRecord()
        }
    }

    override fun initListener() {
    }

    private fun initTabFragment() {
        //设置tab
        mBinding.tabLayout.setTabData(
            arrayListOf(
                TabEntity(
                    getString(R.string.tab_home_page),
                    R.drawable.icon_tab_home_selected,
                    R.drawable.icon_tab_home_unselected
                ),
                TabEntity(
                    getString(R.string.tab_instrument),
                    R.drawable.icon_tab_instrument_selected,
                    R.drawable.icon_tab_instrument_unselected
                ),
                TabEntity(
                    getString(R.string.tab_mine),
                    R.drawable.icon_tab_me_selected,
                    R.drawable.icon_tab_me_unselected
                )
            )
        )
        mBinding.tabLayout.setOnTabSelectListener(object : OnTabSelectListener {
            override fun onTabSelect(position: Int) {
                mBinding.vp.currentItem = position
            }

            override fun onTabReselect(position: Int) {
            }
        })

        //设置viewPager
        val fragments = arrayListOf(
            HomeFragment(),
            InstrumentFragment(),
            MeFragment()
        )
        mBinding.vp.adapter = object : FragmentPagerAdapter(supportFragmentManager) {
            override fun getCount(): Int = fragments.size

            override fun getItem(position: Int): Fragment = fragments[position]

        }
        mBinding.vp.offscreenPageLimit = 3
        mBinding.vp.addOnPageChangeListener(object : ViewPager.OnPageChangeListener {
            override fun onPageScrolled(
                position: Int,
                positionOffset: Float,
                positionOffsetPixels: Int
            ) {
            }

            override fun onPageSelected(position: Int) {
                mBinding.tabLayout.currentTab = position
            }

            override fun onPageScrollStateChanged(state: Int) {
            }

        })
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        when (requestCode) {
            RC_OPEN_BT -> if (resultCode == RESULT_OK) {
                Device.getInstance().requestConnect()
            } else {
                toast(R.string.bt_not_enabled)
            }
        }
    }

    private var exitTime: Long = 0

    override fun onKeyDown(keyCode: Int, event: KeyEvent?): Boolean {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if ((System.currentTimeMillis() - exitTime) > 2000) {
                toast("再按一次退出程序")
                exitTime = System.currentTimeMillis()
            } else {
                finish()
            }
            return true
        }
        return super.onKeyDown(keyCode, event)
    }

}