package com.project.survey.ui.approval

import android.net.Uri
import androidx.activity.viewModels
import com.bingce.surveyor.agentweb.AgentWebActivity
import com.project.survey.BuildConfig
import com.project.survey.R
import com.project.survey.constants.SPConstants
import com.project.survey.databinding.ActivityApprovalBinding
import com.project.survey.extend.isVisibleOrGone
import com.project.survey.extend.setOnClickNoRepeatListener
import com.project.survey.extend.smartDismiss
import com.project.survey.extend.toast
import com.project.survey.logic.viewmodel.ProjectViewModel
import com.project.survey.ui.approval.adapter.ApprovalAdapter
import com.project.survey.ui.approval.adapter.ApprovalWebAdapter
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.util.ActivityNavUtil
import com.project.survey.util.SPUtils
import com.project.survey.widget.decoration.TransparentDividerDecoration
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener

/**
 * 待处理，已处理，已发起
 */
class ApprovalActivity : BaseBindingActivity<ActivityApprovalBinding>(), OnRefreshLoadMoreListener {

    companion object {
        const val TYPE = "type"

        const val PI_PREAPPROV = "PI_PREAPPROV"//待处理
        const val PI_APPROVED = "PI_APPROVED"//已处理
        const val PI_OWNER = "PI_OWNER"//所有的流程
        //        const val TYPE_INITIATED = "TYPE_INITIATED"//已发起

        fun start(type: String = PI_OWNER) {
            ActivityNavUtil.startActivity<ApprovalActivity> {
                putString(TYPE, type)
            }
        }
    }

    override fun getBinding(): ActivityApprovalBinding =
        ActivityApprovalBinding.inflate(layoutInflater)

    private val viewModel: ProjectViewModel by viewModels()

    private var type: String = ""

    private val adapter by lazy { ApprovalWebAdapter() }
    private var currentPage = 1
    private var searchContent: String? = null
    private var end: Int? = null //type为PI_APPROVED（已办理）时生效，表示是否完结：0,未完结；1，已完结

    override fun initView() {
        type = intent.getStringExtra(TYPE) ?: ""
        immersionToolbar(
            mBinding.toolbar,
            when (type) {
                PI_PREAPPROV -> R.string.to_be_processed
                PI_APPROVED -> R.string.processed
                else -> R.string.initiated
            }
        )
        // type为已办理和所有时，显示
        mBinding.llScreen.isVisibleOrGone(type == PI_APPROVED || type == PI_OWNER)
        end = if (type == PI_APPROVED || type == PI_OWNER) {
            mBinding.tvUncompleted.isSelected = true
            mBinding.tvCompleted.isSelected = false
            0
        } else null
//        val beginTransaction = supportFragmentManager.beginTransaction()
//        val fragemnt = when (type) {
//            PI_PREAPPROV -> ApprovalWaitFragment()
//            PI_APPROVED -> ApprovalProcessedFragment()
//            PI_OWNER -> ApprovalInitiatedFragment()
//            else -> {
//                throw IllegalArgumentException("type is error")
//            }
//        }
//        beginTransaction.replace(R.id.container, fragemnt)
//        beginTransaction.commit()

        mBinding.smartRefreshLayout.setOnRefreshLoadMoreListener(this)
//        mBinding.recyclerView.addItemDecoration(TransparentDividerDecoration(this))
        mBinding.recyclerView.adapter = adapter
        adapter.isStateViewEnable = true
    }

    override fun initData() {
        viewModel.errorResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
        }
        viewModel.workTaskListResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
            it.data?.let { data ->
                if (currentPage == 1) {
                    adapter.submitList(data.rows)
                    if (data.rows.isEmpty()) {
                        mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                        adapter.setStateViewLayout(this, R.layout.layout_no_data)
                    } else {
                        if (adapter.itemCount < data.totalCount) {
                            currentPage++
                        } else {
                            mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                        }
                    }
                } else {
                    if (data.rows.isEmpty()) {
                        mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    } else {
                        if (adapter.itemCount < data.totalCount) {
                            adapter.addAll(data.rows)
                            currentPage++
                        } else {
                            mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                        }
                    }
                }
            }
        }
        // 加载功能权限
        viewModel.loadFuncPermErrorResponse.observe(this) {
            toast(it)
        }
        viewModel.loadFuncPermResponse.observe(this) {
            // http://10.190.183.37/app/micro/func/#/pages/func/form/{功能编码}/{流程ID}?authorization={登录token}&appId=2024-0926-1104-0165
            val token = SPUtils.getString(SPConstants.TOKEN)
            val url =
                BuildConfig.BASE_URL + "app/micro/func/#/pages/func/form/${it.funcCode}/${it.pkValue}?authorization=${token}&appId=2024-0926-1104-0165"
            AgentWebActivity.navigation2(this, Uri.parse(url))
        }

//        if (type == PI_PREAPPROV) {
//            viewModel.fetchWorkTask(SPUtils.getString(SPConstants.PROJECT_ID) ?: "", type)
//        } else if(type == PI_APPROVED) {
//            EasyHttpUtil.post("je/workflow/currentUserTask/getTask", SPUtils.getString(SPConstants.PROJECT_ID) ?: "", PI_APPROVED)
//        } else {
//            EasyHttpUtil.get("je/workflow/currentUserTask/getTask")
//        }
        fetchData()
    }

    private fun fetchData() {
        viewModel.fetchWorkTask(type, searchContent, end, currentPage)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        currentPage = 1
        fetchData()
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        fetchData()
    }

    override fun initListener() {
        // 搜索
        mBinding.searchWidget.setOnEditorSearchListener {
            searchContent = it
            currentPage = 1
            fetchData()
        }
        // 未完结
        mBinding.tvUncompleted.setOnClickNoRepeatListener {
            end = 0
            mBinding.tvUncompleted.isSelected = true
            mBinding.tvCompleted.isSelected = false
            currentPage = 1
            fetchData()
        }
        // 已完结
        mBinding.tvCompleted.setOnClickNoRepeatListener {
            end = 1
            mBinding.tvUncompleted.isSelected = false
            mBinding.tvCompleted.isSelected = true
            currentPage = 1
            fetchData()
        }
        adapter.setOnItemClickListener { _, _, position ->
            adapter.getItem(position)?.let { bean ->
                // 加载功能权限
                viewModel.loadFuncPerm(bean)
            }
        }
    }

}