package com.project.survey.ui.approval.adapter

import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.chad.library.adapter4.BaseMultiItemAdapter
import com.project.survey.databinding.ItemApprovalApprovedBinding
import com.project.survey.databinding.ItemApprovalOwnerBinding
import com.project.survey.extend.isVisibleOrGone
import com.project.survey.model.ApprovalBean
import com.project.survey.ui.approval.ApprovalActivity

class ApprovalAdapter : BaseMultiItemAdapter<ApprovalBean>() {

    private var type = ApprovalActivity.PI_PREAPPROV

    init {
        addItemType(0, object : OnMultiItemAdapterListener<ApprovalBean, ItemPreapprovVH> {
            override fun onCreate(
                context: Context,
                parent: ViewGroup,
                viewType: Int
            ): ItemPreapprovVH {
                val viewBinding =
                    ItemApprovalApprovedBinding.inflate(LayoutInflater.from(context), parent, false)
                return ItemPreapprovVH(viewBinding)
            }

            override fun onBind(holder: ItemPreapprovVH, position: Int, item: ApprovalBean?) {
                holder.binding.ivAgree.isVisibleOrGone(false)
                item?.let { bean ->
                    holder.binding.tvName.text = bean.title
                    holder.binding.tvNo.text = bean.funcCode
                    holder.binding.tvTime.text = bean.submitTime
                }
            }
        }).addItemType(1, object : OnMultiItemAdapterListener<ApprovalBean, ItemApprovedVH> {
            override fun onCreate(
                context: Context,
                parent: ViewGroup,
                viewType: Int
            ): ItemApprovedVH {
                val viewBinding =
                    ItemApprovalApprovedBinding.inflate(LayoutInflater.from(context), parent, false)
                return ItemApprovedVH(viewBinding)
            }

            override fun onBind(holder: ItemApprovedVH, position: Int, item: ApprovalBean?) {
                holder.binding.ivAgree.isVisibleOrGone(true)
                item?.let { bean ->
                    holder.binding.tvName.text = bean.title
                    holder.binding.tvNo.text = bean.funcCode
                    holder.binding.tvTime.text = bean.submitTime
                }
            }
        }).addItemType(2, object : OnMultiItemAdapterListener<ApprovalBean, ItemOwnerVH> {
            override fun onCreate(
                context: Context,
                parent: ViewGroup,
                viewType: Int
            ): ItemOwnerVH {
                val viewBinding =
                    ItemApprovalOwnerBinding.inflate(LayoutInflater.from(context), parent, false)
                return ItemOwnerVH(viewBinding)
            }

            override fun onBind(holder: ItemOwnerVH, position: Int, item: ApprovalBean?) {
                item?.let { bean ->
                    holder.binding.tvName.text = bean.title
                    holder.binding.tvNo.text = bean.funcCode
                    holder.binding.tvStartTime.text = bean.submitTime
                    holder.binding.tvEndTime.text = ""
                }
            }
        }).onItemViewType { _, _ ->
            when (type) {
                ApprovalActivity.PI_PREAPPROV -> 0
                ApprovalActivity.PI_APPROVED -> 1
                else -> 2
            }
        }
    }

    fun submitList(list: List<ApprovalBean>?, type: String) {
        this.type = type
        super.submitList(list)
    }

    class ItemPreapprovVH(val binding: ItemApprovalApprovedBinding) :
        RecyclerView.ViewHolder(binding.root)

    class ItemApprovedVH(val binding: ItemApprovalApprovedBinding) :
        RecyclerView.ViewHolder(binding.root)

    class ItemOwnerVH(val binding: ItemApprovalOwnerBinding) :
        RecyclerView.ViewHolder(binding.root)

//    companion object {
//        private const val TYPE_0 = 0
//        private const val TYPE_1 = 1
//        private const val TYPE_2 = 2
//    }

}