package com.project.survey.ui.base;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;

import androidx.annotation.DrawableRes;
import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;

import com.bingce.AppChannel;
import com.bingce.constants.KeyEventAction;
import com.bingce.coordlib.model.Blh;
import com.bingce.coordlib.model.Coordinate;
import com.bingce.device.Device;
import com.bingce.device.enums.DeviceTypeEnum;
import com.bingce.device.ui.manager.DeviceConnectManager;
import com.bingce.device.ui.manager.RtkManager;
import com.bingce.device.ui.manager.TsAnglesRefreshManager;
import com.bingce.device.ui.manager.TsBatteryRefreshManager;
import com.bingce.device.ui.manager.TsChangeHrManager;
import com.bingce.device.ui.manager.TsMeasCoordinateManager;
import com.bingce.device.ui.manager.TsMeasSdAnglesManager;
import com.bingce.device.ui.utils.TSSettingDialogUtils;
import com.bingce.rtk.command.RTK;
import com.bingce.rtk.gnss.Gnss;
import com.bingce.surveyor.util.SurveyDevicePoleHrHeightUtils;
import com.bingce.totalstation.TsListener;
import com.bingce.utils.PropertiesGet;
import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.dialog.BubblePopWindow;
import com.project.survey.ui.instrument.satellitosis.SatelliteStatusActivity;
import com.project.survey.ui.interfacee.IUIProvider;
import com.project.survey.util.AnimUtil;
import com.project.survey.util.BaseSurveyActivityUtils;
import com.project.survey.util.CountDownUtils;
import com.project.survey.util.DeviceConnectUtil;
import com.project.survey.util.DrawableUtils;
import com.project.survey.util.SystemUtils;

import org.polaric.colorful.Colorful;
import org.polaric.colorful.ColorfulActivity;

import java.util.List;

import blankj.utilcode.util.LogUtils;
import blankj.utilcode.util.ToastUtils;

public abstract class BaseSurveyNewActivity extends ColorfulActivity implements IUIProvider, Device.IDeviceConnectListener, RtkManager.IRtkListener,
        TsListener.MeasCoordinateListener, TsListener.MeasSdAnglesListener, TsListener.AnglesRefreshListener, TsListener.BatteryRefreshListener, TsListener.ChangeHrListener,
        CountDownUtils.ICountDownSwitcher {
    public final BaseSurveyActivityUtils baseSurveyActivityUtils = new BaseSurveyActivityUtils(this, this, onRootSetBackgroundColor());

    private final CountDownUtils countDownUtils = new CountDownUtils(() -> this, this);
    public final DeviceConnectManager deviceConnectManager = new DeviceConnectManager(this, this);
    protected final RtkManager rtkManager = new RtkManager(this, this);
    protected final TsMeasCoordinateManager tsMeasCoordinateManager = new TsMeasCoordinateManager(this, this);
    protected final TsMeasSdAnglesManager tsMeasSdAnglesManager = new TsMeasSdAnglesManager(this, this);
    protected final TsAnglesRefreshManager tsAnglesRefreshManager = new TsAnglesRefreshManager(this, this);
    protected final TsBatteryRefreshManager tsBatteryRefreshManager = new TsBatteryRefreshManager(this, this);
    protected final TsChangeHrManager tsChangeHrManager = new TsChangeHrManager(this, this);
    protected final TSSettingDialogUtils tsSettingDialogUtils = new TSSettingDialogUtils(this);

    final protected void hideSurveyorInfoLayout() {
        baseSurveyActivityUtils.activityBaseSurveyBinding.rlSurveyInfo.setVisibility(View.INVISIBLE);
    }

    //最右侧的按钮图片点击事件
    protected void onCommonRightClicked(View view) {
    }

    public int onRootSetBackgroundColor() {
        return -1;
    }

    public boolean isUseSystemStyleNavigationBar() {
        return false;
    }

    public boolean isShowTopBarHrPoleLayout() {
        return true;
    }

    public int customSystemStyleNavigationBarColor() {
        return R.color.theme_gray;
    }

    public void setFragmentTitle(int fragmentTitle) {
        setStayMainPage(false, getString(fragmentTitle));
    }

    public void setStayMainPage(boolean isStayMainPage) {
        setStayMainPage(isStayMainPage, "");
    }

    public void setStayMainPage(boolean isStayMainPage, String fragmentTitle) {
        if (isStayMainPage) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.rlSurveyInfo.setVisibility(View.GONE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.rlBack.setVisibility(View.GONE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.llRoadItemSurveyInfo.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.llTitleBarRightIcon.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.rlMenuSideslip.setVisibility(View.VISIBLE);
        } else {
            if (!TextUtils.isEmpty(fragmentTitle)) {
                baseSurveyActivityUtils.activityBaseSurveyBinding.rlSurveyInfo.setVisibility(View.GONE);
                baseSurveyActivityUtils.activityBaseSurveyBinding.rlBack.setVisibility(View.VISIBLE);
                baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setVisibility(View.VISIBLE);
                baseSurveyActivityUtils.activityBaseSurveyBinding.llRoadItemSurveyInfo.setVisibility(View.GONE);
                baseSurveyActivityUtils.activityBaseSurveyBinding.llTitleBarRightIcon.setVisibility(View.GONE);
            } else {
                baseSurveyActivityUtils.activityBaseSurveyBinding.rlSurveyInfo.setVisibility(View.VISIBLE);
                baseSurveyActivityUtils.activityBaseSurveyBinding.rlBack.setVisibility(View.VISIBLE);
                baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setVisibility(View.GONE);
                baseSurveyActivityUtils.activityBaseSurveyBinding.llRoadItemSurveyInfo.setVisibility(View.VISIBLE);
                baseSurveyActivityUtils.activityBaseSurveyBinding.llTitleBarRightIcon.setVisibility(View.GONE);
            }
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setText(fragmentTitle);
            baseSurveyActivityUtils.activityBaseSurveyBinding.rlMenuSideslip.setVisibility(View.GONE);
        }
    }

    public BubblePopWindow newBubblePopWindow(Context context) {
        return new BubblePopWindow(context, false);
    }

    private BubblePopWindow defaultMenu = null;

    protected void showMenu() {
        if (defaultMenu == null) {
            defaultMenu = newBubblePopWindow(this);
            defaultMenu.setOnDismissListener(() -> toggleBright());
            defaultMenu.setHideHelpDoc(isHideHelpDoc());
            defaultMenu.setOnHelpDocClickListener(() -> {
                onHelpClicked();
            });

            defaultMenu.setHideTiltSwitch(isHideTiltSwitch());
            defaultMenu.setOnTiltSwitchClickListener(() -> {
                onTiltSwitchClick();
            });
        }
        defaultMenu.showAsDropDown(baseSurveyActivityUtils.activityBaseSurveyBinding.ivMore, -150, 20);
    }

    /**
     * 隐藏三点更多
     */
    public void hideMore() {
        baseSurveyActivityUtils.activityBaseSurveyBinding.ivMore.setVisibility(View.GONE);
    }

    public boolean isHideHelpDoc() {
        return false;
    }

    public boolean isHideTiltSwitch() {
        return true;
    }

    private AnimUtil animUtil;
    private float bgAlpha = 1f;
    private boolean bright = false;
    private static final long DURATION = 400;
    private static final float START_ALPHA = 0.6f;
    private static final float END_ALPHA = 1f;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Colorful.applyTheme(this);
        baseSurveyActivityUtils.onCreate();
        if (App.instance.isThemeDark()) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.cutDownProgressbar.setBackgroundColor(ContextCompat.getColor(this, R.color.dark));
            baseSurveyActivityUtils.activityBaseSurveyBinding.baseView.setBackgroundColor(ContextCompat.getColor(this, R.color.dark));
        }
        if (!isUseSystemStyleNavigationBar()) {
            if (onRootSetBackgroundColor() != -1) {
                baseSurveyActivityUtils.activityBaseSurveyBinding.getRoot().setBackgroundColor(getColor(onRootSetBackgroundColor()));
                DrawableUtils.tintModifyColorVector(baseSurveyActivityUtils.activityBaseSurveyBinding.ivBack, R.drawable.icon_system_back, R.color.black);
                DrawableUtils.tintModifyColorVector(baseSurveyActivityUtils.activityBaseSurveyBinding.ivMore, R.drawable.icon_white_more, R.color.black);
                baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setTextColor(getColor(R.color.black));
            }
        } else {
            if (customSystemStyleNavigationBarColor() == R.color.theme_gray) {
                if (App.instance.isThemeDark()) {
                    Colorful.ThemeColor primaryColor = Colorful.getThemeDelegate().getPrimaryColor();
                    baseSurveyActivityUtils.activityBaseSurveyBinding.getRoot().setBackgroundColor(getColor(primaryColor.getColorRes()));
                    DrawableUtils.tintModifyColorVector(baseSurveyActivityUtils.activityBaseSurveyBinding.ivBack, R.drawable.icon_system_back, R.color.white);
                    DrawableUtils.tintModifyColorVector(baseSurveyActivityUtils.activityBaseSurveyBinding.ivMore, R.drawable.icon_white_more, R.color.white);
                    baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setTextColor(getColor(R.color.white));
                } else {
                    baseSurveyActivityUtils.activityBaseSurveyBinding.getRoot().setBackgroundColor(getColor(R.color.white));
                    DrawableUtils.tintModifyColorVector(baseSurveyActivityUtils.activityBaseSurveyBinding.ivBack, R.drawable.icon_system_back, R.color.black);
                    DrawableUtils.tintModifyColorVector(baseSurveyActivityUtils.activityBaseSurveyBinding.ivMore, R.drawable.icon_white_more, R.color.black);
                    baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setTextColor(getColor(R.color.black));
                }
            } else {
                baseSurveyActivityUtils.activityBaseSurveyBinding.getRoot().setBackgroundColor(getColor(customSystemStyleNavigationBarColor()));
                DrawableUtils.tintModifyColorVector(baseSurveyActivityUtils.activityBaseSurveyBinding.ivBack, R.drawable.icon_system_back, R.color.white);
                DrawableUtils.tintModifyColorVector(baseSurveyActivityUtils.activityBaseSurveyBinding.ivMore, R.drawable.icon_white_more, R.color.white);
                baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setTextColor(getColor(R.color.white));
            }
        }

        baseSurveyActivityUtils.activityBaseSurveyBinding.llPoleHighHr.setVisibility(isShowTopBarHrPoleLayout() ? View.VISIBLE : View.GONE);

        SurveyDevicePoleHrHeightUtils.setDevicePoleHrHigh(this, baseSurveyActivityUtils.activityBaseSurveyBinding.llPoleHighHr, baseSurveyActivityUtils.activityBaseSurveyBinding.ivPoleHighHr, baseSurveyActivityUtils.activityBaseSurveyBinding.tvPoleHighHr, true);

        baseSurveyActivityUtils.activityBaseSurveyBinding.llCommonRight.setOnClickListener(this::onCommonRightClicked);

        baseSurveyActivityUtils.activityBaseSurveyBinding.llIconHomeSatellite.setOnClickListener(v -> {
            //激光
            if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
                tsSettingDialogUtils.showTotalStationSettingDialog();
            }
//            else {
//                startActivity(new Intent(this, SatelliteStatusActivity.class));
//            }
        });

//        baseSurveyActivityUtils.activityBaseSurveyBinding.llRtkStatus.setOnClickListener(v -> {
//            DeviceConnectUtil.startDeviceConnected(this, baseSurveyActivityUtils.activityBaseSurveyBinding);
//        });

        animUtil = new AnimUtil();

        baseSurveyActivityUtils.activityBaseSurveyBinding.rlMore.setOnClickListener(v -> {
            showMenu();
            toggleBright();
        });

        initViewMap(savedInstanceState);
        initView();
        initData();
    }

    @Override
    protected void onResume() {
        super.onResume();
        //按照设置选项，保持屏幕常亮，放在onResume而不放在onCreate里是因为，用户更改设置后返回主页就要开始常亮了
        SystemUtils.setScreenAlwaysLight(this);
    }

    final public void toggleBright() {
        // 三个参数分别为：起始值 结束值 时长，那么整个动画回调过来的值就是从0.5f--1f的
        animUtil.setValueAnimator(START_ALPHA, END_ALPHA, DURATION);
        animUtil.addUpdateListener(progress -> {
            // 此处系统会根据上述三个值，计算每次回调的值是多少，我们根据这个值来改变透明度
            bgAlpha = bright ? progress : (START_ALPHA + END_ALPHA - progress);
            backgroundAlpha(bgAlpha);
        });
        animUtil.addEndListner(animator -> {
            // 在一次动画结束的时候，翻转状态
            bright = !bright;
        });
        animUtil.startAnimator();
    }

    final public void backgroundAlpha(float bgAlpha) {
        WindowManager.LayoutParams lp = getWindow().getAttributes();
        lp.alpha = bgAlpha; //0.0-1.0
        getWindow().setAttributes(lp);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_DIM_BEHIND);
    }

    protected void initViewMap(Bundle savedInstanceState) {

    }

    /**
     * 设置标题
     *
     * @param title
     */
    final protected void setTitle(String title) {
        setTitle(title, null, "", R.color.white);
    }

    final protected void setTitle(String title, String rightText) {
        setTitle(title, null, rightText, R.color.white);
    }

    final protected void setTitle(String title, String rightText, int rightTextColor) {
        setTitle(title, null, rightText, rightTextColor);
    }

    @Override
    public void setTitle(int titleId) {
        setTitle(getString(titleId));
    }

    @Override
    public void setTitle(CharSequence title) {
        setTitle(title.toString());
    }

    final protected void setTitle(String title, Integer rightIcon, String rightText, int rightTextColor) {
        if (!TextUtils.isEmpty(title)) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setText(title);
            baseSurveyActivityUtils.activityBaseSurveyBinding.rlSurveyInfo.setVisibility(View.GONE);
        } else {
            baseSurveyActivityUtils.activityBaseSurveyBinding.rlSurveyInfo.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setText("");
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonTitle.setVisibility(View.GONE);
        }

        if (rightIcon == null && TextUtils.isEmpty(rightText)) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.llCommonRight.setVisibility(View.GONE);
        } else {
            baseSurveyActivityUtils.activityBaseSurveyBinding.llCommonRight.setVisibility(View.VISIBLE);
        }
        if (rightIcon != null) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivCommonRightIcon.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivCommonRightIcon.setImageResource(rightIcon);
        } else {
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivCommonRightIcon.setVisibility(View.GONE);
        }
        if (!TextUtils.isEmpty(rightText)) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonRightText.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonRightText.setText(rightText);
        } else {
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonRightText.setVisibility(View.GONE);
        }

        if (rightTextColor > 0) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.tvCommonRightText.setTextColor(getColor(rightTextColor));
        }
    }

    protected void setTitleBar(List<CustomMainMenuItem> itemList) {
        baseSurveyActivityUtils.activityBaseSurveyBinding.llTitleBarRightIcon.setVisibility(!itemList.isEmpty() ? View.VISIBLE : View.GONE);
        if (itemList.size() == 1) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon02.setVisibility(View.GONE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon03.setVisibility(View.GONE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setImageDrawable(ContextCompat.getDrawable(this, itemList.get(0).iconRes));

            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setOnClickListener(v -> itemList.get(0).action.run());

        } else if (itemList.size() == 2) {
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon02.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon03.setVisibility(View.GONE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setImageDrawable(ContextCompat.getDrawable(this, itemList.get(0).iconRes));
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon02.setImageDrawable(ContextCompat.getDrawable(this, itemList.get(1).iconRes));

            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setOnClickListener(v -> itemList.get(0).action.run());
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon02.setOnClickListener(v -> itemList.get(1).action.run());

        } else {
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon02.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon03.setVisibility(View.VISIBLE);
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setImageDrawable(ContextCompat.getDrawable(this, itemList.get(0).iconRes));
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon02.setImageDrawable(ContextCompat.getDrawable(this, itemList.get(1).iconRes));
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon03.setImageDrawable(ContextCompat.getDrawable(this, itemList.get(2).iconRes));

            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon01.setOnClickListener(v -> itemList.get(0).action.run());
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon02.setOnClickListener(v -> itemList.get(1).action.run());
            baseSurveyActivityUtils.activityBaseSurveyBinding.ivTitleBarRightIcon03.setOnClickListener(v -> itemList.get(2).action.run());

        }
    }

    public static class CustomMainMenuItem {
        public final @DrawableRes int iconRes;
        public final Runnable action;


        public CustomMainMenuItem(@DrawableRes int iconRes, @NonNull Runnable action) {
            this.iconRes = iconRes;
            this.action = action;
        }
    }

    @Override
    public int getLayoutId() {
        return -1;
    }

    @Override
    public View getContentView() {
        return null;
    }

    protected abstract void initView();

    protected abstract void initData();

    @Override
    public boolean isShowTimeStatusBar() {
        return true;
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        switch (keyCode) {
            case KeyEvent.KEYCODE_VOLUME_UP:
                switch (/*((App) Utils.getApp()).*/volumeUpAction()) {
                    case KeyEventAction.NONE:
                        return super.onKeyDown(keyCode, event);
                    case KeyEventAction.CAL:
                        break;
                    case KeyEventAction.MEASURE:
                        performMeasure();
                        break;
                    case KeyEventAction.RECORD:
                        performRecord();
                        break;
                    case KeyEventAction.LAST:
                        performLast();
                        break;
                    case KeyEventAction.NEXT:
                        performNext();
                        break;
                    case KeyEventAction.MEASURE_NOT_SEND:
                        performMeasureNotSend();
                        break;
                    case KeyEventAction.MEASURE_AND_RECORD:
                        performMeasureRecord();
                        break;
                }
                return true;
            case KeyEvent.KEYCODE_VOLUME_DOWN:
                switch (volumeDownAction()) {
                    case KeyEventAction.NONE:
                        return super.onKeyDown(keyCode, event);
                    case KeyEventAction.CAL:
                        break;
                    case KeyEventAction.MEASURE:
                        performMeasure();
                        break;
                    case KeyEventAction.RECORD:
                        performRecord();
                        break;
                    case KeyEventAction.LAST:
                        performLast();
                        break;
                    case KeyEventAction.NEXT:
                        performNext();
                        break;
                    case KeyEventAction.MEASURE_NOT_SEND:
                        performMeasureNotSend();
                        break;
                    case KeyEventAction.MEASURE_AND_RECORD:
                        performMeasureRecord();
                        break;
                }
                return true;
            case 220://中海达记录快捷键
                if (PropertiesGet.getModel() != null) {
                    if ((PropertiesGet.getModel().startsWith("iHand") || PropertiesGet.getModel().startsWith("Q3"))) {
                        performRecord();
                    }
                }
                break;
            case 131://丰疆手薄记录快捷键
                if (AppChannel.customChannel == AppChannel.CUSTOM_FJD) {
                    performRecord();
                }
                break;
        }
        return super.onKeyDown(keyCode, event);
    }

    protected int volumeUpAction() {
//        return BingCeBaseSurveyPresenter.volumeUpAction();
        return 0;
    }

    protected int volumeDownAction() {
//        return BingCeBaseSurveyPresenter.volumeDownAction();
        return 0;
    }

    protected void performMeasure() {

    }

    protected void performRecord() {

    }

    protected void performMeasureRecord() {
    }

    protected void performMeasureNotSend() {
    }

    protected void performNext() {

    }

    protected void performLast() {
    }


    protected void onHelpClicked() {

    }

    protected void onTiltSwitchClick() {
        if (!Gnss.getInstance().getDevice().isSupportTilt()) {
            ToastUtils.showShort(R.string.current_device_not_support_tilt);
        } else {
            if (Gnss.getInstance().getDevice().isTiltOpen()) {
                RTK.getInstance().sendCloseTiltCmd();
            } else {
                RTK.getInstance().sendOpenTiltCmd();
            }
        }
    }

    @Override
    final public void setContentView(View view) {
        //判断view的背景色
        ColorfulActivity.autoSwitchRootViewBgColor(view);
        super.setContentView(view);
    }

    @Override
    public void onDeviceConnectLost() {
        baseSurveyActivityUtils.resetTopStatus();
    }

    @Override
    public void onDeviceConnectFailed() {
        baseSurveyActivityUtils.resetTopStatus();
    }

    @Override
    public void onDeviceConnectNeedOpenBT() {

    }

    @Override
    public void onDeviceConnectSucceed() {

    }

    @Override
    public void onRtkPositionReceived(double antennaHeight, Blh blh, Blh originalBlh, Coordinate coordinate, double hdop, double vdop, double pdop) {

    }

    @Override
    public void onRtkStatusChanged() {
        baseSurveyActivityUtils.refreshRtkStatus();
    }

    @Override
    public void onRtkError(String errorMsg) {
        ToastUtils.showShort(errorMsg);
    }

    @Override
    public void onRtkPoleHeightChanged() {

    }

    @Override
    public void onRtkTiltSwitched(boolean onOff) {

    }

    @Override
    public void onSatelliteSkyViewRefresh() {

    }

    @Override
    public void onMeasCoordinateSuccess(Coordinate coordinate) {

    }

    @Override
    public void onMeasCoordinateFailed(String errorMsg, String original) {

    }

    @Override
    public void onMeasSdAnglesSuccess(double originalHa, double originalVa, double sd, double clockwiseHa, double topVa, double hd, double vd) {
        baseSurveyActivityUtils.onMeasSdAngleSuccess(originalHa, originalVa, sd, clockwiseHa, topVa, hd, vd);
    }

    @Override
    public void onMeasSdAnglesFailed(String errorMsg, String original) {

    }

    @Override
    public void onAnglesRefresh(double originalHa, double originalVa) {

    }

    @Override
    public void onBatteryRefresh(int percent) {
        LogUtils.d("onBatteryRefresh:" + percent);
        baseSurveyActivityUtils.refreshBatteryLevel(percent);
    }

    @Override
    public void onChangeHrSuccess(double hr) {

    }

    @Override
    public void onChangeHrFailed(String errorMsg) {

    }

    @Override
    public boolean isVipCountDownOpen() {
        return true;
    }

    public interface ISurveyorUIMenuContainer {
        BubblePopWindow uiMenu();
    }

    public interface IFragmentBtnHelp {
        void onHelpBtnClicked();
    }

    public interface IFragmentBtnTilt {
        void onTiltBtnClicked();
    }
}