package com.project.survey.ui.controlnet

import androidx.activity.viewModels
import com.project.survey.R
import com.project.survey.databinding.ActivityControlPointBinding
import com.project.survey.extend.smartDismiss
import com.project.survey.logic.viewmodel.ControlNetViewModel
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.ui.controlnet.adapter.SpecialControlNetworkInfoAdapter
import com.project.survey.util.ActivityNavUtil
import com.project.survey.widget.decoration.TransparentDividerDecoration
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener

/**
 * 控制点详情
 */
class ControlPointActivity : BaseBindingActivity<ActivityControlPointBinding>(),
    OnRefreshLoadMoreListener {

    companion object {
        fun start(controlNetworkName: String) {
            ActivityNavUtil.startActivity<ControlPointActivity> {
                putString("controlNetworkName", controlNetworkName)
            }
        }
    }

    override fun getBinding(): ActivityControlPointBinding =
        ActivityControlPointBinding.inflate(layoutInflater)

    private val viewModel: ControlNetViewModel by viewModels()

    private val adapter by lazy { SpecialControlNetworkInfoAdapter() }
    private var controlNetworkName = ""
    private var searchContent: String? = null
    private var currentPage = 1

    override fun initView() {
        controlNetworkName = intent.getStringExtra("controlNetworkName") ?: ""
        immersionToolbar(mBinding.toolbar, "控制点详情")

        mBinding.smartRefreshLayout.setOnRefreshLoadMoreListener(this)
        mBinding.recyclerView.addItemDecoration(TransparentDividerDecoration(this))
        mBinding.recyclerView.adapter = adapter
        adapter.isStateViewEnable = true
    }

    override fun initData() {
        viewModel.errorResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
        }
        viewModel.controlNetworkInfoResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
            if (currentPage == 1) {
                adapter.submitList(it.rows)
                if (it.rows.isEmpty()) {
                    mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    adapter.setStateViewLayout(this, R.layout.layout_no_data)
                } else {
                    if (adapter.itemCount < it.totalCount) {
                        currentPage++
                    } else {
                        mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    }
                }
            } else {
                if (it.rows.isEmpty()) {
                    mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                } else {
                    if (adapter.itemCount < it.totalCount){
                        adapter.addAll(it.rows)
                        currentPage++
                    } else {
                        mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    }
                }
            }
        }
        fetchData()
    }

    override fun initListener() {
        // 搜索
        mBinding.searchWidget.setOnEditorSearchListener {
            searchContent = it.ifBlank { null }
            currentPage = 1
            fetchData()
        }
    }

    private fun fetchData() {
        viewModel.fetchSpecialControlNetworkInfo(controlNetworkName, searchContent, currentPage)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        currentPage = 1
        fetchData()
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        fetchData()
    }

}