package com.project.survey.ui.controlnet

import android.graphics.drawable.BitmapDrawable
import android.view.View
import android.widget.AdapterView
import android.widget.AdapterView.OnItemClickListener
import android.widget.ArrayAdapter
import androidx.activity.viewModels
import androidx.appcompat.widget.ListPopupWindow
import com.bingce.utils.ThreadPoolUtil
import com.project.survey.R
import com.project.survey.databinding.ActivityFirstControlNetworkBinding
import com.project.survey.extend.setOnClickNoRepeatListener
import com.project.survey.extend.smartDismiss
import com.project.survey.logic.viewmodel.ControlNetViewModel
import com.project.survey.model.ControlNetVersionBean
import com.project.survey.model.FirstControlNetworkVersionBean
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.ui.controlnet.adapter.ControlNetFirstAdapter
import com.project.survey.util.ActivityNavUtil
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener

/**
 * 首级控制网列表
 */
class FirstControlNetworkActivity : BaseBindingActivity<ActivityFirstControlNetworkBinding>(),
    OnRefreshLoadMoreListener {

    companion object {
        fun start() {
            ActivityNavUtil.startActivity<FirstControlNetworkActivity> { }
        }
    }

    override fun getBinding(): ActivityFirstControlNetworkBinding {
        return ActivityFirstControlNetworkBinding.inflate(layoutInflater)
    }

    private val viewModel: ControlNetViewModel by viewModels()

    private val adapter by lazy { ControlNetFirstAdapter() }
    private var currentPage = 1
    private var currentVersion: String? = null
    private val listPopupWindow by lazy { ListPopupWindow(this) }
    private val versionList = arrayListOf(FirstControlNetworkVersionBean("全部版本"))

    override fun initView() {
        immersionToolbar(mBinding.toolbar, R.string.head_control_network)

        listPopupWindow.anchorView = mBinding.llSpinner

        mBinding.smartRefreshLayout.setOnRefreshLoadMoreListener(this)
//        mBinding.recyclerView.addItemDecoration(TransparentDividerDecoration(this))
        mBinding.recyclerView.adapter = adapter
        adapter.isStateViewEnable = true
    }

    override fun initData() {
        viewModel.errorResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
        }
        // 版本号列表
        viewModel.firstControlNetworkVersionResponse.observe(this) {
            versionList.addAll(it)
//            mBinding.spinner.setItems(list)

            val adapter = ArrayAdapter<String>(this, R.layout.list_popup_item_spinner, versionList.map { it.KZWZBCG_VERSION })
            listPopupWindow.setAdapter(adapter)
            mBinding.tvSpinner.text = versionList[0].KZWZBCG_VERSION

        }
        viewModel.controlNetCGListResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
            if (currentPage == 1) {
                adapter.submitList(it.rows)
                if (it.rows.isEmpty()) {
                    mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    adapter.setStateViewLayout(this, R.layout.layout_no_data)
                } else {
                    if (adapter.itemCount < it.totalCount) {
                        currentPage++
                    } else {
                        mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    }
                }

            } else {
                if (it.rows.isEmpty()) {
                    mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                } else {
                    if (adapter.itemCount < it.totalCount) {
                        adapter.addAll(it.rows)
                        currentPage++
                    } else {
                        mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    }
                }
            }
        }

        // 获取列表数据
        fetchData()
        // 获取版本号列表
        viewModel.fetchFirstControlNetworkVersion()
    }

    override fun initListener() {
        mBinding.llSpinner.setOnClickNoRepeatListener {
            listPopupWindow.show()
        }
        listPopupWindow.setOnItemClickListener { adapterView, view, i, l ->
            listPopupWindow.dismiss()
            currentVersion = if (i == 0) null else versionList[i].KZWZBCG_VERSION
            mBinding.tvSpinner.text = versionList[i].KZWZBCG_VERSION
            currentPage = 1
            fetchData()
        }
//        mBinding.spinner.setOnItemSelectedListener { _, _, _, item ->
//            val versionBean = item as FirstControlNetworkVersionBean
//            currentVersion =
//                if ("版本号" == versionBean.KZWZBCG_VERSION) null else versionBean.KZWZBCG_VERSION
//            currentPage = 1
//            fetchData()
//            ThreadPoolUtil.executeInMainDelay300 {
//                mBinding.spinner.collapse()
//            }
//        }
    }

    private fun fetchData() {
        viewModel.fetchFistControlNetworkList(currentVersion, currentPage)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        currentPage = 1
        fetchData()
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        fetchData()
    }

}