package com.project.survey.ui.controlnet

import android.widget.ArrayAdapter
import androidx.activity.viewModels
import androidx.appcompat.widget.ListPopupWindow
import com.bingce.utils.ThreadPoolUtil
import com.project.survey.R
import com.project.survey.databinding.ActivitySpecialControlNetworkBinding
import com.project.survey.extend.setOnClickNoRepeatListener
import com.project.survey.extend.smartDismiss
import com.project.survey.logic.viewmodel.ControlNetViewModel
import com.project.survey.model.ControlNetVersionBean
import com.project.survey.model.FirstControlNetworkVersionBean
import com.project.survey.model.SpecialControlNetworkVersionBean
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.ui.controlnet.adapter.SpecialControlNetworkAdapter
import com.project.survey.util.ActivityNavUtil
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener

/**
 * 专用控制网列表
 */
class SpecialControlNetworkActivity :
    BaseBindingActivity<ActivitySpecialControlNetworkBinding>(), OnRefreshLoadMoreListener {

    companion object {
        fun start() {
            ActivityNavUtil.startActivity<SpecialControlNetworkActivity> { }
        }
    }

    override fun getBinding(): ActivitySpecialControlNetworkBinding =
        ActivitySpecialControlNetworkBinding.inflate(layoutInflater)

    private val viewModel: ControlNetViewModel by viewModels()

    private val adapter by lazy { SpecialControlNetworkAdapter() }
    private var currentPage = 1
    private var currentVersion: String? = null
    private var currentName: String? = null // 控制网名称

    //    private var versionList = ArrayList<SpecialControlNetworkVersionBean>()
    private var versionNameList =
        arrayListOf(SpecialControlNetworkVersionBean("全部版本", "全部名称"))
    private var versionList = ArrayList<String>()

    private val nameListPopupWindow by lazy { ListPopupWindow(this) }
    private val versionListPopupWindow by lazy { ListPopupWindow(this) }
    private var versionAdapter: ArrayAdapter<String>? = null

    override fun initView() {
        immersionToolbar(mBinding.toolbar, R.string.dedicated_control_network)

        nameListPopupWindow.anchorView = mBinding.tvSpinnerName
        versionListPopupWindow.anchorView = mBinding.tvSpinnerVersion

        mBinding.smartRefreshLayout.setOnRefreshLoadMoreListener(this)
//        mBinding.recyclerView.addItemDecoration(TransparentDividerDecoration(this))
        mBinding.recyclerView.adapter = adapter
        adapter.isStateViewEnable = true
    }

    override fun initData() {
        viewModel.errorResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
        }
        // 版本号列表
        viewModel.specialControlNetworkVersionResponse.observe(this) {
            versionNameList.addAll(it)
//            versionList.add(0, SpecialControlNetworkVersionBean("版本号", "控制网名称"))
//            mBinding.spinnerName.setItems(versionList.map { v -> v.KZW_NAME })
            val nameAdapter = ArrayAdapter<String>(
                this,
                R.layout.list_popup_item_spinner,
                versionNameList.map { it.KZW_NAME })
            nameListPopupWindow.setAdapter(nameAdapter)
            mBinding.tvSpinnerName.text = versionNameList[0].KZW_NAME

            versionList.clear()
            versionList.addAll(versionNameList[0].version_array.split(",").toList())
            versionAdapter =
                ArrayAdapter<String>(this, R.layout.list_popup_item_spinner, versionList)
            versionListPopupWindow.setAdapter(versionAdapter)
            mBinding.tvSpinnerVersion.text = versionList[0]
//            mBinding.spinnerVersion.setItems(versionList)
        }
        viewModel.controlNetworkListResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
            if (currentPage == 1) {
                adapter.submitList(it.rows)
                if (it.rows.isEmpty()) {
                    mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    adapter.setStateViewLayout(this, R.layout.layout_no_data)
                } else {
                    if (adapter.itemCount < it.totalCount) {
                        currentPage++
                    } else {
                        mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    }
                }
            } else {
                if (it.rows.isEmpty()) {
                    mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                } else {
                    if (adapter.itemCount < it.totalCount) {
                        adapter.addAll(it.rows)
                        currentPage++
                    } else {
                        mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    }
                }
            }
        }

        // 获取列表数据
        fetchData()
        // 获取版本号列表
        viewModel.fetchSpecialControlNetworkVersion()
    }

    override fun initListener() {
        // 控制网名称
        mBinding.tvSpinnerName.setOnClickNoRepeatListener {
            nameListPopupWindow.show()
        }
        nameListPopupWindow.setOnItemClickListener { adapterView, view, i, l ->
            nameListPopupWindow.dismiss()
            currentName = if (i == 0) null else versionNameList[i].KZW_NAME
            mBinding.tvSpinnerName.text = currentName ?: versionNameList[0].KZW_NAME

            versionList.clear()
            versionList.addAll(versionNameList[i].version_array.split(",").toList())
//            val versionList = versionList[position].version_array.split(",").toList()
            versionAdapter?.notifyDataSetChanged()
            currentVersion =
                if (currentName == null) null else if (versionList.isEmpty()) null else versionList[0]

            mBinding.tvSpinnerVersion.text = currentVersion ?: versionNameList[0].version_array

            currentPage = 1
            fetchData()
        }
//        mBinding.spinnerName.setOnItemSelectedListener { view, position, id, item ->
//            val bean = item as String
//            currentName = if ("控制网名称" == bean) null else bean
//
//            val versionList = versionList[position].version_array.split(",").toList()
//            mBinding.spinnerVersion.setItems(versionList)
//            currentVersion = if ("控制网名称" == bean)
//                null
//            else {
//                if (versionList.isEmpty()) null else versionList[0].toString()
//            }
//            currentPage = 1
//            fetchData()
//            ThreadPoolUtil.executeInMainDelay300 {
//                mBinding.spinnerName.collapse()
//            }
//        }
        // 版本号
        mBinding.tvSpinnerVersion.setOnClickNoRepeatListener {
            versionListPopupWindow.show()
        }
        versionListPopupWindow.setOnItemClickListener { adapterView, view, i, l ->
            versionListPopupWindow.dismiss()
            currentVersion = if (i == 0) null else versionList[i]
            mBinding.tvSpinnerVersion.text = versionList[i]

            currentPage = 1
            fetchData()
        }
//        mBinding.spinnerVersion.setOnItemSelectedListener { _, _, _, item ->
//            val version = item as String
//            currentVersion =
//                if ("版本号" == version) null else version
//            currentPage = 1
//            fetchData()
//            ThreadPoolUtil.executeInMainDelay300 {
//                mBinding.spinnerVersion.collapse()
//            }
//        }
        adapter.setOnItemClickListener { adapter, _, position ->
            adapter.getItem(position)?.let { bean ->
                ControlPointActivity.start(bean.KZW_NAME)
            }
        }
    }

    private fun fetchData() {
        viewModel.fetchSpecialControlNetworkList(currentName, currentVersion, currentPage)
    }

    override fun onRefresh(refreshLayout: RefreshLayout) {
        currentPage = 1
        fetchData()
    }

    override fun onLoadMore(refreshLayout: RefreshLayout) {
        fetchData()
    }


}