package com.project.survey.ui.home

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.fragment.app.viewModels
import com.project.survey.constants.Constants
import com.project.survey.constants.EventConstants
import com.project.survey.constants.SPConstants
import com.project.survey.databinding.FragmentHomeBinding
import com.project.survey.extend.setOnClickNoRepeatListener
import com.project.survey.logic.viewmodel.MainViewModel
import com.project.survey.model.ProjectBean
import com.project.survey.ui.approval.ApprovalActivity
import com.project.survey.ui.base.BaseBindingFragment
import com.project.survey.ui.controlnet.FirstControlNetworkActivity
import com.project.survey.ui.controlnet.SpecialControlNetworkActivity
import com.project.survey.ui.lofting.LoftingListActivity
import com.project.survey.ui.pointmeasure.PointMeasureListActivity
import com.project.survey.util.SPUtils

/**
 * 首页
 */
class HomeFragment : BaseBindingFragment<FragmentHomeBinding>() {

    override fun getBinding(inflater: LayoutInflater, container: ViewGroup?): FragmentHomeBinding {
        return FragmentHomeBinding.inflate(inflater, container, false)
    }

    private val viewModel: MainViewModel by viewModels()

    override fun initView() {
    }

    override fun initData() {
        viewModel.msgEvent.observe(this) {
            if (it.msg == EventConstants.UPDATE_PROJECT) {
                updateViewByData()
            }
        }
        updateViewByData()
//        mBinding.itemToBeProcessed.setBadge(4)
    }

    private fun updateViewByData() {
        SPUtils.getParcelable(SPConstants.CURRENT_PROJECT, ProjectBean::class.java)?.let {
            mBinding.tvProject.text = it.XMXX_NAME
        }
    }

    override fun initListener() {
        // 测量计划
        mBinding.itemSurveyPlan.setOnClickNoRepeatListener {
            PointMeasureListActivity.start(Constants.TYPE_BD_CLJH)
        }
        // 工程计量
        mBinding.itemEngineeringMetrology.setOnClickNoRepeatListener {
            PointMeasureListActivity.start(Constants.TYPE_BD_CLGZ_GCJL)
        }
//        //点测量
//        mBinding.itemPointMeasure.setOnClickNoRepeatListener {
//            PointMeasureListActivity.start()
//        }
        //放样
        mBinding.itemLofting.setOnClickNoRepeatListener {
            LoftingListActivity.start()
        }
        mBinding.itemToBeProcessed.setOnClickNoRepeatListener {
            ApprovalActivity.start(ApprovalActivity.PI_PREAPPROV)
        }
        mBinding.itemProcessed.setOnClickNoRepeatListener {
            ApprovalActivity.start(ApprovalActivity.PI_APPROVED)
        }
        mBinding.itemInitiated.setOnClickNoRepeatListener {
            ApprovalActivity.start()
        }
        // 首级控制网
        mBinding.itemControlNetFirst.setOnClickNoRepeatListener {
            FirstControlNetworkActivity.start()
        }
        // 专用控制网
        mBinding.itemControlNetSpecial.setOnClickNoRepeatListener {
            SpecialControlNetworkActivity.start()
        }
    }
}