package com.project.survey.ui.home

import android.content.Intent
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.fragment.app.viewModels
import blankj.utilcode.util.AppUtils
import com.bingce.device.Device
import com.bingce.device.enums.ConnectTypeEnum
import com.bingce.device.enums.DeviceTypeEnum
import com.bingce.device.ui.ConnectRtkActivity
import com.bingce.device.ui.ConnectTSActivity
import com.bingce.rtk.command.RTK
import com.bingce.rtk.config.GnssConfigManager
import com.project.survey.R
import com.project.survey.constants.EventConstants
import com.project.survey.constants.SPConstants
import com.project.survey.databinding.FragmentInstrumentBinding
import com.project.survey.extend.setOnClickNoRepeatListener
import com.project.survey.extend.toast
import com.project.survey.logic.viewmodel.MainViewModel
import com.project.survey.model.ProjectBean
import com.project.survey.ui.base.BaseBindingFragment
import com.project.survey.ui.instrument.basestationtranslation.RtkBaseCorrectActivity
import com.project.survey.ui.instrument.coordinatesystem.BingCeCoordinateSystemActivity
import com.project.survey.ui.instrument.mobilestationmode.GnssConfigSetActivity
import com.project.survey.ui.instrument.mobilestationmode.base.RtkPointCorrectActivity
import com.project.survey.ui.instrument.satellitosis.SatelliteStatusActivity
import com.project.survey.ui.instrument.setupstation.InstrumentSetFragmentSetupStationTypeUtils
import com.project.survey.util.ActivityNavUtil
import com.project.survey.util.SPUtils

/**
 * 仪器
 */
class InstrumentFragment : BaseBindingFragment<FragmentInstrumentBinding>() {

    override fun getBinding(
        inflater: LayoutInflater,
        container: ViewGroup?
    ): FragmentInstrumentBinding = FragmentInstrumentBinding.inflate(inflater, container, false)

    private val viewModel: MainViewModel by viewModels()

    override fun initView() {

    }

    override fun initData() {
        viewModel.msgEvent.observe(this) {
            if (it.msg == EventConstants.UPDATE_PROJECT) {
                updateViewByData()
            }
        }
        updateViewByData()
    }

    private fun updateViewByData() {
        SPUtils.getParcelable(SPConstants.CURRENT_PROJECT, ProjectBean::class.java)?.let {
            mBinding.tvProject.text = it.XMXX_NAME
        }
    }

    override fun initListener() {
        //全站仪联机
        mBinding.itemConnectTotalStation.setOnClickNoRepeatListener {
            ConnectTSActivity.start()
        }
        //RTK联机
        mBinding.itemConnectRTK.setOnClickNoRepeatListener {
            ConnectRtkActivity.start()
        }
        //卫星状态
        mBinding.itemSatelliteStatus.setOnClickNoRepeatListener {
            ActivityNavUtil.startActivity<SatelliteStatusActivity> { }
        }
        //全站仪设站
        mBinding.itemTotalStationSetupStation.setOnClickListener {
            InstrumentSetFragmentSetupStationTypeUtils.chooseSetupStationType(this)
        }
        //移动站模式
        mBinding.itemMobileStationMode.setOnClickListener {
            if (Device.getInstance().isDeviceConnected || AppUtils.isAppDebug()) {
                if (RTK.getInstance().isSupportRoverSetting) {
                    GnssConfigManager.reset() //重置配置，防止未退出重新连接其他品牌型号导致配置集未改变的问题
                    val roverIntent = Intent(requireContext(), GnssConfigSetActivity::class.java)
                    roverIntent.putExtra("sysMode", 0)
                    startActivity(roverIntent)
                } else {
                    toast(R.string.device_not_support_this_mode)
                }
            } else {
                toast(R.string.rtk_not_connected)
            }
        }
        //基准站模式
        mBinding.itemBaseStationMode.setOnClickListener {
            if (Device.getInstance().isDeviceConnected || AppUtils.isAppDebug()) {
                if (RTK.getInstance().isSupportBaseSetting) {
                    GnssConfigManager.reset() //重置配置，防止未退出重新连接其他品牌型号导致配置集未改变的问题
                    val baseIntent = Intent(requireContext(), GnssConfigSetActivity::class.java)
                    baseIntent.putExtra("sysMode", 1)
                    startActivity(baseIntent)
                } else {
                    toast(R.string.device_not_support_this_mode)
                }
            } else {
                toast(R.string.rtk_not_connected)
            }
        }
        //基站平移
        mBinding.itemBaseStationTranslation.setOnClickListener {
            startBaseStationTranslation()
        }
        //坐标系统
        mBinding.itemCoordinateSystem.setOnClickListener {
//            BingCeCoordinateSystemActivity.view(requireContext(), "坐标系统", "")
            BingCeCoordinateSystemActivity.viewAndModifyCurrentCoordinateSystem(requireContext())
        }
        //点校正
        mBinding.itemPointCorrection.setOnClickListener {
            ActivityNavUtil.startActivity<RtkPointCorrectActivity> { }
        }
    }

    private fun startBaseStationTranslation() {
        if (!AppUtils.isAppDebug()) {
            if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
                toast("请选择RTK仪器连接后继续")
                return
            }
            if (Device.getInstance().connectType == ConnectTypeEnum.CONNECT_TYPE_PHONE_LOCATION) {
                toast(R.string.rtk_native_unnecessary)
                return
            }
            if (!Device.getInstance().isDeviceConnected) {
                toast(R.string.please_connect_bluetooth)
                return
            }
        }
        ActivityNavUtil.startActivity<RtkBaseCorrectActivity> { }
    }
}