package com.project.survey.ui.home

import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.fragment.app.viewModels
import blankj.utilcode.util.ActivityUtils
import com.afollestad.materialdialogs.MaterialDialog
import com.project.survey.BuildConfig
import com.project.survey.R
import com.project.survey.constants.EventConstants
import com.project.survey.constants.SPConstants
import com.project.survey.databinding.FragmentMeBinding
import com.project.survey.extend.setOnClickNoRepeatListener
import com.project.survey.logic.event.Message
import com.project.survey.logic.viewmodel.LoginViewModel
import com.project.survey.logic.viewmodel.MainViewModel
import com.project.survey.model.ProjectBean
import com.project.survey.ui.MainActivity
import com.project.survey.ui.base.BaseBindingFragment
import com.project.survey.ui.login.LoginActivity
import com.project.survey.ui.project.ProjectListActivity
import com.project.survey.util.ActivityNavUtil
import com.project.survey.util.LogoutUtil
import com.project.survey.util.SPUtils

/**
 * 我的
 */
class MeFragment : BaseBindingFragment<FragmentMeBinding>() {


    override fun getBinding(inflater: LayoutInflater, container: ViewGroup?): FragmentMeBinding =
        FragmentMeBinding.inflate(inflater, container, false)

    private val viewModel: MainViewModel by viewModels()

    override fun initView() {
    }

    override fun initData() {
        viewModel.msgEvent.observe(this) {
            if (it.msg == EventConstants.UPDATE_PROJECT) {
                updateViewByData()
            }
        }
        mBinding.tvName.text = SPUtils.getString(SPConstants.ACCOUNT)
//        mBinding.tvVersion.text = "v${BuildConfig.VERSION_NAME}"

        updateViewByData()
    }

    private fun updateViewByData() {
        SPUtils.getParcelable(SPConstants.CURRENT_PROJECT, ProjectBean::class.java)?.let {
            mBinding.tvProject.text = it.XMXX_NAME
        }
    }

    override fun initListener() {
        mBinding.tvLoginOut.setOnClickNoRepeatListener {
            // 退出登录
            MaterialDialog.Builder(mContext)
                .title("确定退出吗?")
                .negativeText("取消")
                .positiveText("退出")
                .onPositive { _, _ ->
                    LogoutUtil.clearSP()
                    ActivityNavUtil.finishActivity<MainActivity>()
                    ActivityNavUtil.startActivity<LoginActivity> { }
//                    viewModel.msgEvent.postValue(Message(EventConstants.LOGIN_STATUS))
                }
                .show()
//            if (SPUtils.getString(SPConstants.TOKEN).isNullOrBlank()) {
//                // 登录
//                LoginActivity.start()
//            } else {
//
//            }
        }
        mBinding.llSwitchProject.setOnClickNoRepeatListener {
            ProjectListActivity.start()
        }
        // TODO:  
        mBinding.llAbout.setOnClickNoRepeatListener {
//            viewModel.sendSMS("17379748209")
        }
    }
}