package com.project.survey.ui.instrument.coordinatesystem;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import androidx.annotation.Nullable;
import androidx.appcompat.widget.Toolbar;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;

import com.bingce.activity.AutoLandscapeBingCeActivity;
import com.bingce.device.ui.coordinatesystem.fragment.AbstractCoordinateSystemContainerFragment;
import com.bingce.device.ui.databinding.ActivityCoordinateSystemBinding;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.StringUtil;
import com.project.survey.R;
import com.project.survey.ui.instrument.coordinatesystem.fragment.CoordinateSystemContainerFragment;

import blankj.utilcode.util.Utils;


public class BingCeCoordinateSystemActivity extends AutoLandscapeBingCeActivity {
    @Override
    protected void onBingCeCreate(@Nullable Bundle savedInstanceState) {
        ActivityCoordinateSystemBinding binding = ActivityCoordinateSystemBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        Toolbar mToolbar = findViewById(R.id.toolbar);
//        mToolbar.setTitle(R.string.coordinate_system);
        setSupportActionBar(mToolbar);
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        boolean isReturnSys = getIntent().getBooleanExtra(CoordinateSystemContainerFragment.KEY_RETURN_SYS, false);
        if (isReturnSys) {
            binding.activityCoordinateSystemBtnCalc.setVisibility(View.GONE);
        }

        Fragment coordinatorFragment = null;
        if (getIntent() != null && getIntent().getExtras() != null) {
            coordinatorFragment = CoordinateSystemContainerFragment.newInstance(getIntent().getExtras());
            String title = getIntent().getStringExtra(CoordinateSystemContainerFragment.KEY_TITLE);
            if (title != null && !title.isEmpty()) {
                setTitle(title);
            }
        }

        if (coordinatorFragment == null) {
            coordinatorFragment = new CoordinateSystemContainerFragment();
        }

        getSupportFragmentManager()
                .beginTransaction()
                .add(R.id.activity_coordinate_system_fragment, coordinatorFragment)
                .commit();

        Fragment finalCoordinatorFragment = coordinatorFragment;
        binding.activityCoordinateSystemBtnQuit.setOnClickListener(v -> {
            //退出
            if (finalCoordinatorFragment instanceof CoordinateSystemContainerFragment) {
                ((CoordinateSystemContainerFragment) finalCoordinatorFragment).onQuitClicked();
            }
        });

        binding.activityCoordinateSystemBtnAccept.setOnClickListener(v -> {
            //接受
            if (finalCoordinatorFragment instanceof CoordinateSystemContainerFragment) {
                ((CoordinateSystemContainerFragment) finalCoordinatorFragment).onAcceptClicked();
            }
        });

        binding.activityCoordinateSystemBtnCalc.setOnClickListener(v -> {
            //参数解算
            if (finalCoordinatorFragment instanceof CoordinateSystemContainerFragment) {
                ((CoordinateSystemContainerFragment) finalCoordinatorFragment).onCalcClicked();
            }
        });

        boolean isReadOnly = IntentUtil.boolExtra(getIntent(), CoordinateSystemContainerFragment.KEY_READ_ONLY);
        binding.bottomLayout.setVisibility(isReadOnly ? View.GONE : View.VISIBLE);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
//        getMenuInflater().inflate(R.menu.menu_activity_rtk_coordinate_system, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int itemId = item.getItemId();
        if (itemId == android.R.id.home) {
            finish();
        } else if (itemId == R.id.menu_help) {
            if (Utils.getApp() instanceof IHelper) {
                ((IHelper) Utils.getApp()).onNavigation2CoordinateSystemHelp();
            }
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        switch (keyCode) {
            case KeyEvent.KEYCODE_BACK:
                finish();
                break;
        }
        return true;
    }

    /**
     * 查看并修改当前坐标系,并且直接跳转到椭球参数页面
     */
    public static void skip2ProjViewAndModifyCurrentCoordinateSystem(Context context) {
        Intent intent = IntentUtil.forceStaticMethodIntent(context, BingCeCoordinateSystemActivity.class);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_READ_ONLY, false);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_RETURN_SYS, false);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_SKIP2PROJ, true);
        context.startActivity(intent);
    }

    /**
     * 查看坐标系，不能修改
     */
    public static void view(Context context, String title, String jsonString) {
        Intent intent = IntentUtil.forceStaticMethodIntent(context, BingCeCoordinateSystemActivity.class);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_TITLE, title);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_TXT, jsonString);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_READ_ONLY, true);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_RETURN_SYS, false);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_SKIP2PROJ, false);
        context.startActivity(intent);
    }

    /**
     * 查看并修改当前坐标系
     */
    public static void viewAndModifyCurrentCoordinateSystem(Context context) {
        Intent intent = IntentUtil.forceStaticMethodIntent(context, BingCeCoordinateSystemActivity.class);
        context.startActivity(intent);
    }

    /**
     * 选择「预设」的坐标系后，进入其详情界面尝试修改，然后返回修改后的json值
     */
    public static void modifyPresetCoordinateSystemBeforeReturn(
            FragmentActivity context, String jsonString, String coordinateSystemName,
            boolean readonly, boolean returnSys, ICoordinateSystem coordinateSystem) {
        Intent intent = IntentUtil.forceStaticMethodIntent(context, BingCeCoordinateSystemActivity.class);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_TITLE, coordinateSystemName);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_TXT, jsonString);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_READ_ONLY, readonly);
        intent.putExtra(CoordinateSystemContainerFragment.KEY_RETURN_SYS, returnSys);

        ActivityUtils.startActivityForResults(context, intent, result -> {
            if (result != null && result.getResultCode() == RESULT_OK) {
                Intent data = result.getData();
                String json = data == null ? null : data.getStringExtra(AbstractCoordinateSystemContainerFragment.KEY_TXT);
                if (coordinateSystem != null && !StringUtil.isEmpty(json)) {
                    coordinateSystem.onGetCoordinateSystemJson(json);
                }
            }
        });
    }

    public interface ICoordinateSystem {
        void onGetCoordinateSystemJson(String jsonString);
    }

    public interface IHelper {
        void onNavigation2CoordinateSystemHelp();
    }
}