package com.project.survey.ui.instrument.coordinatesystem;

import android.content.Context;

import androidx.annotation.WorkerThread;
import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LiveData;
import androidx.lifecycle.MutableLiveData;

import com.bingce.coordlib.model.CoordinateSystem;
import com.bingce.data.cache.CacheHelper;
import com.bingce.data.cache.CachedObserver;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.KeyValueDb;
import com.bingce.data.database.ProjectDb;
import com.bingce.data.surveyor.designdata.kv.KeyValueConstants;
import com.bingce.data.surveyor.designdata.project.ProjectConstants;
import com.bingce.data.surveyor.designdata.project.ProjectRecord;
import com.bingce.utils.IProvider;
import com.bingce.utils.KVLiveDataIgnoreUserSwitch;

import blankj.utilcode.util.Utils;

public class CachedCurrentCoordinateSystem extends CacheHelper {
    private final KVLiveDataIgnoreUserSwitch currentProjectKV = new KVLiveDataIgnoreUserSwitch(KeyValueConstants.KEY_CURRENT_PROJECT_ID);
    private final IProvider<Context> context;
    //当前项目实例
    private LiveData<ProjectRecord> projectRecordLiveData = null;
    //当前坐标系实例
    private final MutableLiveData<CoordinateSystem> currentCoordinateSystemMutableLiveData = new MutableLiveData<>();

    public CachedCurrentCoordinateSystem(AppCompatActivity activity) {
        super(activity.getLifecycle());
        context = () -> activity;
        addLifeCycle(getLifecycle());
    }

    public CachedCurrentCoordinateSystem(Fragment fragment) {
        super(fragment.getLifecycle());
        context = fragment::getContext;
        addLifeCycle(getLifecycle());
    }

    protected void addLifeCycle(Lifecycle lifecycle) {
        //1.监听当前坐标系id
        currentProjectKV
                .observe(this, configRecord -> {
                    if (configRecord == null) {
                        return;
                    }
                    if (projectRecordLiveData != null) {
                        projectRecordLiveData.removeObservers(this);
                    }
                    projectRecordLiveData = ProjectDb.getInstance()
                            .rawQueryLiveData(DBQueryConstant.findById(ProjectConstants.DB_NAME, configRecord.value));

                    projectRecordLiveData.observe(this, projectRecord -> {
                        CoordinateSystem coordinateSystem = projectRecord == null ? null : projectRecord.coordinateSystem;
                        currentCoordinateSystemMutableLiveData.setValue(coordinateSystem == null ? CoordinateSystem.newCoordinateSystem(context.value()) : coordinateSystem);
                    });
//                    currentCoordinateSystemMutableLiveData.setValue(CoordinateSystem.newCoordinateSystem(context.value()));

                });
    }

    public CoordinateSystem coordinateSystem() {
        CoordinateSystem coordinateRecord = currentCoordinateSystemMutableLiveData.getValue();
        if (coordinateRecord == null) {
            throw new RuntimeException("coordinate system not ready");
        }
        return coordinateRecord;
    }

    public boolean isDataReady() {
        return currentCoordinateSystemMutableLiveData.getValue() != null;
    }

    public void observe(CachedObserver<CoordinateSystem> observer) {
        currentCoordinateSystemMutableLiveData.observe(this, observer);
    }


    @WorkerThread
    public static CoordinateSystem currentCoordinateSystem() {
        String projectId = KeyValueDb.currentProjectId();
        ProjectRecord projectRecord = ProjectDb.getInstance()
                .rawQueryData(DBQueryConstant.findById(ProjectConstants.DB_NAME, projectId));
        return projectRecord == null ? CoordinateSystem.newCoordinateSystem(Utils.getApp()) : projectRecord.coordinateSystem;
    }

    @WorkerThread
    public static void saveCurrentCoordinateSystem(CoordinateSystem system) {
        String projectId = KeyValueDb.currentProjectId();
        ProjectRecord projectRecord = ProjectDb.getInstance()
                .rawQueryData(DBQueryConstant.findById(ProjectConstants.DB_NAME, projectId));
        if (projectRecord == null) {
            return;
        }
        projectRecord.coordinateSystem = system;
        ProjectDb.getInstance().save(projectRecord);
    }
}
