package com.project.survey.ui.instrument.coordinatesystem

import android.R
import android.os.Bundle
import android.view.MenuItem
import androidx.activity.viewModels
import com.bingce.coordlib.util.CoordinateSystemUtil
import com.project.survey.constants.EventConstants
import com.project.survey.databinding.ActivityCoordinateSystemListBinding
import com.project.survey.extend.setOnClickNoRepeatListener
import com.project.survey.extend.toast
import com.project.survey.logic.event.Message
import com.project.survey.logic.viewmodel.CoordinateSystemViewModel
import com.project.survey.util.ActivityNavUtil
import org.polaric.colorful.ColorfulActivity

/**
 * 坐标系统列表
 */
class CoordinateSystemListActivity : ColorfulActivity() {

    private lateinit var binding: ActivityCoordinateSystemListBinding
    private val adapter by lazy { CoordinateSystemAdapter() }
    private val viewModel: CoordinateSystemViewModel by viewModels()

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityCoordinateSystemListBinding.inflate(layoutInflater, null, false)
        setContentView(binding.root)
        setSupportActionBar(binding.toolbar)
        supportActionBar?.setDisplayHomeAsUpEnabled(true)

        binding.recyclerView.itemAnimator = null
        binding.recyclerView.adapter = adapter

        viewModel.downloadCoordinateSystemResponse.observe(this) {
            adapter.submitList(it.map { data ->
                CoordinateSystemUtil.importCoordSysString(data.values.ZBZHCS_ZHCS)
            })
        }
        viewModel.downloadCoordinateSystem()

        // 选择
        binding.btnChoose.setOnClickNoRepeatListener {
            if (adapter.getSelectedPosition() == -1) {
                toast("请选择坐标系统")
                return@setOnClickNoRepeatListener
            }
            if (adapter.getSelectedPosition() < adapter.itemCount) {
                adapter.getItem(adapter.getSelectedPosition())?.let { item ->
                    viewModel.msgEvent.postValue(
                        Message(EventConstants.CHOOSE_COORDINATE_SYSTEM, obj = item)
                    )
                    finish()
                }
            }
        }
    }

    override fun onOptionsItemSelected(item: MenuItem): Boolean {
        if (R.id.home == item.itemId) {
            onBackPressed()
            return true
        }
        return super.onOptionsItemSelected(item)
    }

    companion object {
        @JvmStatic
        fun start() {
            ActivityNavUtil.startActivity<CoordinateSystemListActivity> {}
        }
    }
}