package com.project.survey.ui.instrument.coordinatesystem.fragment;

import android.app.Activity;
import android.content.Intent;
import android.graphics.Color;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentPagerAdapter;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.viewpager.widget.ViewPager;

import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.coordlib.model.CoordinateSystem;
import com.bingce.coordlib.util.CoordinateSystemUtil;
import com.bingce.data.base.user.UserConfig;
import com.bingce.data.cache.AbstractCachedObserver;
import com.bingce.data.database.AppDataBase;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.KeyValueDb;
import com.bingce.data.database.ProjectDb;
import com.bingce.data.surveyor.designdata.kv.KeyValueConstants;
import com.bingce.data.surveyor.designdata.kv.KeyValueRecord;
import com.bingce.data.surveyor.designdata.project.ProjectConstants;
import com.bingce.data.surveyor.designdata.project.ProjectRecord;
import com.bingce.device.Device;
import com.bingce.device.ui.coordinatesystem.event.UpdateCoordinateSystemEvent;
import com.bingce.device.ui.coordinatesystem.fragment.AbstractCoordinateSystemContainerFragment;
import com.bingce.device.ui.coordinatesystem.fragment.BaseCorrectionFragment;
import com.bingce.device.ui.coordinatesystem.fragment.CoordinatorFragmentInterface;
import com.bingce.device.ui.coordinatesystem.fragment.DatumTransformFragment;
import com.bingce.device.ui.coordinatesystem.fragment.ElevationFittingFragment;
import com.bingce.device.ui.coordinatesystem.fragment.EllipsoidFragment;
import com.bingce.device.ui.coordinatesystem.fragment.GridTransformFragment;
import com.bingce.device.ui.coordinatesystem.fragment.ProjectionFragment;
import com.bingce.device.ui.utils.CoordSysShareUtil;
import com.bingce.error.BingCeErrorCode;
import com.bingce.surveyorBase.R;
import com.bingce.utils.SoftKeyUtils;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.ValidateUtil;
import com.flyco.tablayout.SlidingTabLayout;
import com.project.survey.constants.EventConstants;
import com.project.survey.constants.SPConstants;
import com.project.survey.logic.event.Message;
import com.project.survey.logic.viewmodel.CoordinateSystemViewModel;
import com.project.survey.ui.instrument.coordinatesystem.CachedCurrentCoordinateSystem;
import com.project.survey.ui.instrument.coordinatesystem.CoordinateSystemListActivity;
import com.project.survey.ui.instrument.mobilestationmode.base.RtkPointCorrectActivity;
import com.project.survey.util.SPUtils;
import com.rengwuxian.materialedittext.MaterialEditText;

import org.greenrobot.eventbus.EventBus;
import org.greenrobot.eventbus.Subscribe;
import org.greenrobot.eventbus.ThreadMode;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.StringUtils;
import blankj.utilcode.util.ToastUtils;

public class CoordinateSystemContainerFragment extends AbstractCoordinateSystemContainerFragment {
    protected final CachedCurrentCoordinateSystem cachedCoordinateSystem = new CachedCurrentCoordinateSystem(this);
    private MaterialEditText met_name;
    private ViewPager mViewPager;
    private SlidingTabLayout mSlidingTabLayout;
    private boolean isReturnSys = false;
    private boolean isReadOnly = false;
    private boolean skip2Proj = false;
    private final CoordSysShareUtil.Export export = new CoordSysShareUtil.Export();
    private final CoordSysShareUtil.Import anImport = new CoordSysShareUtil.Import();

    private CoordinateSystemViewModel viewModel;

    @Override
    public void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        EventBus.getDefault().register(this);
    }

    protected View contentView(LayoutInflater inflater, @Nullable ViewGroup container) {
        return inflater.inflate(R.layout.fragment_coordinate_system, container, false);
    }

    @Override
    public View onCreateView(LayoutInflater inflater, @Nullable ViewGroup container, @Nullable Bundle savedInstanceState) {
        View view = contentView(inflater, container);
        met_name = view.findViewById(R.id.fragment_coordinate_system_name);
        mViewPager = view.findViewById(R.id.fragment_coordinate_system_viewpager);
        mSlidingTabLayout = view.findViewById(R.id.fragment_coordinate_system_viewpager_tab);
        return view;
    }

    @Override
    protected void onViewReady(CoordinatorFragmentInterface.ICoordinatorContainerFragment container, CoordinateSystem coordinateSystem) {
        setHasOptionsMenu(true);
        initMaterialEditTextColor();
        met_name.setText(coordinateSystem.getName());

        String[] mTitles = new String[]{getString(R.string.ellipsoid), getString(R.string.projection)
                , getString(R.string.datum_transform), getString(R.string.plane_transform)
                , getString(R.string.elevation_fitting), getString(R.string.base_correction_parameters)};

        if (getArguments() != null) {
            isReadOnly = getArguments().getBoolean(KEY_READ_ONLY, false);
            //如果是只读，需要隐藏底部button,将name设置为只读
            if (isReadOnly && getView() != null) {
                View bottomLayout = getView().findViewById(R.id.bottom_layout);
                if (bottomLayout != null) {
                    bottomLayout.setVisibility(View.INVISIBLE);
                }
                met_name.setEnabled(false);
            }
        }
        ArrayList<Fragment> mFragments = new ArrayList<>();
        mFragments.add(EllipsoidFragment.newInstance(getArguments()));
        mFragments.add(ProjectionFragment.newInstance(getArguments()));
        mFragments.add(DatumTransformFragment.newInstance(getArguments()));
        mFragments.add(GridTransformFragment.newInstance(getArguments()));
        mFragments.add(ElevationFittingFragment.newInstance(getArguments()));
        mFragments.add(BaseCorrectionFragment.newInstance(getArguments()));

        for (Fragment fragment : mFragments) {
            if (fragment instanceof CoordinatorFragmentInterface.ICoordinatorAttributeFragment) {
                container.register((CoordinatorFragmentInterface.ICoordinatorAttributeFragment) fragment);
            }
        }

        mViewPager.setAdapter(new MyPagerAdapter(getChildFragmentManager(), mTitles, mFragments));
        mViewPager.setOffscreenPageLimit(10);

        mSlidingTabLayout.setViewPager(mViewPager);

        mViewPager.setElevation(0);
        mSlidingTabLayout.setElevation(10);

        if (getArguments() != null) {
            if (getArguments().containsKey("tabColor") && mSlidingTabLayout != null)
                mSlidingTabLayout.setBackgroundColor(getArguments().getInt("tabColor"));
            isReturnSys = getArguments().getBoolean(KEY_RETURN_SYS, false);
            skip2Proj = getArguments().getBoolean(KEY_SKIP2PROJ, false);
            if (skip2Proj) {
                mSlidingTabLayout.setCurrentTab(1, true);
            }
        }

        viewModel = new ViewModelProvider(this).get(CoordinateSystemViewModel.class);
        viewModel.getMsgEvent().observe(this, new Observer<Message>() {
            @Override
            public void onChanged(Message message) {
                if (message.getMsg().equals(EventConstants.CHOOSE_COORDINATE_SYSTEM)) {
                    CoordinateSystem system = (CoordinateSystem) message.getObj();
                    onCoordinateSystemChanged(system);
                }
            }
        });
        // 分享坐标系统（上传到服务器）
        viewModel.getUploadCoordinateSystemResponse().observe(this, s -> {
            ToastUtils.showShort(s);
        });
        // 更新坐标系统
        viewModel.getUpdateCoordinateSystemResponse().observe(this, s ->{
            ToastUtils.showShort(s);
        });
        // 导入坐标系统
//        viewModel.getDownloadCoordinateSystemResponse().observe(this, data -> {
//            CoordinateSystem system = CoordinateSystemUtil.importCoordSysString(data.getZBZHCS_ZHCS());
//            onCoordinateSystemChanged(system);
//            ToastUtils.showShort("导入成功");
//        });
    }

    @Override
    protected void observeCurrentCoordinateSystem(AbstractCachedObserver<CoordinateSystem> observer) {
        cachedCoordinateSystem.observe(observer);
    }

    public void onQuitClicked() {
        getActivity().finish();
    }

    public void onAcceptClicked() {
        SoftKeyUtils.hideSoftKey();
        save(() -> {
            ToastUtils.showShort(R.string.save_success);
//            getActivity().finish();
        });
    }

    public void onCalcClicked() {
        mSlidingTabLayout.setCurrentTab(1, true);
        new MaterialDialog.Builder(requireContext())
                .title(R.string.hint)
                .content(R.string.seven_or_four_para_hint)
                .positiveText(R.string.go_to_config)
                .negativeText(R.string.configured)
                .onNegative((dialog, which) -> {
                    save(() -> {
                        Intent intent = new Intent(requireContext(), RtkPointCorrectActivity.class);
                        intent.putExtra("from", 1);
                        startActivity(intent);
//                        EventBus.getDefault().post(new SingleMessageEvent("coordinate_system_parameter_calc"));
                    });
                })
                .show();
    }

    private void save(Runnable success) {
        String name = ValidateUtil.filterSpecialChar(met_name.getText().toString().trim());
        if (StringUtils.isEmpty(name)) {
            return;
        }
        setCoordinateSystemName(name);
        checkFragmentsAndSyncData2Object(new CoordinatorFragmentInterface.ISaveCallback() {
            @Override
            public void onSuccess(CoordinateSystem coordinateSystem) {
                if (isReturnSys) {
                    Intent intent = new Intent();
                    intent.putExtra(KEY_TXT, CoordinateSystemUtil.exportCoordSysString(coordinateSystem));
                    getActivity().setResult(Activity.RESULT_OK, intent);
                    success.run();
                } else {
                    ThreadPoolUtil.execute(() -> {
                        //搜索当前坐标系
                        KeyValueRecord currentProjectConfig = KeyValueDb.getInstance().findData(KeyValueConstants.KEY_CURRENT_PROJECT_ID);
                        if (currentProjectConfig == null) {
                            BingCeErrorCode.toast(BingCeErrorCode.NO_CURRENT_PROJECT);
                            return;
                        }
                        // 存坐标系
                        ProjectRecord currentProjectRecord = ProjectDb.getInstance()
                                .rawQueryData(DBQueryConstant.findById(ProjectConstants.DB_NAME, currentProjectConfig.value));
                        if (currentProjectRecord == null) {
                            BingCeErrorCode.toast(BingCeErrorCode.NO_FOUND_PROJECT_BY_ID);
                            return;
                        }
                        currentProjectRecord.coordinateSystem = coordinateSystem;
                        UserConfig userConfig = AppDataBase.getInstance().userConfigDao().findByLeanCloudUserId(SPUtils.INSTANCE.getString(SPConstants.PERSON_ID, ""));
                        ProjectDb.getInstance().userSafe(userConfig).save(currentProjectRecord);
                        //切换线路
                        ThreadPoolUtil.executeInMain(success);
                    });
                }
            }

            @Override
            public void onFailure(int index, CoordinatorFragmentInterface.ICoordinatorAttributeFragment listener) {
                mSlidingTabLayout.setCurrentTab(index, true);
            }
        });
    }

    @Override
    public void onCreateOptionsMenu(Menu menu, MenuInflater inflater) {
        super.onCreateOptionsMenu(menu, inflater);
        if (getArguments() != null) {
            isReadOnly = getArguments().getBoolean(KEY_READ_ONLY, false);
        }
        if (!isReadOnly && !isReturnSys) {//如果只读，就不显示右上角菜单
            inflater.inflate(R.menu.menu_fragment_coordinate_system, menu);
        }
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int itemId = item.getItemId();
        /*if (itemId == R.id.menu_fragment_coordinate_system_library) {
            FragmentActivity activity = getActivity();
            if (activity != null) {
                PresetCoordinateSystemV2Activity.selectCoordinateSystem(activity, coordinateSystem -> {
                    if (coordinateSystem != null) {
                        met_name.setText(coordinateSystem.getName());
                        onCoordinateSystemChanged(coordinateSystem);
                    }
                });
            }
        } else if (itemId == R.id.menu_fragment_coordinate_system_save) {
            //先收集界面内数据
            String name = ValidateUtil.filterSpecialChar(met_name.getText().toString().trim());
            if (StringUtils.isEmpty(name)) {
                ToastUtils.showShort(R.string.coordinate_system_name_cannot_be_empty);
                return true;
            }
            setCoordinateSystemName(name);
            checkFragmentsAndSyncData2Object(new CoordinatorFragmentInterface.ISaveCallback() {
                @Override
                public void onSuccess(CoordinateSystem coordinateSystem) {
                    ThreadPoolUtil.execute(() -> {
                        List<CoordinateSystemRecord> records = CoordinateSystemDb.getInstance()
                                .rawQueryListData(DBQueryConstant.findAll(CoordinateSystemConstants.DB_NAME));
                        //检测是否有重名
                        List<CoordinateSystemRecord> sameNameList = new ArrayList<>();
                        if (records != null) {
                            for (CoordinateSystemRecord record : records) {
                                if (record == null) {
                                    continue;
                                }
                                if (record.coordinateSystem.getName().equals(coordinateSystem.getName())) {
                                    sameNameList.add(record);
                                }
                            }
                        }
                        if (sameNameList.isEmpty()) {
                            CoordinateSystemRecord record = new CoordinateSystemRecord(coordinateSystem, new Date());
                            CoordinateSystemDb.getInstance().save(record);
                            ToastUtils.showShort(R.string.save_success);
                        } else {
                            ThreadPoolUtil.executeInMain(() -> {
                                DialogUtils.tips(getContext(),
                                        getString(R.string.warning),
                                        getString(R.string.already_has_same_name_coordinate_system_r_u_sure_overwrite),
                                        () -> {
                                            //覆盖存储
                                            ThreadPoolUtil.execute(() -> {
                                                for (CoordinateSystemRecord record : sameNameList) {
                                                    record.coordinateSystem = coordinateSystem;
                                                }
                                                CoordinateSystemDb.getInstance().save(sameNameList);
                                                ToastUtils.showShort(R.string.save_success);
                                            });
                                        },
                                        null);
                            });
                        }
                    });
                }

                @Override
                public void onFailure(int index, CoordinatorFragmentInterface.ICoordinatorAttributeFragment listener) {
                    mSlidingTabLayout.setCurrentTab(index, true);
                }
            });
        } else*/
        if (itemId == R.id.menu_fragment_coordinate_system_share) { //上传
//            showShareParametersDialog();
            new MaterialDialog.Builder(requireContext())
                    .title("提示")
                    .content("上传坐标系统参数后，其他用户可下载使用，确认将该坐标参数上传至平台吗？")
                    .positiveText(R.string.confirm)
                    .negativeText(R.string.cancel)
                    .onPositive((materialDialog, dialogAction) -> {
                        String coordinateSystemString = CoordinateSystemUtil.exportCoordSysString(currentCoordinateSystem());
                        String name = currentCoordinateSystem().getName();
                        String coordinateSystemId = SPUtils.INSTANCE.getString(SPConstants.CURRENT_COORDINATE_SYSTEM_ID, "");
                        if (coordinateSystemId.isEmpty()) { //上传新的
                            viewModel.uploadCoordinateSystem(coordinateSystemString, name);
                        } else {  //修改
                            viewModel.updateCoordinateSystem(coordinateSystemId, coordinateSystemString, name);
                        }
                    })
                    .show();
        } else if (itemId == R.id.menu_fragment_coordinate_system_import) { //下载
//            showImportParametersDialog();
            CoordinateSystemListActivity.start();
        }
        return super.onOptionsItemSelected(item);
    }

    private void showShareParametersDialog() {
        export.showSharedDialog(getContext(), currentCoordinateSystem());
    }

    private void showImportParametersDialog() {
        anImport.showImportDialog(getContext(), system -> {
            met_name.setText(system.getName());
            onCoordinateSystemChanged(system);
        });
    }

    @Subscribe(threadMode = ThreadMode.MAIN)
    public void onMessageEvent(UpdateCoordinateSystemEvent event) {
        onCoordinateSystemChanged(null);
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        EventBus.getDefault().unregister(this);
    }

    private void initMaterialEditTextColor() {
        if (Device.getInstance().isThemeDark) {
            met_name.setMetTextColor(Color.WHITE);

            met_name.setPrimaryColor(Color.LTGRAY);

            met_name.setFocusFraction(1.0f);

            met_name.setMetHintTextColor(Color.LTGRAY);

            met_name.setUnderlineColor(Color.LTGRAY);
        }
    }

    public static class MyPagerAdapter extends FragmentPagerAdapter {
        private final String[] mTitle;
        private final List<Fragment> mFragments;

        public MyPagerAdapter(@NonNull FragmentManager fm, String[] mTitle, List<Fragment> lists) {
            super(fm);
            this.mTitle = mTitle;
            this.mFragments = lists;
        }

        @NonNull
        @Override
        public Fragment getItem(int position) {
            return mFragments.get(position);
        }

        @Nullable
        @Override
        public CharSequence getPageTitle(int position) {
            return mTitle[position];
        }

        @Override
        public int getCount() {
            return mFragments != null && mFragments.size() == 0 ? 0 : mFragments.size();
        }
    }

    public Fragment getViewPagerFragment(int viewId, int index) {
        String tag = "android:switcher:" + viewId + ":" + index;
        return getChildFragmentManager().findFragmentByTag(tag);
    }

    public static Fragment newInstance(Bundle args) {
        CoordinateSystemContainerFragment fragment = new CoordinateSystemContainerFragment();
        fragment.setArguments(args);
        return fragment;
    }
}
