package com.project.survey.ui.instrument.mobilestationmode.base;

import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.coordlib.enums.EllipsoidEnum;
import com.bingce.coordlib.model.CoordinateSystem;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.KeyValueDb;
import com.bingce.data.database.ProjectDb;
import com.bingce.data.surveyor.designdata.project.ProjectConstants;
import com.bingce.utils.IOnSingleGetCallback;
import com.bingce.utils.ThreadPoolUtil;
import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.ui.instrument.basestationtranslation.RtkBaseCorrectActivity;
import com.project.survey.ui.instrument.setupstation.BingCeBaseSurveyPresenter;
import com.project.survey.ui.instrument.setupstation.ControlPointsNewActivity;
import com.project.survey.ui.instrument.setupstation.CoordinatePointsLibraryActivity;
import com.project.survey.ui.instrument.setupstation.StakingNewJobActivity;

import blankj.utilcode.util.Utils;


public class RtkMatchingPointsActivity extends AbstractRtkMatchingPointsActivity {
    @Override
    protected void hideSoftKey() {
        BingCeBaseSurveyPresenter.hideSoftKey();
    }

    @Override
    protected void speak(String string) {
        BingCeBaseSurveyPresenter.speak(string);
    }

    @Override
    protected void updateLastRTKLocation() {
        BingCeBaseSurveyPresenter.updateLastRTKLocation();
    }

    @Override
    protected boolean checkRegister() {
        return BingCeBaseSurveyPresenter.checkRegister();
    }

    @Override
    protected int volumeUpAction() {
        return BingCeBaseSurveyPresenter.volumeUpAction();
    }

    @Override
    protected int volumeDownAction() {
        return BingCeBaseSurveyPresenter.volumeDownAction();
    }

    // 大地坐标数据源
    @Override
    protected void selectBLH() {
        new MaterialDialog.Builder(RtkMatchingPointsActivity.this)
                .title(R.string.data_source)
                .items(new String[]{getString(R.string.point_survey_point_library)})
                .itemsCallback((dialog, itemView, which, text) -> {
                    switch (which) {
                        case 0:
                            CoordinatePointsLibraryActivity.pickPoint(this, (name, code, x, y, z, b, l, h) -> {
                                onPicked(b, l, h);
                            });
                            break;
                    }
                }).show();
    }

    // 平面坐标数据源
    @Override
    protected void selectXYZ() {
        // 放样点库
        StakingNewJobActivity.pickPoint(this, (name, x, y, z) -> {
            onPicked(name, x, y, z);
        });
//        new MaterialDialog.Builder(RtkMatchingPointsActivity.this)
//                .title(R.string.data_source)
//                .items(new String[]{getString(R.string.road_control_point), getString(R.string.global_control_point), getString(R.string.point_survey_point_library), getString(R.string.staking_point_library)})
//                .itemsCallback((dialog, itemView, which, text) -> {
//                    switch (which) {
//                        case 0:
//                            ControlPointsNewActivity.pickPoint(this, false, (name, code, x, y, z, remarks) -> {
//                                onPicked(name, x, y, z);
//                            });
//                            break;
//                        case 1:
//                            ControlPointsNewActivity.pickPoint(this, true, (name, code, x, y, z, remarks) -> {
//                                onPicked(name, x, y, z);
//                            });
//                            break;
//                        case 2:
//                            CoordinatePointsLibraryActivity.pickPoint(this, (name, code, x, y, z, b, l, h) -> {
//                                onPicked(name, x, y, z);
//                            });
//                            break;
//                        case 3:
//                            StakingNewJobActivity.pickPoint(this, (name, x, y, z) -> {
//                                onPicked(name, x, y, z);
//                            });
//                            break;
//                    }
//                }).show();
    }

    @Override
    protected boolean isThemeDark() {
        return App.instance.isThemeDark();
    }

    @Override
    protected void save(CoordinateSystem coordinateSystem) {
        ProjectDb.getInstance().updateCurrentCoordinator(coordinateSystem);
    }

    @Override
    protected void loadCurrentCoordinateSystem(IOnSingleGetCallback<CoordinateSystem> callback) {
        // 获取坐标系
        ThreadPoolUtil.execute(() -> {
            String currentProjectId = KeyValueDb.currentProjectId();
            runOnUiThread(() -> {
                ProjectDb.getInstance().rawQueryLiveData(
                                DBQueryConstant.findById(ProjectConstants.DB_NAME, currentProjectId))
                        .observe(this, projectRecord -> {
                            if (projectRecord == null) {
                                return;
                            }
                            callback.onGet(projectRecord.coordinateSystem);
                        });
            });
        });

//        CoordinateSystem coordinateSystem = new CoordinateSystem("2000");
//        coordinateSystem.setEllipsoid(EllipsoidEnum.AGD_84);
//        callback.onGet(coordinateSystem);
    }
}
