package com.project.survey.ui.instrument.mobilestationmode.base;

import android.content.Context;
import android.content.Intent;
import android.view.Menu;
import android.view.MenuItem;

import com.bingce.coordlib.enums.EllipsoidEnum;
import com.bingce.coordlib.model.CoordinateSystem;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.KeyValueDb;
import com.bingce.data.database.ProjectDb;
import com.bingce.data.surveyor.designdata.project.ProjectConstants;
import com.bingce.device.ui.adapter.AbstractMatchingPointRecyclerViewAdapter;
import com.bingce.surveyor.agentweb.HelpWebActivity;
import com.bingce.ui.coordinatesystem.BingCeCoordinateSystemActivity;
import com.bingce.utils.IOnSingleGetCallback;
import com.bingce.utils.ThreadPoolUtil;
import com.project.survey.R;
import com.project.survey.ui.instrument.mobilestationmode.adapter.MatchingPointRecyclerViewAdapter;

/**
 * 点校正（参数解算）
 */
public class RtkPointCorrectActivity extends AbstractRtkPointCorrectActivity {

    @Override
    protected void observeCurrentCoordinate(IOnSingleGetCallback<CoordinateSystem> callback) {
        // 获取坐标系
        ThreadPoolUtil.execute(() -> {
            String currentProjectId = KeyValueDb.currentProjectId();
            runOnUiThread(() -> {
                ProjectDb.getInstance().rawQueryLiveData(
                                DBQueryConstant.findById(ProjectConstants.DB_NAME, currentProjectId))
                        .observe(this, projectRecord -> {
                            if (projectRecord == null) {
                                return;
                            }
                            callback.onGet(projectRecord.coordinateSystem);
                        });
            });
        });

//        CoordinateSystem coordinateSystem = new CoordinateSystem("2000");
//        coordinateSystem.setEllipsoid(EllipsoidEnum.AGD_84);
//        callback.onGet(coordinateSystem);
    }

    @Override
    protected AbstractMatchingPointRecyclerViewAdapter newMatchingPointRecyclerViewAdapter(Context context, CoordinateSystem coordinateSystem) {
        return new MatchingPointRecyclerViewAdapter(context, coordinateSystem);
    }

    // 新建匹配点
    @Override
    protected void startRtkMatchingPointsActivityForResult(int requestCode) {
        Intent intent = new Intent(RtkPointCorrectActivity.this, RtkMatchingPointsActivity.class);
        startActivityForResult(intent, requestCode);
    }

    @Override
    protected void skip2ProjViewAndModifyCurrentCoordinateSystem() {
        // 点击提示弹框的“去配置”按钮
        BingCeCoordinateSystemActivity.skip2ProjViewAndModifyCurrentCoordinateSystem(this);
    }

    @Override
    protected void save(CoordinateSystem coordinateSystem) {
        ProjectDb.getInstance().updateCurrentCoordinator(coordinateSystem);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
//        getMenuInflater().inflate(R.menu.menu_activity_rtk_point_correct, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (android.R.id.home == item.getItemId()) {
            onBackPressed();
            return true;
        } else if (R.id.menu_help == item.getItemId()) {
            HelpWebActivity.navigation2RtkPointCorrectHelp(this);
            return true;
        }
        return super.onOptionsItemSelected(item);
    }
}
