package com.project.survey.ui.instrument.satellitosis;

import android.content.Context;
import android.content.Intent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;

import com.bingce.device.Device;
import com.bingce.rtk.command.RTK;
import com.bingce.rtk.command.geomax.GeoMaxRTK;
import com.bingce.rtk.enums.SateSystemEnum;
import com.bingce.rtk.model.GnssPosition;
import com.bingce.rtk.parser.nmea0183.SatInfo;
import com.bingce.rtk.parser.nmea0183.SatelliteData;
import com.project.survey.R;
import com.project.survey.databinding.ActivitySatelliteStatusBinding;
import com.project.survey.dialog.BubblePopWindow;
import com.project.survey.ui.base.BaseSurveyNewActivity;

import java.util.Arrays;


public class SatelliteStatusActivity extends BaseSurveyNewActivity {
    private ActivitySatelliteStatusBinding binding;

    @Override
    public View getContentView() {
        binding = ActivitySatelliteStatusBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        baseSurveyActivityUtils.activityBaseSurveyBinding.rlMore.setVisibility(View.VISIBLE);
    }

    @Override
    protected void initData() {
        onSatelliteSkyViewRefresh();
    }


    @Override
    public void onPause() {
        super.onPause();
        if (Device.getInstance().isDeviceConnected()) {
            if (RTK.getInstance() instanceof GeoMaxRTK) {
                ((GeoMaxRTK) RTK.getInstance()).stopOutputSatelliteInfo();
            }
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (Device.getInstance().isDeviceConnected() && RTK.getInstance() instanceof GeoMaxRTK) {
            ((GeoMaxRTK) RTK.getInstance()).startOutputSatelliteInfo();
        }
    }

    @Override
    public void onSatelliteSkyViewRefresh() {
        super.onSatelliteSkyViewRefresh();
        binding.activitySatelliteStatusSkyview.update();
        binding.activitySatelliteStatusViewSatelliteNumber.setText(String.format("%s %s", getString(R.string.tracked_satellite_number), GnssPosition.getInstance().getSatTrackNum()));
        binding.activitySatelliteStatusSolutionSatelliteNumber.setText(String.format("%s %s", getString(R.string.solution_satellite_number), GnssPosition.getInstance().getSatSolutionNum()));

        StringBuilder systemResult = new StringBuilder(getString(R.string.satellite_system) + "\r\n");
        StringBuilder prnResult = new StringBuilder(getString(R.string.satellite_prn) + "\r\n");
        StringBuilder elevationResult = new StringBuilder(getString(R.string.satellite_elevation) + "\r\n");
        StringBuilder azimuthResult = new StringBuilder(getString(R.string.satellite_azimuth) + "\r\n");
        StringBuilder snrResult = new StringBuilder(getString(R.string.satellite_snr) + "\r\n");
        for (int i = 0; i < SatelliteData.getInstance().getSatInfoArray().size(); i++) {
            SatInfo satInfo = SatelliteData.getInstance().getSatInfoArray().get(i);
            if (satInfo.getSateSystem() == SateSystemEnum.GPS) {
                if (satInfo.getElevation() <= 0.0) continue;
                double snr = satInfo.snrL1B1 != 0 ? satInfo.snrL1B1 : satInfo.snrL2B2 != 0 ? satInfo.snrL2B2 : satInfo.snrL5B3;
                String systemName = "GPS";
                systemResult.append(systemName + "\r\n");
                prnResult.append(satInfo.getPrn() + "\r\n");
                elevationResult.append(satInfo.getElevation() + "\r\n");
                azimuthResult.append(satInfo.getAzimuth() + "\r\n");
                snrResult.append(snr + "\r\n");
            }
        }
        for (int i = 0; i < SatelliteData.getInstance().getSatInfoArray().size(); i++) {
            SatInfo satInfo = SatelliteData.getInstance().getSatInfoArray().get(i);
            if (satInfo.getSateSystem() == SateSystemEnum.GLONASS) {
                if (satInfo.getElevation() <= 0.0) continue;
                double snr = satInfo.snrL1B1 != 0 ? satInfo.snrL1B1 : satInfo.snrL2B2 != 0 ? satInfo.snrL2B2 : satInfo.snrL5B3;
                String systemName = "GLONASS";
                systemResult.append(systemName + "\r\n");
                prnResult.append(satInfo.getPrn() + "\r\n");
                elevationResult.append(satInfo.getElevation() + "\r\n");
                azimuthResult.append(satInfo.getAzimuth() + "\r\n");
                snrResult.append(snr + "\r\n");
            }
        }
        for (int i = 0; i < SatelliteData.getInstance().getSatInfoArray().size(); i++) {
            SatInfo satInfo = SatelliteData.getInstance().getSatInfoArray().get(i);
            if (satInfo.getSateSystem() == SateSystemEnum.BDS) {
                if (satInfo.getElevation() <= 0.0) continue;
                double snr = satInfo.snrL1B1 != 0 ? satInfo.snrL1B1 : satInfo.snrL2B2 != 0 ? satInfo.snrL2B2 : satInfo.snrL5B3;
                String systemName = "BDS";
                systemResult.append(systemName + "\r\n");
                prnResult.append(satInfo.getPrn() + "\r\n");
                elevationResult.append(satInfo.getElevation() + "\r\n");
                azimuthResult.append(satInfo.getAzimuth() + "\r\n");
                snrResult.append(snr + "\r\n");
            }
        }
        for (int i = 0; i < SatelliteData.getInstance().getSatInfoArray().size(); i++) {
            SatInfo satInfo = SatelliteData.getInstance().getSatInfoArray().get(i);
            if (satInfo.getSateSystem() == SateSystemEnum.GALILEO) {
                if (satInfo.getElevation() <= 0.0) continue;
                double snr = satInfo.snrL1B1 != 0 ? satInfo.snrL1B1 : satInfo.snrL2B2 != 0 ? satInfo.snrL2B2 : satInfo.snrL5B3;
                String systemName = "GALILEO";
                systemResult.append(systemName + "\r\n");
                prnResult.append(satInfo.getPrn() + "\r\n");
                elevationResult.append(satInfo.getElevation() + "\r\n");
                azimuthResult.append(satInfo.getAzimuth() + "\r\n");
                snrResult.append(snr + "\r\n");
            }
        }
        for (int i = 0; i < SatelliteData.getInstance().getSatInfoArray().size(); i++) {
            SatInfo satInfo = SatelliteData.getInstance().getSatInfoArray().get(i);
            if (satInfo.getSateSystem() == SateSystemEnum.QZSS) {
                if (satInfo.getElevation() <= 0.0) continue;
                double snr = satInfo.snrL1B1 != 0 ? satInfo.snrL1B1 : satInfo.snrL2B2 != 0 ? satInfo.snrL2B2 : satInfo.snrL5B3;
                String systemName = "QZSS";
                systemResult.append(systemName + "\r\n");
                prnResult.append(satInfo.getPrn() + "\r\n");
                elevationResult.append(satInfo.getElevation() + "\r\n");
                azimuthResult.append(satInfo.getAzimuth() + "\r\n");
                snrResult.append(snr + "\r\n");
            }
        }
        for (int i = 0; i < SatelliteData.getInstance().getSatInfoArray().size(); i++) {
            SatInfo satInfo = SatelliteData.getInstance().getSatInfoArray().get(i);
            if (satInfo.getSateSystem() == SateSystemEnum.SBAS) {
                if (satInfo.getElevation() <= 0.0) continue;
                double snr = satInfo.snrL1B1 != 0 ? satInfo.snrL1B1 : satInfo.snrL2B2 != 0 ? satInfo.snrL2B2 : satInfo.snrL5B3;
                String systemName = "SBAS";
                systemResult.append(systemName + "\r\n");
                prnResult.append(satInfo.getPrn() + "\r\n");
                elevationResult.append(satInfo.getElevation() + "\r\n");
                azimuthResult.append(satInfo.getAzimuth() + "\r\n");
                snrResult.append(snr + "\r\n");
            }
        }
        for (int i = 0; i < SatelliteData.getInstance().getSatInfoArray().size(); i++) {
            SatInfo satInfo = SatelliteData.getInstance().getSatInfoArray().get(i);
            if (satInfo.getSateSystem() == SateSystemEnum.IRNSS) {
                if (satInfo.getElevation() <= 0.0) continue;
                double snr = satInfo.snrL1B1 != 0 ? satInfo.snrL1B1 : satInfo.snrL2B2 != 0 ? satInfo.snrL2B2 : satInfo.snrL5B3;
                String systemName = "IRNSS";
                systemResult.append(systemName + "\r\n");
                prnResult.append(satInfo.getPrn() + "\r\n");
                elevationResult.append(satInfo.getElevation() + "\r\n");
                azimuthResult.append(satInfo.getAzimuth() + "\r\n");
                snrResult.append(snr + "\r\n");
            }
        }
        for (int i = 0; i < SatelliteData.getInstance().getSatInfoArray().size(); i++) {
            SatInfo satInfo = SatelliteData.getInstance().getSatInfoArray().get(i);
            if (satInfo.getSateSystem() == SateSystemEnum.OTHER) {
                if (satInfo.getElevation() <= 0.0) continue;
                double snr = satInfo.snrL1B1 != 0 ? satInfo.snrL1B1 : satInfo.snrL2B2 != 0 ? satInfo.snrL2B2 : satInfo.snrL5B3;
                String systemName = "OTHER";
                systemResult.append(systemName + "\r\n");
                prnResult.append(satInfo.getPrn() + "\r\n");
                elevationResult.append(satInfo.getElevation() + "\r\n");
                azimuthResult.append(satInfo.getAzimuth() + "\r\n");
                snrResult.append(snr + "\r\n");
            }
        }

        binding.activitySatelliteStatusSystem.setText(systemResult.toString());
        binding.activitySatelliteStatusPrn.setText(prnResult.toString());
        binding.activitySatelliteStatusAzimuth.setText(azimuthResult.toString());
        binding.activitySatelliteStatusElevation.setText(elevationResult.toString());
        binding.activitySatelliteStatusSnr.setText(snrResult.toString());
    }

//    @Override
//    public boolean onCreateOptionsMenu(Menu menu) {
//        getMenuInflater().inflate(R.menu.menu_activity_satellite_system_switch, menu);
//        return super.onCreateOptionsMenu(menu);
//    }
//
//    @Override
//    public boolean onOptionsItemSelected(MenuItem item) {
//        switch (item.getItemId()) {
//            case android.R.id.home:
//                finish();
//                break;
//            case R.id.menu_satellite_system_switch:
//                Intent intent = new Intent(this, SatelliteSystemSwitchActivity.class);
//                startActivity(intent);
//                break;
//        }
//        return super.onOptionsItemSelected(item);
//    }

    @Override
    public boolean isHideHelpDoc() {
        return true;
    }

    @Override
    public BubblePopWindow newBubblePopWindow(Context context) {
        return new BubblePopWindow(
                context,
                Arrays.asList(
                        new BubblePopWindow.CustomMenuItem(R.drawable.icon_function_switch_off, "卫星系统开关", () -> {
                            Intent intent = new Intent(context, SatelliteSystemSwitchActivity.class);
                            startActivity(intent);
                        })
                ));
    }
}
