package com.project.survey.ui.instrument.setupstation;

import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.activity.result.ActivityResultLauncher;


import com.bingce.surveyor.util.ConstUtils;

import com.bingce.utils.IntentUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.google.gson.Gson;
import com.project.survey.R;
import com.project.survey.databinding.ActivityAddResultFormatBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.db.RecordsFixedDataBase;
import com.project.survey.ui.instrument.setupstation.db.resultformat.FormatRecord;
import com.project.survey.util.SystemUtils;
import com.zhy.view.flowlayout.FlowLayout;
import com.zhy.view.flowlayout.TagAdapter;


import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;


public class AddResultFormatActivity extends BaseSurveyNewActivity {

    public ActivityAddResultFormatBinding binding;

    private final int[] mItemsFileHeader = new int[]{R.string.yes, R.string.no};
    private List<String> mItemsFileHeaderArrayList;
    private int indexOfFileHeader = 0;

    private final int[] mItemsSuffix = new int[]{R.string.xls, R.string.dat, R.string.txt,R.string.csv};
    private List<String> mItemsSuffixArrayList;
    private int indexOfSuffix = 0;

    private final int[] mItemsAngleFormat = new int[]{R.string.af_d_ms, R.string.af_d_m_s, R.string.af_d, R.string.af_radian};
    private List<String> mItemsAngleFormatArrayList;
    private int indexOfAngleFormat = 0;

    private final int[] mItemsDividedSymbols = new int[]{R.string.split_comma, R.string.split_at, R.string.split_space, R.string.split_tab, R.string.split_enter};
    private List<String> mItemsDividedSymbolsArrayList;
    private int indexOfDividedSymbols = 0;

    private TagAdapter<String> selAdapter,tagAdapter;
    private List<String> tagTab = new ArrayList();
    private List<String> selTab = new ArrayList();
    private String uid;
    private boolean isNewCraet;
    private List<String> allTagList = new ArrayList<>();
    public static int SUFFIX_XLS_NUM = 0;
    public static int SUFFIX_DAT_NUM = 1;
    public static int SUFFIX_TXT_NUM = 2;
    public static int SUFFIX_CSV_NUM = 3;

    private int SYMBOLS_COMMMA = 0;
    private int SYMBOLS_AT = 1;
    private int SYMBOLS_SPACE = 2;
    private int SYMBOLS_TAB = 3;
    private int SYMBOLS_ENTER = 4;

    public static int ANGLE_FORMAT_D_MS = 0;
    public static int ANGLE_FORMAT_D_M_S = 1;
    public static int ANGLE_FORMAT_D = 2;
    public static int ANGLE_FORMAT_R = 3;

    @Override
    public View getContentView() {
        binding = ActivityAddResultFormatBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {

        isNewCraet = IntentUtil.boolExtra(this, ConstUtils.intentConst.keyIsNewCreateResultFormat);

        if (isNewCraet){
            setTitle(getString(R.string.new_export_format));
        }else {
            setTitle(getString(R.string.edit_export_format));
        }
        binding.tvFormatName.setText(getString(R.string.format_name) + "：");
        binding.tvFileHeader.setText(getString(R.string.file_header) + "：");
        binding.tvFileSuffix.setText(getString(R.string.file_suffix) + "：");
        binding.tvAngleFormat.setText(getString(R.string.angle_formats) + "：");
        binding.tvDividedSymbols.setText(getString(R.string.divided_symbols) + "：");

        binding.btnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String formatName = binding.etFormatName.getText().toString();
                if (TextUtils.isEmpty(formatName)){
                    ToastUtils.showShort(getString(R.string.enter_format_name));
                    return;
                }

                if (selTab != null && selTab.size() == 0){
                    ToastUtils.showShort(getString(R.string.select_custom_format_description));
                    return;
                }

                StringBuilder stringBuilder = new StringBuilder();
                for (int i = 0; i < selTab.size(); i++) {
                    if (i == selTab.size() - 1){
                        stringBuilder.append(selTab.get(i));
                    }else {
                        stringBuilder.append(selTab.get(i)).append(binding.sherlockSymbol.getText().toString());
                    }
                }

                ThreadPoolUtil.execute(new Runnable() {
                    @Override
                    public void run() {
                        FormatRecord formatRecord = new FormatRecord();
                        formatRecord.id = uid;
                        formatRecord.format_name =  formatName;
                        formatRecord.format_content = stringBuilder.toString();
                        formatRecord.divided_symbols = mItemsDividedSymbolsArrayList.get(indexOfDividedSymbols);
                        formatRecord.file_suffix = indexOfSuffix;
                        formatRecord.file_header = indexOfFileHeader == 0 ? true : false;
                        formatRecord.angle_format = indexOfAngleFormat;
                        RecordsFixedDataBase.getInstance().formatRecordDao().save(formatRecord);
                        Intent intent = new Intent();
                        intent.putExtra(ConstUtils.intentConst.keyResultFormatRecord, new Gson().toJson(formatRecord));
                        intent.putExtra(ConstUtils.intentConst.keyIsNewCreateResultFormat, isNewCraet);
                        setResult(LauncherEvent.launcher_results_export,intent);
                        finish();
                    }
                });
            }
        });
    }

    @Override
    protected void initData() {
        allTagList.add(getString(R.string.point_name));
        allTagList.add(getString(R.string.point_code));
        allTagList.add(getString(R.string.latitude));
        allTagList.add(getString(R.string.longitude));
        allTagList.add(getString(R.string.altitude));
        allTagList.add(getString(R.string.antenna_type));
        allTagList.add(getString(R.string.antenna_height));
        allTagList.add(getString(R.string.northing));
        allTagList.add(getString(R.string.easting));
        allTagList.add(getString(R.string.elevation));
        allTagList.add(getString(R.string.solution_state));
        allTagList.add(getString(R.string.satellite_num));
        allTagList.add(getString(R.string.satellite_track_num));
        allTagList.add(getString(R.string.pdop));
        allTagList.add(getString(R.string.hrms));
        allTagList.add(getString(R.string.vrms));
        allTagList.add(getString(R.string.diff_delay));
        allTagList.add(getString(R.string.remark));
        allTagList.add(getString(R.string.acquisition_time));
        allTagList.add(getString(R.string.tilt_survey));

        if (isNewCraet){
            uid = SystemUtils.getUUID();
            for (int i = 0; i < allTagList.size(); i++) {
                if (i == 0){
                    selTab.add(allTagList.get(i));
                }else {
                    tagTab.add(allTagList.get(i));
                }
            }

            TagFormatData();

            TagFlowData();

        }else {
            uid = IntentUtil.stringExtra(this, ConstUtils.intentConst.keyResultsExportFormatId);

            ThreadPoolUtil.execute(new Runnable() {
                @Override
                public void run() {
                    FormatRecord formatRecord = RecordsFixedDataBase.getInstance().formatRecordDao().findById(uid);
                    ThreadPoolUtil.executeInMain(new Runnable() {
                        @Override
                        public void run() {

                            binding.etFormatName.setText(formatRecord.format_name);

                            String[] split = formatRecord.format_content.split(formatRecord.divided_symbols);
                            for (int i = 0; i < split.length; i++) {
                                selTab.add(split[i]);
                            }

                            for (int i = 0; i < allTagList.size(); i++) {
                                if (!selTab.contains(allTagList.get(i))){
                                    tagTab.add(allTagList.get(i));
                                }
                            }

                            if (formatRecord.file_header){
                                indexOfFileHeader = 0;
                            }else {
                                indexOfFileHeader = 1;
                            }

                            indexOfSuffix = formatRecord.file_suffix;

                            if (indexOfSuffix == SUFFIX_TXT_NUM || indexOfSuffix == SUFFIX_CSV_NUM){
                                binding.llSherlockSymbol.setVisibility(View.VISIBLE);
                            }else {
                                binding.llSherlockSymbol.setVisibility(View.GONE);
                            }

                            indexOfAngleFormat = formatRecord.angle_format;

                            if (formatRecord.divided_symbols.equals(getString(R.string.split_comma))){
                                indexOfDividedSymbols = SYMBOLS_COMMMA;
                            }else if (formatRecord.divided_symbols.equals(getString(R.string.split_at))){
                                indexOfDividedSymbols = SYMBOLS_AT;
                            }else if (formatRecord.divided_symbols.equals(getString(R.string.split_space))){
                                indexOfDividedSymbols = SYMBOLS_SPACE;
                            }else if (formatRecord.divided_symbols.equals(getString(R.string.split_tab))){
                                indexOfDividedSymbols = SYMBOLS_TAB;
                            }else if (formatRecord.divided_symbols.equals(getString(R.string.split_enter))){
                                indexOfDividedSymbols = SYMBOLS_ENTER;
                            }

                            TagFormatData();

                            TagFlowData();
                        }
                    });
                }
            });
        }
    }

    private void TagFormatData() {
        mItemsFileHeaderArrayList = new ArrayList<>();
        for (int mItem : mItemsFileHeader) {
            mItemsFileHeaderArrayList.add(getResources().getString(mItem));
        }
        ArrayAdapter<String> fileHeaderAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, mItemsFileHeaderArrayList);
        binding.sherlockFileHeader.setAdapter(fileHeaderAdapter);
        binding.sherlockFileHeader.setText(mItemsFileHeaderArrayList.get(indexOfFileHeader));
        binding.sherlockFileHeader.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                indexOfFileHeader = position;
                binding.sherlockFileHeader.setText(mItemsFileHeaderArrayList.get(indexOfFileHeader));
            }
        });

        mItemsSuffixArrayList = new ArrayList<>();
        for (int mItem : mItemsSuffix) {
            mItemsSuffixArrayList.add(getResources().getString(mItem));
        }
        ArrayAdapter<String> suffixAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, mItemsSuffixArrayList);
        binding.sherlockSuffix.setAdapter(suffixAdapter);
        binding.sherlockSuffix.setText(mItemsSuffixArrayList.get(indexOfSuffix));
        if (indexOfSuffix == SUFFIX_XLS_NUM){
            binding.rlFileHeader.setVisibility(View.VISIBLE);
        }else {
            binding.rlFileHeader.setVisibility(View.GONE);
        }
        binding.sherlockSuffix.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                indexOfSuffix = position;
                binding.sherlockSuffix.setText(mItemsSuffixArrayList.get(indexOfSuffix));
                if (indexOfSuffix == SUFFIX_TXT_NUM || indexOfSuffix == SUFFIX_CSV_NUM){
                    binding.llSherlockSymbol.setVisibility(View.VISIBLE);
                }else {
                    indexOfDividedSymbols = SYMBOLS_COMMMA;
                    binding.sherlockSymbol.setText(mItemsDividedSymbolsArrayList.get(indexOfDividedSymbols));
                    binding.llSherlockSymbol.setVisibility(View.GONE);
                }

                if (indexOfSuffix == SUFFIX_XLS_NUM){
                    binding.rlFileHeader.setVisibility(View.VISIBLE);
                    indexOfFileHeader = 0;
                }else {
                    binding.rlFileHeader.setVisibility(View.GONE);
                    indexOfFileHeader = 1;
                }
                binding.sherlockFileHeader.setText(mItemsFileHeaderArrayList.get(indexOfFileHeader));
            }
        });

        mItemsAngleFormatArrayList = new ArrayList<>();
        for (int mItem : mItemsAngleFormat) {
            mItemsAngleFormatArrayList.add(getResources().getString(mItem));
        }
        ArrayAdapter<String> angleFormatAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, mItemsAngleFormatArrayList);
        binding.sherlockAngleFormat.setAdapter(angleFormatAdapter);
        binding.sherlockAngleFormat.setText(mItemsAngleFormatArrayList.get(indexOfAngleFormat));
        binding.sherlockAngleFormat.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                indexOfAngleFormat = position;
                binding.sherlockAngleFormat.setText(mItemsAngleFormatArrayList.get(indexOfAngleFormat));
            }
        });

        mItemsDividedSymbolsArrayList = new ArrayList<>();
        for (int mItem : mItemsDividedSymbols) {
            mItemsDividedSymbolsArrayList.add(getResources().getString(mItem));
        }
        ArrayAdapter<String> dividedSymbolsAdapter = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, mItemsDividedSymbolsArrayList);
        binding.sherlockSymbol.setAdapter(dividedSymbolsAdapter);
        binding.sherlockSymbol.setText(mItemsDividedSymbolsArrayList.get(indexOfDividedSymbols));
        binding.sherlockSymbol.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                indexOfDividedSymbols = position;
                binding.sherlockSymbol.setText(mItemsDividedSymbolsArrayList.get(indexOfDividedSymbols));
            }
        });
    }

    private void TagFlowData() {

        selAdapter = new TagAdapter<String>(selTab){
            @Override
            public View getView(FlowLayout parent, final int position, String s) {
                RelativeLayout view = (RelativeLayout) LayoutInflater.from(AddResultFormatActivity.this).inflate( R.layout.layout_format_flowlayout, binding.flowlayout,false);
                TextView viewById = (TextView) view.findViewById(R.id.tv_name);
                viewById.setText(s);
                ImageView imageView = (ImageView) view.findViewById(R.id.iv_delete);
                imageView.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        tagTab.add(selTab.get(position));
                        tagAdapter.notifyDataChanged();

                        selTab.remove(position);
                        selAdapter.notifyDataChanged();
                    }
                });
                return view;
            }
        };
        binding.flowlayout.setAdapter(selAdapter);

        tagAdapter = new TagAdapter<String>(tagTab){
            @Override
            public View getView(FlowLayout parent, final int position, String s) {
                LinearLayout view = (LinearLayout) LayoutInflater.from(AddResultFormatActivity.this).inflate( R.layout.layout_format_flowlayout_tag, binding.tagFlowlayout,false);
                TextView viewById = (TextView) view.findViewById(R.id.tv_name);
                viewById.setText(s);
                //如果点击就添加元素并刷新adapter
                view.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        //将新加入的数据加到集合的最后一个位置,而原来的添加图标会到 +1 的位置
                        selTab.add(tagTab.get(position));
                        selAdapter.notifyDataChanged();

                        tagTab.remove(position);
                        tagAdapter.notifyDataChanged();
                    }
                });
                return view ;
            }
        };
        binding.tagFlowlayout.setAdapter(tagAdapter);
    }


    public static void start(ActivityResultLauncher<Intent> launcher, Context context, boolean isNewCraet){
        start(launcher,context,isNewCraet,"");
    }

    public static void start(ActivityResultLauncher<Intent> launcher,Context context, boolean isNewCraet, String uid) {
        Intent intent = new Intent(context,AddResultFormatActivity.class);
        intent.putExtra(ConstUtils.intentConst.keyIsNewCreateResultFormat,isNewCraet);
        intent.putExtra(ConstUtils.intentConst.keyResultsExportFormatId,uid);
        launcher.launch(intent);
    }
}