package com.project.survey.ui.instrument.setupstation;

import static com.bingce.road.parameter.RoadAttributeControlPointConverter.readControlPoint;

import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;

import androidx.activity.result.ActivityResult;
import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.MutableLiveData;
import androidx.recyclerview.widget.RecyclerView;

import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.data.cache.CachedCurrentProject;
import com.bingce.data.cache.CachedCurrentRoad;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.KeyValueDb;
import com.bingce.data.database.PointDb;
import com.bingce.data.database.RoadDb;
import com.bingce.data.surveyor.designdata.kv.KeyValueConstants;
import com.bingce.data.surveyor.designdata.kv.KeyValueRecord;
import com.bingce.data.surveyor.designdata.road.RoadConstants;
import com.bingce.data.surveyor.designdata.road.RoadRecord;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.device.ui.dialog.DialogUtil;
import com.bingce.error.BingCeErrorCode;
import com.bingce.file.FileOperator;
import com.bingce.list.SwipeDeleteListLayoutHelper;
import com.bingce.poi.excel.ExcelExportHelper;
import com.bingce.road.RoadIntentUtil;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.DateUtils;
import com.bingce.utils.FileUtil;
import com.bingce.utils.FragmentUtils;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.project.survey.BuildConfig;
import com.project.survey.R;
import com.project.survey.databinding.ActivityPointsBinding;
import com.project.survey.databinding.RecyclerviewItemControlPointLayoutBinding;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsClearDialogUtils;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsDetailDialogUtils;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsExportDialogUtils;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsFilterDialogUtils;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsImportConfigDialogUtils;

import org.polaric.colorful.ColorfulActivity;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;
import cn.liuyanbing.surveyor.model.base.Point;
import cn.liuyanbing.surveyor.model.base.PointArray;
import cn.liuyanbing.surveyor.model.base.Road;

/**
 * 控制点
 */
public class ControlPointsActivity extends ColorfulActivity {
    private String[] mHeaders;
    private ProgressDialog mProgressDialog;
    private final List<List<String>> exportResult = new ArrayList<>();
    private final ControlPointsFilterDialogUtils controlPointsFilterDialogUtils = new ControlPointsFilterDialogUtils(() -> this);
    private final ControlPointsDetailDialogUtils controlPointsDetailDialogUtils = new ControlPointsDetailDialogUtils(this);
    private final ControlPointsExportDialogUtils controlPointsExportDialogUtils = new ControlPointsExportDialogUtils(this);
    private final ControlPointsClearDialogUtils controlPointsClearDialogUtils = new ControlPointsClearDialogUtils(this);
    private final ControlPointsImportConfigDialogUtils controlPointsImportConfigDialogUtils = new ControlPointsImportConfigDialogUtils(this);

    private final SwipeDeleteListLayoutHelper<DATA, VH> listLayoutHelper = new SwipeDeleteListLayoutHelper<DATA, VH>(this) {
        @Override
        protected void onDelete(int index, DATA instance, IOnDeleteListener<DATA> listener) {
            DialogUtil.mNullCancelAlertDialog(ControlPointsActivity.this, null, R.string.confirm_delete, (dialogInterface, i) -> {
                if (StringUtil.isEmpty(instance.roadId)) {//如果是全局控制点
                    ThreadPoolUtil.execute(() -> {
                        PointDb.getInstance().delete(instance.pointRecord);
                        runOnUiThread(() -> listener.completeDelete(instance));
                    });
                } else {//如果是线路控制点，则需要修改线路数据
                    ThreadPoolUtil.execute(() -> {
                        RoadRecord roadRecord = RoadDb.getInstance()
                                .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, instance.roadId));
                        if (roadRecord == null) {
                            BingCeErrorCode.toast(BingCeErrorCode.NO_FOUND_ROAD_BY_ID);
                            return;
                        }
                        Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                        if (road == null) {
                            BingCeErrorCode.toast(BingCeErrorCode.ROAD_RECORD_CONVERT_2_ROAD_FAILURE);
                            return;
                        }
                        if (instance.index >= 0 && instance.index < road.getControlPointList().size()) {
                            road.getControlPointList().remove(instance.index);
                            RoadDb.getInstance().save(road);
                        }
                    });
                }

                binding.fabNewPoint.setVisibility(View.VISIBLE);
            });
        }

        @Override
        protected VH newItemContentVH(@NonNull ViewGroup parent, int viewType) {
            return new VH(RecyclerviewItemControlPointLayoutBinding.inflate(getLayoutInflater(), parent, false));
        }

        @Override
        protected void onItemClicked(int index, DATA data) {
            boolean needReturn = IntentUtil.boolExtra(getIntent(), KEY_IS_RETURN_DATA);
            if (needReturn) {
                Intent intent = new Intent();
                PointRecord instance = data.pointRecord;
                intent.putExtra(RESULT_ID, instance.id);
                intent.putExtra(RESULT_NAME, instance.name);
                intent.putExtra(RESULT_CODE, instance.code);
                intent.putExtra(RESULT_REMARK, instance.remarks);
                intent.putExtra(RESULT_X, instance.x);
                intent.putExtra(RESULT_Y, instance.y);
                intent.putExtra(RESULT_Z, instance.h);
                setResult(RESULT_OK, intent);
                finish();
            } else {
                controlPointsDetailDialogUtils.showControlPointDialog(
                        data.roadId, data, getString(R.string.edit_control_point));
            }
        }

        @Override
        protected void doBindContent(VH vh, DATA data) {
            PointRecord instance = data.pointRecord;
            vh.itemBinding.recyclerviewItemSwipePointTitle1.setText(instance.name);
            vh.itemBinding.recyclerviewItemSwipePointTitle2.setText(instance.code);
            vh.itemBinding.recyclerviewItemSwipePointBody1.setText("X：" + Util.formatDouble2StringDotAuto(instance.x));
            vh.itemBinding.recyclerviewItemSwipePointBody2.setText("Y：" + Util.formatDouble2StringDotAuto(instance.y));
            vh.itemBinding.recyclerviewItemSwipePointBody3.setText("Z：" + Util.formatDouble2StringDotAuto(instance.h));
            vh.itemBinding.recyclerviewItemSwipePointBody4.setText(
                    StringUtil.isEmpty(data.roadId) ? DateUtils.toFull(instance.createDate) : data.timeString
            );
            vh.itemBinding.recyclerviewItemSwipePointBody5.setText(instance.remarks);
        }

        @Override
        protected void onMenuOpen() {
            binding.fabNewPoint.setVisibility(View.GONE);
        }

        @Override
        protected void onMenuClosed() {
            binding.fabNewPoint.setVisibility(View.VISIBLE);
        }

        @Override
        protected RecyclerView recyclerView() {
            return binding.recyclerviewPoints;
        }
    };
    private final MutableLiveData<List<DATA>> pointRecordsMutableLiveData = new MutableLiveData<>();
    private ActivityPointsBinding binding;

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityPointsBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        setSupportActionBar(binding.toolbar);
        if (getSupportActionBar() != null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        }

        mHeaders = new String[]{getString(R.string.category),
                getString(R.string.point_name),
                getString(R.string.point_code),
                "X", "Y", "Z",
                getString(R.string.remarks),
                getString(R.string.record_time)};

        //监控数据变化
        pointRecordsMutableLiveData.observe(this, listLayoutHelper::onDataChanged);
        String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
        binding.fabNewPoint.setOnClickListener(v -> {
            controlPointsDetailDialogUtils.showControlPointDialog(
                    roadId, null, getString(R.string.new_control_point));
        });


        boolean isGlobal = IntentUtil.boolExtra(this, KEY_IS_GLOBAL);

        ThreadPoolUtil.execute(() -> {
            if (isGlobal) {
                String projectId = CachedCurrentProject.currentProjectId();
                runOnUiThread(() -> {
                    PointDb.getInstance()
                            .rawQueryListLiveData(PointConstants.findControlPointByProject(projectId))
                            .observe(this, pointRecords -> {
                                List<DATA> dataList = new ArrayList<>();
                                for (PointRecord record : pointRecords) {
                                    dataList.add(new DATA(record));
                                }
                                pointRecordsMutableLiveData.setValue(dataList);
                            });
                });
            } else {
                if (!StringUtil.isEmpty(roadId)) {//
                    runOnUiThread(() -> {
                        RoadDb.getInstance()
                                .rawQueryLiveData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId))
                                .observe(this, roadRecord -> {
                                    if (roadRecord == null){
                                        return;
                                    }
                                    PointArray pointArray = new PointArray();
                                    String error = readControlPoint(roadRecord.controlPointJson, roadRecord.name, pointArray);
                                    if (!StringUtil.isEmpty(error)) {
                                        ToastUtils.showShort(error);
                                        return;
                                    }
                                    List<DATA> dataList = new ArrayList<>();
                                    for (int index = 0; index < pointArray.size(); index++) {
                                        Point point = pointArray.get(index);
                                        dataList.add(new DATA(roadId, index,
                                                new PointRecord(null, null, null,
                                                        -1,
                                                        point.getName(), point.getCode(), point.getRemarks(),
                                                        point.getX(), point.getY(), point.getZ(),
                                                        0, 0, 0,
                                                        PointConstants.POINT_FORMAT_XYZ, null), point.getTime()));
                                    }
                                    pointRecordsMutableLiveData.setValue(dataList);
                                });
                    });
                } else {
                    //查询当前线路
                    String projectId = CachedCurrentProject.currentProjectId();
                    String currentRoadId = CachedCurrentRoad.currentRoadId(projectId);
                    runOnUiThread(() -> {
                        RoadDb.getInstance()
                                .rawQueryLiveData(DBQueryConstant.findById(RoadConstants.DB_NAME, currentRoadId))
                                .observe(this, roadRecord -> {
                                    PointArray pointArray = new PointArray();
                                    String error = readControlPoint(roadRecord.controlPointJson, roadRecord.name, pointArray);
                                    if (!StringUtil.isEmpty(error)) {
                                        ToastUtils.showShort(error);
                                        return;
                                    }
                                    List<DATA> dataList = new ArrayList<>();
                                    for (int index = 0; index < pointArray.size(); index++) {
                                        Point point = pointArray.get(index);
                                        dataList.add(new DATA(roadId, index,
                                                new PointRecord(null, null, null,
                                                        -1,
                                                        point.getName(), point.getCode(), point.getRemarks(),
                                                        point.getX(), point.getY(), point.getZ(),
                                                        0, 0, 0,
                                                        PointConstants.POINT_FORMAT_XYZ, null), point.getTime()));
                                    }
                                    pointRecordsMutableLiveData.setValue(dataList);
                                });
                    });
                }
            }
        });
    }

    private void updateData(String filterKey) {
        String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
        boolean isGlobal = IntentUtil.boolExtra(this, KEY_IS_GLOBAL);
        ThreadPoolUtil.execute(() -> {
            if (isGlobal) {//处理全局控制点
                String projectId = CachedCurrentProject.currentProjectId();
                List<PointRecord> pointRecords = PointDb.getInstance()
                        .rawQueryListData(PointConstants.findGlobalControlByNameLike(projectId, filterKey));
                List<DATA> dataList = new ArrayList<>();
                for (PointRecord record : pointRecords) {
                    dataList.add(new DATA(record));
                }
                runOnUiThread(() -> {
                    pointRecordsMutableLiveData.setValue(dataList);
                });
            } else {//处理线路控制点
                if (!StringUtil.isEmpty(roadId)) {//
                    RoadRecord roadRecord = RoadDb.getInstance()
                            .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
                    PointArray pointArray = new PointArray();
                    String error = readControlPoint(roadRecord.controlPointJson, roadRecord.name, pointArray);
                    if (!StringUtil.isEmpty(error)) {
                        ToastUtils.showShort(error);
                        return;
                    }
                    runOnUiThread(() -> updatePointList(roadId, pointArray, filterKey));
                } else {
                    //查询当前线路
                    String projectId = CachedCurrentProject.currentProjectId();
                    String currentRoadId = CachedCurrentRoad.currentRoadId(projectId);
                    RoadRecord roadRecord = RoadDb.getInstance()
                            .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, currentRoadId));
                    PointArray pointArray = new PointArray();
                    String error = readControlPoint(roadRecord.controlPointJson, roadRecord.name, pointArray);
                    if (!StringUtil.isEmpty(error)) {
                        ToastUtils.showShort(error);
                        return;
                    }
                    runOnUiThread(() -> updatePointList(roadId, pointArray, filterKey));
                }
            }
        });
    }

    private void updatePointList(String roadId, PointArray pointArray, String filterKey) {
        List<DATA> dataList = new ArrayList<>();
        if (pointArray != null) {
            for (int index = 0; index < pointArray.size(); index++) {
                Point point = pointArray.get(index);
                if (StringUtil.isEmpty(filterKey) || point.getName().contains(filterKey)) {
                    dataList.add(new DATA(roadId, index,
                            new PointRecord(null, null, null,
                                    -1,
                                    point.getName(), point.getCode(), point.getRemarks(),
                                    point.getX(), point.getY(), point.getZ(),
                                    0, 0, 0,
                                    PointConstants.POINT_FORMAT_XYZ, null), point.getTime()));
                }
            }
        }
        pointRecordsMutableLiveData.setValue(dataList);
    }

    @SuppressLint("StaticFieldLeak")
    private void exportExcelAsync(final String name, final String[] headers) {
        new AsyncTask<Integer, Integer, Boolean>() {

            @Override
            protected void onPreExecute() {
                mProgressDialog = new ProgressDialog(ControlPointsActivity.this);
                mProgressDialog.setMessage(getString(R.string.export_points));
                mProgressDialog.setOnCancelListener(dialog -> cancel(true));
                mProgressDialog.setCancelable(true);
                mProgressDialog.show();
            }

            @Override
            protected Boolean doInBackground(Integer... params) {
                exportResult.clear();

                for (int i = 0; i < listLayoutHelper.itemSize(); i++) {
                    DATA data = listLayoutHelper.getItem(i);
                    PointRecord point = data.pointRecord;

                    ArrayList<String> rowResult = new ArrayList<>();

                    String type = getString(R.string.control_point);

                    rowResult.add(type);
                    rowResult.add(point.name);
                    rowResult.add(point.code);
                    rowResult.add(Util.formatDouble2StringDotAuto(point.x));
                    rowResult.add(Util.formatDouble2StringDotAuto(point.y));
                    rowResult.add(Util.formatDouble2StringDotAuto(point.h));
                    rowResult.add(point.remarks);
                    rowResult.add(DateUtils.toFull(point.createDate));
                    exportResult.add(rowResult);
                }
                if (!exportResult.isEmpty()) {
                    new ExcelExportHelper().exportExcelForBigDataAndSave(headers, exportResult, getString(R.string.point_data), ExcelExportHelper.MORE_SHEET_FLAG, FileUtil.getSDPath() + "/" + getString(R.string.surveyor_exported_file) + "/" + getString(R.string.point_data), name);
                }
                return true;
            }

            @Override
            protected void onPostExecute(Boolean result) {
                super.onPostExecute(result);
                mProgressDialog.dismiss();
                File file = new File(FileUtil.getSDPath() + "/" + getString(R.string.surveyor_exported_file) + "/" + getString(R.string.point_data) + "/" + name + ".xls");
                if (file.exists()) {
                    FileUtil.scanFile(ControlPointsActivity.this, file);
                    showExportOverDialog(file);
                }
            }
        }.execute();
    }

    private void showExportOverDialog(final File file) {
        new MaterialDialog.Builder(this).title(R.string.export_successful)
                .content("导出文件保存在手机根目录的[工程测量导出文件]文件夹内")
                .positiveText(R.string.send_to)
                .negativeText(R.string.open)
                .neutralText(R.string.close)
                .onPositive((dialog, which) -> FileOperator.shareFile(ControlPointsActivity.this, file, BuildConfig.APPLICATION_ID))
                .onNegative((dialog, which) -> FileUtil.openFile(ControlPointsActivity.this, file))
                .show();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        getMenuInflater().inflate(R.menu.menu_activity_points, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                break;
            case R.id.menu_point_search:
                controlPointsFilterDialogUtils.showFilterDialog(instance -> {
                    updateData(instance);
                    binding.tvFilterKey.setVisibility(StringUtil.isEmpty(instance) ? View.GONE : View.VISIBLE);
                    binding.tvFilterKey.setText(getString(R.string.filter) + ":[" + instance + "]");
                });
                break;
            case R.id.menu_point_import:
                controlPointsImportConfigDialogUtils.showImportConfigDialog();
                break;
            case R.id.menu_point_export:
                controlPointsExportDialogUtils.showExportDialog(name -> {
                    exportExcelAsync(name, mHeaders);
                });
                break;
            case R.id.menu_point_clear:
                controlPointsClearDialogUtils.showClearDialog(() -> {
                    ThreadPoolUtil.execute(() -> {
                        boolean isGlobal = IntentUtil.boolExtra(this, KEY_IS_GLOBAL);
                        if (isGlobal) {//清空当前项目所有全局控制点
                            KeyValueRecord currentProjectConfig = KeyValueDb.getInstance().findData(KeyValueConstants.KEY_CURRENT_PROJECT_ID);
                            if (currentProjectConfig == null) {
                                BingCeErrorCode.toast(BingCeErrorCode.NO_CURRENT_PROJECT);
                                return;
                            }
                            PointDb.getInstance().deleteControlPoints(currentProjectConfig.value);
                        } else {//尝试清空当前线路所有控制点
                            String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
                            if (!StringUtil.isEmpty(roadId)) {
                                ThreadPoolUtil.execute(() -> {
                                    RoadRecord roadRecord = RoadDb.getInstance()
                                            .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
                                    if (roadRecord == null) {
                                        BingCeErrorCode.toast(BingCeErrorCode.NO_FOUND_ROAD_BY_ID);
                                        return;
                                    }
                                    Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                                    road.getControlPointList().clear();
                                    RoadDb.getInstance().save(road);
                                });
                            }
                        }
                    });
                    ToastUtils.showShort(R.string.clean_over);
                });
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        switch (keyCode) {
            case KeyEvent.KEYCODE_BACK:
                finish();
                break;
        }
        return true;
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        boolean isGlobal = IntentUtil.boolExtra(this, KEY_IS_GLOBAL);
        controlPointsImportConfigDialogUtils.onActivityResult(isGlobal, requestCode, resultCode, data);
    }

    public static class DATA {
        public final String roadId;
        public final int index;
        public final PointRecord pointRecord;
        public final String timeString;

        public DATA(PointRecord pointRecord) {
            this(null, -1, pointRecord, null);
        }

        DATA(String roadId, int index, PointRecord pointRecord, String timeString) {
            this.roadId = roadId;
            this.index = index;
            this.pointRecord = pointRecord;
            this.timeString = timeString;
        }

        @Override
        public String toString() {
            return "DATA{" +
                    "roadId='" + roadId + '\'' +
                    ", pointRecord=" + pointRecord.toString() +
                    ", timeString='" + timeString + '\'' +
                    '}';
        }

        @Override
        public boolean equals(Object o) {
            if (o instanceof DATA){
                DATA pointRecord = (DATA) o;
                return pointRecord.toString().equals(this.toString());
            }
            return false;
        }
    }

    private static class VH extends RecyclerView.ViewHolder {
        private final RecyclerviewItemControlPointLayoutBinding itemBinding;

        public VH(@NonNull RecyclerviewItemControlPointLayoutBinding binding) {
            super(binding.getRoot());
            itemBinding = binding;
        }
    }

    private final static String KEY_IS_GLOBAL = "is global";//是否查看全局控制点
    private final static String KEY_IS_RETURN_DATA = "is return data";//是否返回选择点

    private final static String RESULT_ID = "id";
    private final static String RESULT_NAME = "name";
    private final static String RESULT_CODE = "code";
    private final static String RESULT_REMARK = "remark";
    private final static String RESULT_X = "X";
    private final static String RESULT_Y = "Y";
    private final static String RESULT_Z = "Z";

    public static void pickPoint(FragmentActivity activity, boolean isGlobal, @NonNull IPickCallback callback) {
        Intent intent2 = new Intent(activity, ControlPointsActivity.class);
        intent2.putExtra(KEY_IS_RETURN_DATA, true);
        intent2.putExtra(KEY_IS_GLOBAL, isGlobal);
        if (isGlobal) {
            ActivityUtils.startActivityForResults(activity, intent2, result -> {
                onPicked(result, callback);
            });
        } else {
            ThreadPoolUtil.execute(() -> {
                String projectId = CachedCurrentProject.currentProjectId();
                String roadId = CachedCurrentRoad.currentRoadId(projectId);
                RoadIntentUtil.setRoadId(intent2, roadId);
                ThreadPoolUtil.executeInMain(() -> {
                    ActivityUtils.startActivityForResults(activity, intent2, result -> {
                        onPicked(result, callback);
                    });
                });
            });
        }
    }

    public static void pickPoint(Fragment fragment, boolean isGlobal, @NonNull IPickCallback callback) {
        Intent intent2 = new Intent(fragment.getContext(), ControlPointsActivity.class);
        intent2.putExtra(KEY_IS_RETURN_DATA, true);
        intent2.putExtra(KEY_IS_GLOBAL, isGlobal);
        if (isGlobal) {
            FragmentUtils.startActivityForResults(fragment, intent2, result -> {
                onPicked(result, callback);
            });
        } else {
            ThreadPoolUtil.execute(() -> {
                String projectId = CachedCurrentProject.currentProjectId();
                String roadId = CachedCurrentRoad.currentRoadId(projectId);
                RoadIntentUtil.setRoadId(intent2, roadId);
                ThreadPoolUtil.executeInMain(() -> {
                    FragmentUtils.startActivityForResults(fragment, intent2, result -> {
                        onPicked(result, callback);
                    });
                });
            });
        }
    }

    private static void onPicked(ActivityResult result, @NonNull IPickCallback callback) {
        if (result == null || result.getResultCode() != RESULT_OK || result.getData() == null) {
            return;
        }
        Intent intent = result.getData();
        callback.onPicked(
                intent.getStringExtra(RESULT_NAME),
                intent.getStringExtra(RESULT_CODE),
                intent.getDoubleExtra(RESULT_X, 0),
                intent.getDoubleExtra(RESULT_Y, 0),
                intent.getDoubleExtra(RESULT_Z, 0),
                intent.getStringExtra(RESULT_REMARK));
    }

    public static void pickPoint(FragmentActivity activity, boolean isGlobal, @NonNull IPickCallback2 callback) {
        Intent intent2 = new Intent(activity, ControlPointsActivity.class);
        intent2.putExtra(KEY_IS_RETURN_DATA, true);
        intent2.putExtra(KEY_IS_GLOBAL, isGlobal);
        if (isGlobal) {
            ActivityUtils.startActivityForResults(activity, intent2, result -> {
                onPicked2(result, callback);
            });
        } else {
            ThreadPoolUtil.execute(() -> {
                String projectId = CachedCurrentProject.currentProjectId();
                String roadId = CachedCurrentRoad.currentRoadId(projectId);
                RoadIntentUtil.setRoadId(intent2, roadId);
                ThreadPoolUtil.executeInMain(() -> {
                    ActivityUtils.startActivityForResults(activity, intent2, result -> {
                        onPicked2(result, callback);
                    });
                });
            });
        }
    }

    private static void onPicked2(ActivityResult result, @NonNull IPickCallback2 callback) {
        if (result == null || result.getResultCode() != RESULT_OK || result.getData() == null) {
            return;
        }
        Intent intent = result.getData();
        callback.onPicked(
                intent.getStringExtra(RESULT_ID),
                intent.getStringExtra(RESULT_NAME),
                intent.getStringExtra(RESULT_CODE),
                intent.getDoubleExtra(RESULT_X, 0),
                intent.getDoubleExtra(RESULT_Y, 0),
                intent.getDoubleExtra(RESULT_Z, 0),
                intent.getStringExtra(RESULT_REMARK));
    }

    public interface IPickCallback {
        void onPicked(String name, String code, double x, double y, double z, String remarks);
    }

    public interface IPickCallback2 {
        void onPicked(String pointId, String name, String code, double x, double y, double z, String remarks);
    }

    public static void startRoadControlListActivity(Context context, String roadId) {
        Intent intent = new Intent(context, ControlPointsActivity.class);
        RoadIntentUtil.setRoadId(intent, roadId);
        intent.putExtra(KEY_IS_GLOBAL, false);
        context.startActivity(intent);
    }

    public static void startGlobalControlListActivity(Context context) {
        Intent intent = new Intent(context, ControlPointsActivity.class);
        intent.putExtra(KEY_IS_GLOBAL, true);
        context.startActivity(intent);
    }
}
