package com.project.survey.ui.instrument.setupstation;

import static com.bingce.road.parameter.RoadAttributeControlPointConverter.readControlPoint;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.text.Editable;
import android.text.TextUtils;
import android.view.Gravity;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.TextView;

import androidx.activity.result.ActivityResult;
import androidx.annotation.NonNull;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.LiveData;
import androidx.paging.Pager;
import androidx.paging.PagingConfig;
import androidx.paging.PagingData;
import androidx.paging.PagingDataTransforms;
import androidx.paging.PagingLiveData;
import androidx.paging.PagingSource;
import androidx.recyclerview.widget.RecyclerView;

import com.afollestad.materialdialogs.MaterialDialog;
import com.afollestad.materialdialogs.internal.MDButton;
import com.bingce.data.cache.CachedCurrentProject;
import com.bingce.data.cache.CachedCurrentRoad;
import com.bingce.data.cache.CachedProject;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.PointDb;
import com.bingce.data.database.RoadDb;
import com.bingce.data.surveyor.designdata.road.RoadConstants;
import com.bingce.data.surveyor.designdata.road.RoadRecord;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.error.BingCeErrorCode;
import com.bingce.file.FileOperator;
import com.bingce.poi.excel.ExcelExportHelper;
import com.bingce.road.RoadIntentUtil;
import com.bingce.ui.BcDialogExt;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.DateUtils;
import com.bingce.utils.FileUtil;
import com.bingce.utils.FragmentUtils;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.SimpleTextWatcher;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.bingce.viewmodel.AbstractCoordinatePointsLibraryViewModelKt;
import com.kongzue.dialogx.dialogs.CustomDialog;
import com.kongzue.dialogx.interfaces.OnBindView;
import com.project.survey.App;
import com.project.survey.BuildConfig;
import com.project.survey.R;
import com.project.survey.databinding.ActivityStakingNewJobBinding;
import com.project.survey.ui.instrument.setupstation.adapter.ControlPointLibraryAdapter;
import com.project.survey.ui.instrument.setupstation.callback.ControlPointDiffCallback;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsClearDialogUtils;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsDetailDialogUtils;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsExportDialogUtils;
import com.project.survey.ui.instrument.setupstation.util.ControlPointsImportConfigDialogUtils;
import com.project.survey.ui.instrument.setupstation.util.StakingPointsLibraryActivitySQLiteUtils;
import com.rengwuxian.materialedittext.MaterialEditText;

import org.polaric.colorful.ColorfulActivity;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ConvertUtils;
import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;
import cn.liuyanbing.surveyor.model.base.Point;
import cn.liuyanbing.surveyor.model.base.PointArray;
import cn.liuyanbing.surveyor.model.base.Road;
import kotlinx.coroutines.Dispatchers;
import kotlinx.coroutines.ExecutorsKt;

public class ControlPointsNewActivity extends ColorfulActivity {
    private RecyclerView.OnScrollListener leftScrollListener = new RecyclerView.OnScrollListener() {
        @Override
        public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
            if (recyclerView.getScrollState() != RecyclerView.SCROLL_STATE_IDLE) {
                binding.bodyRv.scrollBy(dx, dy);
            }
        }

        @Override
        public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
            super.onScrollStateChanged(recyclerView, newState);
            binding.bodyHsv.setIntercept(newState != RecyclerView.SCROLL_STATE_IDLE);
        }
    };
    private RecyclerView.OnScrollListener rightScrollListener = new RecyclerView.OnScrollListener() {
        @Override
        public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
            if (!isFinishing() && recyclerView.getScrollState() != RecyclerView.SCROLL_STATE_IDLE) {
                binding.bodyNameRv.scrollBy(dx, dy);
            }
        }

        @Override
        public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
            super.onScrollStateChanged(recyclerView, newState);
            binding.bodyNameRl.setIntercept(newState != RecyclerView.SCROLL_STATE_IDLE);
        }
    };
    private static final int EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL = 330;
    private static final int EX_FILE_PICKER_RESULT_SURVEYOR_TXT = 331;
    private static final int EX_FILE_PICKER_RESULT_CASS_DAT = 332;
    private ActivityStakingNewJobBinding binding;
    private String[] mHeaders;

    private final ControlPointLibraryAdapter controlPointLibraryAdapter = new ControlPointLibraryAdapter(new ControlPointDiffCallback(), this, false);
    private final ControlPointLibraryAdapter controlPointNameLibraryAdapter = new ControlPointLibraryAdapter(new ControlPointDiffCallback(), this, true);

    //    private final ControlPointLibraryAdapter controlPointLibraryAdapter = new ControlPointLibraryAdapter(new ControlPointDiffCallback(),this,false);
//    private final ControlPointLibraryAdapter controlPointNameLibraryAdapter = new ControlPointLibraryAdapter(new ControlPointDiffCallback(),this,true);
    private String projectId;
    private AbstractCoordinatePointsLibraryViewModelKt.FilterParameter filterParameter = new AbstractCoordinatePointsLibraryViewModelKt.FilterParameter();
    private String roadKey;
    private Pager<Integer, PointRecord> globalPager = new Pager<>(
            new PagingConfig(100),
            this::dataSourse);
    private Pager<Integer, RoadRecord> roadPager = new Pager<>(
            new PagingConfig(100),
            this::dataSourseRoad);

    private PagingSource<Integer, PointRecord> dataSourse() {
        return PointDb.getInstance().rawQueryPagingSource(StakingPointsLibraryActivitySQLiteUtils.searchSQLit(
                projectId,
                filterParameter, false, binding.etEnterNameCode.getText().toString().trim()
        ));
    }

    private List<PointRecord> getAllPoint() {
        return PointDb.getInstance().rawQueryListData(StakingPointsLibraryActivitySQLiteUtils.searchSQLit(
                projectId,
                filterParameter, false, binding.etEnterNameCode.getText().toString().trim()
        ));
    }

    private PagingSource<Integer, RoadRecord> dataSourseRoad() {
        String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
        if (TextUtils.isEmpty(roadId)) {
            projectId = CachedCurrentProject.currentProjectId();
            roadId = CachedCurrentRoad.currentRoadId(projectId);
        }
        return RoadDb.getInstance().rawQueryPagingSource(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
    }

    private List<RoadRecord> getAllRoadPoint() {
        String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
        if (TextUtils.isEmpty(roadId)) {
            projectId = CachedCurrentProject.currentProjectId();
            roadId = CachedCurrentRoad.currentRoadId(projectId);
        }
        return RoadDb.getInstance().rawQueryListData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
    }

    private final ControlPointsImportConfigDialogUtils controlPointsImportConfigDialogUtils = new ControlPointsImportConfigDialogUtils(this);
    private final ControlPointsExportDialogUtils controlPointsExportDialogUtils = new ControlPointsExportDialogUtils(this);
    private final ControlPointsDetailDialogUtils controlPointsDetailDialogUtils = new ControlPointsDetailDialogUtils(this);
    private final ControlPointsClearDialogUtils controlPointsClearDialogUtils = new ControlPointsClearDialogUtils(this);
    private static boolean isPickPoint;

    private LiveData<PagingData<PointRecord>> pagingDataLiveData;
    private LiveData<PagingData<RoadRecord>> pagingDataLiveDataRoad;
    private final FilterHandlerNew filterHandler = new FilterHandlerNew(new FilterHandlerNew.FilterCallBack() {
        @Override
        public void fireByKey(String key) {
            ThreadPoolUtil.execute(() -> {
                boolean isGlobal = IntentUtil.boolExtra(ControlPointsNewActivity.this, KEY_IS_GLOBAL);
                if (isGlobal) {
                    projectId = CachedProject.currentProjectId();
                    ThreadPoolUtil.executeInMain(() -> {
                        pagingDataLiveData.removeObservers(ControlPointsNewActivity.this);
                        pagingDataLiveData = PagingLiveData.cachedIn(PagingLiveData.getLiveData(globalPager), getLifecycle());
                        pagingDataLiveData.observe(ControlPointsNewActivity.this, downloadRecordPagingData -> {
                            refreshAdapter(downloadRecordPagingData);
                        });
                    });
                } else {
                    ThreadPoolUtil.executeInMain(() -> {
                        roadKey = key;
                        pagingDataLiveDataRoad = PagingLiveData.cachedIn(PagingLiveData.getLiveData(roadPager), getLifecycle());
                        pagingDataLiveDataRoad.observe(ControlPointsNewActivity.this, downloadRecordPagingData -> {
                            refreshAdapterRoad(downloadRecordPagingData);
                        });
                    });
                }
            });
        }
    });

    private void refreshAdapter(PagingData<PointRecord> downloadRecordPagingData) {
//        boolean isGlobal = IntentUtil.boolExtra(this, KEY_IS_GLOBAL);
//        if (isGlobal) {
        PagingData<ControlPointsActivity.DATA> map = PagingDataTransforms.map(downloadRecordPagingData, ExecutorsKt.asExecutor(Dispatchers.getIO()), ControlPointsActivity.DATA::new);
        controlPointLibraryAdapter.submitData(getLifecycle(), map);
        controlPointNameLibraryAdapter.submitData(getLifecycle(), map);
        controlPointLibraryAdapter.resetConfig();
        controlPointNameLibraryAdapter.resetConfig();
        controlPointLibraryAdapter.notifyDataSetChanged();
        controlPointNameLibraryAdapter.notifyDataSetChanged();
        new Handler().postDelayed(() -> binding.tvPointNumber.setText(controlPointLibraryAdapter.getItemCount() + ""), 500);
//        }else {

//        }
    }

    private List<ControlPointsActivity.DATA> tempData = new ArrayList<>();

    private void refreshAdapterRoad(PagingData<RoadRecord> downloadRecordPagingData) {
        PagingDataTransforms.map(downloadRecordPagingData, ExecutorsKt.asExecutor(Dispatchers.getIO()), item -> {
            item.toString();
            return null;
        });
        ThreadPoolUtil.execute(() -> {
            String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
            if (TextUtils.isEmpty(roadId)) {
                projectId = CachedCurrentProject.currentProjectId();
                roadId = CachedCurrentRoad.currentRoadId(projectId);
            }
            String finalRoadId = roadId;
            ThreadPoolUtil.executeInMain(() -> {
                PagingData<ControlPointsActivity.DATA> map = PagingDataTransforms.map(downloadRecordPagingData, ExecutorsKt.asExecutor(Dispatchers.getIO()), item -> {
                    if (item == null) return null;
                    PointArray pointArray = new PointArray();
                    String error = readControlPoint(item.controlPointJson, item.name, pointArray);
                    if (!StringUtil.isEmpty(error)) {
                        ToastUtils.showShort(error);
                        return null;
                    }
                    List<ControlPointsActivity.DATA> data = new ArrayList<>();
                    for (int index = 0; index < pointArray.size(); index++) {
                        Point point = pointArray.get(index);
                        if (!TextUtils.isEmpty(roadKey) && !point.getName().contains(roadKey) && !point.getCode().contains(roadKey))
                            continue;
                        data.add(new ControlPointsActivity.DATA(finalRoadId, index,
                                new PointRecord(null, null, null,
                                        -1,
                                        point.getName(), point.getCode(), point.getRemarks(),
                                        point.getX(), point.getY(), point.getZ(),
                                        0, 0, 0,
                                        PointConstants.POINT_FORMAT_XYZ, null), point.getTime()));
                    }
                    tempData.clear();
                    tempData.addAll(data);
                    ThreadPoolUtil.executeInMain(() -> {
                        controlPointLibraryAdapter.submitData(getLifecycle(), PagingData.Companion.from(data));
                        controlPointNameLibraryAdapter.submitData(getLifecycle(), PagingData.Companion.from(data));
                        controlPointLibraryAdapter.resetConfig();
                        controlPointNameLibraryAdapter.resetConfig();
                        controlPointLibraryAdapter.notifyDataSetChanged();
                        controlPointNameLibraryAdapter.notifyDataSetChanged();
                        binding.tvPointNumber.setText("" + data.size());
                    });

                    return new ControlPointsActivity.DATA(null);
                });
                controlPointLibraryAdapter.submitData(getLifecycle(), map);
            });
        });


    }

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = ActivityStakingNewJobBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        setSupportActionBar(binding.toolbar);
        if (getSupportActionBar() != null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        }
//        setTitle(getString(R.string.staking_point_library)/*,getString(R.string.clear)*/);
        mHeaders = new String[]{getString(R.string.category),
                getString(R.string.point_name),
                getString(R.string.point_code),
                "X", "Y", "Z",
                getString(R.string.remarks),
                getString(R.string.record_time)};
        binding.bodyRv.setAdapter(controlPointLibraryAdapter);
        binding.bodyNameRv.setAdapter(controlPointNameLibraryAdapter);
        binding.bodyHsv.setScrollView(binding.headerHsv);
        binding.headerHsv.setScrollView(binding.bodyHsv);
        binding.bodyRv.addOnScrollListener(rightScrollListener);
        binding.bodyNameRv.addOnScrollListener(leftScrollListener);
        binding.bodyRv.addOnItemTouchListener(new SimpleRecyclerViewItemClickListener(binding.bodyRv, new SimpleRecyclerViewItemClickListener.OnItemClickListener() {
            @Override
            public void onItemClick(View view, int position) {
                controlPointLibraryAdapter.setSelItemPos(position);
                controlPointNameLibraryAdapter.setSelItemPos(position);
            }
        }));
        binding.bodyNameRv.addOnItemTouchListener(new SimpleRecyclerViewItemClickListener(binding.bodyNameRv, new SimpleRecyclerViewItemClickListener.OnItemClickListener() {
            @Override
            public void onItemClick(View view, int position) {
                controlPointLibraryAdapter.setSelItemPos(position);
                controlPointNameLibraryAdapter.setSelItemPos(position);
            }
        }));


        boolean isGlobal = IntentUtil.boolExtra(this, KEY_IS_GLOBAL);
        ThreadPoolUtil.execute(() -> {
            if (isGlobal) {
                projectId = CachedProject.currentProjectId();
                ThreadPoolUtil.executeInMain(() -> {
                    pagingDataLiveData = PagingLiveData.cachedIn(PagingLiveData.getLiveData(globalPager), getLifecycle());
                    pagingDataLiveData
                            .observe(this, downloadRecordPagingData -> {
                                refreshAdapter(downloadRecordPagingData);
                            });

                });
            } else {
                ThreadPoolUtil.executeInMain(() -> {
                    pagingDataLiveDataRoad = PagingLiveData.cachedIn(PagingLiveData.getLiveData(roadPager), getLifecycle());
                    pagingDataLiveDataRoad.observe(this, downloadRecordPagingData -> {
                        refreshAdapterRoad(downloadRecordPagingData);
                    });
                });
            }
        });
//        if (isGlobal) {
//
//            holderInActivity.getViewModel().getLiveData().observe(this, pointRecordUIData -> {
//                if (pointRecordUIData == null) {
//                    return;
//                }
//                data.clear();
//                List<PointRecord> pointRecords = pointRecordUIData.getRecords();
//                holderInActivity.getViewModel().updateUI(this, binding.pageLayout, pointRecordUIData);
//                for (PointRecord record : pointRecords) {
//                    data.add(new ControlPointsActivity.DATA(record));
//                }
//
//                ThreadPoolUtil.execute(() -> onPointDataChanged(data));
//            });
//        }else {
//            ThreadPoolUtil.execute(() -> {
//                String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
//                if (TextUtils.isEmpty(roadId)) {
//                    String projectId = CachedCurrentProject.currentProjectId();
//                    roadId = CachedCurrentRoad.currentRoadId(projectId);
//                }
//                String finalRoadId = roadId;
//                runOnUiThread(() -> {
//                    RoadDb.getInstance()
//                            .rawQueryLiveData(DBQueryConstant.findById(RoadConstants.DB_NAME, finalRoadId))
//                            .observe(this, roadRecord -> {
//                                if (roadRecord == null) {
//                                    return;
//                                }
//                                PointArray pointArray = new PointArray();
//                                String error = readControlPoint(roadRecord.controlPointJson, roadRecord.name, pointArray);
//                                if (!StringUtil.isEmpty(error)) {
//                                    ToastUtils.showShort(error);
//                                    return;
//                                }
//                                data.clear();
//                                for (int index = 0; index < pointArray.size(); index++) {
//                                    Point point = pointArray.get(index);
//                                    data.add(new ControlPointsActivity.DATA(finalRoadId, index,
//                                            new PointRecord(null, null, null,
//                                                    -1,
//                                                    point.getName(), point.getCode(), point.getRemarks(),
//                                                    point.getX(), point.getY(), point.getZ(),
//                                                    0, 0, 0,
//                                                    PointConstants.POINT_FORMAT_XYZ, null), point.getTime()));
//                                }
//                                ThreadPoolUtil.execute(() -> onPointDataChanged(data));
//                            });
//                });
//            });
//        }

//        if (isGlobal) {
//            String projectId = CachedCurrentProject.currentProjectId();
//
//            runOnUiThread(() -> {
//                PointDb.getInstance()
//                        .rawQueryListLiveData(PointConstants.findControlPointByProject(projectId))
//                        .observe(this, pointRecords -> {
//                            List<ControlPointsActivity.DATA> dataList = new ArrayList<>();
//                            for (PointRecord record : pointRecords) {
//                                dataList.add(new ControlPointsActivity.DATA(record));
//                            }
//                            ThreadPoolUtil.execute(() -> onPointDataChanged(dataList));
//                        });
//            });
//        }


        binding.pointsLibraryBtnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                int row = controlPointLibraryAdapter.getSelItemPos();
                ControlPointsActivity.DATA data = controlPointLibraryAdapter.getSelItem();
                if (data == null) {
                    ToastUtils.showShort(R.string.please_select_one_data_item);
                    return;
                }
                Intent intent = new Intent();
                PointRecord instance = data.pointRecord;
                intent.putExtra(RESULT_ID, instance.id);
                intent.putExtra(RESULT_NAME, instance.name);
                intent.putExtra(RESULT_CODE, instance.code);
                intent.putExtra(RESULT_REMARK, instance.remarks);
                intent.putExtra(RESULT_X, instance.x);
                intent.putExtra(RESULT_Y, instance.y);
                intent.putExtra(RESULT_Z, instance.h);
                intent.putExtra(RESULT_B, instance.latitude);
                intent.putExtra(RESULT_L, instance.longitude);
                intent.putExtra(RESULT_H, instance.altitude);
                setResult(RESULT_OK, intent);
                finish();
            }
        });
        binding.pointsLibraryBtnNewCreate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
                controlPointsDetailDialogUtils.showControlPointDialog(
                        roadId, null, getString(R.string.new_control_point), new ControlPointsDetailDialogUtils.DataCallBack() {
                            @Override
                            public void add(ControlPointsActivity.DATA data) {
//                                datasource().onNewPointAdded(data);
                            }
                        });
            }
        });

        boolean isSelect = getIntent().getBooleanExtra(KEY_IS_SELECT, false);

        if (isGlobal) {
            binding.pointsLibraryBtnFilter.setVisibility(View.VISIBLE);
        } else {
            binding.pointsLibraryBtnFilter.setVisibility(View.GONE);
        }
        if (isSelect) {
            binding.pointsLibraryBtnConfirm.setVisibility(View.VISIBLE);
            binding.pointsLibraryBtnDelete.setVisibility(View.GONE);
            binding.pointsLibraryBtnMore.setVisibility(View.GONE);
        } else {
            binding.pointsLibraryBtnConfirm.setVisibility(View.GONE);
            binding.pointsLibraryBtnDelete.setVisibility(View.VISIBLE);
            binding.pointsLibraryBtnMore.setVisibility(View.VISIBLE);
        }
        binding.etEnterNameCode.addTextChangedListener(new SimpleTextWatcher() {
            @Override
            public void afterTextChanged(Editable s) {
                filterHandler.onFilterStringChanged(s.toString());
            }
        });
        binding.pointsLibraryBtnFilter.setOnClickListener(v -> {
//            boolean forceSurvey = IntentUtil.boolExtra(StakingNewJobActivity.this, StakingNewJobActivity.FORCE_SURVEY_POINT_LIBRARY);
            PointFilterActivity.startFilter(ControlPointsNewActivity.this, false,
                    filterParameter.getNeedSurvey(), filterParameter.getNeedInput(), filterParameter.getNeedControl(),
                    filterParameter.getDateStart(), filterParameter.getDateEnd(),
                    filterParameter.getNameKey(), filterParameter.getCodeKey(), filterParameter.getRemarkKey(),
                    (needSurvey, needInput, needControl, dateStart, dateEnd, nameKey, codeKey, remarksKey) -> {
//                        AbstractCoordinatePointsLibraryViewModelKt.FilterParameter filterParameter = holderInActivity.getViewModel().getFilterParameter();
                        filterParameter.setNeedSurvey(needSurvey);
                        filterParameter.setNeedInput(needInput);
                        filterParameter.setNeedControl(needControl);
                        filterParameter.setDateStart(dateStart);
                        filterParameter.setDateEnd(dateEnd);
                        filterParameter.setNameKey(nameKey);
                        filterParameter.setCodeKey(codeKey);
                        filterParameter.setRemarkKey(remarksKey);
//                        holderInActivity.getViewModel().update(filterParameter);
                        ThreadPoolUtil.execute(() -> {
                            projectId = CachedProject.currentProjectId();
                            ThreadPoolUtil.executeInMain(() -> {
                                pagingDataLiveData.removeObservers(this);
                                pagingDataLiveData =
                                        PagingLiveData.cachedIn(PagingLiveData.getLiveData(globalPager), getLifecycle());
                                pagingDataLiveData.observe(this, downloadRecordPagingData -> {
                                    refreshAdapter(downloadRecordPagingData);
                                });
                            });
                        });
                    });
        });

        binding.pointsLibraryBtnNewCreate.setOnClickListener(v -> {

            String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
            controlPointsDetailDialogUtils.showControlPointDialog(
                    roadId, null, getString(R.string.new_control_point), new ControlPointsDetailDialogUtils.DataCallBack() {
                        @Override
                        public void add(ControlPointsActivity.DATA data) {
//                            datasource().onNewPointAdded(data);
                        }
                    });
        });

        binding.pointsLibraryBtnDelete.setOnClickListener(v -> {
            int row = controlPointLibraryAdapter.getSelItemPos();
            ControlPointsActivity.DATA record = controlPointLibraryAdapter.getSelItem();
            if (record == null) {
                ToastUtils.showShort(R.string.please_select_one_data_item);
                return;
            }
            BcDialogExt.create().showMessage(getString(R.string.warning), getString(R.string.want_to_delete_point_library), getString(R.string.confirm), getString(R.string.cancel), (baseDialog, view) -> {
                ThreadPoolUtil.execute(() -> {
                    if (isGlobal) {
                        PointDb.getInstance().deleteById(record.pointRecord.id);
                    } else {
                        RoadRecord roadRecord = RoadDb.getInstance()
                                .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, record.roadId));
                        if (roadRecord == null) {
                            BingCeErrorCode.toast(BingCeErrorCode.NO_FOUND_ROAD_BY_ID);
                            return;
                        }
                        Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                        if (road == null) {
                            BingCeErrorCode.toast(BingCeErrorCode.ROAD_RECORD_CONVERT_2_ROAD_FAILURE);
                            return;
                        }

                        if (record.index >= 0 && record.index < road.getControlPointList().size()) {
                            road.getControlPointList().remove(record.index);
                            RoadDb.getInstance().save(road);
                        }
                    }
                    setResult(LauncherEvent.launcher_delete_point_lib);
                });
                return false;
            }, (baseDialog, view) -> false);


//            tableAdapter.datasource().onDeleteSelected(StakingNewJobActivity.this);
//            datasource().onDelete(row,isGlobal);

        });

        binding.pointsLibraryBtnMore.setOnClickListener(v -> {

            CustomDialog.show(new OnBindView<CustomDialog>(R.layout.layout_menu_more) {

                        private TextView tvImport;
                        private TextView tvExport;
                        private TextView tvEdit;

                        @Override
                        public void onBind(final CustomDialog dialog, View v) {
                            if (App.instance.isThemeDark()) {
                                v.setBackground(getDrawable(R.drawable.round_dark));
                            }
                            tvImport = v.findViewById(R.id.tv_import);
                            tvExport = v.findViewById(R.id.tv_export);
                            tvEdit = v.findViewById(R.id.tv_edit);
                            tvImport.setOnClickListener(v13 -> {
                                controlPointsImportConfigDialogUtils.showImportConfigDialog();
                                dialog.dismiss();
                            });
                            tvExport.setOnClickListener(v12 -> controlPointsExportDialogUtils.showExportDialog(name -> {
                                exportExcelAsync(name, mHeaders);
                                dialog.dismiss();
                            }));
                            tvEdit.setOnClickListener(v1 -> {
                                int row = controlPointLibraryAdapter.getSelItemPos();
                                ControlPointsActivity.DATA data = controlPointLibraryAdapter.getSelItem();
                                if (data == null) {
                                    ToastUtils.showShort(R.string.please_select_one_data_item);
                                    return;
                                }
                                controlPointsDetailDialogUtils.showControlPointDialog(
                                        data.roadId, data, getString(R.string.edit_control_point), new ControlPointsDetailDialogUtils.DataCallBack() {
                                            @Override
                                            public void edit(ControlPointsActivity.DATA data) {
                                                filterHandler.onFilterStringChanged(binding.etEnterNameCode.getText().toString());
                                            }
                                        });
                                dialog.dismiss();
                            });
                        }
                    }, CustomDialog.ALIGN.RIGHT_TOP)
                    .setCancelable(false)
                    .setCancelable(true)
                    .setEnterAnimResId(R.anim.anim_dialogx_bottom_enter)
                    .setExitAnimResId(R.anim.anim_dialogx_bottom_exit)
//                    .setMaskColor(getColor(R.color.black30))
                    .setAlignBaseViewGravity(
                            binding.pointsLibraryBtnMore,
                            Gravity.TOP | Gravity.RIGHT
                    )
                    .setBaseViewMarginLeft(ConvertUtils.dp2px(-106f))
                    .show();
//            controlPointsExportDialogUtils.showExportDialog(name -> {
//                exportExcelAsync(name, mHeaders);
//            });
//            showImportConfigDialog();
//            AbstractCoordinatePointsLibraryViewModelKt.FilterParameter filterParameter = holderInActivity.getViewModel().getFilterParameter();
//            ResultsTheExportActivity.start(StakingNewJobActivity.this, getString(R.string.export), false,
//                    filterParameter.getNameKey(),
//                    filterParameter.getCodeKey(),
//                    filterParameter.getRemarkKey(),
//                    filterParameter.getDateStart(),
//                    filterParameter.getDateEnd());
        });
    }

    private final List<List<String>> exportResult = new ArrayList<>();

    @SuppressLint("StaticFieldLeak")
    private void exportExcelAsync(final String name, final String[] headers) {
        new AsyncTask<Integer, Integer, Boolean>() {

            @Override
            protected void onPreExecute() {
                showLoading();
            }

            @Override
            protected Boolean doInBackground(Integer... params) {
                exportResult.clear();

                for (int i = 0; i < controlPointLibraryAdapter.getItemCount(); i++) {
                    ControlPointsActivity.DATA _data = controlPointLibraryAdapter.item(i);
                    if (_data == null) continue;
                    PointRecord point = _data.pointRecord;
                    ArrayList<String> rowResult = new ArrayList<>();

                    String type = getString(R.string.control_point);

                    rowResult.add(type);
                    rowResult.add(point.name);
                    rowResult.add(point.code);
                    rowResult.add(Util.formatDouble2StringDotAuto(point.x));
                    rowResult.add(Util.formatDouble2StringDotAuto(point.y));
                    rowResult.add(Util.formatDouble2StringDotAuto(point.h));
                    rowResult.add(point.remarks);
                    rowResult.add(DateUtils.toFull(point.createDate));
                    exportResult.add(rowResult);
                }
                if (!exportResult.isEmpty()) {
                    new ExcelExportHelper().exportExcelForBigDataAndSave(headers, exportResult, getString(R.string.point_data), ExcelExportHelper.MORE_SHEET_FLAG, FileUtil.getSDPath() + "/" + getString(R.string.surveyor_exported_file) + "/" + getString(R.string.point_data), name);
                }
                return true;
            }

            @Override
            protected void onPostExecute(Boolean result) {
                super.onPostExecute(result);
                hideLoading();
                File file = new File(FileUtil.getSDPath() + "/" + getString(R.string.surveyor_exported_file) + "/" + getString(R.string.point_data) + "/" + name + ".xls");
                if (file.exists()) {
                    FileUtil.scanFile(ControlPointsNewActivity.this, file);
                    showExportOverDialog(file);
                }
            }
        }.execute();
    }

    private void showExportOverDialog(final File file) {
        new MaterialDialog.Builder(this).title(R.string.export_successful)
                .content("导出文件保存在手机根目录的[工程测量导出文件]文件夹内")
                .positiveText(R.string.send_to)
                .negativeText(R.string.open)
                .neutralText(R.string.close)
                .onPositive((dialog, which) -> FileOperator.shareFile(ControlPointsNewActivity.this, file, BuildConfig.APPLICATION_ID))
                .onNegative((dialog, which) -> FileUtil.openFile(ControlPointsNewActivity.this, file))
                .show();
    }

    private MaterialEditText met_name, met_code, met_x, met_y, met_z, met_remarks;
    private MDButton mPositiveAction;

//    @WorkerThread
//    public void onPointDataChanged(List<ControlPointsActivity.DATA> pointSurveyRecords) {
//        //显示loading
//        ThreadPoolUtil.executeInMain(this::showLoading);
//        if (pointSurveyRecords == null) {
//            pointSurveyRecords = new ArrayList<>();
//        }
//        IDatasourceControl datasource = datasource();
//        boolean isEditMode = datasource.isEditMode();
//        datasource.onDataChanged(pointSurveyRecords, isEditMode);
//        //隐藏loading
//        ThreadPoolUtil.executeInMain(this::hideLoading);
//    }

//    IDatasourceControl datasource() {
//        return tableAdapter.datasource();
//    }

    void showLoading() {
        if (binding.loadingLayout.getVisibility() != View.VISIBLE) {
            binding.loadingLayout.setVisibility(View.VISIBLE);
        }
    }

    void hideLoading() {
        if (binding.loadingLayout.getVisibility() == View.VISIBLE) {
            binding.loadingLayout.setVisibility(View.GONE);
        }
    }

    private boolean isOverrideImport;
    private boolean cassXY = false;//cass格式按照xy排序，否则yx排序

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        boolean isGlobal = IntentUtil.boolExtra(this, KEY_IS_GLOBAL);
        controlPointsImportConfigDialogUtils.onActivityResult(isGlobal, requestCode, resultCode, data);
    }

    //    public final ActivityResultLauncher<Intent> launcher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
//        if (result != null) {
//            if (result.getResultCode() == LauncherEvent.launcher_update_point_library) {
//                holderInActivity.getViewModel().refreshCurrentPage();
//            }
//        }
//    });
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        boolean isSelect = getIntent().getBooleanExtra(KEY_IS_SELECT, false);
        if (!isSelect) {
            getMenuInflater().inflate(R.menu.menu_activity_staking_clear, menu);
        }
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        String roadId = RoadIntentUtil.getRoadIdFromIntent(getIntent());
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                break;
//            case R.id.menu_job_create:
//                controlPointsDetailDialogUtils.showControlPointDialog(
//                        roadId, null, getString(R.string.new_control_point), new ControlPointsDetailDialogUtils.DataCallBack() {
//                            @Override
//                            public void add(ControlPointsActivity.DATA data) {
//                                datasource().onNewPointAdded(data);
//                            }
//                        });
//                break;
            case R.id.menu_job_clear:
//                controlPointsClearDialogUtils.showClearDialog(() -> {
                BcDialogExt.create().showMessageWithTick(getString(R.string.warning), getString(R.string.want_to_empty_point_library), getString(R.string.confirm), getString(R.string.cancel), (baseDialog, view) -> {
                    ThreadPoolUtil.execute(() -> {
                        boolean isGlobal = IntentUtil.boolExtra(this, KEY_IS_GLOBAL);
                        if (isGlobal) {//清空当前项目所有全局控制点
                            controlPointLibraryAdapter.clear(getAllPoint());
                        } else {//尝试清空当前线路所有控制点
                            if (!StringUtil.isEmpty(roadId)) {
                                ThreadPoolUtil.execute(() -> {
                                    RoadRecord roadRecord = RoadDb.getInstance()
                                            .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
                                    if (roadRecord == null) {
                                        BingCeErrorCode.toast(BingCeErrorCode.NO_FOUND_ROAD_BY_ID);
                                        return;
                                    }
                                    Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                                    if (road == null) {
                                        BingCeErrorCode.toast(BingCeErrorCode.ROAD_RECORD_CONVERT_2_ROAD_FAILURE);
                                        return;
                                    }

                                    for (int i = controlPointLibraryAdapter.getItemCount() - 1; i >= 0; i--) {
                                        ControlPointsActivity.DATA item1 = tempData.get(i);
                                        if (item1 != null) {
                                            road.getControlPointList().remove(item1.index);
                                        }
                                    }
                                    RoadDb.getInstance().save(road);
                                });
                            }
                        }
                    });
                    ToastUtils.showShort(R.string.clean_over);
                    return false;
                }, (baseDialog, view) -> false);

//                });
                break;
        }
        return super.onOptionsItemSelected(item);
    }

    private final static String KEY_IS_SELECT = "is_selected";
    private final static String KEY_IS_GLOBAL = "is global";//是否查看全局控制点
    private final static String RESULT_ID = "id";
    private final static String RESULT_NAME = "name";
    private final static String RESULT_CODE = "code";
    private final static String RESULT_REMARK = "remark";
    private final static String RESULT_X = "X";
    private final static String RESULT_Y = "Y";
    private final static String RESULT_Z = "Z";
    private final static String RESULT_B = "B";
    private final static String RESULT_L = "L";
    private final static String RESULT_H = "H";

    public static void pickPoint(FragmentActivity activity, boolean isGlobal, @NonNull IPickXyzCallback callback) {
        Intent intent2 = new Intent(activity, ControlPointsNewActivity.class);
        intent2.putExtra(KEY_IS_SELECT, true);
        intent2.putExtra(KEY_IS_GLOBAL, isGlobal);
        if (isGlobal) {
            ActivityUtils.startActivityForResults(activity, intent2, result -> {
                onPicked(result, callback);
            });
        } else {
            ThreadPoolUtil.execute(() -> {
                String projectId = CachedCurrentProject.currentProjectId();
                String roadId = CachedCurrentRoad.currentRoadId(projectId);
                RoadIntentUtil.setRoadId(intent2, roadId);
                ThreadPoolUtil.executeInMain(() -> {
                    ActivityUtils.startActivityForResults(activity, intent2, result -> {
                        onPicked(result, callback);
                    });
                });
            });
        }
    }

    public static void pickPoint(Fragment fragment, boolean isGlobal, @NonNull IPickXyzCallback callback) {
        Intent intent2 = new Intent(fragment.getContext(), ControlPointsNewActivity.class);
        intent2.putExtra(KEY_IS_SELECT, true);
        intent2.putExtra(KEY_IS_GLOBAL, isGlobal);
        if (isGlobal) {
            FragmentUtils.startActivityForResults(fragment, intent2, result -> {
                onPicked(result, callback);
            });
        } else {
            ThreadPoolUtil.execute(() -> {
                String projectId = CachedCurrentProject.currentProjectId();
                String roadId = CachedCurrentRoad.currentRoadId(projectId);
                RoadIntentUtil.setRoadId(intent2, roadId);
                ThreadPoolUtil.executeInMain(() -> {
                    FragmentUtils.startActivityForResults(fragment, intent2, result -> {
                        onPicked(result, callback);
                    });
                });
            });
        }
    }

    public static void pickPoint(FragmentActivity activity, boolean isGlobal, @NonNull IPickXyzBlhCallback callback) {
        Intent intent2 = new Intent(activity, ControlPointsNewActivity.class);
        intent2.putExtra(KEY_IS_SELECT, true);
        intent2.putExtra(KEY_IS_GLOBAL, isGlobal);
        if (isGlobal) {
            ActivityUtils.startActivityForResults(activity, intent2, result -> {
                onPicked(result, callback);
            });
        } else {
            ThreadPoolUtil.execute(() -> {
                String projectId = CachedCurrentProject.currentProjectId();
                String roadId = CachedCurrentRoad.currentRoadId(projectId);
                RoadIntentUtil.setRoadId(intent2, roadId);
                ThreadPoolUtil.executeInMain(() -> {
                    ActivityUtils.startActivityForResults(activity, intent2, result -> {
                        onPicked(result, callback);
                    });
                });
            });
        }
    }

    public static void pickPoint(Fragment fragment, boolean isGlobal, @NonNull IPickXyzBlhCallback callback) {
        Intent intent2 = new Intent(fragment.getContext(), ControlPointsNewActivity.class);
        intent2.putExtra(KEY_IS_SELECT, true);
        intent2.putExtra(KEY_IS_GLOBAL, isGlobal);
        if (isGlobal) {
            FragmentUtils.startActivityForResults(fragment, intent2, result -> {
                onPicked(result, callback);
            });
        } else {
            ThreadPoolUtil.execute(() -> {
                String projectId = CachedCurrentProject.currentProjectId();
                String roadId = CachedCurrentRoad.currentRoadId(projectId);
                RoadIntentUtil.setRoadId(intent2, roadId);
                ThreadPoolUtil.executeInMain(() -> {
                    FragmentUtils.startActivityForResults(fragment, intent2, result -> {
                        onPicked(result, callback);
                    });
                });
            });
        }
    }

    private static void onPicked(ActivityResult result, @NonNull IPickXyzCallback callback) {
        if (result == null || result.getResultCode() != RESULT_OK || result.getData() == null) {
            return;
        }
        Intent intent = result.getData();
        callback.onPicked(
                intent.getStringExtra(RESULT_NAME),
                intent.getStringExtra(RESULT_CODE),
                intent.getDoubleExtra(RESULT_X, 0),
                intent.getDoubleExtra(RESULT_Y, 0),
                intent.getDoubleExtra(RESULT_Z, 0),
                intent.getStringExtra(RESULT_REMARK));
    }

    private static void onPicked(ActivityResult result, @NonNull IPickXyzBlhCallback callback) {
        if (result == null || result.getResultCode() != RESULT_OK || result.getData() == null) {
            return;
        }
        Intent intent = result.getData();
        callback.onPicked(
                intent.getStringExtra(RESULT_NAME),
                intent.getStringExtra(RESULT_CODE),
                intent.getDoubleExtra(RESULT_X, 0),
                intent.getDoubleExtra(RESULT_Y, 0),
                intent.getDoubleExtra(RESULT_Z, 0),
                intent.getDoubleExtra(RESULT_B, 0),
                intent.getDoubleExtra(RESULT_L, 0),
                intent.getDoubleExtra(RESULT_H, 0),
                intent.getStringExtra(RESULT_REMARK));
    }

    /**
     *  isGlobal false 线路控制点 true 工程控制点
     *
     */
    public static void pickPoint(FragmentActivity activity, boolean isGlobal, @NonNull IPickXyzCallback2 callback) {
        Intent intent2 = new Intent(activity, ControlPointsNewActivity.class);
        intent2.putExtra(KEY_IS_SELECT, true);
        intent2.putExtra(KEY_IS_GLOBAL, isGlobal);
        if (isGlobal) {
            ActivityUtils.startActivityForResults(activity, intent2, result -> {
                onPicked2(result, callback);
            });
        } else {
            ThreadPoolUtil.execute(() -> {
                String projectId = CachedCurrentProject.currentProjectId();
                String roadId = CachedCurrentRoad.currentRoadId(projectId);
                RoadIntentUtil.setRoadId(intent2, roadId);
                ThreadPoolUtil.executeInMain(() -> {
                    ActivityUtils.startActivityForResults(activity, intent2, result -> {
                        onPicked2(result, callback);
                    });
                });
            });
        }
    }

    private static void onPicked2(ActivityResult result, @NonNull IPickXyzCallback2 callback) {
        if (result == null || result.getResultCode() != RESULT_OK || result.getData() == null) {
            return;
        }
        Intent intent = result.getData();
        callback.onPicked(
                intent.getStringExtra(RESULT_ID),
                intent.getStringExtra(RESULT_NAME),
                intent.getStringExtra(RESULT_CODE),
                intent.getDoubleExtra(RESULT_X, 0),
                intent.getDoubleExtra(RESULT_Y, 0),
                intent.getDoubleExtra(RESULT_Z, 0),
                intent.getStringExtra(RESULT_REMARK));
    }

    private static void onPicked2(ActivityResult result, @NonNull IPickXyzBlhCallback2 callback) {
        if (result == null || result.getResultCode() != RESULT_OK || result.getData() == null) {
            return;
        }
        Intent intent = result.getData();
        callback.onPicked(
                intent.getStringExtra(RESULT_ID),
                intent.getStringExtra(RESULT_NAME),
                intent.getStringExtra(RESULT_CODE),
                intent.getDoubleExtra(RESULT_X, 0),
                intent.getDoubleExtra(RESULT_Y, 0),
                intent.getDoubleExtra(RESULT_Z, 0),
                intent.getDoubleExtra(RESULT_B, 0),
                intent.getDoubleExtra(RESULT_L, 0),
                intent.getDoubleExtra(RESULT_H, 0),
                intent.getStringExtra(RESULT_REMARK));
    }

    public static void startRoadControlListActivity(Context context, String roadId) {
        Intent intent = new Intent(context, ControlPointsNewActivity.class);
        RoadIntentUtil.setRoadId(intent, roadId);
        intent.putExtra(KEY_IS_SELECT, false);
        intent.putExtra(KEY_IS_GLOBAL, false);
        context.startActivity(intent);
    }

    public static void startGlobalControlListActivity(Context context) {
        Intent intent = new Intent(context, ControlPointsNewActivity.class);
        intent.putExtra(KEY_IS_SELECT, false);
        intent.putExtra(KEY_IS_GLOBAL, true);
        context.startActivity(intent);
    }

    public interface IPickXyzCallback {
        void onPicked(String name, String code, double x, double y, double z, String remarks);
    }

    public interface IPickXyzCallback2 {
        void onPicked(String pointId, String name, String code, double x, double y, double z, String remarks);
    }

    public interface IPickXyzBlhCallback {
        void onPicked(String name, String code, double x, double y, double z, double lat, double lon, double alt, String remarks);
    }

    public interface IPickXyzBlhCallback2 {
        void onPicked(String pointId, String name, String code, double x, double y, double z, double lat, double lon, double alt, String remarks);
    }

    @Override
    protected void onDestroy() {
        try {
            binding.bodyNameRv.removeOnScrollListener(leftScrollListener);
            binding.bodyRv.removeOnScrollListener(rightScrollListener);
            leftScrollListener = null;
            rightScrollListener = null;
        } catch (Exception e) {
            e.printStackTrace();
        }
        super.onDestroy();
    }
}