package com.project.survey.ui.instrument.setupstation

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.paging.PagingDataAdapter
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.RecyclerView
import com.bingce.data.database.PointDb
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord
import com.bingce.utils.DateUtils
import com.bingce.utils.ThreadPoolUtil
import com.bingce.utils.Util
import com.project.survey.databinding.ItemCoordinatePointLibraryBinding
import com.project.survey.databinding.ItemCoordinatePointNameLibraryBinding

class CoordinatePointLibraryAdapter(
    diffCallback: DiffUtil.ItemCallback<PointRecord>,
    val context: Context,
    private val onlyPointName: Boolean
) : PagingDataAdapter<PointRecord, CoordinatePointLibraryAdapter.BaseVH>(diffCallback) {
    private var mSelItemPos = mutableSetOf<Int>()
    fun toggleSelItemPos(pos: Int) {
        if (mSelItemPos.contains(pos)) {
            mSelItemPos.remove(pos)
        } else {
            mSelItemPos.add(pos)
        }
        notifyDataSetChanged()
    }

    fun selAll() {
        mSelItemPos.addAll((0 until itemCount))
        notifyDataSetChanged()
    }

    fun getSelItemPos(): List<Int> {
        return mSelItemPos.toList()
    }

    fun resetConfig() {
        mSelItemPos.clear()
    }

    fun getSelItem(): MutableList<PointRecord>? {
        if (mSelItemPos.size == 0) return null
        val list = mutableListOf<PointRecord>()
        mSelItemPos.forEach { getItem(it)?.let { it1 -> list.add(it1) } }
        return list
    }

    fun resetSelPos() {
        mSelItemPos.clear()
    }

    fun clear() {
        val list = ArrayList<PointRecord>()
        ThreadPoolUtil.execute {
            (0 until itemCount).forEach {
                getItem(it)?.let { it1 -> list.add(it1) }
            }
            mSelItemPos.clear()
            PointDb.getInstance().delete(list)
        }
    }

    fun clear(pointRecords: List<PointRecord>) {
        mSelItemPos.clear()
        PointDb.getInstance().delete(pointRecords)
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseVH {
        return if (onlyPointName) {
            VH2(
                ItemCoordinatePointNameLibraryBinding.inflate(
                    LayoutInflater.from(context),
                    parent,
                    false
                )
            )
        } else {
            VH(
                ItemCoordinatePointLibraryBinding.inflate(
                    LayoutInflater.from(context),
                    parent,
                    false
                )
            )
        }
    }

//    override fun getItemCount(): Int {
//        return arrayList.size
//    }

    override fun onBindViewHolder(holder: BaseVH, position: Int) {
        getItem(position)?.let { holder.update(it) }
    }

    override fun onBindViewHolder(holder: BaseVH, position: Int, payloads: MutableList<Any>) {
        if (payloads.isEmpty()) {
            onBindViewHolder(holder, position)
        } else {
            val bundle = payloads[0]
            getItem(position)?.let { holder.updatePart(it, bundle as Bundle) }
        }
    }

    inner class VH2(val vb: ItemCoordinatePointNameLibraryBinding) : BaseVH(vb.root) {
        override fun update(pointRecord: PointRecord) {
            vb.tvName.text = pointRecord.name
            if (mSelItemPos.contains(absoluteAdapterPosition)) {
                vb.root.setBackgroundColor(Color.rgb(226, 236, 215))
                vb.checkBox.isChecked = true
            } else {
                vb.root.background = null
                vb.checkBox.isChecked = false
            }
//            vb.checkBox.setOnCheckedChangeListener{_,isChecked->
//                if (isChecked){
//                    mSelItemPos.add(absoluteAdapterPosition)
//                }else{
//                    mSelItemPos.remove(absoluteAdapterPosition)
//                }
//                notifyItemChanged(absoluteAdapterPosition)
//            }
        }

        override fun updatePart(pointRecord: PointRecord, bundle: Bundle) {
            for (key in bundle.keySet()) {
                val value = bundle.getString(key) ?: ""
                when (key) {
                    "name" -> vb.tvName.text = value
                    else -> {}
                }
            }
        }
    }

    inner class VH(val vb: ItemCoordinatePointLibraryBinding) : BaseVH(vb.root) {
        override fun update(pointRecord: PointRecord) {
            vb.tvCode.text = pointRecord.code
            vb.tvX.text = Util.formatDouble2StringDotAuto(pointRecord.x)
            vb.tvY.text = Util.formatDouble2StringDotAuto(pointRecord.y)
            vb.tvH.text = Util.formatDouble2StringDotAuto(pointRecord.h)
            vb.tvLatitude.text =
                Util.radianToDmsDoubleString(Math.toRadians(pointRecord.latitude), 6, false)
            vb.tvLongitude.text =
                Util.radianToDmsDoubleString(Math.toRadians(pointRecord.longitude), 6, false)
            vb.tvEarthH.text = Util.formatDouble2String(pointRecord.altitude, 4)
            vb.tvTime.text = DateUtils.toFull(pointRecord.createDate)
            if (mSelItemPos.contains(absoluteAdapterPosition)) {
                vb.root.setBackgroundColor(Color.rgb(226, 236, 215))
            } else {
                vb.root.background = null
            }
        }

        override fun updatePart(pointRecord: PointRecord, bundle: Bundle) {
            for (key in bundle.keySet()) {
                val value = bundle.getString(key) ?: ""
                when (key) {
                    "code" -> vb.tvCode.text = value
                    "x" -> vb.tvX.text = value
                    "y" -> vb.tvY.text = value
                    "h" -> vb.tvH.text = value
                    "time" -> vb.tvTime.text = value
                    else -> {}
                }
            }
        }
    }

    abstract class BaseVH(val view: View) : RecyclerView.ViewHolder(view) {
        abstract fun update(pointRecord: PointRecord)
        abstract fun updatePart(pointRecord: PointRecord, bundle: Bundle)
    }
}