package com.project.survey.ui.instrument.setupstation;

import android.content.Context;
import android.content.Intent;
import android.os.Handler;
import android.text.Editable;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.LiveData;
import androidx.paging.Pager;
import androidx.paging.PagingConfig;
import androidx.paging.PagingData;
import androidx.paging.PagingDataTransforms;
import androidx.paging.PagingLiveData;
import androidx.paging.PagingSource;
import androidx.recyclerview.widget.RecyclerView;

import com.afollestad.materialdialogs.DialogAction;
import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.data.cache.CachedCurrentJob;
import com.bingce.data.cache.CachedProject;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.JobDb;
import com.bingce.data.database.PointDb;
import com.bingce.data.surveyor.designdata.job.JobConstants;
import com.bingce.data.surveyor.designdata.job.JobRecord;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.error.BingCeErrorCode;
import com.bingce.ui.BcDialogExt;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.SimpleTextWatcher;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.viewmodel.AbstractCoordinatePointsLibraryViewModelKt;
import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.databinding.ActivityCoordinatePointsLibraryBinding;
import com.project.survey.dialog.BubblePopWindow;
import com.project.survey.ui.base.BaseSurveyNewActivity;

import java.util.Arrays;
import java.util.List;

import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;

import kotlinx.coroutines.Dispatchers;
import kotlinx.coroutines.ExecutorsKt;

public class CoordinatePointsLibraryActivity extends BaseSurveyNewActivity {

    private ActivityCoordinatePointsLibraryBinding binding;
    private RecyclerView.OnScrollListener leftScrollListener = new RecyclerView.OnScrollListener() {
        @Override
        public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
            if (recyclerView.getScrollState() != RecyclerView.SCROLL_STATE_IDLE) {
                binding.bodyRv.scrollBy(dx, dy);
            }
        }

        @Override
        public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
            super.onScrollStateChanged(recyclerView, newState);
            binding.bodyHsv.setIntercept(newState != RecyclerView.SCROLL_STATE_IDLE);
        }
    };
    private RecyclerView.OnScrollListener rightScrollListener = new RecyclerView.OnScrollListener() {
        @Override
        public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
            if (!isFinishing() && recyclerView.getScrollState() != RecyclerView.SCROLL_STATE_IDLE) {
                binding.bodyNameRv.scrollBy(dx, dy);
            }
        }

        @Override
        public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
            super.onScrollStateChanged(recyclerView, newState);
            binding.bodyNameRl.setIntercept(newState != RecyclerView.SCROLL_STATE_IDLE);
        }
    };

    private final CoordinatePointLibraryAdapter coordinatePointLibraryAdapter = new CoordinatePointLibraryAdapter(new PointRecordDiffCallback2(), this, false);
    private final CoordinatePointLibraryAdapter coordinatePointNameLibraryAdapter = new CoordinatePointLibraryAdapter(new PointRecordDiffCallback2(), this, true);

    private String jobId;
    private AbstractCoordinatePointsLibraryViewModelKt.FilterParameter filterParameter = new AbstractCoordinatePointsLibraryViewModelKt.FilterParameter();
    private Pager<Integer, PointRecord> pager = new Pager<>(
            new PagingConfig(100),
            this::dataSourse);

    private PagingSource<Integer, PointRecord> dataSourse() {
        return PointDb.getInstance().rawQueryPagingSource(CoordinatePointsLibraryActivitySQLiteUtils.searchSQLit(
                jobId,
                filterParameter, binding.etEnterNameCode.getText().toString().trim()
        ));
    }

    private List<PointRecord> getAllPoint() {
        return PointDb.getInstance().rawQueryListData(CoordinatePointsLibraryActivitySQLiteUtils.searchSQLit(
                jobId,
                filterParameter, binding.etEnterNameCode.getText().toString().trim()
        ));
    }

    private LiveData<PagingData<PointRecord>> pagingDataLiveData;
    private final FilterHandlerNew filterHandler = new FilterHandlerNew(new FilterHandlerNew.FilterCallBack() {
        @Override
        public void fireByKey(String key) {
            ThreadPoolUtil.execute(() -> {
//                String projectId = CachedProject.currentProjectId();
//                jobId = CachedCurrentJob.currentJobId(projectId);
                ThreadPoolUtil.executeInMain(() -> {
                    pagingDataLiveData.removeObservers(CoordinatePointsLibraryActivity.this);
                    pagingDataLiveData = PagingLiveData.cachedIn(PagingLiveData.getLiveData(pager), getLifecycle());
                    pagingDataLiveData.observe(CoordinatePointsLibraryActivity.this, downloadRecordPagingData -> {
                        refreshAdapter(downloadRecordPagingData);
                    });

                });
            });
        }
    });

    private void refreshAdapter(PagingData<PointRecord> downloadRecordPagingData) {
        coordinatePointLibraryAdapter.submitData(getLifecycle(), downloadRecordPagingData);
        coordinatePointNameLibraryAdapter.submitData(getLifecycle(), downloadRecordPagingData);
        coordinatePointLibraryAdapter.resetConfig();
        coordinatePointNameLibraryAdapter.resetConfig();
        coordinatePointLibraryAdapter.notifyDataSetChanged();
        coordinatePointNameLibraryAdapter.notifyDataSetChanged();
        new Handler().postDelayed(() -> binding.tvPointNumber.setText(coordinatePointLibraryAdapter.getItemCount() + ""), 500);
    }

    private void resetSelPos() {
        coordinatePointLibraryAdapter.resetSelPos();
        coordinatePointNameLibraryAdapter.resetSelPos();
    }

    @Override
    public View getContentView() {
        binding = ActivityCoordinatePointsLibraryBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.point_survey_point_library));
        jobId = IntentUtil.stringExtra(this, KEY_JOB_ID);

        if (App.instance.isThemeDark()) {
            binding.topLayout.setBackgroundColor(getColor(R.color.theme_dark_black));
        }

        binding.bodyRv.setAdapter(coordinatePointLibraryAdapter);
        binding.bodyNameRv.setAdapter(coordinatePointNameLibraryAdapter);
        binding.bodyHsv.setScrollView(binding.headerHsv);
        binding.headerHsv.setScrollView(binding.bodyHsv);

        binding.tvLatitude.setText(getString(R.string.latitude) + "(dd.mmss)");
        binding.tvLongitude.setText(getString(R.string.longitude) + "(dd.mmss)");

        binding.bodyRv.addOnScrollListener(rightScrollListener);

        binding.bodyNameRv.addOnScrollListener(leftScrollListener);
        binding.bodyRv.addOnItemTouchListener(new SimpleRecyclerViewItemClickListener(binding.bodyRv, (view, position) -> {
            coordinatePointLibraryAdapter.toggleSelItemPos(position);
            coordinatePointNameLibraryAdapter.toggleSelItemPos(position);
        }));
        binding.bodyNameRv.addOnItemTouchListener(new SimpleRecyclerViewItemClickListener(binding.bodyNameRv, (view, position) -> {
            coordinatePointLibraryAdapter.toggleSelItemPos(position);
            coordinatePointNameLibraryAdapter.toggleSelItemPos(position);
        }));
        binding.tvSelAll.setOnClickListener(v -> {
            if (coordinatePointLibraryAdapter.getItemCount() == coordinatePointLibraryAdapter.getSelItemPos().size()) {
                coordinatePointLibraryAdapter.resetSelPos();
                coordinatePointNameLibraryAdapter.resetSelPos();
                coordinatePointLibraryAdapter.notifyDataSetChanged();
                coordinatePointNameLibraryAdapter.notifyDataSetChanged();
            } else {
                coordinatePointLibraryAdapter.resetSelPos();
                coordinatePointNameLibraryAdapter.resetSelPos();
                coordinatePointLibraryAdapter.selAll();
                coordinatePointNameLibraryAdapter.selAll();
                coordinatePointLibraryAdapter.notifyDataSetChanged();
                coordinatePointNameLibraryAdapter.notifyDataSetChanged();
            }
        });

        ThreadPoolUtil.execute(() -> {
//            String projectId = CachedProject.currentProjectId();
//            jobId = CachedCurrentJob.currentJobId(projectId);
            ThreadPoolUtil.executeInMain(() -> {
                pagingDataLiveData = PagingLiveData.cachedIn(PagingLiveData.getLiveData(pager), getLifecycle());
                pagingDataLiveData
                        .observe(this, downloadRecordPagingData -> {
                            PagingDataTransforms.map(downloadRecordPagingData, ExecutorsKt.asExecutor(Dispatchers.getIO()), item -> {
                                return item;
                            });
                            refreshAdapter(downloadRecordPagingData);
                        });

            });
        });

        binding.etEnterNameCode.addTextChangedListener(new SimpleTextWatcher() {
            @Override
            public void afterTextChanged(Editable s) {
                filterHandler.onFilterStringChanged(s.toString());
            }
        });

        binding.pointsLibraryBtnFilter.setOnClickListener(v -> {
            PointFilterActivity.startFilter(CoordinatePointsLibraryActivity.this, false,
                    filterParameter.getNeedSurvey(), filterParameter.getNeedInput(), filterParameter.getNeedControl(),
                    filterParameter.getDateStart(), filterParameter.getDateEnd(),
                    filterParameter.getNameKey(), filterParameter.getCodeKey(), filterParameter.getRemarkKey(),
                    (needSurvey, needInput, needControl, dateStart, dateEnd, nameKey, codeKey, remarksKey) -> {
                        filterParameter.setNeedSurvey(needSurvey);
                        filterParameter.setNeedInput(needInput);
                        filterParameter.setNeedControl(needControl);
                        filterParameter.setDateStart(dateStart);
                        filterParameter.setDateEnd(dateEnd);
                        filterParameter.setNameKey(nameKey);
                        filterParameter.setCodeKey(codeKey);
                        filterParameter.setRemarkKey(remarksKey);
                        ThreadPoolUtil.execute(() -> {
//                            String projectId = CachedProject.currentProjectId();
//                            jobId = CachedCurrentJob.currentJobId(projectId);
                            ThreadPoolUtil.executeInMain(() -> {
                                pagingDataLiveData.removeObservers(this);
                                pagingDataLiveData =
                                        PagingLiveData.cachedIn(PagingLiveData.getLiveData(pager), getLifecycle());
                                pagingDataLiveData.observe(this, downloadRecordPagingData -> {
                                    refreshAdapter(downloadRecordPagingData);
                                });
                            });
                        });
                    });
        });

        binding.pointsLibraryBtnEdit.setOnClickListener(v -> {
            List<PointRecord> selItem = coordinatePointNameLibraryAdapter.getSelItem();
            if (selItem == null || selItem.size() != 1) {
                ToastUtils.showShort(R.string.please_select_one_data_item);
                return;
            }
            EditCoordinatePointActivity.start(launcher, CoordinatePointsLibraryActivity.this, selItem.get(0).id);
        });

        binding.pointsLibraryBtnDelete.setVisibility(View.GONE);
        binding.pointsLibraryBtnDelete.setOnClickListener(v -> {
            List<PointRecord> selItem = coordinatePointLibraryAdapter.getSelItem();
            if (selItem == null || selItem.size() == 0) {
                ToastUtils.showShort(R.string.please_select_one_data_item);
                return;
            }
            new MaterialDialog.Builder(this)
                    .title(R.string.warning)
                    .content("确定要删除选择的点吗？")
                    .negativeText(R.string.cancel)
                    .positiveText(R.string.confirm)
                    .onPositive((materialDialog, dialogAction) -> {
                        ThreadPoolUtil.execute(() -> {
                            PointDb.getInstance().delete(selItem);
                            resetSelPos();
                        });
                    }).show();
//            BcDialogExt.create().showMessage(getString(R.string.warning),"确定要删除选择的点吗？",
//                    getString(R.string.confirm), getString(R.string.cancel), (baseDialog, view) -> {
//                ThreadPoolUtil.execute(() -> {
//                    PointDb.getInstance().delete(selItem);
//                    resetSelPos();
//                });
//                return false;
//            }, (baseDialog, view) -> false);

        });

        // 导出
        binding.pointsLibraryBtnExport.setOnClickListener(v ->
                ResultsTheExportActivity.start(
                        this,
                        jobId,
                        false,
                        filterParameter.getNameKey(),
                        filterParameter.getCodeKey(),
                        filterParameter.getRemarkKey(),
                        filterParameter.getDateStart(),
                        filterParameter.getDateEnd())
        );

        binding.pointsLibraryBtnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                List<PointRecord> selItem = coordinatePointLibraryAdapter.getSelItem();
                if (selItem == null || selItem.size() != 1) {
                    ToastUtils.showShort(R.string.please_select_one_data_item);
                    return;
                }
                PointRecord pointRecord = selItem.get(0);

                // 数据是使用Intent返回
                Intent intent = new Intent();
                // 把返回数据存入Intent
                intent.putExtra(RESULT_ID, pointRecord.id);
                intent.putExtra(RESULT_NAME, pointRecord.name);
                intent.putExtra(RESULT_CODE, pointRecord.code);
                intent.putExtra(RESULT_X, pointRecord.x);
                intent.putExtra(RESULT_Y, pointRecord.y);
                intent.putExtra(RESULT_Z, pointRecord.h);
                intent.putExtra(RESULT_B, pointRecord.latitude);
                intent.putExtra(RESULT_L, pointRecord.longitude);
                intent.putExtra(RESULT_H, pointRecord.altitude);
                // 设置返回数据
                setResult(RESULT_OK, intent);
                // 关闭Activity
                finish();
            }
        });

        boolean isSelect = getIntent().getBooleanExtra(KEY_IS_PICK, false);

        if (isSelect) {
            binding.pointsLibraryBtnConfirm.setVisibility(View.VISIBLE);
            binding.pointsLibraryBtnEdit.setVisibility(View.GONE);
//            binding.pointsLibraryBtnDelete.setVisibility(View.GONE);
            binding.pointsLibraryBtnExport.setVisibility(View.GONE);
        } else {
            binding.pointsLibraryBtnConfirm.setVisibility(View.GONE);
            binding.pointsLibraryBtnEdit.setVisibility(View.VISIBLE);
//            binding.pointsLibraryBtnDelete.setVisibility(View.VISIBLE);
            binding.pointsLibraryBtnExport.setVisibility(View.VISIBLE);
        }
    }

    @Override
    protected void initData() {

    }

    @Override
    public BubblePopWindow newBubblePopWindow(Context context) {
        return new BubblePopWindow(this, Arrays.asList(new BubblePopWindow.CustomMenuItem(R.drawable.icon_function_delete, getString(R.string.batch_delete), () -> {
            List<PointRecord> selItemPos = coordinatePointLibraryAdapter.getSelItem();
            if (selItemPos != null && selItemPos.size() > 0) {
                BcDialogExt.create().showMessageWithTick(getString(R.string.warning), getString(R.string.want_to_batch_delete_points), getString(R.string.confirm), getString(R.string.cancel), (baseDialog, view) -> {
                    ThreadPoolUtil.execute(() -> {
                        PointDb.getInstance().delete(selItemPos);
                        resetSelPos();
                    });
                    return false;
                }, (baseDialog, view) -> false);
            } else {
                ToastUtils.showShort(getString(R.string.select_batch_delete_data));
            }
        })));
    }

    @Override
    public boolean isHideHelpDoc() {
        return true;
    }

    public final static String KEY_JOB_ID = "JOBID";
    final static String FORCE_SURVEY_POINT_LIBRARY = "should survey point library";

    public static void start4Result(ActivityResultLauncher<Intent> launcher, Context context, String jobId) {
        Intent intent = new Intent(context, CoordinatePointsLibraryActivity.class);
        intent.putExtra(KEY_JOB_ID, jobId);
        intent.putExtra(KEY_IS_PICK, false);
        launcher.launch(intent);
    }

    public static void start(Context context, String jobId) {
        Intent intent = new Intent(context, CoordinatePointsLibraryActivity.class);
        intent.putExtra(KEY_JOB_ID, jobId);
        intent.putExtra(KEY_IS_PICK, false);
        context.startActivity(intent);
    }

    public final ActivityResultLauncher<Intent> launcher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result != null) {
            if (result.getResultCode() == LauncherEvent.launcher_update_point_library) {
//                holderInActivity.getViewModel().refreshCurrentPage();
            }
        }
    });

    public static void start(FragmentActivity fragmentActivity) {
        ThreadPoolUtil.execute(() -> {
            String projectId = CachedProject.currentProjectId();
            String jobId = CachedCurrentJob.currentJobId(projectId);
            JobRecord jobRecord = JobDb.getInstance()
                    .rawQueryData(DBQueryConstant.findById(JobConstants.DB_NAME, jobId));
            if (jobRecord == null) {
                BingCeErrorCode.toast(BingCeErrorCode.NO_CURRENT_JOB);
                return;
            }
            Intent intent = new Intent(fragmentActivity, CoordinatePointsLibraryActivity.class);
            intent.putExtra(KEY_JOB_ID, jobId);
            fragmentActivity.startActivity(intent);
        });
    }

    private static final String RESULT_ID = "id";
    private static final String RESULT_NAME = "name";
    private static final String RESULT_CODE = "code";
    private static final String RESULT_X = "x";
    private static final String RESULT_Y = "y";
    private static final String RESULT_Z = "z";
    private static final String RESULT_B = "b";
    private static final String RESULT_L = "l";
    private static final String RESULT_H = "h";

    private static final String KEY_IS_PICK = "is pick";

    public static void pickPoint(@NonNull FragmentActivity fragmentActivity, @NonNull ICallback callback) {
        ThreadPoolUtil.execute(() -> {
            String projectId = CachedProject.currentProjectId();
            String jobId = CachedCurrentJob.currentJobId(projectId);
            JobRecord jobRecord = JobDb.getInstance()
                    .rawQueryData(DBQueryConstant.findById(JobConstants.DB_NAME, jobId));
            if (jobRecord == null) {
                BingCeErrorCode.toast(BingCeErrorCode.NO_CURRENT_JOB);
                return;
            }
            Intent intent = new Intent(fragmentActivity, CoordinatePointsLibraryActivity.class);
            intent.putExtra(KEY_IS_PICK, true);
            intent.putExtra(KEY_JOB_ID, jobId);
            fragmentActivity.runOnUiThread(() -> {
                ActivityUtils.startActivityForResults(fragmentActivity, intent, result -> {
                    if (result == null) {
                        return;
                    }
                    if (result.getResultCode() != RESULT_OK) {
                        return;
                    }
                    Intent data = result.getData();
                    if (data == null) {
                        return;
                    }
                    callback.onPicked(
                            data.getStringExtra(RESULT_NAME),
                            data.getStringExtra(RESULT_CODE),
                            data.getDoubleExtra(RESULT_X, 0),
                            data.getDoubleExtra(RESULT_Y, 0),
                            data.getDoubleExtra(RESULT_Z, 0),
                            data.getDoubleExtra(RESULT_B, 0),
                            data.getDoubleExtra(RESULT_L, 0),
                            data.getDoubleExtra(RESULT_H, 0));
                });
            });
        });
    }

    public interface ICallback {
        void onPicked(String name, String code,
                      double x, double y, double z,
                      double b, double l, double h);
    }

    public static void pickPoint(@NonNull FragmentActivity fragmentActivity, @NonNull ICallback2 callback) {
        ThreadPoolUtil.execute(() -> {
            String projectId = CachedProject.currentProjectId();
            String jobId = CachedCurrentJob.currentJobId(projectId);
            JobRecord jobRecord = JobDb.getInstance()
                    .rawQueryData(DBQueryConstant.findById(JobConstants.DB_NAME, jobId));
            if (jobRecord == null) {
                BingCeErrorCode.toast(BingCeErrorCode.NO_CURRENT_JOB);
                return;
            }
            Intent intent = new Intent(fragmentActivity, CoordinatePointsLibraryActivity.class);
            intent.putExtra(KEY_IS_PICK, true);
            intent.putExtra(KEY_JOB_ID, jobId);
            fragmentActivity.runOnUiThread(() -> {
                ActivityUtils.startActivityForResults(fragmentActivity, intent, result -> {
                    if (result == null) {
                        return;
                    }
                    if (result.getResultCode() != RESULT_OK) {
                        return;
                    }
                    Intent data = result.getData();
                    if (data == null) {
                        return;
                    }
                    callback.onPicked(
                            data.getStringExtra(RESULT_ID),
                            data.getStringExtra(RESULT_NAME),
                            data.getStringExtra(RESULT_CODE),
                            data.getDoubleExtra(RESULT_X, 0),
                            data.getDoubleExtra(RESULT_Y, 0),
                            data.getDoubleExtra(RESULT_Z, 0),
                            data.getDoubleExtra(RESULT_B, 0),
                            data.getDoubleExtra(RESULT_L, 0),
                            data.getDoubleExtra(RESULT_H, 0));
                });
            });
        });
    }


    public interface ICallback2 {
        void onPicked(String pointId, String name, String code,
                      double x, double y, double z,
                      double b, double l, double h);
    }

    @Override
    protected void onDestroy() {
        try {
            binding.bodyNameRv.removeOnScrollListener(leftScrollListener);
            binding.bodyRv.removeOnScrollListener(rightScrollListener);
            leftScrollListener = null;
            rightScrollListener = null;
        } catch (Exception e) {
            e.printStackTrace();
        }
        super.onDestroy();
    }
}