package com.project.survey.ui.instrument.setupstation;

import static com.bingce.data.database.DBQueryConstant.NOT_DELETED;
import static com.bingce.data.database.DBQueryConstant.WHERE;
import static com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants.surveyorPoint;

import android.text.TextUtils;

import androidx.annotation.NonNull;
import androidx.sqlite.db.SimpleSQLiteQuery;
import androidx.sqlite.db.SupportSQLiteQuery;

import com.bingce.data.BingCeDbConstant;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.utils.RoomFilterBuilder;
import com.bingce.utils.StringUtil;
import com.bingce.viewmodel.AbstractCoordinatePointsLibraryViewModelKt.FilterParameter;


public class CoordinatePointsLibraryActivitySQLiteUtils {
    @NonNull
    static SimpleSQLiteQuery getSimpleSQLiteQuery(FilterParameter parameter, StringBuilder stringBuilder, String jobId, int pageIndex, int pageSize, String nameOrCodeStr) {
        if (parameter == null) {
            return new SimpleSQLiteQuery(stringBuilder.toString());
        }
        stringBuilder.append(WHERE).append(NOT_DELETED);
        boolean filtered = RoomFilterBuilder.appendEqualsFilter(true, stringBuilder, BingCeDbConstant.DB_KEY_JOB_ID, jobId);
        filtered = surveyorPoint(filtered, stringBuilder);

        if (!StringUtil.isEmpty(parameter.getNameKey())) {
            filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, PointConstants.COL_NAME, parameter.getNameKey());
        }
        if (!StringUtil.isEmpty(parameter.getCodeKey())) {
            filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, PointConstants.COL_CODE, parameter.getCodeKey());
        }
        if (!StringUtil.isEmpty(parameter.getRemarkKey())) {
            filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, BingCeDbConstant.DB_KEY_REMARKS, parameter.getRemarkKey());
        }
        if (!TextUtils.isEmpty(nameOrCodeStr)){
            stringBuilder.append(" AND ");
            stringBuilder.append(" (");
            stringBuilder.append(RoomFilterBuilder.filterLike(PointConstants.COL_NAME, nameOrCodeStr));
            RoomFilterBuilder.appendOrLikesFilter(filtered,stringBuilder,PointConstants.COL_CODE,nameOrCodeStr);
            stringBuilder.append(") ");
        }

        if (parameter.getDateStart() != null && parameter.getDateEnd() != null) {
            filtered = RoomFilterBuilder.appendBetweenDate(filtered, stringBuilder, BingCeDbConstant.DB_KEY_CREATED_DATE,
                    parameter.getDateStart(), parameter.getDateEnd());
        } else if (parameter.getDateStart() != null) {
            filtered = RoomFilterBuilder.appendDateFilter(filtered, stringBuilder, parameter.getDateStart(), BingCeDbConstant.DB_KEY_CREATED_DATE, true);
        } else if (parameter.getDateEnd() != null) {
            filtered = RoomFilterBuilder.appendDateFilter(filtered, stringBuilder, parameter.getDateEnd(), BingCeDbConstant.DB_KEY_CREATED_DATE, false);
        }

//        if (pageIndex < 0 || pageSize < 0) {
            RoomFilterBuilder.appendDateOrder(stringBuilder, BingCeDbConstant.DB_KEY_CREATED_DATE, false);
//        } else {
//            if (pageIndex == 0) {
//                RoomFilterBuilder.appendDateOrderAndNumLimit(stringBuilder, BingCeDbConstant.DB_KEY_CREATED_DATE, pageSize, false);
//            } else {
//                RoomFilterBuilder.appendDateOrderAndNumLimitOffset(stringBuilder, BingCeDbConstant.DB_KEY_CREATED_DATE,
//                        pageSize, pageSize * pageIndex, false);
//            }
//        }

        return new SimpleSQLiteQuery(stringBuilder.toString());
    }


    static SupportSQLiteQuery countSQLit(String jobId, FilterParameter parameter) {
        StringBuilder stringBuilder = new StringBuilder(DBQueryConstant.COUNT_FROM)
                .append(PointConstants.DB_NAME);
        return getSimpleSQLiteQuery(parameter, stringBuilder, jobId, -1, -1,null);
    }

    static SupportSQLiteQuery searchSQLit(String jobId, FilterParameter parameter, int pageIndex, int pageSize) {
        StringBuilder stringBuilder = new StringBuilder(DBQueryConstant.SELECT_FROM)
                .append(PointConstants.DB_NAME);
        return getSimpleSQLiteQuery(parameter, stringBuilder, jobId, pageIndex, pageSize,null);
    }
    public static SupportSQLiteQuery searchSQLit(String jobId, FilterParameter parameter, String nameOrCode){
        StringBuilder stringBuilder = new StringBuilder(DBQueryConstant.SELECT_FROM)
                .append(PointConstants.DB_NAME);
        return getSimpleSQLiteQuery(parameter, stringBuilder, jobId, -1, -1,nameOrCode);
    }
}
