package com.project.survey.ui.instrument.setupstation;


import static com.project.survey.ui.instrument.setupstation.LauncherEvent.launcher_update_point_library;

import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;

import com.bingce.data.DeprecatedDeviceData;
import com.bingce.data.DeviceInfoData;
import com.bingce.data.DeviceInfoDataUtils;
import com.bingce.data.RTKStatusData;
import com.bingce.data.TSStatusData;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.JobDb;
import com.bingce.data.database.PointDb;
import com.bingce.data.surveyor.designdata.job.JobConstants;
import com.bingce.data.surveyor.designdata.job.JobRecord;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.utils.DateUtils;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.project.survey.R;
import com.project.survey.databinding.ActivityEditCoordinatePointBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;

import blankj.utilcode.util.ToastUtils;

public class EditCoordinatePointActivity extends BaseSurveyNewActivity {

    private ActivityEditCoordinatePointBinding binding;
    private PointRecord pointRecord;
    private double recordHigh, recordAltitude, recordHr;

    @Override
    public View getContentView() {
        binding = ActivityEditCoordinatePointBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {

        binding.editCoordinatePointBtnConfirm.setOnClickListener(v -> {

            if (pointRecord != null) {
                if (TextUtils.isEmpty(binding.etPointName.getText().toString())) {
                    ToastUtils.showShort("请输入点名");
                    return;
                }
                if (TextUtils.isEmpty(binding.etAntennaHeight.getText().toString())) {
                    ToastUtils.showShort(getString(R.string.enter_pole_height));
                    return;
                }
                pointRecord.name = binding.etPointName.getText().toString();
                pointRecord.code = binding.etCode.getText().toString();
                pointRecord.remarks = binding.etRemark.getText().toString();

                String inputHrString = binding.etAntennaHeight.getText().toString();
                double inputHrDouble = Double.parseDouble(inputHrString);

                if (pointRecord.deviceInfoData.tsStatusData != null) {
                    TSStatusData pointRecordTSInfo = pointRecord.deviceInfoData.tsStatusData;
                    TSStatusData tsStatusData = new TSStatusData(pointRecordTSInfo.deviceType, pointRecordTSInfo.x, pointRecordTSInfo.y, pointRecordTSInfo.z, inputHrDouble, pointRecordTSInfo.setupStationMethod, pointRecordTSInfo.stationName, pointRecordTSInfo.stationX, pointRecordTSInfo.stationY, pointRecordTSInfo.stationZ, pointRecordTSInfo.hi, pointRecordTSInfo.ha, pointRecordTSInfo.va, pointRecordTSInfo.sd, pointRecordTSInfo.angleDifference, pointRecordTSInfo.powerLevel, pointRecordTSInfo.backSightPoints, pointRecordTSInfo.checkPoints, pointRecordTSInfo.deviceModel, pointRecordTSInfo.deviceSn, pointRecordTSInfo.prismTypeCode, pointRecordTSInfo.prismConstant, pointRecordTSInfo.prismTypeName);
                    DeviceInfoData deviceInfoData = new DeviceInfoData(tsStatusData);
                    pointRecord.deviceInfoData = deviceInfoData;
                }
                if (pointRecord.deviceInfoData.rtkStatusData != null) {
                    RTKStatusData pointRecordRTKInfo = pointRecord.deviceInfoData.rtkStatusData;
                    RTKStatusData rtkStatusData = new RTKStatusData(pointRecordRTKInfo.deviceType, pointRecordRTKInfo.workMode, pointRecordRTKInfo.posType, pointRecordRTKInfo.solutionState, pointRecordRTKInfo.latitude, pointRecordRTKInfo.longitude, pointRecordRTKInfo.altitude, pointRecordRTKInfo.x, pointRecordRTKInfo.y, pointRecordRTKInfo.z, pointRecordRTKInfo.isTiltOpen, pointRecordRTKInfo.isTiltEnable, pointRecordRTKInfo.diffAge, pointRecordRTKInfo.pdop, pointRecordRTKInfo.hrms, pointRecordRTKInfo.vrms, inputHrDouble, pointRecordRTKInfo.powerLevel, pointRecordRTKInfo.satTrackNum, pointRecordRTKInfo.satSolutionNum, pointRecordRTKInfo.deviceModel, pointRecordRTKInfo.deviceSn);
                    DeviceInfoData deviceInfoData = new DeviceInfoData(rtkStatusData);
                    pointRecord.deviceInfoData = deviceInfoData;
                }
                if (pointRecord.deviceInfoData.deprecatedDeviceData != null) {
                    DeprecatedDeviceData pointRecordDeprecatedInfo = pointRecord.deviceInfoData.deprecatedDeviceData;
                    DeprecatedDeviceData deprecatedDeviceData = new DeprecatedDeviceData(pointRecordDeprecatedInfo.satellite, binding.etAntennaHeight.getText().toString(), pointRecordDeprecatedInfo.electricity, pointRecordDeprecatedInfo.posType, pointRecordDeprecatedInfo.hrms, pointRecordDeprecatedInfo.vrms, pointRecordDeprecatedInfo.posType, pointRecordDeprecatedInfo.diffAge, pointRecordDeprecatedInfo.stationType, pointRecordDeprecatedInfo.solutionState);
                    DeviceInfoData deviceInfoData = new DeviceInfoData(deprecatedDeviceData);
                    pointRecord.deviceInfoData = deviceInfoData;
                }

                pointRecord.h = recordHigh + recordHr - inputHrDouble;
                pointRecord.altitude = recordAltitude + recordHr - inputHrDouble;

                ThreadPoolUtil.execute(() -> {
                    PointDb.getInstance().save(pointRecord);
                    runOnUiThread(() -> {
                        setResult(launcher_update_point_library);
                        finish();
                    });
                });
            }
        });
    }

    @Override
    protected void initData() {
        String recordId = IntentUtil.stringExtra(getIntent(), KEY_RECORD);
        if (StringUtil.isEmpty(recordId)) {
            finish();
            ToastUtils.showShort("无记录");
            return;
        }
        ThreadPoolUtil.execute(() -> {
            pointRecord = PointDb.getInstance()
                    .rawQueryData(DBQueryConstant.findById(PointConstants.DB_NAME, recordId));
            JobRecord jobRecord = JobDb.getInstance()
                    .rawQueryData(DBQueryConstant.findById(JobConstants.DB_NAME, pointRecord.jobId));
            String jobName = jobRecord == null ? getString(R.string.empty) : jobRecord.name;

            recordHigh = pointRecord.h;
            recordAltitude = pointRecord.altitude;
            recordHr = Double.parseDouble(DeviceInfoDataUtils.hrString(pointRecord.deviceInfoData));

            setRecord(pointRecord, jobName);
        });
    }

    private void setRecord(PointRecord results, String jobName) {
        setTitle(jobName);
        binding.etPointName.setText(results.name);
        binding.etCode.setText(results.code);
        binding.etAntennaHeight.setText(DeviceInfoDataUtils.hrString(results.deviceInfoData));
        binding.etRemark.setText(results.remarks);
        binding.tvX.setText(Util.formatDouble2StringDotAuto(results.x));
        binding.tvY.setText(Util.formatDouble2StringDotAuto(results.y));
        binding.tvZ.setText(Util.formatDouble2StringDotAuto(results.h));
        binding.tvLatitude.setText(Util.radianToDmsDoubleString(Math.toRadians(results.latitude), 6, false));
        binding.tvLongitude.setText(Util.radianToDmsDoubleString(Math.toRadians(results.longitude), 6, false));
        binding.tvAltitude.setText(Util.formatDouble2StringDotAuto(results.altitude));
        binding.tvHrms.setText(DeviceInfoDataUtils.hrmsString(results.deviceInfoData));
        binding.tvVrms.setText(DeviceInfoDataUtils.vrmsString(results.deviceInfoData));
        binding.tvPdop.setText(DeviceInfoDataUtils.pdopString(results.deviceInfoData));
        binding.tvDiffAge.setText(DeviceInfoDataUtils.diffAgeString(this, results.deviceInfoData));
        binding.tvSolutionStatus.setText(DeviceInfoDataUtils.solutionStateString(results.deviceInfoData));
        binding.tvSolutionSatelliteNumber.setText(DeviceInfoDataUtils.satelliteString(this, results.deviceInfoData));
        binding.tvRecordTime.setText(DateUtils.toFull(results.createDate));
    }

    private static final String KEY_RECORD = "record";

    public static void start(ActivityResultLauncher<Intent> launcher, Context context, String recordId) {
        Intent intent = new Intent(context, EditCoordinatePointActivity.class);
        intent.putExtra(KEY_RECORD, recordId);
        launcher.launch(intent);
    }
}
