package com.project.survey.ui.instrument.setupstation;

import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import androidx.annotation.NonNull;

import com.bingce.utils.ThreadPoolUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

public final class FilterHandlerNew extends Handler {
    private static final int MSG_WORK = 0;//过滤关键字发生变化
    private static final List<String> filterKey = new ArrayList<>();
    private final AtomicBoolean isWorking = new AtomicBoolean(false);
    private FilterCallBack iData;
    public interface FilterCallBack{
        void fireByKey(String key);
    }

    public FilterHandlerNew(FilterCallBack callBack) {
        super(Looper.getMainLooper());
        this.iData = callBack;
    }
    @Override
    public void handleMessage(@NonNull Message msg) {
        if (!isWorking.compareAndSet(false, true)) {
            return;
        }
        String targetKey = null;
        synchronized (this) {
            if (!filterKey.isEmpty()) {
                targetKey = filterKey.get(filterKey.size() - 1);
                filterKey.clear();
            }
        }
        //检测是否有key
        if (targetKey == null) {
            isWorking.set(false);
            return;
        }
        //执行数据更新
        String finalTargetKey = targetKey;
        ThreadPoolUtil.executeInQueue(() -> {
            iData.fireByKey(finalTargetKey);
            //执行完毕后，不管后续是否还有其他消息，都要再检查一下
            isWorking.set(false);
            startLoop();
        });
    }

    public void onFilterStringChanged(String string) {
        removeMessages(MSG_WORK);
        synchronized (this) {
            filterKey.add(string);
            startLoop();
        }
    }

    /**
     * 启动消息循环
     */
    private void startLoop() {
        Message msg = Message.obtain();
        msg.what = MSG_WORK;
        sendMessageDelayed(msg,400);
    }
}
