package com.project.survey.ui.instrument.setupstation;

import android.content.Intent;
import android.view.View;

import androidx.appcompat.app.AppCompatActivity;

import com.bingce.dialog.TimePicker;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.IntentUtil;
import com.project.survey.R;
import com.project.survey.databinding.ActivityPointFilterBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;

import java.util.Calendar;
import java.util.Date;


public class PointFilterActivity extends BaseSurveyNewActivity {
    private ActivityPointFilterBinding binding;
    private TimePicker timePicker = null;
    private long startTime = -1;
    private long endTime = -1;

    @Override
    public View getContentView() {
        binding = ActivityPointFilterBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    private void onTodayClicked() {
        setBtnDayBackground(0, true);
        setBtnDayBackground(1, false);
        setBtnDayBackground(2, false);
        setCalendarBackground(0, false);
        setCalendarBackground(1, false);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 1);
        startTime = calendar.getTime().getTime();
        endTime = startTime + 1000 * 60 * 60 * 24 - 2000;//减2s到23:59:59
        binding.tvStartDate.setText(TimePicker.getTime(new Date(startTime)));
        binding.tvEndDate.setText(TimePicker.getTime(new Date(endTime)));
    }

    private void on7DayClicked() {
        setBtnDayBackground(0, false);
        setBtnDayBackground(1, true);
        setBtnDayBackground(2, false);
        setCalendarBackground(0, false);
        setCalendarBackground(1, false);
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        endTime = calendar.getTime().getTime();
        startTime = endTime - 1000 * 60 * 60 * 24 * 7 + 2000;//加2s，到0：0：1
        binding.tvStartDate.setText(TimePicker.getTime(new Date(startTime)));
        binding.tvEndDate.setText(TimePicker.getTime(new Date(endTime)));
    }

    private void onAllDayClicked() {
        setBtnDayBackground(0, false);
        setBtnDayBackground(1, false);
        setBtnDayBackground(2, true);
        setCalendarBackground(0, false);
        setCalendarBackground(1, false);

        startTime = -1;
        endTime = -1;
        binding.tvStartDate.setText("");
        binding.tvEndDate.setText("");
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.filter));
        timePicker = new TimePicker(this, R.color.theme_green);
        boolean forceSurvey = IntentUtil.boolExtra(this, KEY_FORCE_SURVEY);
        if (forceSurvey) {
            binding.typeLabel.setVisibility(View.GONE);
            binding.typeCheckBoxes.setVisibility(View.GONE);
        }
        setBtnDayBackground(0, true);
        binding.tvToday.setOnClickListener(v -> onTodayClicked());
        binding.tvNearly7Days.setOnClickListener(v -> on7DayClicked());
        binding.tvAllDays.setOnClickListener(v -> onAllDayClicked());
        binding.rlStartDate.setOnClickListener(v -> {
            timePicker.select(instance -> {
                setBtnDayBackground(0, false);
                setBtnDayBackground(1, false);
                setBtnDayBackground(2, false);
                setCalendarBackground(0, true);
                binding.tvStartDate.setText(TimePicker.getTime(new Date(instance)));
                startTime = instance;
            });
        });
        binding.rlEndDate.setOnClickListener(v -> {
            timePicker.select(binding.tvStartDate.getText().toString(), instance -> {
                setBtnDayBackground(0, false);
                setBtnDayBackground(1, false);
                setBtnDayBackground(2, false);
                setCalendarBackground(1, true);
                binding.tvEndDate.setText(TimePicker.getTime(new Date(instance)));
                endTime = instance;
            });
        });

        binding.filterBtnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent();
                if (forceSurvey) {
                    intent.putExtra(KEY_NEED_SURVEY, true);
                } else {
                    intent.putExtra(KEY_NEED_SURVEY, binding.cbSurvey.isChecked());
                    intent.putExtra(KEY_NEED_INPUT, binding.cbInput.isChecked());
                    intent.putExtra(KEY_NEED_CONTROL, binding.cbControl.isChecked());
                }
                if (startTime > 0) {
                    intent.putExtra(KEY_DATE_START, startTime);
                }
                if (endTime > 0) {
                    intent.putExtra(KEY_DATE_END, endTime);
                }
                intent.putExtra(KEY_NAME, binding.etName.getText().toString());
                intent.putExtra(KEY_CODE, binding.etCode.getText().toString());
                intent.putExtra(KEY_REMARKS, binding.etRemarks.getText().toString());

                setResult(RESULT_OK, intent);
                finish();
            }
        });

        binding.filterBtnCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
        //除时间外，设置默认值
        binding.cbSurvey.setChecked(IntentUtil.boolExtra(this, KEY_NEED_SURVEY));
        binding.cbInput.setChecked(IntentUtil.boolExtra(this, KEY_NEED_INPUT));
        binding.cbControl.setChecked(IntentUtil.boolExtra(this, KEY_NEED_CONTROL));

        binding.etName.setText(IntentUtil.stringExtra(this, KEY_NAME));
        binding.etCode.setText(IntentUtil.stringExtra(this, KEY_CODE));
        binding.etRemarks.setText(IntentUtil.stringExtra(this, KEY_REMARKS));

        //隐藏三点更多
        hideMore();
    }

    @Override
    protected void initData() {
        long start = IntentUtil.longExtra(this, KEY_DATE_START);
        long end = IntentUtil.longExtra(this, KEY_DATE_END);
        if (start < 0 && end < 0) {
            onAllDayClicked();
        } else {
            if (end - start == 1000 * 60 * 60 * 24 * 7 - 2000) {
                on7DayClicked();
            } else {
                onTodayClicked();
            }
        }
    }

    /**
     * @param selectedIndex 0--今天，1--7天，2--全部
     */
    private void setBtnDayBackground(int selectedIndex, boolean selected) {
        if (0 == selectedIndex) {
            binding.tvToday.setBackgroundResource(buttonBg(selected));
            binding.tvToday.setTextColor(getResources().getColor(buttonColor(selected)));
        } else if (1 == selectedIndex) {
            binding.tvNearly7Days.setBackgroundResource(buttonBg(selected));
            binding.tvNearly7Days.setTextColor(getResources().getColor(buttonColor(selected)));
        } else if (2 == selectedIndex) {
            binding.tvAllDays.setBackgroundResource(buttonBg(selected));
            binding.tvAllDays.setTextColor(getResources().getColor(buttonColor(selected)));
        }
    }

    private static int buttonBg(boolean selected) {
        return selected ?
                R.drawable.rectangle_radius_5_theme_green_press_full :
                R.drawable.rectangle_radius_5_theme_green_line;
    }

    private static int buttonColor(boolean selected) {
        return selected ? R.color.white : R.color.theme_green;
    }

    private void setCalendarBackground(int selectedIndex, boolean selected) {
        if (0 == selectedIndex) {
            binding.rlStartDate.setBackgroundResource(selected ?
                    R.drawable.rectangle_radius_5_theme_green_line :
                    R.drawable.rectangle_radius_5_theme_gray_line);
            binding.tvStartDate.setTextColor(getResources().getColor(selected ?
                    R.color.theme_green : R.color.color_575757));
            binding.ivStartCalendar.setImageResource(selected ?
                    R.mipmap.icon_calendar_green : R.mipmap.icon_calendar_gray);
        } else if (1 == selectedIndex) {
            binding.rlEndDate.setBackgroundResource(selected ?
                    R.drawable.rectangle_radius_5_theme_green_line :
                    R.drawable.rectangle_radius_5_theme_gray_line);
            binding.tvEndDate.setTextColor(getResources().getColor(selected ?
                    R.color.theme_green : R.color.color_575757));
            binding.ivEndCalendar.setImageResource(selected ?
                    R.mipmap.icon_calendar_green : R.mipmap.icon_calendar_gray);
        }
    }

    public interface IFilterCallback {
        void onFilter(boolean needSurvey, boolean needInput, boolean needControl,
                      Date dateStart, Date dateEnd, String nameKey, String codeKey, String remarksKey);
    }

    private static final String KEY_FORCE_SURVEY = "FORCE SURVEY";//只在测量点中过滤

    private static final String KEY_NEED_SURVEY = "NEED_SURVEY";
    private static final String KEY_NEED_INPUT = "NEED INPUT";
    private static final String KEY_NEED_CONTROL = "NEED CONTROL";
    private static final String KEY_DATE_START = "date start";
    private static final String KEY_DATE_END = "date end";
    private static final String KEY_NAME = "name key";
    private static final String KEY_CODE = "code key";
    private static final String KEY_REMARKS = "remarks key";

    public static void startFilter(AppCompatActivity activity, boolean forceSurvey,
                                   boolean needSurvey, boolean needInput, boolean needControl,
                                   Date dateStart, Date dateEnd, String nameKey, String codeKey, String remarksKey,
                                   IFilterCallback callback) {
        Intent intent2StartActivity = new Intent(activity, PointFilterActivity.class);
        intent2StartActivity.putExtra(KEY_FORCE_SURVEY, forceSurvey);
        intent2StartActivity.putExtra(KEY_NEED_SURVEY, needSurvey);
        intent2StartActivity.putExtra(KEY_NEED_INPUT, needInput);
        intent2StartActivity.putExtra(KEY_NEED_CONTROL, needControl);
        intent2StartActivity.putExtra(KEY_DATE_START, dateStart == null ? -1 : dateStart.getTime());
        intent2StartActivity.putExtra(KEY_DATE_END, dateEnd == null ? -1 : dateEnd.getTime());
        intent2StartActivity.putExtra(KEY_NAME, nameKey);
        intent2StartActivity.putExtra(KEY_CODE, codeKey);
        intent2StartActivity.putExtra(KEY_REMARKS, remarksKey);

        ActivityUtils.startActivityForResults(activity, intent2StartActivity, result -> {
            if (result == null || result.getData() == null || result.getResultCode() != RESULT_OK) {
                return;
            }
            Intent intent = result.getData();
            long start = intent.getLongExtra(KEY_DATE_START, -1);
            long end = intent.getLongExtra(KEY_DATE_END, -1);
            callback.onFilter(
                    intent.getBooleanExtra(KEY_NEED_SURVEY, false),
                    intent.getBooleanExtra(KEY_NEED_INPUT, false),
                    intent.getBooleanExtra(KEY_NEED_CONTROL, false),
                    start > 0 ? new Date(start) : null,
                    end > 0 ? new Date(end) : null,
                    intent.getStringExtra(KEY_NAME),
                    intent.getStringExtra(KEY_CODE),
                    intent.getStringExtra(KEY_REMARKS));
        });
    }
}