package com.project.survey.ui.instrument.setupstation;

import android.annotation.SuppressLint;
import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.surveyor.util.dialog.CustomDialog;
import com.project.survey.R;
import com.project.survey.ui.instrument.setupstation.db.resultformat.FormatRecord;

import java.util.List;


public class ResultsExportAdapter extends RecyclerView.Adapter<ResultsExportAdapter.ViewHolder> {
    private Context context;
    private OnItemClickListener onClickListener;
    private int selectPosition;
    private List<FormatRecord> formatRecordList;

    public ResultsExportAdapter(Context context, List<FormatRecord> formatRecordList, int defPosition) {
        this.selectPosition = defPosition;
        this.context = context;
        this.formatRecordList = formatRecordList;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        ViewHolder holder = new ViewHolder(LayoutInflater.from(
                context).inflate(R.layout.layout_results_export_item, parent,
                false));
        return holder;
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder viewHolder, @SuppressLint("RecyclerView") int position) {
        FormatRecord formatRecord = formatRecordList.get(position);

        if (TextUtils.isEmpty(formatRecord.id)) {
            viewHolder.iv_format_fixed.setVisibility(View.VISIBLE);
        } else {
            viewHolder.iv_format_fixed.setVisibility(View.GONE);
        }
        viewHolder.tv_name.setText(formatRecord.format_name);

        StringBuilder stringBuilder = new StringBuilder();
        String[] split = formatRecord.format_content.split(formatRecord.divided_symbols);
        if (position == 5) {
            viewHolder.tv_view_all.setVisibility(View.VISIBLE);
        } else {
            viewHolder.tv_view_all.setVisibility(View.GONE);
        }

        viewHolder.tv_view_all.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                new CustomDialog.Builder(context).setContent(formatRecord.format_content).create().show();
            }
        });

        for (int i = 0; i < split.length; i++) {
            switch (split[i]) {
                case "点名":
                    split[i] = context.getString(R.string.point_name);
                    break;
                case "编码":
                    split[i] = "编码";
                    break;
                case "纬度":
                    split[i] = "纬度";
                    break;
                case "经度":
                    split[i] = context.getString(R.string.longitude);
                    break;
                case "大地高":
                    split[i] = "大地高";
                    break;
                case "天线类型":
                    split[i] = "天线类型";
                    break;
                case "天线高度":
                    split[i] = context.getString(R.string.antenna_height);
                    break;
                case "北坐标":
                    split[i] = "北坐标";
                    break;
                case "东坐标":
                    split[i] = "东坐标";
                    break;
                case "高程":
                    split[i] = "高程";
                    break;
                case "解状态":
                    split[i] = "解状态";
                    break;
                case "解算卫星":
                    split[i] = "解算卫星";
                    break;
                case "可见卫星":
                    split[i] = "可见卫星";
                    break;
                case "PDOP":
                    split[i] = "PDOP";
                    break;
                case "HRMS":
                    split[i] = "HRMS";
                    break;
                case "VRMS":
                    split[i] = "VRMS";
                    break;
                case "差分延迟":
                    split[i] = "差分延迟";
                    break;
                case "采集时间":
                    split[i] = "采集时间";
                    break;
                case "基站ID":
                    split[i] = "基站ID";
                    break;
                case "基站纬度":
                    split[i] = "基站纬度";
                    break;
                case "基站经度":
                    split[i] = "基站经度";
                    break;
                case "基站距离":
                    split[i] = "基站距离";
                    break;
                case "惯导测量":
                    split[i] = "惯导测量";
                    break;
            }

            if (i == split.length - 1) {
                stringBuilder.append("[" + split[i] + "]");
            } else {
                stringBuilder.append("[" + split[i] + "]" + formatRecord.divided_symbols);
            }
        }

        viewHolder.tv_content.setText(stringBuilder.toString());
        switch (formatRecord.file_suffix) {
            case 0:
                viewHolder.tv_suffix.setText(".xls");
                break;
            case 1:
                viewHolder.tv_suffix.setText(".dat");
                break;
            case 2:
                viewHolder.tv_suffix.setText(".txt");
                break;
            case 3:
                viewHolder.tv_suffix.setText(".csv");
                break;
            case 4:
                viewHolder.tv_suffix.setText(".dxf");
                break;
            case 5:
                viewHolder.tv_suffix.setText(".kml");
                break;
        }

        viewHolder.tv_file_header.setText("文件表头" + "：" + (formatRecord.file_header ? context.getString(R.string.yes) : context.getString(R.string.no)));

        if (formatRecord.angle_format == 0) {
            viewHolder.tv_angle_formats.setText("度.分秒");
        } else if (formatRecord.angle_format == 1) {
            viewHolder.tv_angle_formats.setText("度°分′秒″");
        } else if (formatRecord.angle_format == 2) {
            viewHolder.tv_angle_formats.setText("度");
        } else if (formatRecord.angle_format == 3) {
            viewHolder.tv_angle_formats.setText("弧度");
        }

        if (selectPosition == position) {
            viewHolder.iv_checkbox.setImageResource(R.mipmap.icon_checkbox_select);
        } else {
            viewHolder.iv_checkbox.setImageResource(R.mipmap.icon_checkbox_unselect);
        }
        viewHolder.itemView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                onClickListener.OnItemClick(position);
                selectPosition = position;
                notifyDataSetChanged();
            }
        });
    }

    @Override
    public int getItemCount() {
        return formatRecordList == null ? 0 : formatRecordList.size();
    }

    public void setUpdateList(int position, List<FormatRecord> formatRecordList) {
        this.selectPosition = position;
        this.formatRecordList = formatRecordList;
        notifyDataSetChanged();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        TextView tv_name, tv_content, tv_suffix, tv_file_header, tv_angle_formats, tv_view_all;
        ImageView iv_checkbox;
        ImageView iv_format_fixed;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            iv_format_fixed = itemView.findViewById(R.id.iv_format_fixed);
            tv_name = itemView.findViewById(R.id.tv_name);
            tv_content = itemView.findViewById(R.id.tv_content);
            tv_suffix = itemView.findViewById(R.id.tv_suffix);
            iv_checkbox = itemView.findViewById(R.id.iv_checkbox);
            tv_file_header = itemView.findViewById(R.id.tv_file_header);
            tv_angle_formats = itemView.findViewById(R.id.tv_angle_formats);
            tv_view_all = itemView.findViewById(R.id.tv_view_all);
        }
    }

    public interface OnItemClickListener {
        void OnItemClick(int position);
    }

    public void setOnClickListener(OnItemClickListener onClickListener) {
        this.onClickListener = onClickListener;
    }
}
