package com.project.survey.ui.instrument.setupstation;

import static androidx.recyclerview.widget.LinearLayoutManager.VERTICAL;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.Nullable;
import androidx.recyclerview.widget.LinearLayoutManager;

import com.bingce.coordlib.model.CoordinateSystem;
import com.bingce.data.cache.CachedCurrentCoordinateSystem;
import com.bingce.data.cache.CachedCurrentJob;
import com.bingce.data.cache.CachedProject;
import com.bingce.data.cache.CachedProjectKt;
import com.bingce.data.database.PointDb;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.surveyor.util.ConstUtils;
import com.bingce.surveyor.util.dialog.CustomDialog;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.google.gson.Gson;
import com.hjq.permissions.OnPermissionCallback;
import com.hjq.permissions.Permission;
import com.hjq.permissions.XXPermissions;
import com.project.survey.R;
import com.project.survey.databinding.ActivityResultsTheExportBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.db.PointSurveyRecord;
import com.project.survey.ui.instrument.setupstation.db.RecordsFixedDataBase;
import com.project.survey.ui.instrument.setupstation.db.resultformat.FormatRecord;
import com.project.survey.util.CassCsvTxtExportUtil;
import com.project.survey.util.DxfUtil;
import com.project.survey.util.ExcelUtil;
import com.project.survey.util.FormatSuffixesUtils;
import com.project.survey.util.SystemUtils;
import com.project.survey.util.TxtUtil;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import blankj.utilcode.util.ActivityUtils;
import blankj.utilcode.util.ToastUtils;


public class ResultsTheExportActivity extends BaseSurveyNewActivity {

    private ActivityResultsTheExportBinding binding;
    private final String filterId = SystemUtils.getUUID();
    private ResultsExportAdapter adapter;
    private String uid;
    private final List<PointSurveyRecord> selectedPoints = new ArrayList<>();
    private int defPosition = 0;
    private List<FormatRecord> formatRecordList = new ArrayList<>();
    private String title;
    private boolean isImport;
    private List<String> fileTypeSuffixList;
    private String txtContent = "";
    private final CachedProjectKt currentProject = new CachedProjectKt(this);

    @Override
    public View getContentView() {
        binding = ActivityResultsTheExportBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        title = IntentUtil.stringExtra(this, ConstUtils.intentConst.keyActivityTitleName);
        isImport = IntentUtil.boolExtra(this, ConstUtils.intentConst.keyIsImport);
        if (StringUtil.isEmpty(title)) {
            setTitle("导出");
        } else {
            setTitle(title);
        }

        binding.pointsLibraryBtnNewCreate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AddResultFormatActivity.start(launcher, ResultsTheExportActivity.this, true);
            }
        });

        binding.pointsLibraryBtnEdit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (formatRecordList != null && formatRecordList.size() > 0) {
                    if (defPosition == -1) {
                        ToastUtils.showShort(getString(R.string.select_format_want_edit));
                        return;
                    }

                    if (StringUtil.isEmpty(formatRecordList.get(defPosition).id)) {
                        ToastUtils.showShort(getString(R.string.default_format_not_allowed_edit_delete));
                        return;
                    }

                    AddResultFormatActivity.start(launcher, ResultsTheExportActivity.this, false, uid);
                }
            }
        });

        binding.pointsLibraryBtnDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (formatRecordList != null && formatRecordList.size() > 0) {
                    if (defPosition == -1) {
                        ToastUtils.showShort(getString(R.string.select_format_want_delete));
                        return;
                    }

                    if (StringUtil.isEmpty(formatRecordList.get(defPosition).id)) {
                        ToastUtils.showShort(getString(R.string.default_format_not_allowed_edit_delete));
                        return;
                    }

                    ThreadPoolUtil.execute(new Runnable() {
                        @Override
                        public void run() {
                            RecordsFixedDataBase.getInstance().formatRecordDao().deleteById(uid);
                            ThreadPoolUtil.executeInMain(new Runnable() {
                                @Override
                                public void run() {
                                    adapter.notifyItemRemoved(defPosition);
                                    formatRecordList.remove(defPosition);
                                    defPosition = -1;
                                    uid = "";
                                    adapter.setUpdateList(defPosition, formatRecordList);
                                }
                            });
                        }
                    });
                }
            }
        });

        binding.pointsLibraryBtnImportExport.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (formatRecordList != null && formatRecordList.size() > 0) {
                    requestStoragePermission();
                } else {
                    ToastUtils.showShort(getString(R.string.create_new_export_format_first));
                }
            }
        });
    }

    public void requestStoragePermission() {
        XXPermissions.with(this)
                .permission(Permission.MANAGE_EXTERNAL_STORAGE)
                .request(new OnPermissionCallback() {
                    @Override
                    public void onGranted(List<String> permissions, boolean all) {
                        if (!isImport) {
                            export();
                        } else {
                            importContent();
                        }
                    }

                    @Override
                    public void onDenied(List<String> permissions, boolean never) {
                        if (never) {
                            ToastUtils.showShort("请手动授予文件读写权限");
                            // 如果是被永久拒绝就跳转到应用权限系统设置页面
                            XXPermissions.startPermissionActivity(ResultsTheExportActivity.this, permissions);
                        } else {
                            requestStoragePermission();
                        }
                    }
                });
    }

    private void export() {
        //导出功能
        ThreadPoolUtil.execute(new Runnable() {
            @Override
            public void run() {
                if (pointBaseRecordList2Export() != null && pointBaseRecordList2Export().size() > 0) {
                    ThreadPoolUtil.executeInMain(new Runnable() {
                        @Override
                        public void run() {
                            switch (formatRecordList.get(defPosition).file_suffix) {
                                case 0:
                                    ExcelUtil.exportFormat(ExcelUtil.ID_POINT_COORDINATE, ResultsTheExportActivity.this, currentProject, formatRecordList.get(defPosition), filePaths -> {
                                        ExcelUtil.exportCoordinator(pointBaseRecordList2Export(), filePaths, ResultsTheExportActivity.this, formatRecordList.get(defPosition));
                                    });
                                    break;
                                case 1:
                                    ExportCassCsvTxt(FormatSuffixesUtils.formatDAT);
                                    break;
                                case 2:
                                    ExportCassCsvTxt(FormatSuffixesUtils.formatTXT);
                                    break;
                                case 3:
                                    ExportCassCsvTxt(FormatSuffixesUtils.formatCSV);
                                    break;
                                case 4:
                                    ExportDxf();
                                    break;
                                case 5:
                                    ExportKml();
                                    break;
                            }
                        }
                    });
                } else {
                    ToastUtils.showShort(getString(R.string.no_exportable_data));
                }
            }
        });
    }

    private void importContent() {
        //导入功能
        fileTypeSuffixList = new ArrayList<>();
        String fileFormatName = "";
        int suffix = formatRecordList.get(defPosition).file_suffix;
        if (suffix == AddResultFormatActivity.SUFFIX_XLS_NUM) {
            fileTypeSuffixList = Arrays.asList("xls", "xlsx");
            fileFormatName = "Excel";
        } else if (suffix == AddResultFormatActivity.SUFFIX_DAT_NUM) {
            fileTypeSuffixList = Arrays.asList("dat", "DAT");
            fileFormatName = "Cass";
        } else if (suffix == AddResultFormatActivity.SUFFIX_TXT_NUM) {
            fileTypeSuffixList = Arrays.asList("txt", "TXT");
            fileFormatName = "Txt";
        } else if (suffix == AddResultFormatActivity.SUFFIX_CSV_NUM) {
            fileTypeSuffixList = Arrays.asList("csv", "CSV");
            fileFormatName = "Csv";
        }
        new CustomDialog.Builder(ResultsTheExportActivity.this)
                .setTitle(getString(R.string.import1) + fileFormatName + getString(R.string.file))
                .setContent("选择导入的文件，请严格按照以下" + "\n\n[" + formatRecordList.get(defPosition).format_content + "]" + "\n" + "以免出现导入失败等情况！")
                .setButtonConfirm(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
//                                        SelectFilePathActivity.select(ResultsTheExportActivity.this, fileTypeSuffixList, instance -> {
//                                            ThreadPoolUtil.execute(() -> {
//                                                List<PointSurveyRecord> pointSurveyRecordList = null;
//                                                String projectId = CachedProject.currentProjectId();
//                                                if (suffix == AddResultFormatActivity.SUFFIX_XLS_NUM){
//                                                    pointSurveyRecordList = ImportCoordinatorUtils.doImport(formatRecordList.get(defPosition),projectId, ResultsTheExportActivity.this, instance);
//                                                }else if (suffix == AddResultFormatActivity.SUFFIX_DAT_NUM || suffix == AddResultFormatActivity.SUFFIX_TXT_NUM || suffix == AddResultFormatActivity.SUFFIX_CSV_NUM){
//                                                    pointSurveyRecordList = CassCsvTxtImportUtil.importControlPoint(formatRecordList.get(defPosition),ResultsTheExportActivity.this, instance);
//                                                }
//                                                Intent intent = new Intent();
//                                                intent.putExtra(ConstUtils.intentConst.keyCoordinatePointsLibrary,new Gson().toJson(pointSurveyRecordList));
//                                                setResult(LauncherEvent.launcher_coordinate_point_library,intent);
//                                                finish();
//                                            });
//                                        });
                    }
                }).create().show();
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == XXPermissions.REQUEST_CODE) {
            if (XXPermissions.isGranted(this, Permission.MANAGE_EXTERNAL_STORAGE)) {
                if (!isImport) {
                    export();
                } else {
                    importContent();
                }
            } else {
                ToastUtils.showLong(getString(R.string.open_read_write_permissions));
            }
        }
    }

    private void ExportKml() {
        ThreadPoolUtil.execute(() -> {
            List<PointSurveyRecord> pointSurveyRecordList = pointBaseRecordList2Export();
            ThreadPoolUtil.executeInMain(new Runnable() {
                @Override
                public void run() {
                    if (pointSurveyRecordList != null && pointSurveyRecordList.size() > 0) {
                        String txtContentHead = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
                                "<kml xmlns=\"http://www.opengis.net/kml/2.2\">\n" +
                                "<Document>\n" +
                                "<name>" + currentProject.currentProject().name + "</name>\n" +
                                "\t<open>1</open>\n" +
                                "\t<Style id=\"khStyle\">\n" +
                                "\t<IconStyle>\n" +
                                "\t<scale>0.6</scale>\n" +
                                "\t<Icon><href>http://maps.google.com/mapfiles/kml/shapes/donut.png</href></Icon>\n" +
                                "\t</IconStyle>\n" +
                                "\t<LabelStyle>\n" +
                                "\t<scale>0.7</scale>\n" +
                                "\t</LabelStyle>\n" +
                                "\t</Style>";

                        for (int i = 0; i < pointSurveyRecordList.size(); i++) {
                            txtContent += "<Placemark>\n" +
                                    "\t<name>" + pointSurveyRecordList.get(i).pointName + "</name>\n" +
                                    "\t<styleUrl>#khStyle</styleUrl>\n" +
                                    "\t<Style>\n" +
                                    "\t<IconStyle>\n" +
                                    "\t<color>ff000000</color>\n" +
                                    "\t</IconStyle>\n" +
                                    "\t</Style>\n" +
                                    "\t<Point>\n" +
                                    "\t<coordinates>\n" +
                                    "\t" + pointSurveyRecordList.get(i).planeN + "," + pointSurveyRecordList.get(i).planeE + "," + pointSurveyRecordList.get(i).altitude + "\n" +
                                    "\t</coordinates>\n" +
                                    "\t</Point>\n" +
                                    "\t</Placemark>";
                        }

                        String txtContentFoot = "</Document>\n" +
                                "</kml>";

                        TxtUtil.exportFormat(ResultsTheExportActivity.this, txtContentHead + txtContent + txtContentFoot, FormatSuffixesUtils.formatKML, currentProject);
                    } else {
                        ToastUtils.showShort(getString(R.string.no_exportable_data));
                    }
                }
            });
        });
    }

    private void ExportDxf() {
        ThreadPoolUtil.execute(() -> {
            List<PointSurveyRecord> pointSurveyRecordList = pointBaseRecordList2Export();
            ThreadPoolUtil.executeInMain(new Runnable() {
                @Override
                public void run() {
                    if (pointSurveyRecordList != null && pointSurveyRecordList.size() > 0) {
                        DxfUtil.exportFormat(ResultsTheExportActivity.this, currentProject, pointSurveyRecordList);
                    } else {
                        ToastUtils.showShort(getString(R.string.no_exportable_data));
                    }
                }
            });
        });
    }

    private void ExportCassCsvTxt(String formatDat) {
        ThreadPoolUtil.execute(new Runnable() {
            @Override
            public void run() {
                CoordinateSystem coordinateSystem = CachedCurrentCoordinateSystem.currentCoordinateSystem();
                List<PointSurveyRecord> pointBaseRecordList = pointBaseRecordList2Export();
                ThreadPoolUtil.executeInMain(new Runnable() {
                    @Override
                    public void run() {
                        CassCsvTxtExportUtil.exportFormat(ResultsTheExportActivity.this, formatDat, currentProject, formatRecordList.get(defPosition), pointBaseRecordList, coordinateSystem);
                    }
                });
            }
        });
    }

    private List<PointSurveyRecord> pointBaseRecordList2Export() {
        if (selectedPoints.isEmpty()) {
            String jobId = IntentUtil.stringExtra(getIntent(), KEY_JOB_ID);
            String filterName = IntentUtil.stringExtra(getIntent(), KEY_NAME);
            String filterCode = IntentUtil.stringExtra(getIntent(), KEY_CODE);
            String filterRemark = IntentUtil.stringExtra(getIntent(), KEY_REMARK);
            Date filterStart = new Date(IntentUtil.longExtra(getIntent(), KEY_START_DATE));
            Date filterEnd = new Date(IntentUtil.longExtra(getIntent(), KEY_END_DATE));
            List<PointRecord> results = PointDb.getInstance().rawQueryListData(PointConstants.findPointSurveyByJobNameCodeRemarkDate(jobId, filterName, filterCode, filterRemark, filterStart, filterEnd));
            if (results != null) {
                for (int i = 0; i < results.size(); i++) {
                    PointRecord pointRecord = results.get(i);
                    if (pointRecord == null) {
                        continue;
                    }
                    PointSurveyRecord pointSurveyRecord = new PointSurveyRecord(pointRecord);
                    pointSurveyRecord.pointId = pointRecord.id;
                    pointSurveyRecord.pointName = pointRecord.name;
                    pointSurveyRecord.code = pointRecord.code;
                    pointSurveyRecord.planeN = pointRecord.x;
                    pointSurveyRecord.planeE = pointRecord.y;
                    pointSurveyRecord.planeH = pointRecord.h;
                    pointSurveyRecord.latitude = pointRecord.latitude;
                    pointSurveyRecord.longitude = pointRecord.longitude;
                    pointSurveyRecord.altitude = pointRecord.altitude;
                    pointSurveyRecord.originalLatitude = pointRecord.originalLatitude;
                    pointSurveyRecord.originalLongitude = pointRecord.originalLongitude;
                    pointSurveyRecord.originalAltitude = pointRecord.originalAltitude;
                    pointSurveyRecord.remarks = pointRecord.remarks;
                    pointSurveyRecord.createdAt = pointRecord.createDate;
                    if (pointRecord.deviceInfoData.rtkStatusData == null && pointRecord.deviceInfoData.deprecatedDeviceData == null) {
                        pointSurveyRecord.solutionState = "";
                    } else if (pointRecord.deviceInfoData.rtkStatusData != null) {
                        pointSurveyRecord.solutionState = pointRecord.deviceInfoData.rtkStatusData.solutionState;
                    } else {
                        pointSurveyRecord.solutionState = pointRecord.deviceInfoData.deprecatedDeviceData.solutionState;
                    }
                    if (pointRecord.deviceInfoData.rtkStatusData != null) {
                        pointSurveyRecord.satelliteNum = pointRecord.deviceInfoData.rtkStatusData.satSolutionNum;
                    } else {
                        if (pointRecord.deviceInfoData.deprecatedDeviceData == null) {
                            pointSurveyRecord.satelliteNum = -1;
                        } else {
                            if (!StringUtil.isEmpty(pointRecord.deviceInfoData.deprecatedDeviceData.satellite)) {
                                pointSurveyRecord.satelliteNum = Integer.parseInt(pointRecord.deviceInfoData.deprecatedDeviceData.satellite);
                            } else {
                                pointSurveyRecord.satelliteNum = -1;
                            }
                        }
                    }
                    pointSurveyRecord.satelliteTrackNum = pointRecord.deviceInfoData.rtkStatusData != null ? pointRecord.deviceInfoData.rtkStatusData.satTrackNum : -1;
                    if (pointRecord.deviceInfoData.rtkStatusData != null) {
                        pointSurveyRecord.pdop = pointRecord.deviceInfoData.rtkStatusData.pdop;
                    } else {
                        if (pointRecord.deviceInfoData.deprecatedDeviceData == null) {
                            pointSurveyRecord.pdop = -1;
                        } else {
                            if (!StringUtil.isEmpty(pointRecord.deviceInfoData.deprecatedDeviceData.pdop)) {
                                pointSurveyRecord.pdop = Double.parseDouble(pointRecord.deviceInfoData.deprecatedDeviceData.pdop);
                            } else {
                                pointSurveyRecord.pdop = -1;
                            }
                        }
                    }
                    if (pointRecord.deviceInfoData.rtkStatusData != null) {
                        pointSurveyRecord.hrms = pointRecord.deviceInfoData.rtkStatusData.hrms;
                    } else {
                        if (pointRecord.deviceInfoData.deprecatedDeviceData == null) {
                            pointSurveyRecord.hrms = -1;
                        } else {
                            if (!StringUtil.isEmpty(pointRecord.deviceInfoData.deprecatedDeviceData.hrms)) {
                                pointSurveyRecord.hrms = Double.parseDouble(pointRecord.deviceInfoData.deprecatedDeviceData.hrms);
                            } else {
                                pointSurveyRecord.hrms = -1;
                            }
                        }
                    }
                    if (pointRecord.deviceInfoData.rtkStatusData != null) {
                        pointSurveyRecord.vrms = pointRecord.deviceInfoData.rtkStatusData.vrms;
                    } else {
                        if (pointRecord.deviceInfoData.deprecatedDeviceData == null) {
                            pointSurveyRecord.vrms = -1;
                        } else {
                            if (!StringUtil.isEmpty(pointRecord.deviceInfoData.deprecatedDeviceData.vrms)) {
                                pointSurveyRecord.vrms = Double.parseDouble(pointRecord.deviceInfoData.deprecatedDeviceData.vrms);
                            } else {
                                pointSurveyRecord.vrms = -1;
                            }
                        }
                    }
                    if (pointRecord.deviceInfoData.rtkStatusData != null) {
                        pointSurveyRecord.diffAge = pointRecord.deviceInfoData.rtkStatusData.diffAge;
                    } else {
                        if (pointRecord.deviceInfoData.deprecatedDeviceData == null) {
                            pointSurveyRecord.diffAge = -1;
                        } else {
                            if (!StringUtil.isEmpty(pointRecord.deviceInfoData.deprecatedDeviceData.diffAge)) {
                                pointSurveyRecord.diffAge = Double.parseDouble(pointRecord.deviceInfoData.deprecatedDeviceData.diffAge);
                            } else {
                                pointSurveyRecord.diffAge = -1;
                            }
                        }
                    }
                    pointSurveyRecord.isTiltEnable = pointRecord.deviceInfoData.rtkStatusData != null && pointRecord.deviceInfoData.rtkStatusData.isTiltEnable;
                    if (pointRecord.deviceInfoData.rtkStatusData != null) {
                        pointSurveyRecord.antennaHeight = pointRecord.deviceInfoData.rtkStatusData.poleHeight;
                    } else {
                        if (pointRecord.deviceInfoData.deprecatedDeviceData == null) {
                            pointSurveyRecord.antennaHeight = -1;
                        } else {
                            if (!StringUtil.isEmpty(pointRecord.deviceInfoData.deprecatedDeviceData.hr)) {
                                pointSurveyRecord.antennaHeight = Double.parseDouble(pointRecord.deviceInfoData.deprecatedDeviceData.hr);
                            } else {
                                pointSurveyRecord.antennaHeight = -1;
                            }
                        }
                    }
                    selectedPoints.add(pointSurveyRecord);
                }
            }
        }
        return selectedPoints;
    }

    @Override
    protected void initData() {
        ThreadPoolUtil.execute(new Runnable() {
            @Override
            public void run() {
                formatRecordList = RecordsFixedDataBase.getInstance().formatRecordDao().findAll();
                ThreadPoolUtil.executeInMain(new Runnable() {
                    @Override
                    public void run() {
                        if (formatRecordList != null && formatRecordList.size() > 0) {
                            uid = formatRecordList.get(defPosition).id;
                        } else {
                            uid = "";
                        }

                        FormatRecord formatRecordCass = new FormatRecord();
                        formatRecordCass.id = "";
                        formatRecordCass.format_name = "Cass";
                        formatRecordCass.format_content = "点名,编码,东坐标,北坐标,高程";
                        formatRecordCass.divided_symbols = ",";
                        formatRecordCass.file_suffix = 1;
                        formatRecordCass.file_header = false;
                        formatRecordCass.angle_format = 0;
                        formatRecordList.add(0, formatRecordCass);

                        FormatRecord formatRecordPlane = new FormatRecord();
                        formatRecordPlane.id = "";
                        formatRecordPlane.format_name = getString(R.string.plane_coordinate);
                        formatRecordPlane.format_content = "点名,北坐标,东坐标,高程,编码";
                        formatRecordPlane.divided_symbols = ",";
                        formatRecordPlane.file_suffix = 3;
                        formatRecordPlane.file_header = false;
                        formatRecordPlane.angle_format = 0;
                        formatRecordList.add(1, formatRecordPlane);

                        FormatRecord formatRecordLatLon = new FormatRecord();
                        formatRecordLatLon.id = "";
                        formatRecordLatLon.format_name = getString(R.string.lat_lon_coordinate);
                        formatRecordLatLon.format_content = "点名,纬度,经度,大地高,编码";
                        formatRecordLatLon.divided_symbols = ",";
                        formatRecordLatLon.file_suffix = 3;
                        formatRecordLatLon.file_header = false;
                        formatRecordLatLon.angle_format = 0;
                        formatRecordList.add(2, formatRecordLatLon);

                        FormatRecord formatRecordDxf = new FormatRecord();
                        formatRecordDxf.id = "";
                        formatRecordDxf.format_name = "dxf";
                        formatRecordDxf.format_content = "点名,经度,纬度,大地高";
                        formatRecordDxf.divided_symbols = ",";
                        formatRecordDxf.file_suffix = 4;
                        formatRecordDxf.file_header = false;
                        formatRecordDxf.angle_format = 0;
                        formatRecordList.add(3, formatRecordDxf);

                        FormatRecord formatRecordKml = new FormatRecord();
                        formatRecordKml.id = "";
                        formatRecordKml.format_name = "google kml";
                        formatRecordKml.format_content = "点名,北坐标,东坐标,高程";
                        formatRecordKml.divided_symbols = ",";
                        formatRecordKml.file_suffix = 5;
                        formatRecordKml.file_header = false;
                        formatRecordKml.angle_format = 0;
                        formatRecordList.add(4, formatRecordKml);

                        FormatRecord formatRecordAllCsv = new FormatRecord();
                        formatRecordAllCsv.id = "";
                        formatRecordAllCsv.format_name = getString(R.string.format_original_data);
                        formatRecordAllCsv.format_content = "点名,编码,纬度,经度,大地高,天线类型,天线高,北坐标,东坐标,高程,解状态,解算卫星,可见卫星,PDOP,HRMS,VRMS,差分延迟,备注,采集时间,惯导测量";
                        formatRecordAllCsv.divided_symbols = ",";
                        formatRecordAllCsv.file_suffix = 3;
                        formatRecordAllCsv.file_header = false;
                        formatRecordAllCsv.angle_format = 0;
                        formatRecordList.add(5, formatRecordAllCsv);

                        adapter = new ResultsExportAdapter(ResultsTheExportActivity.this, formatRecordList, defPosition);
                        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(ResultsTheExportActivity.this);
                        linearLayoutManager.setOrientation(VERTICAL);
                        binding.recyclerView.setLayoutManager(linearLayoutManager);
                        binding.recyclerView.setAdapter(adapter);

                        adapter.setOnClickListener(new ResultsExportAdapter.OnItemClickListener() {
                            @Override
                            public void OnItemClick(int position) {
                                defPosition = position;
                                uid = formatRecordList.get(position).id;
                            }
                        });
                    }
                });
            }
        });
    }

    public final ActivityResultLauncher<Intent> launcher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result != null) {
            switch (result.getResultCode()) {
                case LauncherEvent.launcher_results_export:
                    FormatRecord formatRecord = new Gson().fromJson(result.getData().getStringExtra(ConstUtils.intentConst.keyResultFormatRecord), FormatRecord.class);
                    if (result.getData().getBooleanExtra(ConstUtils.intentConst.keyIsNewCreateResultFormat, false)) {
                        formatRecordList.add(formatRecord);
                        if (formatRecordList != null && formatRecordList.size() > 0) {
                            adapter.setUpdateList(formatRecordList.size() - 1, formatRecordList);
                            defPosition = formatRecordList.size() - 1;
                        } else {
                            adapter.setUpdateList(0, formatRecordList);
                            defPosition = 0;
                        }
                        uid = formatRecord.id;
                    } else {
                        formatRecordList.set(defPosition, formatRecord);
                        adapter.setUpdateList(defPosition, formatRecordList);
                    }
                    break;
            }
        }
    });

    /**
     * 根据成果导出的格式，经纬度进行转换导出
     *
     * @param angle_format_type
     * @param l
     * @return
     */
    private static String converAngleFormat(int angle_format_type, double l) {
        if (angle_format_type == AddResultFormatActivity.ANGLE_FORMAT_D_MS) {
            //度.分秒
            return Util.radianToDmsDoubleString(Math.toRadians(l), 6, false);
        } else if (angle_format_type == AddResultFormatActivity.ANGLE_FORMAT_D_M_S) {
            //度°分′秒″
            return Util.radianToDmsString(Math.toRadians(l), 6, true);
        } else if (angle_format_type == AddResultFormatActivity.ANGLE_FORMAT_D) {
            //度
            return Util.formatDouble2String(l, 6);
        } else {
            //弧度
            return Util.formatDouble2String(Math.toRadians(l), 6);
        }
    }

    private final static String KEY_JOB_ID = "JOBID";
    private final static String KEY_NAME = "KEYNAME";
    private final static String KEY_CODE = "KEYCODE";
    private final static String KEY_REMARK = "KEYREMARK";
    private final static String KEY_START_DATE = "KEYSTARTDATE";
    private final static String KEY_END_DATE = "KEYENDDATE";

//    public static void start(Context context, String title, Boolean isImport, String name, String code, String remark, Date startDate, Date endDate) {
//        ThreadPoolUtil.execute(() -> {
//            String jobId = CachedCurrentJob.currentJobId(CachedProject.currentProjectId());
//            Intent intent = new Intent(context, ResultsTheExportActivity.class);
//            intent.putExtra(ConstUtils.intentConst.keyActivityTitleName, title);
//            intent.putExtra(ConstUtils.intentConst.keyIsImport, isImport);
//            intent.putExtra(KEY_JOB_ID, jobId);
//            intent.putExtra(KEY_NAME, name);
//            intent.putExtra(KEY_CODE, code);
//            intent.putExtra(KEY_REMARK, remark);
//            intent.putExtra(KEY_START_DATE, startDate == null ? null : startDate.getTime());
//            intent.putExtra(KEY_END_DATE, endDate == null ? SystemUtils.getSimpleDateTimeStamp() : endDate.getTime());
//            context.startActivity(intent);
//        });
//    }

    public static void start(Context context, String jobId, Boolean isImport, String name, String code, String remark, Date startDate, Date endDate) {
//        Bundle bundle = new Bundle();
//        bundle.putString(ConstUtils.intentConst.keyActivityTitleName, "导出");
//        bundle.putBoolean(ConstUtils.intentConst.keyIsImport, isImport);
//        bundle.putString(KEY_JOB_ID, jobId);
//        bundle.putString(KEY_NAME, name);
//        bundle.putString(KEY_CODE, code);
//        bundle.putString(KEY_REMARK, remark);
//        bundle.putLong(KEY_START_DATE, startDate == null ? null : startDate.getTime());
//        bundle.putLong(KEY_END_DATE, endDate == null ? SystemUtils.getSimpleDateTimeStamp() : endDate.getTime());
//        ActivityUtils.startActivity(bundle, ResultsTheExportActivity.class);
        Intent intent = new Intent(context, ResultsTheExportActivity.class);
        intent.putExtra(ConstUtils.intentConst.keyActivityTitleName, "导出");
        intent.putExtra(ConstUtils.intentConst.keyIsImport, isImport);
        intent.putExtra(KEY_JOB_ID, jobId);
        intent.putExtra(KEY_NAME, name);
        intent.putExtra(KEY_CODE, code);
        intent.putExtra(KEY_REMARK, remark);
        intent.putExtra(KEY_START_DATE, startDate == null ? null : startDate.getTime());
        intent.putExtra(KEY_END_DATE, endDate == null ? SystemUtils.getSimpleDateTimeStamp() : endDate.getTime());
        context.startActivity(intent);
    }
}