package com.project.survey.ui.instrument.setupstation;

import android.graphics.Rect;
import android.view.GestureDetector;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;

import androidx.annotation.IdRes;
import androidx.core.view.GestureDetectorCompat;
import androidx.recyclerview.widget.RecyclerView;

/**
 * description: recyclerView的单击与长按事件监听器
 * version:
 * state: done
 */
public class SimpleRecyclerViewItemClickListener extends RecyclerView.SimpleOnItemTouchListener {

    private OnItemClickListener mListener;
    private GestureDetectorCompat mGestureDetector;
    private RecyclerView mRecyclerView;
    private int[] ids;

    public SimpleRecyclerViewItemClickListener(RecyclerView rv, OnItemClickListener listener, @IdRes int... ids) {
        this.mListener = listener;
        this.mRecyclerView = rv;
        this.ids = ids;
    }
    public SimpleRecyclerViewItemClickListener(RecyclerView rv, OnItemClickListener listener) {
        this.mListener = listener;
        this.mRecyclerView = rv;
    }

    @Override
    public boolean onInterceptTouchEvent(RecyclerView rv, MotionEvent e) {
        if (mGestureDetector == null) {
            initGestureDetector(mRecyclerView);
        }
        return mGestureDetector.onTouchEvent(e);// 把事件交给GestureDetector处理
    }

    private Rect getRelativeRectTo(RecyclerView recyclerView,View otherView,View view){
        Rect rootRect = new Rect();
        recyclerView.getGlobalVisibleRect(rootRect);
        Rect otherRect = new Rect();
        otherView.getGlobalVisibleRect(otherRect);
        Rect rect = new Rect();
        view.getGlobalVisibleRect(rect);
        int relativeLeft = rect.left - otherRect.left;
        int relativeTop = rect.top - rootRect.top;
        int relativeRight = relativeLeft + (rect.right - rect.left);
        int relativeBottom = relativeTop + (rect.bottom - rect.top);
        return new Rect(relativeLeft, relativeTop, relativeRight, relativeBottom);
    }
    /**
     * 初始化GestureDetector
     */
    private void initGestureDetector(final RecyclerView recyclerView) {
        mGestureDetector = new GestureDetectorCompat(recyclerView.getContext(), new GestureDetector.SimpleOnGestureListener() { // 这里选择SimpleOnGestureListener实现类，可以根据需要选择重写的方法

            /**
             * 单击事件
             */
            @Override
            public boolean onSingleTapUp(MotionEvent e) {
                View childView = recyclerView.findChildViewUnder(e.getX(), e.getY());
                if (childView != null && mListener != null) {
                    if (childView instanceof ViewGroup && ids != null && ids.length != 0) {
                        for (int id:ids) {
                            if (getRelativeRectTo(recyclerView,childView, childView.findViewById(id)).contains((int) e.getX(), (int) e.getY())) {
                                mListener.onItemClick(childView.findViewById(id), recyclerView.getChildLayoutPosition(childView));
                                return false;
                            }
                        }
                    }
                    mListener.onItemClick(childView, recyclerView.getChildLayoutPosition(childView));
                }
                return false;
            }

            /**
             * 长按事件
             */
            @Override
            public void onLongPress(MotionEvent e) {
                View childView = recyclerView.findChildViewUnder(e.getX(), e.getY());
                if (childView != null && mListener != null) {
                    if (childView instanceof ViewGroup && ids != null && ids.length != 0) {
                        for (int id:ids) {
                            if (getRelativeRectTo(recyclerView,childView, childView.findViewById(id)).contains((int) e.getX(), (int) e.getY())) {
                                mListener.onItemLongClick(childView.findViewById(id), recyclerView.getChildLayoutPosition(childView));
                                return;
                            }
                        }
                    }
                    mListener.onItemLongClick(childView, recyclerView.getChildLayoutPosition(childView));
                }
            }

        });

    }

    public interface OnItemClickListener {

        /**
         * 当ItemView的单击事件触发时调用
         */
        void onItemClick(View view, int position);

        /**
         * 当ItemView的长按事件触发时调用
         */
        default void onItemLongClick(View view, int position){};

    }
}