package com.project.survey.ui.instrument.setupstation;

import android.app.Activity;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Color;
import android.os.Handler;
import android.text.Editable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.LiveData;
import androidx.paging.Pager;
import androidx.paging.PagingConfig;
import androidx.paging.PagingData;
import androidx.paging.PagingLiveData;
import androidx.paging.PagingSource;
import androidx.recyclerview.widget.RecyclerView;

import com.afollestad.materialdialogs.DialogAction;
import com.afollestad.materialdialogs.MaterialDialog;
import com.afollestad.materialdialogs.internal.MDButton;
import com.bingce.data.cache.CachedProject;
import com.bingce.data.database.PointDb;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.surveyor.agentweb.AgentWebActivity;
import com.bingce.surveyor.util.dialog.CustomDialog;
import com.bingce.ui.BcDialogExt;
import com.bingce.utils.ActivityUtils;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.PointLibraryUtil;
import com.bingce.utils.SimpleTextWatcher;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.bingce.viewmodel.AbstractCoordinatePointsLibraryViewModelKt;
import com.kongzue.dialogx.interfaces.OnBindView;
import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.constants.SPConstants;
import com.project.survey.databinding.ActivityStakingNewJob2Binding;
import com.project.survey.dialog.BubblePopWindow;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.adapter.StakingPointLibraryAdapter;
import com.project.survey.ui.instrument.setupstation.util.StakingPointsLibraryActivitySQLiteUtils;
import com.project.survey.util.CommonUtils;
import com.project.survey.util.SPUtils;
import com.rengwuxian.materialedittext.MaterialEditText;

import java.io.File;
import java.util.Arrays;
import java.util.List;

import blankj.utilcode.util.ConvertUtils;
import blankj.utilcode.util.ToastUtils;
import ru.bartwell.exfilepicker.ExFilePicker;
import ru.bartwell.exfilepicker.data.ExFilePickerResult;

public class StakingNewJobActivity extends BaseSurveyNewActivity {
    private RecyclerView.OnScrollListener leftScrollListener = new RecyclerView.OnScrollListener() {
        @Override
        public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
            if (recyclerView.getScrollState() != RecyclerView.SCROLL_STATE_IDLE) {
                binding.bodyRv.scrollBy(dx, dy);
            }
        }

        @Override
        public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
            super.onScrollStateChanged(recyclerView, newState);
            binding.bodyHsv.setIntercept(newState != RecyclerView.SCROLL_STATE_IDLE);
        }
    };
    private RecyclerView.OnScrollListener rightScrollListener = new RecyclerView.OnScrollListener() {
        @Override
        public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
            if (!isFinishing() && recyclerView.getScrollState() != RecyclerView.SCROLL_STATE_IDLE) {
                binding.bodyNameRv.scrollBy(dx, dy);
            }
        }

        @Override
        public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
            super.onScrollStateChanged(recyclerView, newState);
            binding.bodyNameRl.setIntercept(newState != RecyclerView.SCROLL_STATE_IDLE);
        }
    };
    private static final int EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL = 330;
    private static final int EX_FILE_PICKER_RESULT_SURVEYOR_TXT = 331;
    private static final int EX_FILE_PICKER_RESULT_CASS_DAT = 332;
    private ActivityStakingNewJob2Binding binding;
    private final StakingPointLibraryAdapter stakingPointLibraryAdapter = new StakingPointLibraryAdapter(new PointRecordDiffCallback2(), this, false);
    private final StakingPointLibraryAdapter stakingPointNameLibraryAdapter = new StakingPointLibraryAdapter(new PointRecordDiffCallback2(), this, true);
    private AbstractCoordinatePointsLibraryViewModelKt.FilterParameter filterParameter = new AbstractCoordinatePointsLibraryViewModelKt.FilterParameter();
    private Pager<Integer, PointRecord> pager = new Pager<>(
            new PagingConfig(100),
            this::dataSourse);
    private final FilterHandlerNew filterHandler = new FilterHandlerNew(new FilterHandlerNew.FilterCallBack() {
        @Override
        public void fireByKey(String key) {
            ThreadPoolUtil.execute(() -> {
//                String projectId = CachedProject.currentProjectId();
//                jobId = CachedCurrentJob.currentJobId(projectId);
                ThreadPoolUtil.executeInMain(() -> {
                    pagingDataLiveData.removeObservers(StakingNewJobActivity.this);
                    pagingDataLiveData = PagingLiveData.cachedIn(PagingLiveData.getLiveData(pager), getLifecycle());
                    pagingDataLiveData.observe(StakingNewJobActivity.this, downloadRecordPagingData -> {
                        refreshAdapter(downloadRecordPagingData);
                    });

                });
            });
        }
    });

    private void refreshAdapter(PagingData<PointRecord> downloadRecordPagingData) {
        stakingPointLibraryAdapter.submitData(getLifecycle(), downloadRecordPagingData);
        stakingPointNameLibraryAdapter.submitData(getLifecycle(), downloadRecordPagingData);
        stakingPointLibraryAdapter.resetConfig();
        stakingPointNameLibraryAdapter.resetConfig();
        stakingPointLibraryAdapter.notifyDataSetChanged();
        stakingPointNameLibraryAdapter.notifyDataSetChanged();
        new Handler().postDelayed(() -> binding.tvPointNumber.setText(stakingPointLibraryAdapter.getItemCount() + ""), 500);
    }

    private final static String KEY_IS_SELECT = "is_selected";
    private final static String RESULT_POINT_ID = "pointId";
    private final static String RESULT_INDEX = "index";
    private final static String RESULT_NAME = "name";
    private final static String RESULT_X = "x";
    private final static String RESULT_Y = "y";
    private final static String RESULT_Z = "z";
    //    private String jobId;
    private String workId;
    private LiveData<PagingData<PointRecord>> pagingDataLiveData;

    public View getContentView() {
        binding = ActivityStakingNewJob2Binding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.staking_point_library));

        workId = IntentUtil.stringExtra(this, KEY_WORK_ID);

        if (App.instance.isThemeDark()) {
            binding.topLayout.setBackgroundColor(getColor(R.color.theme_dark_black));
            binding.tvNumber.setTextColor(getColor(R.color.white));
        }

        binding.bodyRv.setAdapter(stakingPointLibraryAdapter);
        binding.bodyNameRv.setAdapter(stakingPointNameLibraryAdapter);
        binding.bodyHsv.setScrollView(binding.headerHsv);
        binding.headerHsv.setScrollView(binding.bodyHsv);
        binding.bodyRv.addOnScrollListener(rightScrollListener);
        binding.bodyNameRv.addOnScrollListener(leftScrollListener);
        binding.bodyRv.addOnItemTouchListener(new SimpleRecyclerViewItemClickListener(binding.bodyRv, (view, position) -> {
            stakingPointLibraryAdapter.setSelItemPos(position);
            stakingPointNameLibraryAdapter.setSelItemPos(position);
        }));
        binding.bodyNameRv.addOnItemTouchListener(new SimpleRecyclerViewItemClickListener(binding.bodyNameRv, (view, position) -> {
            stakingPointLibraryAdapter.setSelItemPos(position);
            stakingPointNameLibraryAdapter.setSelItemPos(position);
        }));

        ThreadPoolUtil.execute(() -> {
//            String projectId = CachedProject.currentProjectId();
//            jobId = CachedCurrentJob.currentJobId(projectId);
            ThreadPoolUtil.executeInMain(() -> {
                pagingDataLiveData = PagingLiveData.cachedIn(PagingLiveData.getLiveData(pager), getLifecycle());
                pagingDataLiveData
                        .observe(this, downloadRecordPagingData -> {
                            refreshAdapter(downloadRecordPagingData);
                        });

            });
        });

        binding.pointsLibraryBtnConfirm.setOnClickListener(v -> {
            int row = stakingPointLibraryAdapter.getSelItemPos();
            PointRecord record = stakingPointLibraryAdapter.getSelItem();
            if (record == null) {
                ToastUtils.showShort(R.string.please_select_one_data_item);
                return;
            }
            Intent intent = new Intent();
            intent.putExtra(RESULT_POINT_ID, record.id);
            intent.putExtra(RESULT_INDEX, row);
            intent.putExtra(RESULT_NAME, record.name);
            intent.putExtra(RESULT_X, record.x);
            intent.putExtra(RESULT_Y, record.y);
            intent.putExtra(RESULT_Z, record.h);
            setResult(RESULT_OK, intent);
            finish();
        });
        binding.pointsLibraryBtnNewCreate.setVisibility(View.GONE);
        binding.pointsLibraryBtnNewCreate.setOnClickListener(v -> showPointDialog(null, getString(R.string.new_point)));

        boolean isSelect = getIntent().getBooleanExtra(KEY_IS_SELECT, true);

        if (isSelect) {
            binding.pointsLibraryBtnConfirm.setVisibility(View.VISIBLE);
            binding.pointsLibraryBtnDelete.setVisibility(View.GONE);
            binding.pointsLibraryBtnMore.setVisibility(View.GONE);
        } else {
            binding.pointsLibraryBtnConfirm.setVisibility(View.GONE);
            binding.pointsLibraryBtnDelete.setVisibility(View.VISIBLE);
            binding.pointsLibraryBtnMore.setVisibility(View.VISIBLE);
        }
        binding.etEnterNameCode.addTextChangedListener(new SimpleTextWatcher() {
            @Override
            public void afterTextChanged(Editable s) {
                filterHandler.onFilterStringChanged(s.toString());
            }
        });
        binding.pointsLibraryBtnFilter.setOnClickListener(v -> {
            PointFilterActivity.startFilter(StakingNewJobActivity.this, false,
                    filterParameter.getNeedSurvey(), filterParameter.getNeedInput(), filterParameter.getNeedControl(),
                    filterParameter.getDateStart(), filterParameter.getDateEnd(),
                    filterParameter.getNameKey(), filterParameter.getCodeKey(), filterParameter.getRemarkKey(),
                    (needSurvey, needInput, needControl, dateStart, dateEnd, nameKey, codeKey, remarksKey) -> {
                        filterParameter.setNeedSurvey(needSurvey);
                        filterParameter.setNeedInput(needInput);
                        filterParameter.setNeedControl(needControl);
                        filterParameter.setDateStart(dateStart);
                        filterParameter.setDateEnd(dateEnd);
                        filterParameter.setNameKey(nameKey);
                        filterParameter.setCodeKey(codeKey);
                        filterParameter.setRemarkKey(remarksKey);
                        ThreadPoolUtil.execute(() -> {
//                            String projectId = CachedProject.currentProjectId();
//                            jobId = CachedCurrentJob.currentJobId(projectId);
                            ThreadPoolUtil.executeInMain(() -> {
                                pagingDataLiveData.removeObservers(this);
                                pagingDataLiveData =
                                        PagingLiveData.cachedIn(PagingLiveData.getLiveData(pager), getLifecycle());
                                pagingDataLiveData.observe(this, downloadRecordPagingData -> {
                                    refreshAdapter(downloadRecordPagingData);
                                });
                            });
                        });
                    });
        });

        // 新建
        binding.pointsLibraryBtnNewCreate.setOnClickListener(v -> {
            showPointDialog(null, getString(R.string.new_point));
        });

        binding.pointsLibraryBtnDelete.setOnClickListener(v -> {
            int row = stakingPointLibraryAdapter.getSelItemPos();
            PointRecord record = stakingPointLibraryAdapter.getSelItem();
            if (record == null) {
                ToastUtils.showShort(R.string.please_select_one_data_item);
                return;
            }
            //删除所选数据
            if (StringUtil.isEmpty(record.id)) {
                return;
            }
            BcDialogExt.create().showMessage(getString(R.string.warning), getString(R.string.want_to_delete_point_library), getString(R.string.confirm), getString(R.string.cancel), (baseDialog, view) -> {
                //删除数据库中数据
                ThreadPoolUtil.execute(() -> {
                    PointDb.getInstance().deleteById(record.id);
                });
                setResult(LauncherEvent.launcher_delete_point_lib);
                return false;
            }, (baseDialog, view) -> false);
        });

        binding.pointsLibraryBtnMore.setOnClickListener(v -> {
            com.kongzue.dialogx.dialogs.CustomDialog.show(new OnBindView<com.kongzue.dialogx.dialogs.CustomDialog>(R.layout.layout_menu_more) {

                        private TextView tvImport;
                        private TextView tvExport;
                        private TextView tvEdit;

                        @Override
                        public void onBind(final com.kongzue.dialogx.dialogs.CustomDialog dialog, View v) {
                            if (App.instance.isThemeDark()) {
                                v.setBackground(getDrawable(R.drawable.round_dark));
                            }
                            tvImport = v.findViewById(R.id.tv_import);
                            tvExport = v.findViewById(R.id.tv_export);
                            tvEdit = v.findViewById(R.id.tv_edit);
                            tvExport.setVisibility(View.GONE);
                            tvImport.setOnClickListener(v12 -> {
                                showImportConfigDialog();
                                dialog.dismiss();
                            });
                            tvEdit.setOnClickListener(v1 -> {
                                int row = stakingPointLibraryAdapter.getSelItemPos();
                                PointRecord record = stakingPointLibraryAdapter.getSelItem();
                                if (record == null) {
                                    ToastUtils.showShort(R.string.please_select_one_data_item);
                                    return;
                                }
                                showPointDialog(record, getString(R.string.edit_known_point));
                                dialog.dismiss();
                            });
                        }
                    })
                    .setCancelable(false)
                    .setCancelable(true)
                    .setEnterAnimResId(R.anim.anim_dialogx_bottom_enter)
                    .setExitAnimResId(R.anim.anim_dialogx_bottom_exit)
                    .setAlignBaseViewGravity(
                            binding.pointsLibraryBtnMore,
                            Gravity.TOP | Gravity.RIGHT
                    )
                    .setBaseViewMarginLeft(ConvertUtils.dp2px(-126f))
                    .show();
        });
    }

    @Override
    protected void initData() {

    }

    private PagingSource<Integer, PointRecord> dataSourse() {
        return PointDb.getInstance().rawQueryPagingSource(StakingPointsLibraryActivitySQLiteUtils.searchSQLit(
                workId,
                filterParameter, true, binding.etEnterNameCode.getText().toString().trim()
        ));
    }

    private List<PointRecord> getAllPoint() {
        return PointDb.getInstance().rawQueryListData(StakingPointsLibraryActivitySQLiteUtils.searchSQLit(
                workId,
                filterParameter, true, binding.etEnterNameCode.getText().toString().trim()
        ));
    }

    private MaterialEditText met_name, met_code, met_x, met_y, met_z, met_remarks;
    private MDButton mPositiveAction;

    public void showPointDialog(final PointRecord pointRecord, String title) {
        MaterialDialog dialog = new MaterialDialog.Builder(this)
                .title(title)
                .customView(R.layout.dialog_new_staking_point, false)
                .onPositive((dialog1, which) -> {
                    ThreadPoolUtil.execute(() -> {
                        PointRecord record = pointRecord;
                        if (record != null) {
                            record.name = met_name.getText().toString().trim();
                            record.code = met_code.getText().toString().trim();
                            record.x = "".equals(met_x.getText().toString().trim()) ? 0 : Double.parseDouble(met_x.getText().toString().trim());
                            record.y = "".equals(met_y.getText().toString().trim()) ? 0 : Double.parseDouble(met_y.getText().toString().trim());
                            record.h = "".equals(met_z.getText().toString().trim()) ? 0 : Double.parseDouble(met_z.getText().toString().trim());
                            record.remarks = met_remarks.getText().toString().trim();
                        } else {
                            String projectId = CachedProject.currentProjectId();
//                            jobId = CachedCurrentJob.currentJobId(projectId);
                            record = new PointRecord(projectId, workId, "",
                                    PointConstants.POINT_TYPE_KNOWN_POINT,
                                    met_name.getText().toString().trim(),
                                    met_code.getText().toString().trim(),
                                    met_remarks.getText().toString().trim(),
                                    "".equals(met_x.getText().toString().trim()) ? 0 : Double.parseDouble(met_x.getText().toString().trim()),
                                    "".equals(met_y.getText().toString().trim()) ? 0 : Double.parseDouble(met_y.getText().toString().trim()),
                                    "".equals(met_z.getText().toString().trim()) ? 0 : Double.parseDouble(met_z.getText().toString().trim()),
                                    0, 0, 0,
                                    PointConstants.POINT_FORMAT_XYZ, null);
                        }
                        PointDb.getInstance().save(record);
                    });
                    CommonUtils.hideSoftInput();
                })
                .positiveText(this.getString(R.string.save))
                .negativeText(this.getString(R.string.cancel)).build();

        met_name = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_name);
        met_code = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_code);
        met_x = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_x);
        met_y = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_y);
        met_z = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_z);
        met_remarks = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_remarks);
        mPositiveAction = dialog.getActionButton(DialogAction.POSITIVE);

        initMaterialEditTextColor();

        if (pointRecord != null) {
            met_name.setText(pointRecord.name);
            met_code.setText(pointRecord.code);
            met_x.setText(Util.formatDouble2StringDotAuto(pointRecord.x));
            met_y.setText(Util.formatDouble2StringDotAuto(pointRecord.y));
            met_z.setText(Util.formatDouble2StringDotAuto(pointRecord.h));
            met_remarks.setText(pointRecord.remarks);
        }

        TextWatcher watcher = new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                String name_text = met_name.getText().toString().trim();
                String x_text = met_x.getText().toString().trim();
                String y_text = met_y.getText().toString().trim();
                String z_text = met_z.getText().toString().trim();

                if (!"".equals(name_text)
                        && !"-".equals(x_text)
                        && !"-".equals(y_text)
                        && !"-".equals(z_text)
                        && !".".equals(x_text)
                        && !".".equals(y_text)
                        && !".".equals(z_text)
                        && !"-.".equals(x_text)
                        && !"-.".equals(y_text)
                        && !"-.".equals(z_text)
                        && !("".equals(x_text) && "".equals(y_text) && "".equals(z_text))) {
                    mPositiveAction.setEnabled(true);
                } else {
                    mPositiveAction.setEnabled(false);
                }

            }
        };
        met_name.addTextChangedListener(watcher);
        met_code.addTextChangedListener(watcher);
        met_x.addTextChangedListener(watcher);
        met_y.addTextChangedListener(watcher);
        met_z.addTextChangedListener(watcher);
        met_remarks.addTextChangedListener(watcher);

        dialog.show();
        mPositiveAction.setEnabled(false);
    }

    private void initMaterialEditTextColor() {
        if (App.instance.isThemeDark()) {
            met_name.setMetTextColor(Color.WHITE);
            met_x.setMetTextColor(Color.WHITE);
            met_y.setMetTextColor(Color.WHITE);
            met_z.setMetTextColor(Color.WHITE);

            met_name.setPrimaryColor(Color.LTGRAY);
            met_x.setPrimaryColor(Color.LTGRAY);
            met_y.setPrimaryColor(Color.LTGRAY);
            met_z.setPrimaryColor(Color.LTGRAY);

            met_name.setFocusFraction(1.0f);
            met_x.setFocusFraction(1.0f);
            met_y.setFocusFraction(1.0f);
            met_z.setFocusFraction(1.0f);

            met_name.setMetHintTextColor(Color.GRAY);
            met_x.setMetHintTextColor(Color.GRAY);
            met_y.setMetHintTextColor(Color.GRAY);
            met_z.setMetHintTextColor(Color.GRAY);

            met_name.setUnderlineColor(Color.GRAY);
            met_x.setUnderlineColor(Color.GRAY);
            met_y.setUnderlineColor(Color.GRAY);
            met_z.setUnderlineColor(Color.GRAY);
        }
    }

    private void showImportConfigDialog() {
        new MaterialDialog.Builder(this)
                .title(R.string.please_choose_import_format)
                .items(new String[]{getString(R.string.excel_xls)
                        , getString(R.string.text_txt)
                        , getString(R.string.cass_dat_txt)})
                .itemsCallback(new MaterialDialog.ListCallback() {
                    @Override
                    public void onSelection(MaterialDialog dialog, View itemView, int which, CharSequence text) {
                        String[] filter = null;
                        int intentResultNumber = EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL;
                        String message = "";
                        switch (which) {
                            case 0:
                                filter = new String[]{"xls", "xlsx"};
                                intentResultNumber = EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL;
                                message = getString(R.string.import_point_excel_guide);
                                break;
                            case 1:
                                filter = new String[]{"txt"};
                                intentResultNumber = EX_FILE_PICKER_RESULT_SURVEYOR_TXT;
                                message = getString(R.string.import_point_text_guide);
                                break;
                            case 2:
                                showImportCassXYDialog();
                                return;
                        }
                        if (filter != null) {
                            final int finalIntentResultNumber = intentResultNumber;
                            final String[] finalFilter = filter;
                            new AlertDialog.Builder(StakingNewJobActivity.this)
                                    .setTitle(R.string.import_points)
                                    .setMessage(message)
                                    .setPositiveButton(R.string.overwrite_import, new DialogInterface.OnClickListener() {
                                        @Override
                                        public void onClick(DialogInterface dialog, int which) {
                                            isOverrideImport = true;
                                            selectFolder(finalIntentResultNumber, finalFilter);
                                        }
                                    })
                                    .setNegativeButton(R.string.additional_import, new DialogInterface.OnClickListener() {
                                        @Override
                                        public void onClick(DialogInterface dialog, int which) {
                                            isOverrideImport = false;
                                            selectFolder(finalIntentResultNumber, finalFilter);
                                        }
                                    })
                                    .setNeutralButton(R.string.view_sample, new DialogInterface.OnClickListener() {
                                        @Override
                                        public void onClick(DialogInterface dialog, int which) {
//                                            Intent intent = new Intent();
//                                            intent.setClass(Utils.getApp(), AgentWebActivity.class);
//                                            intent.putExtra("short_url", "android_url_import_point_sample");
//                                            startActivity(intent);
                                            AgentWebActivity.navigation2(
                                                    StakingNewJobActivity.this,
                                                    "android_url_import_point_sample");
                                        }
                                    })
                                    .show();
                        }
                    }
                }).show();
    }

    private void selectFolder(final int intentResultNumber, final String[] filter) {
        ExFilePicker exFilePicker = new ExFilePicker();
        exFilePicker.setLandscape(App.instance.isLandscape());
        exFilePicker.setShowOnlyExtensions(filter);
        exFilePicker.start(StakingNewJobActivity.this, intentResultNumber);
    }

    private boolean isOverrideImport;
    private boolean cassXY = false;//cass格式按照xy排序，否则yx排序

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (resultCode == Activity.RESULT_OK) {
            if (data != null) {
                ExFilePickerResult object = ExFilePickerResult.getFromIntent(data);
                if (object != null && object.getCount() > 0) {
                    ThreadPoolUtil.execute(() -> {
                        String projectId = SPUtils.INSTANCE.getString(SPConstants.PROJECT_ID, "");
//                        jobId = CachedCurrentJob.currentJobId(projectId);
                        try {
                            String path = object.getPath() + object.getNames().get(0);
                            String result = null;
                            switch (requestCode) {
                                case EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL:
                                    PointLibraryUtil.importStakingJobPointExcel(new File(path), projectId, workId, isOverrideImport, (status, msg) -> ToastUtils.showShort(msg));
                                    break;
                                case EX_FILE_PICKER_RESULT_SURVEYOR_TXT:
                                    PointLibraryUtil.importStakingJobPointTxt(new File(path), projectId, workId, isOverrideImport, (status, msg) -> ToastUtils.showShort(msg));
                                    break;
                                case EX_FILE_PICKER_RESULT_CASS_DAT:
                                    PointLibraryUtil.importStakingJobPointCassDat(new File(path), projectId, workId, isOverrideImport, cassXY, (status, msg) -> ToastUtils.showShort(msg));
                                    break;
                            }
                        } catch (Exception e) {
                            Log.e("StakingNewJobActivity", "File select error", e);
                        }
                    });
                }
            }
        }
    }

    private void showImportCassXYDialog() {
        cassXY = false;
        String message;
        if (cassXY) {
            message = getString(R.string.import_point_cass_xy_dat);
        } else {
            message = getString(R.string.import_point_cass_yx_dat);
        }
        new AlertDialog.Builder(StakingNewJobActivity.this)
                .setTitle(R.string.import_points)
                .setMessage(message)
                .setPositiveButton(R.string.overwrite_import, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        isOverrideImport = true;
                        selectFolder(EX_FILE_PICKER_RESULT_CASS_DAT, new String[]{"dat", "txt"});
                    }
                })
                .setNegativeButton(R.string.additional_import, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        isOverrideImport = false;
                        selectFolder(EX_FILE_PICKER_RESULT_CASS_DAT, new String[]{"dat", "txt"});
                    }
                })
                .setNeutralButton(R.string.view_sample, (dialog, which) -> AgentWebActivity.navigation2(
                        StakingNewJobActivity.this,
                        "android_url_import_point_sample"))
                .show();
    }

    protected void onCommonRightClicked(View view) {
        if (stakingPointLibraryAdapter.getItemCount() != 0) {
            new CustomDialog.Builder(this)
                    .setContent(getString(R.string.want_to_empty_point_library))
                    .setButtonConfirm(v ->
                            ThreadPoolUtil.execute(() -> {
                                stakingPointLibraryAdapter.clear(getAllPoint());
                                runOnUiThread(() -> setResult(LauncherEvent.launcher_delete_point_lib));
                            }))
                    .create().show();
        } else {
            ToastUtils.showShort(getString(R.string.no_data_clear));
        }
    }


    @Override
    public BubblePopWindow newBubblePopWindow(Context context) {
        return new BubblePopWindow(this, Arrays.asList(new BubblePopWindow.CustomMenuItem(R.drawable.icon_function_delete, getString(R.string.clear), () -> {
            if (stakingPointLibraryAdapter.getItemCount() != 0) {
                BcDialogExt.create().showMessageWithTick(getString(R.string.warning), getString(R.string.want_to_empty_point_library), getString(R.string.confirm), getString(R.string.cancel), (baseDialog, view) -> {
                    ThreadPoolUtil.execute(() -> {
                        stakingPointLibraryAdapter.clear(getAllPoint());
                        runOnUiThread(() -> setResult(LauncherEvent.launcher_delete_point_lib));
                    });
                    return false;
                }, (baseDialog, view) -> false);
            } else {
                ToastUtils.showShort(getString(R.string.no_data_clear));
            }
        })));
    }

    private static final String KEY_WORK_ID = "keyWorkId";

    public static void start(Context context, String workId) {
        Intent intent = new Intent(context, StakingNewJobActivity.class);
        intent.putExtra(KEY_IS_SELECT, false);
        intent.putExtra(KEY_WORK_ID, workId);
        context.startActivity(intent);
    }

    public static void start(FragmentActivity context, String workId, OnSelectedPointListener listener) {
        Intent intent = new Intent(context, StakingNewJobActivity.class);
        intent.putExtra(KEY_IS_SELECT, true);
        intent.putExtra(KEY_WORK_ID, workId);
        ActivityUtils.startActivityForResults(context, intent, result -> {
            if (result.getResultCode() == RESULT_OK && result.getData() != null) {
                String pointId = result.getData().getStringExtra(RESULT_POINT_ID);
                if (listener != null) {
                    listener.onSelected(pointId);
                }
            }
        });
    }

    public static void pickPoint(FragmentActivity context, ICallback callback) {
        pickPoint(context, (pointId, index, name, x, y, z) -> callback.onPicked(name, x, y, z));
    }

    public static void pickPoint(FragmentActivity context, ICallbackWithIndex callback) {
        Intent intent = new Intent(context, StakingNewJobActivity.class);
        intent.putExtra(KEY_IS_SELECT, true);
        ActivityUtils.startActivityForResults(context, intent, result -> {
            if (result.getResultCode() == RESULT_OK && result.getData() != null) {
                String pointId = result.getData().getStringExtra(RESULT_POINT_ID);
                int index = result.getData().getIntExtra(RESULT_INDEX, -1);
                String name = result.getData().getStringExtra(RESULT_NAME);
                double x = result.getData().getDoubleExtra(RESULT_X, 0);
                double y = result.getData().getDoubleExtra(RESULT_Y, 0);
                double z = result.getData().getDoubleExtra(RESULT_Z, 0);
                if (callback != null) {
                    callback.onPicked(pointId, index, name, x, y, z);
                }
            }
        });
    }

    public interface ICallback {
        void onPicked(String name, double x, double y, double z);
    }

    public interface ICallbackWithIndex {
        void onPicked(String pointId, int index, String name, double x, double y, double z);
    }

    public interface OnSelectedPointListener {
        void onSelected(String pointId);
    }

    @Override
    protected void onDestroy() {
        try {
            binding.bodyNameRv.removeOnScrollListener(leftScrollListener);
            binding.bodyRv.removeOnScrollListener(rightScrollListener);
            leftScrollListener = null;
            rightScrollListener = null;
        } catch (Exception e) {
            e.printStackTrace();
        }
        super.onDestroy();
    }

    @Override
    public boolean isHideHelpDoc() {
        return true;
    }
}