package com.project.survey.ui.instrument.setupstation.adapter

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.paging.PagingDataAdapter
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.RecyclerView
import com.bingce.data.database.PointDb
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord
import com.bingce.utils.DateUtils
import com.bingce.utils.Util
import com.project.survey.databinding.ItemStakingPointLibraryBinding
import com.project.survey.databinding.ItemStakingPointNameLibraryBinding
import com.project.survey.ui.instrument.setupstation.ControlPointsActivity

class ControlPointLibraryAdapter(diffCallback: DiffUtil.ItemCallback<ControlPointsActivity.DATA>, val context: Context, private val onlyPointName:Boolean): PagingDataAdapter<ControlPointsActivity.DATA,ControlPointLibraryAdapter.BaseVH>(diffCallback) {
    private var mSelItemPos = -1
    fun setSelItemPos(pos:Int){
        mSelItemPos = pos
        notifyDataSetChanged()
    }
    fun getSelItemPos():Int{
        return mSelItemPos
    }
    fun getSelItem():ControlPointsActivity.DATA?{
        if (mSelItemPos == -1) return null
        return getItem(mSelItemPos)
    }
    fun resetConfig(){
        mSelItemPos = -1
    }
    fun item(index:Int):ControlPointsActivity.DATA?{
        return getItem(index)
    }
    //clear全局控制点
//    fun clear(){
//        ThreadPoolUtil.execute {
//            val list = ArrayList<PointRecord>()
//            (0 until itemCount).forEach {
//                getItem(it)?.let { it1 -> list.add(it1.pointRecord) }
//            }
//            mSelItemPos = -1
//            PointDb.getInstance().delete(list)
//        }
//    }
    fun clear(pointRecords:List<PointRecord>){
        mSelItemPos = -1
        PointDb.getInstance().delete(pointRecords)
    }
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseVH {
        return if (onlyPointName){
            VH2(ItemStakingPointNameLibraryBinding.inflate(LayoutInflater.from(context),parent,false))
        }else {
            VH(
                ItemStakingPointLibraryBinding.inflate(
                    LayoutInflater.from(context),
                    parent,
                    false
                )
            )
        }
    }

//    override fun getItemCount(): Int {
//        return arrayList.size
//    }

    override fun onBindViewHolder(holder: BaseVH, position: Int) {
        getItem(position)?.let { holder.update(it) }
    }

    override fun onBindViewHolder(holder: BaseVH, position: Int, payloads: MutableList<Any>) {
        if (payloads.isEmpty()){
            onBindViewHolder(holder,position)
        }else{
            val bundle = payloads[0]
            getItem(position)?.let { holder.updatePart(it,bundle as Bundle) }
        }
    }
    inner class VH2(val vb: ItemStakingPointNameLibraryBinding): BaseVH(vb.root) {
        override fun update(data: ControlPointsActivity.DATA) {
            if (data.pointRecord == null) return
            vb.tvName.text = data.pointRecord.name
            if (mSelItemPos == absoluteAdapterPosition){
                vb.root.setBackgroundColor(Color.rgb(226, 236, 215))
            }else{
                vb.root.background = null
            }

        }
        override fun updatePart(data: ControlPointsActivity.DATA, bundle: Bundle) {
            for (key in bundle.keySet()) {
                val value = bundle.getString(key)?:""
                when (key) {
                    "name"-> vb.tvName.text = value
                    else -> {}
                }
            }
        }
    }

    inner class VH(val vb: ItemStakingPointLibraryBinding): BaseVH(vb.root) {
        override fun update(data: ControlPointsActivity.DATA) {
            if (data.pointRecord == null) return
            vb.tvCode.text = data.pointRecord.code
            vb.tvX.text = Util.formatDouble2StringDotAuto(data.pointRecord.x)
            vb.tvY.text = Util.formatDouble2StringDotAuto(data.pointRecord.y)
            vb.tvH.text = Util.formatDouble2StringDotAuto(data.pointRecord.h)
            vb.tvTime.text = DateUtils.toFull(data.pointRecord.createDate)
            vb.tvRemark.text = data.pointRecord.remarks
            if (mSelItemPos == absoluteAdapterPosition){
                vb.root.setBackgroundColor(Color.rgb(226, 236, 215))
            }else{
                vb.root.background = null
            }
        }
        override fun updatePart(data: ControlPointsActivity.DATA, bundle: Bundle) {
            for (key in bundle.keySet()) {
                val value = bundle.getString(key)?:""
                when (key) {
                    "code"-> vb.tvCode.text = value
                    "x"->vb.tvX.text = value
                    "y"->vb.tvY.text = value
                    "h"->vb.tvH.text = value
                    "time"->vb.tvTime.text = value
                    else -> {}
                }
            }
        }
    }
    abstract class BaseVH(val view: View):RecyclerView.ViewHolder(view){
        abstract fun update(data:ControlPointsActivity.DATA)
        abstract fun updatePart(data: ControlPointsActivity.DATA, bundle: Bundle)
    }
}