package com.project.survey.ui.instrument.setupstation.adapter

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.paging.PagingDataAdapter
import androidx.recyclerview.widget.DiffUtil
import androidx.recyclerview.widget.RecyclerView
import com.bingce.data.database.PointDb
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord
import com.bingce.utils.DateUtils
import com.bingce.utils.Util
import com.project.survey.databinding.ItemStakingPointLibraryBinding
import com.project.survey.databinding.ItemStakingPointNameLibraryBinding

class StakingPointLibraryAdapter(diffCallback: DiffUtil.ItemCallback<PointRecord>, val context: Context, private val onlyPointName:Boolean): PagingDataAdapter<PointRecord,StakingPointLibraryAdapter.BaseVH>(diffCallback) {
    private var mSelItemPos = -1
    fun setSelItemPos(pos:Int){
        mSelItemPos = pos
        notifyDataSetChanged()
    }
    fun getSelItemPos():Int{
        return mSelItemPos
    }
    fun resetConfig(){
        mSelItemPos = -1
    }
    fun getSelItem():PointRecord?{
        if (mSelItemPos == -1) return null
        return getItem(mSelItemPos)
    }
    fun resetSelPos(){
        mSelItemPos = -1
    }

    fun clear(pointRecords:List<PointRecord>){
        mSelItemPos = -1
        PointDb.getInstance().delete(pointRecords)
    }
    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): BaseVH {
        return if (onlyPointName){
            VH2(ItemStakingPointNameLibraryBinding.inflate(LayoutInflater.from(context),parent,false))
        }else {
            VH(
                ItemStakingPointLibraryBinding.inflate(
                    LayoutInflater.from(context),
                    parent,
                    false
                )
            )
        }
    }

//    override fun getItemCount(): Int {
//        return arrayList.size
//    }

    override fun onBindViewHolder(holder: BaseVH, position: Int) {
        getItem(position)?.let { holder.update(it) }
    }

    override fun onBindViewHolder(holder: BaseVH, position: Int, payloads: MutableList<Any>) {
        if (payloads.isEmpty()){
            onBindViewHolder(holder,position)
        }else{
            val bundle = payloads[0]
            getItem(position)?.let { holder.updatePart(it,bundle as Bundle) }
        }
    }
    inner class VH2(val vb: ItemStakingPointNameLibraryBinding): BaseVH(vb.root) {
        override fun update(pointRecord: PointRecord) {
            vb.tvName.text = pointRecord.name
            if (mSelItemPos == absoluteAdapterPosition){
                vb.root.setBackgroundColor(Color.rgb(226, 236, 215))
            }else{
                vb.root.background = null
            }
        }
        override fun updatePart(pointRecord: PointRecord, bundle: Bundle) {
            for (key in bundle.keySet()) {
                val value = bundle.getString(key)?:""
                when (key) {
                    "name"-> vb.tvName.text = value
                    else -> {}
                }
            }
        }
    }

    inner class VH(val vb: ItemStakingPointLibraryBinding): BaseVH(vb.root) {
        override fun update(pointRecord: PointRecord) {
            vb.tvCode.text = pointRecord.code
            vb.tvX.text = Util.formatDouble2StringDotAuto(pointRecord.x)
            vb.tvY.text = Util.formatDouble2StringDotAuto(pointRecord.y)
            vb.tvH.text = Util.formatDouble2StringDotAuto(pointRecord.h)
            vb.tvTime.text = DateUtils.toFull(pointRecord.createDate)
            vb.tvRemark.text = pointRecord.remarks
            if (mSelItemPos == absoluteAdapterPosition){
                vb.root.setBackgroundColor(Color.rgb(226, 236, 215))
            }else{
                vb.root.background = null
            }
        }
        override fun updatePart(pointRecord: PointRecord, bundle: Bundle) {
            for (key in bundle.keySet()) {
                val value = bundle.getString(key)?:""
                when (key) {
                    "code"-> vb.tvCode.text = value
                    "x"->vb.tvX.text = value
                    "y"->vb.tvY.text = value
                    "h"->vb.tvH.text = value
                    "time"->vb.tvTime.text = value
                    else -> {}
                }
            }
        }
    }
    abstract class BaseVH(val view: View):RecyclerView.ViewHolder(view){
        abstract fun update(pointRecord:PointRecord)
        abstract fun updatePart(pointRecord: PointRecord, bundle: Bundle)
    }
}