package com.project.survey.ui.instrument.setupstation.callback

import android.os.Bundle
import android.text.TextUtils
import androidx.recyclerview.widget.DiffUtil
import com.bingce.utils.DateUtils
import com.bingce.utils.Util
import com.project.survey.ui.instrument.setupstation.ControlPointsActivity

class ControlPointDiffCallback: DiffUtil.ItemCallback<ControlPointsActivity.DATA>() {

    override fun areItemsTheSame(oldItem: ControlPointsActivity.DATA, newItem: ControlPointsActivity.DATA): Boolean {
        if (oldItem.pointRecord == null || newItem.pointRecord == null || TextUtils.isEmpty(oldItem.pointRecord.id) || TextUtils.isEmpty(newItem.pointRecord.id)){
            return false
        }
        return oldItem.pointRecord.id == newItem.pointRecord.id
    }

    override fun areContentsTheSame(oldItem: ControlPointsActivity.DATA, newItem: ControlPointsActivity.DATA): Boolean {
        return oldItem == newItem
    }

    override fun getChangePayload(oldItem: ControlPointsActivity.DATA, newItem: ControlPointsActivity.DATA): Any? {
        val bundle = Bundle()

        if (oldItem.pointRecord.name != newItem.pointRecord.name){
            bundle.putString("name",newItem.pointRecord.name)
        }
        if (oldItem.pointRecord.code != newItem.pointRecord.code){
            bundle.putString("code",newItem.pointRecord.code)
        }
        if (oldItem.pointRecord.x != newItem.pointRecord.x){
            bundle.putString("x", Util.formatDouble2StringDotAuto(newItem.pointRecord.x))
        }
        if (oldItem.pointRecord.y != newItem.pointRecord.y){
            bundle.putString("y", Util.formatDouble2StringDotAuto(newItem.pointRecord.y))
        }
        if (oldItem.pointRecord.h != newItem.pointRecord.h){
            bundle.putString("h", Util.formatDouble2StringDotAuto(newItem.pointRecord.h))
        }
        if (oldItem.pointRecord.createDate.time != newItem.pointRecord.createDate.time){
            bundle.putString("time", DateUtils.toFull(newItem.pointRecord.createDate))
        }
        if (bundle.size() == 0) return null
        return bundle
    }
}