package com.project.survey.ui.instrument.setupstation.util;

import android.content.Context;

import androidx.annotation.WorkerThread;

import com.bingce.TimeCount;
import com.bingce.data.database.ProjectCoordinateDatabase;
import com.bingce.data.database.SurveyRecordDatabase;
import com.bingce.data.database.SyncRecordDatabase;
import com.bingce.utils.FileUtil;
import com.bingce.utils.StringUtil;
import com.hjq.permissions.Permission;
import com.hjq.permissions.XXPermissions;
import com.project.survey.R;

import java.io.File;
import java.io.IOException;

import blankj.utilcode.util.ToastUtils;

/**
 * 应用首次启动时执行的操作
 */
public class CollectLocalDataInAppStartFirstTimeUtil {
    static final String COLLECTED_FILE_NAME = "collected2";
    static final String GENERATOR_COLLECTED_FILE_FAILURE = "generator_collect_failure";

    @WorkerThread
    public static void collectData(Context activity, ILog log, TimeCount.ITimeCounter timeCounter) {
        log.log("执行数据库清除");
        //1.先清除数据库
        SurveyRecordDatabase.getInstance().clearAllTables();
        ProjectCoordinateDatabase.getInstance().clearAllTables();
        SyncRecordDatabase.getInstance().clearAllTables();

        log.log("读取本地数据");
        //2.开始收集本地数据
        collect(activity, log, timeCounter);
    }

    @WorkerThread
    private static void collect(Context appCompatActivity, ILog log, TimeCount.ITimeCounter timeCounter) {
        //1.优先尝试读取私有目录
        File appFolder = new File(appCompatActivity.getFilesDir(), "Project");
        if (!CollectAndroidDataFolderUtils.collectFromDataFolder(appCompatActivity, appFolder, log, timeCounter)) {
            if (XXPermissions.isGranted(appCompatActivity, Permission.MANAGE_EXTERNAL_STORAGE)) {
                //2.尝试读取根目录下数据--如果有权限 ，就尝试收集根目录，否则以后收集
                collectFromDisk(appCompatActivity, log, timeCounter);
            }
        }
    }

    @WorkerThread
    public static void collectFromDisk(Context appCompatActivity, ILog log, TimeCount.ITimeCounter timeCounter) {
        log.log("检测BingCe目录下数据");
        //1.检测‘BingCe’目录
        String sdPath = FileUtil.getSDPath();
        if (StringUtil.isEmpty(sdPath)) {
            log.log("读取手机根目录失败.");
            return;
        }
        File rootFolder = new File(FileUtil.getSDPath());
        if (!rootFolder.exists()) {
            log.log("读取手机根目录失败..");
            ToastUtils.showShort("读取手机根目录失败!");
            return;
        }
        //1.检测冰测目录
        File bingceFolder = new File(rootFolder, "Bingce");
        File surveyorFolder = new File(bingceFolder, "Surveyor");
        File mark = new File(surveyorFolder, COLLECTED_FILE_NAME);
        if (mark.exists() || !surveyorFolder.exists() || !MigrationBingCeFolderDataUtils.collectFromBingCeFolder(appCompatActivity, surveyorFolder, log, timeCounter)) {
            //2.检测‘测量员’目录
            File celiangyuanFolder = new File(rootFolder, appCompatActivity.getString(R.string.app_name));
            mark = new File(celiangyuanFolder, COLLECTED_FILE_NAME);
            if (!mark.exists() && celiangyuanFolder.exists()) {
                MigrationCeliangyuanFolderDataUtils.collectFromCeliangyuanFolder(appCompatActivity, celiangyuanFolder, log, timeCounter);
            }
        }
    }

    static boolean generatorCollectedFile(File folder) {
        if (folder.exists()) {
            File tagFile = new File(folder, COLLECTED_FILE_NAME);
            if (!tagFile.exists()) {
                String result = "";
                try {
                    if (!tagFile.createNewFile()) {
                        result = "create Failure";
                    }
                } catch (IOException ioException) {
                    result = "generatorCollectedFile of " + folder.getAbsolutePath() + "  " + ioException.toString();
                }
                if (!result.isEmpty()) {
                    return false;
                }
            }
        }
        return true;
    }

    public interface ILog {
        void log(String msg);
    }
}
