package com.project.survey.ui.instrument.setupstation.util;


import android.content.Context;
import android.graphics.Color;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;

import androidx.annotation.StringRes;

import com.afollestad.materialdialogs.DialogAction;
import com.afollestad.materialdialogs.MaterialDialog;
import com.afollestad.materialdialogs.internal.MDButton;
import com.bingce.data.cache.CachedCurrentJob;
import com.bingce.data.cache.CachedCurrentProject;
import com.bingce.data.cache.CachedCurrentRoad;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.PointDb;
import com.bingce.data.database.RoadDb;
import com.bingce.data.surveyor.designdata.road.RoadConstants;
import com.bingce.data.surveyor.designdata.road.RoadRecord;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.device.util.DeviceUtils;
import com.bingce.utils.DateUtils;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.ui.instrument.setupstation.ControlPointsActivity;
import com.rengwuxian.materialedittext.MaterialEditText;

import java.util.Date;

import blankj.utilcode.util.Utils;
import cn.liuyanbing.surveyor.model.base.Point;
import cn.liuyanbing.surveyor.model.base.Road;

public class ControlPointsDetailDialogUtils {
    private final Context context;

    private MDButton mPositiveAction;
    private MaterialEditText met_name, met_code, met_x, met_y, met_z, met_remarks, met_date;

    public ControlPointsDetailDialogUtils(Context context) {
        this.context = context;
    }

    public interface DataCallBack {
        default void edit(ControlPointsActivity.DATA data) {
        }

        default void add(ControlPointsActivity.DATA data) {
        }
    }

    public void showControlPointDialog(String roadId, final ControlPointsActivity.DATA data, String title, DataCallBack callBack) {
        MaterialDialog dialog = new MaterialDialog.Builder(context)
                .title(title)
                .customView(R.layout.dialog_new_control_point, false)
                .onPositive((dialog1, which) -> {
                    String name = met_name.getText().toString().trim();
                    String code = met_code.getText().toString().trim();
                    double x = "".equals(met_x.getText().toString().trim()) ? 0 : Double.parseDouble(met_x.getText().toString().trim());
                    double y = "".equals(met_y.getText().toString().trim()) ? 0 : Double.parseDouble(met_y.getText().toString().trim());
                    double z = "".equals(met_z.getText().toString().trim()) ? 0 : Double.parseDouble(met_z.getText().toString().trim());
                    String remarks = met_remarks.getText().toString().trim();
                    if (data != null) {//修改
                        ThreadPoolUtil.execute(() -> {
                            if (StringUtil.isEmpty(data.roadId)) {//全局控制点
                                PointRecord pointRecord = data.pointRecord;
                                pointRecord.name = name;
                                pointRecord.code = code;
                                pointRecord.x = x;
                                pointRecord.y = y;
                                pointRecord.h = z;
                                pointRecord.remarks = remarks;

                                PointDb.getInstance().save(pointRecord);
                                if (callBack != null) callBack.edit(data);
                            } else {//线路控制点
                                RoadRecord roadRecord = RoadDb.getInstance()
                                        .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, data.roadId));
                                Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                                if (road != null && road.getControlPointList() != null && data.index >= 0 && data.index < road.getControlPointList().size()) {
                                    road.getControlPointList().set(data.index, Point.CONTROL, name, code, x, y, z, remarks, DateUtils.toFull(new Date()));
                                    RoadDb.getInstance().save(road);
                                }
                            }
                        });
                    } else {//添加
                        ThreadPoolUtil.execute(() -> {
                            if (StringUtil.isEmpty(roadId)) {//添加全局控制点
                                String currentProjectId = CachedCurrentProject.currentProjectId();
                                PointRecord pointRecord = new PointRecord(
                                        currentProjectId, CachedCurrentJob.currentJobId(currentProjectId), CachedCurrentRoad.currentRoadId(currentProjectId),
                                        PointConstants.POINT_TYPE_GLOBAL_CONTROL_POINT,
                                        name, code, remarks,
                                        x, y, z, 0, 0, 0, PointConstants.POINT_FORMAT_XYZ,
                                        DeviceUtils.collectDeviceInfo());
                                PointDb.getInstance().save(pointRecord);
                                if (callBack != null)
                                    callBack.add(new ControlPointsActivity.DATA(pointRecord));
                            } else {
                                RoadRecord roadRecord = RoadDb.getInstance()
                                        .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
                                Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                                if (road != null && road.getControlPointList() != null) {
                                    road.getControlPointList().add(Point.CONTROL, name, code, x, y, z, remarks);
                                    RoadDb.getInstance().save(road);
                                }
                            }
                        });
                    }
                })
                .positiveText(this.getString(R.string.save))
                .negativeText(this.getString(R.string.cancel)).build();

        met_name = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_name);
        met_code = dialog.getCustomView().findViewById(R.id.met_dialog_new_point_code);
        met_x = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_x);
        met_y = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_y);
        met_z = dialog.getCustomView().findViewById(R.id.met_dialog_new_staking_point_z);
        met_remarks = dialog.getCustomView().findViewById(R.id.met_dialog_new_point_remarks);
        met_date = dialog.getCustomView().findViewById(R.id.met_dialog_new_point_date);
        mPositiveAction = dialog.getActionButton(DialogAction.POSITIVE);

        initMaterialEditTextColor();

        if (/*position >= 0 && position < allControlPoints.size()*/data != null) {
            PointRecord pointRecord = data.pointRecord;
            met_name.setText(pointRecord.name);
            met_code.setText(pointRecord.code);
            met_x.setText(Util.formatDouble2StringDotAuto(pointRecord.x));
            met_y.setText(Util.formatDouble2StringDotAuto(pointRecord.y));
            met_z.setText(Util.formatDouble2StringDotAuto(pointRecord.h));
            met_remarks.setText(pointRecord.remarks);
            met_date.setText(StringUtil.isEmpty(data.roadId) ? DateUtils.toFull(pointRecord.createDate) : data.timeString);
        } else {
            met_date.setVisibility(View.GONE);
        }
        TextWatcher watcher = new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {
            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {
            }

            @Override
            public void afterTextChanged(Editable s) {
                String name_text = met_name.getText().toString().trim();
                String x_text = met_x.getText().toString().trim();
                String y_text = met_y.getText().toString().trim();
                String z_text = met_z.getText().toString().trim();

                if (!"".equals(name_text)
                        && !"-".equals(x_text)
                        && !"-".equals(y_text)
                        && !"-".equals(z_text)
                        && !".".equals(x_text)
                        && !".".equals(y_text)
                        && !".".equals(z_text)
                        && !"-.".equals(x_text)
                        && !"-.".equals(y_text)
                        && !"-.".equals(z_text)
                        && !("".equals(x_text)
                        && "".equals(y_text)
                        && "".equals(z_text))) {
                    mPositiveAction.setEnabled(true);
                } else {
                    mPositiveAction.setEnabled(false);
                }

            }
        };
        met_name.addTextChangedListener(watcher);
        met_code.addTextChangedListener(watcher);
        met_x.addTextChangedListener(watcher);
        met_y.addTextChangedListener(watcher);
        met_z.addTextChangedListener(watcher);
        met_remarks.addTextChangedListener(watcher);

        dialog.show();
        mPositiveAction.setEnabled(false);
    }

    public void showControlPointDialog(String roadId, final ControlPointsActivity.DATA data, String title) {
        showControlPointDialog(roadId, data, title, null);
    }

    private void initMaterialEditTextColor() {
        if (App.instance.isThemeDark()) {
            met_name.setMetTextColor(Color.WHITE);
            met_code.setMetTextColor(Color.WHITE);
            met_x.setMetTextColor(Color.WHITE);
            met_y.setMetTextColor(Color.WHITE);
            met_z.setMetTextColor(Color.WHITE);
            met_remarks.setMetTextColor(Color.WHITE);
            met_date.setMetTextColor(Color.WHITE);

            met_name.setPrimaryColor(Color.LTGRAY);
            met_code.setPrimaryColor(Color.LTGRAY);
            met_x.setPrimaryColor(Color.LTGRAY);
            met_y.setPrimaryColor(Color.LTGRAY);
            met_z.setPrimaryColor(Color.LTGRAY);
            met_remarks.setPrimaryColor(Color.LTGRAY);
            met_date.setPrimaryColor(Color.LTGRAY);

            met_name.setFocusFraction(1.0f);
            met_code.setFocusFraction(1.0f);
            met_x.setFocusFraction(1.0f);
            met_y.setFocusFraction(1.0f);
            met_z.setFocusFraction(1.0f);
            met_remarks.setFocusFraction(1.0f);
            met_date.setFocusFraction(1.0f);

            met_name.setMetHintTextColor(Color.GRAY);
            met_code.setMetHintTextColor(Color.GRAY);
            met_x.setMetHintTextColor(Color.GRAY);
            met_y.setMetHintTextColor(Color.GRAY);
            met_z.setMetHintTextColor(Color.GRAY);
            met_remarks.setMetHintTextColor(Color.GRAY);
            met_date.setMetHintTextColor(Color.GRAY);

            met_name.setUnderlineColor(Color.GRAY);
            met_code.setUnderlineColor(Color.GRAY);
            met_x.setUnderlineColor(Color.GRAY);
            met_y.setUnderlineColor(Color.GRAY);
            met_z.setUnderlineColor(Color.GRAY);
            met_remarks.setUnderlineColor(Color.GRAY);
            met_date.setUnderlineColor(Color.GRAY);
        }
    }

    private String getString(@StringRes int id) {
        return context.getString(id);
    }
}
