package com.project.survey.ui.instrument.setupstation.util;

import static com.bingce.road.parameter.RoadAttributeControlPointConverter.exportPoints;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.Intent;
import android.util.Log;

import androidx.annotation.StringRes;
import androidx.appcompat.app.AlertDialog;

import com.afollestad.materialdialogs.MaterialDialog;
import com.bingce.TimeCount;
import com.bingce.data.cache.CachedCurrentJob;
import com.bingce.data.cache.CachedCurrentProject;
import com.bingce.data.cache.CachedCurrentRoad;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.PointDb;
import com.bingce.data.database.RoadDb;
import com.bingce.data.surveyor.designdata.road.RoadConstants;
import com.bingce.data.surveyor.designdata.road.RoadRecord;
import com.bingce.error.BingCeErrorCode;
import com.bingce.road.RoadIntentUtil;
import com.bingce.road.RoadRecordConvertUtils;
import com.bingce.surveyor.agentweb.AgentWebActivity;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.project.survey.App;
import com.project.survey.R;

import java.io.File;

import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;
import cn.liuyanbing.surveyor.model.base.Point;
import cn.liuyanbing.surveyor.model.base.PointArray;
import cn.liuyanbing.surveyor.model.base.Road;
import ru.bartwell.exfilepicker.ExFilePicker;
import ru.bartwell.exfilepicker.data.ExFilePickerResult;

public class ControlPointsImportConfigDialogUtils {
    private static final int EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL = 330;
    private static final int EX_FILE_PICKER_RESULT_SURVEYOR_TXT = 331;
    private static final int EX_FILE_PICKER_RESULT_CASS_DAT = 332;
    private static final int EX_FILE_PICKER_RESULT_ROADSTAR_EXCEL = 333;

    private final Activity context;
    private boolean cassXY = false;//cass格式按照xy排序，否则yx排序
    private boolean isOverrideImport;

    public ControlPointsImportConfigDialogUtils(Activity context) {
        this.context = context;
    }

   public void showImportConfigDialog() {
        new MaterialDialog.Builder(context)
                .title(R.string.please_choose_import_format)
                .items(new String[]{getString(R.string.excel_xls)
                        , getString(R.string.text_txt)
                        , getString(R.string.cass_dat_txt)
                        /* , getString(R.string.road_star_point_data_excel_xls)*/})
                .itemsCallback((dialog, itemView, which, text) -> {
                    String[] filter = null;
                    int intentResultNumber = EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL;
                    String message = "";
                    switch (which) {
                        case 0:
                            filter = new String[]{"xls", "xlsx"};
                            intentResultNumber = EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL;
                            message = getString(R.string.import_point_excel_guide);
                            break;
                        case 1:
                            filter = new String[]{"txt"};
                            intentResultNumber = EX_FILE_PICKER_RESULT_SURVEYOR_TXT;
                            message = getString(R.string.import_point_text_guide);
                            break;
                        case 2:
                            showImportCassXYDialog();
                            return;
                        case 3:
                            filter = new String[]{"xls", "xlsx"};
                            intentResultNumber = EX_FILE_PICKER_RESULT_ROADSTAR_EXCEL;
                            message = getString(R.string.import_point_road_star_excel_guide);
                            break;
                    }
                    if (filter != null) {
                        final int finalIntentResultNumber = intentResultNumber;
                        final String[] finalFilter = filter;
                        new AlertDialog.Builder(context)
                                .setTitle(R.string.import_points)
                                .setMessage(message)
                                .setPositiveButton(R.string.overwrite_import, new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        isOverrideImport = true;
                                        selectFolder(finalIntentResultNumber, finalFilter);
                                    }
                                })
                                .setNegativeButton(R.string.additional_import, new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        isOverrideImport = false;
                                        selectFolder(finalIntentResultNumber, finalFilter);
                                    }
                                })
                                .setNeutralButton(R.string.view_sample, new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
//                                        Intent intent = new Intent();
//                                        intent.setClass(Utils.getApp(), AgentWebActivity.class);
//                                        intent.putExtra("short_url", "android_url_import_point_sample");
//                                        context.startActivity(intent);
                                        AgentWebActivity.navigation2(
                                                context,
                                                "android_url_import_point_sample"
                                        );
                                    }
                                })
                                .show();
                    }
                }).show();
    }

    private void showImportCassXYDialog() {
        cassXY = false;
        String message;
        if (cassXY) {
            message = getString(R.string.import_point_cass_xy_dat);
        } else {
            message = getString(R.string.import_point_cass_yx_dat);
        }
        new AlertDialog.Builder(context)
                .setTitle(R.string.import_points)
                .setMessage(message)
                .setPositiveButton(R.string.overwrite_import, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        isOverrideImport = true;
                        selectFolder(EX_FILE_PICKER_RESULT_CASS_DAT, new String[]{"dat", "txt"});
                    }
                })
                .setNegativeButton(R.string.additional_import, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        isOverrideImport = false;
                        selectFolder(EX_FILE_PICKER_RESULT_CASS_DAT, new String[]{"dat", "txt"});
                    }
                })
                .setNeutralButton(R.string.view_sample, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
//                        Intent intent = new Intent();
//                        intent.setClass(Utils.getApp(), AgentWebActivity.class);
//                        intent.putExtra("short_url", "android_url_import_point_sample");
//                        startActivity(intent);
                        AgentWebActivity.navigation2(
                                context,
                                "android_url_import_point_sample");
                    }
                })
                .show();
    }
//    private void showImportCassXYDialog() {
//        new MaterialDialog.Builder(context)
//                .title(R.string.coordinate_order)
//                .items(new String[]{"X-Y-Z", "Y-X-Z"})
//                .itemsCallback(new MaterialDialog.ListCallback() {
//                    @Override
//                    public void onSelection(MaterialDialog dialog, View itemView, int which, CharSequence text) {
//                        String message = getString(R.string.import_point_cass_xy_dat);
//                        switch (which) {
//                            case 0:
//                                cassXY = true;
//                                message = getString(R.string.import_point_cass_xy_dat);
//                                break;
//                            case 1:
//                                cassXY = false;
//                                message = getString(R.string.import_point_cass_yx_dat);
//                                break;
//                        }
//                        new AlertDialog.Builder(context)
//                                .setTitle(R.string.import_points)
//                                .setMessage(message)
//                                .setPositiveButton(R.string.overwrite_import, new DialogInterface.OnClickListener() {
//                                    @Override
//                                    public void onClick(DialogInterface dialog, int which) {
//                                        isOverrideImport = true;
//                                        selectFolder(EX_FILE_PICKER_RESULT_CASS_DAT, new String[]{"dat", "txt"});
//                                    }
//                                })
//                                .setNegativeButton(R.string.additional_import, new DialogInterface.OnClickListener() {
//                                    @Override
//                                    public void onClick(DialogInterface dialog, int which) {
//                                        isOverrideImport = false;
//                                        selectFolder(EX_FILE_PICKER_RESULT_CASS_DAT, new String[]{"dat", "txt"});
//                                    }
//                                })
//                                .setNeutralButton(R.string.view_sample, new DialogInterface.OnClickListener() {
//                                    @Override
//                                    public void onClick(DialogInterface dialog, int which) {
////                                        Intent intent = new Intent();
////                                        intent.setClass(Utils.getApp(), AgentWebActivity.class);
////                                        intent.putExtra("short_url", "android_url_import_point_sample");
////                                        context.startActivity(intent);
//                                        AgentWebActivity.navigation2(
//                                                context, "android_url_import_point_sample");
//                                    }
//                                })
//                                .show();
//                    }
//                }).show();
//    }

    private void selectFolder(final int intentResultNumber, final String[] filter) {
        ExFilePicker exFilePicker = new ExFilePicker();
        exFilePicker.setLandscape(App.instance.isLandscape());
        exFilePicker.setShowOnlyExtensions(filter);
        exFilePicker.start(context, intentResultNumber);
    }

    public void onActivityResult(boolean isGlobal, int requestCode, int resultCode, Intent data) {
        if (resultCode == Activity.RESULT_OK) {
            if (data != null) {
                ExFilePickerResult object = ExFilePickerResult.getFromIntent(data);
                if (object != null && object.getCount() > 0) {
                    try {
                        String path = object.getPath() + object.getNames().get(0);
                        switch (requestCode) {
                            case EX_FILE_PICKER_RESULT_SURVEYOR_EXCEL: {
                                ThreadPoolUtil.execute(() -> {
                                    String projectId = CachedCurrentProject.currentProjectId();
                                    if (isGlobal) {
                                        String jobId = CachedCurrentJob.currentJobId(projectId);
                                        if (isOverrideImport) {
                                            //删除之前控制点
                                            PointDb.getInstance().deleteControlPoints(projectId);
                                        }
                                        _6GlobalControlPointCollectUtils.importGlobalControlPointExcel(projectId, jobId,
                                                new File(path),
                                                TimeCount.timeCounter());
                                    } else {
                                        String roadId = RoadIntentUtil.getRoadIdFromIntent(context.getIntent());
                                        if (StringUtil.isEmpty(roadId)) {
                                            roadId = CachedCurrentRoad.currentRoadId(projectId);
                                        }
                                        RoadRecord roadRecord = RoadDb.getInstance()
                                                .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
                                        if (roadRecord == null) {
                                            BingCeErrorCode.toast(BingCeErrorCode.NO_FOUND_ROAD_BY_ID);
                                            return;
                                        }
                                        PointArray pointArray = new PointArray();
                                        _6GlobalControlPointCollectUtils.Result result = _6GlobalControlPointCollectUtils.importGlobalControlPointExcel(new File(path), pointArray);
                                        if (isOverrideImport) {
                                            roadRecord.controlPointJson = exportPoints(pointArray);
                                        } else {
                                            Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                                            if (road != null) {
                                                for (int index = 0; index < pointArray.size(); index++) {
                                                    Point point = pointArray.get(index);
                                                    road.getControlPointList().add(point.getCategory(), point.getName(), point.getCode(),
                                                            point.getX(), point.getY(), point.getZ(), point.getRemarks(), point.getTime());
                                                }
                                            }
                                            RoadRecordConvertUtils.updateRecord(roadRecord, road);
                                        }
                                        RoadDb.getInstance().save(roadRecord);
                                        ToastUtils.showShort(result.describe);
                                    }
                                });
                            }
                            break;
                            case EX_FILE_PICKER_RESULT_SURVEYOR_TXT:
                                ThreadPoolUtil.execute(() -> {
                                    String projectId = CachedCurrentProject.currentProjectId();
                                    if (isGlobal) {
                                        String jobId = CachedCurrentJob.currentJobId(projectId);
                                        if (isOverrideImport) {
                                            //删除之前控制点
                                            PointDb.getInstance().deleteControlPoints(projectId);
                                        }
                                        _6GlobalControlPointCollectUtils.importStakingJobPointTxt(new File(path), projectId, jobId,

                                                TimeCount.timeCounter());
                                    } else {
                                        String roadId = RoadIntentUtil.getRoadIdFromIntent(context.getIntent());
                                        if (StringUtil.isEmpty(roadId)) {
                                            roadId = CachedCurrentRoad.currentRoadId(projectId);
                                        }
                                        RoadRecord roadRecord = RoadDb.getInstance()
                                                .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
                                        if (roadRecord == null) {
                                            BingCeErrorCode.toast(BingCeErrorCode.NO_FOUND_ROAD_BY_ID);
                                            return;
                                        }
                                        PointArray pointArray = new PointArray();
                                        _6GlobalControlPointCollectUtils.Result result = _6GlobalControlPointCollectUtils.importStakingJobPointTxt(new File(path), pointArray);
                                        if (isOverrideImport) {
                                            roadRecord.controlPointJson = exportPoints(pointArray);
                                        } else {
                                            Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                                            if (road != null) {
                                                for (int index = 0; index < pointArray.size(); index++) {
                                                    Point point = pointArray.get(index);
                                                    road.getControlPointList().add(point.getCategory(), point.getName(), point.getCode(),
                                                            point.getX(), point.getY(), point.getZ(), point.getRemarks(), point.getTime());
                                                }
                                            }
                                            RoadRecordConvertUtils.updateRecord(roadRecord, road);
                                        }
                                        RoadDb.getInstance().save(roadRecord);
                                        ToastUtils.showShort(result.describe);
                                    }
                                });
                                break;
                            case EX_FILE_PICKER_RESULT_CASS_DAT:
                                ThreadPoolUtil.execute(() -> {
                                    String projectId = CachedCurrentProject.currentProjectId();
                                    if (isGlobal) {
                                        String jobId = CachedCurrentJob.currentJobId(projectId);
                                        if (isOverrideImport) {
                                            //删除之前控制点
                                            PointDb.getInstance().deleteControlPoints(projectId);
                                        }
                                        _6GlobalControlPointCollectUtils.importStakingJobPointCassDat(new File(path), projectId, jobId,

                                                TimeCount.timeCounter(), false);
                                    } else {
                                        String roadId = RoadIntentUtil.getRoadIdFromIntent(context.getIntent());
                                        if (StringUtil.isEmpty(roadId)) {
                                            roadId = CachedCurrentRoad.currentRoadId(projectId);
                                        }
                                        RoadRecord roadRecord = RoadDb.getInstance()
                                                .rawQueryData(DBQueryConstant.findById(RoadConstants.DB_NAME, roadId));
                                        if (roadRecord == null) {
                                            BingCeErrorCode.toast(BingCeErrorCode.NO_FOUND_ROAD_BY_ID);
                                            return;
                                        }
                                        PointArray pointArray = new PointArray();
                                        _6GlobalControlPointCollectUtils.Result result = _6GlobalControlPointCollectUtils.importStakingJobPointCassDat(new File(path), pointArray, false);
                                        if (isOverrideImport) {
                                            roadRecord.controlPointJson = exportPoints(pointArray);
                                        } else {
                                            Road road = RoadDb.getInstance().roadConverter().convert(roadRecord);
                                            if (road != null) {
                                                for (int index = 0; index < pointArray.size(); index++) {
                                                    Point point = pointArray.get(index);
                                                    road.getControlPointList().add(point.getCategory(), point.getName(), point.getCode(),
                                                            point.getX(), point.getY(), point.getZ(), point.getRemarks(), point.getTime());
                                                }
                                            }
                                            RoadRecordConvertUtils.updateRecord(roadRecord, road);
                                        }
                                        RoadDb.getInstance().save(roadRecord);
                                        ToastUtils.showShort(result.describe);
                                    }
                                });
                                break;
//                            case EX_FILE_PICKER_RESULT_ROADSTAR_EXCEL:
//                                String result4 = isGlobal
//                                        ? PointLibraryUtil.importGlobalControlPointRoadStarExcel(new File(path), isOverrideImport)
//                                        : ControlPointUtil.importControlPointRoadStarExcel(new File(path), road, isOverrideImport);
//                                ToastUtils.showLong(result4);
//                                break;
                        }
//                        updateRecyclerView();
//                        isPrjChanged = true;
                    } catch (Exception e) {
                        Log.e(ControlPointsImportConfigDialogUtils.class.getName(), "File select error", e);
                    }
                }
            }
        }
    }

    private String getString(@StringRes int id) {
        return context.getString(id);
    }
}
