package com.project.survey.ui.instrument.setupstation.util;

import android.content.Context;
import android.content.SharedPreferences;
import android.text.TextUtils;

import androidx.annotation.WorkerThread;

import com.bingce.TimeCount;
import com.bingce.constants.FileNameConstants;
import com.bingce.data.database.JobDb;
import com.bingce.data.database.KeyValueDb;
import com.bingce.data.database.PointDb;
import com.bingce.data.surveyor.designdata.job.JobConstants;
import com.bingce.data.surveyor.designdata.job.JobRecord;
import com.bingce.data.surveyor.designdata.kv.KeyValueConstants;
import com.bingce.data.surveyor.designdata.kv.KeyValueRecord;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.log.BCLog;
import com.bingce.poi.excel.ExcelReadHelper;
import com.bingce.utils.AppUtil;
import com.bingce.utils.StringUtil;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonParser;
import com.project.survey.R;
import com.project.survey.ui.instrument.setupstation.bean.StakingJob;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import blankj.utilcode.util.StringUtils;
import cn.liuyanbing.surveyor.model.base.Point;
import cn.liuyanbing.surveyor.model.base.PointArray;

public class JobDbUtils {
    @WorkerThread
    static void collect(Context context, String projectId, File projectFolder, CollectLocalDataInAppStartFirstTimeUtil.ILog log, TimeCount.ITimeCounter time) {
        if (StringUtil.isEmpty(projectId) || projectFolder == null || !projectFolder.exists() || projectFolder.isFile()) {
            return;
        }
        log.log("读取面积测量作业");
        log.log("读取点测量作业");
        //收集点测量作业
        collectPointSurveyJob(projectId, projectFolder, time);
        log.log("读取已知点放样作业");
        //收集已知点放样作业
        collectPointStakingJob(projectId, projectFolder, time);
        log.log("读取断面测量作业");
    }

    /*
     * 收集项目下的点测量作业
     */
    @WorkerThread
    private static void collectPointSurveyJob(String projectId, File projectFolder, TimeCount.ITimeCounter time) {
        String fileName = FileNameConstants.POINT_SURVEYOR;
        collectXJob(projectId, projectFolder, FileNameConstants.POINT_SURVEYOR_JOB_PREFIX, fileName, time);
    }

    @WorkerThread
    private static void collectPointStakingJob(String projectId, File projectFolder, TimeCount.ITimeCounter timeCounter) {
        File stakingFile = new File(projectFolder, FileNameConstants.CONFIG_FILE_KNOWN_POINT_STAKING);
        if (!stakingFile.exists()) {
            stakingFile = new File(projectFolder, "放样点库.xls");
        }
        if (!stakingFile.exists()) {
            return;
        }
        List<StakingJob> stakingJobs = new ArrayList<>();
        String error = readStakingJob(stakingFile, stakingJobs);
        BCLog.log(log -> log.e("collect_job", projectId + "__" + error));
        for (StakingJob job : stakingJobs) {
            String stakingJobName = job.getName();
            JobRecord record = JobDb.getInstance().rawQueryData(
                    JobConstants.findByProjectAndName(projectId, FileNameConstants.POINT_STAKE_JOB_PREFIX + stakingJobName));
            if (record != null) {
                BCLog.log(log -> log.d("collect_warning", "stakingJob 重复收集-" + stakingJobName));
                continue;
            }
            JobRecord jobRecord = new JobRecord(FileNameConstants.POINT_STAKE_JOB_PREFIX + stakingJobName, projectId, timeCounter.time());
            JobDb.getInstance().save(jobRecord);
            String jobId = jobRecord.id;
            //存储放样任务设置点坐标
            PointArray pointArray = job.getPointList();
            if (pointArray == null || pointArray.isEmpty()) {
                continue;
            }
            List<PointRecord> pointRecords = new ArrayList<>();
            for (int index = 0; index < pointArray.size(); index++) {
                Point point = pointArray.get(index);
                PointRecord pointRecord = new PointRecord(projectId, jobId, "",
                        PointConstants.POINT_TYPE_KNOWN_POINT,
                        point.getName(), point.getCode(), point.getRemarks(),
                        point.getX(), point.getY(), point.getZ(),
                        0, 0, 0,
                        PointConstants.POINT_FORMAT_XYZ, null);
                pointRecord.createDate = new Date(timeCounter.time());
                pointRecords.add(pointRecord);
            }
            PointDb.getInstance().save(pointRecords);
        }
    }

    private static String readStakingJob(File stakingFile, List<StakingJob> stakingJobs) {
        String result = "";
        int line = 0;
        try {
            List<List<List<String>>> importResult = ExcelReadHelper.excelReadBySheet(stakingFile, ExcelReadHelper.FIRST_ROW_NOT_IGNORE);
            if (importResult == null) return "";
            for (int i = 0; i < importResult.size(); i++) {
                line = 0;
                List<List<String>> sheetResult = importResult.get(i);
                if (sheetResult == null || sheetResult.isEmpty()) {
                    if (AppUtil.isZhCN()) {
                        return stakingFile.getAbsolutePath() + "，导入失败，原因：导入excel为空";
                    } else {
                        result = stakingFile.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed);
                    }
                }
                String sheetName = sheetResult.get(0).isEmpty() ? "default" : sheetResult.get(0).get(0);
                StakingJob stakeOutJob = new StakingJob(sheetName);
                for (int j = 1; j < sheetResult.size(); j++) {
                    List<String> row = sheetResult.get(j);

                    line = j + 1;
                    if (row.isEmpty() || ExcelReadHelper.isRowEmpty(row)) {
                        continue;
                    }
                    if (row.size() != 8) {
                        if (AppUtil.isZhCN()) {
                            result = stakingFile.getAbsolutePath() + "/" + sheetName + "，导入失败，原因：第" + line + "行，元素个数必须为8";
                        } else {
                            result = stakingFile.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed);
                        }
                        break;
                    }
                    stakeOutJob.getPointList().add(Integer.parseInt(row.get(0)), row.get(1), row.get(2), Double.parseDouble(row.get(3))
                            , Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)), row.get(6), row.get(7));
                }
                stakingJobs.add(stakeOutJob);
            }
            if ("".equals(result)) {
                result = stakingFile.getAbsolutePath() + "," + "," + StringUtils.getString(R.string.import_success);
            }
        } catch (NumberFormatException ex) {
            if (AppUtil.isZhCN()) {
                result = stakingFile.getAbsolutePath() + "，导入失败，原因：第" + line + "行，发现非数字字符串";
            } else {
                result = stakingFile.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }

        return result;
    }

    /**
     * 收集面积测量任务和点测量任务
     */
    @WorkerThread
    static void collectXJob(String projectId, File projectFolder, String prefix, String jsonArrayFileName, TimeCount.ITimeCounter time) {
        if (projectId == null || projectFolder == null || !projectFolder.exists()) {
            return;
        }
        JsonStringArray jsonStringArray = readListStringJson(projectFolder, jsonArrayFileName);
        if (jsonStringArray != null && jsonStringArray.getStrings() != null) { //1.尝试读取「.json」文件
            List<String> surveyJobList = jsonStringArray.getStrings();
            for (String jobString : surveyJobList) {
                String jobName = prefix + jobString;
                JobRecord record = JobDb.getInstance().rawQueryData(
                        JobConstants.findByProjectAndName(projectId, jobName));
                if (record != null) {
                    BCLog.log(log -> log.d("collect_warning", "测量任务 重复收集-" + jobName + " in " + projectId));
                    continue;
                }
                JobRecord jobRecord = new JobRecord(jobName, projectId, time.time());
                JobDb.getInstance().save(jobRecord);
            }
        } else { //2.尝试读取「.job」文件
            //todo 暂时放弃
        }
    }

    /**
     * 读取面积测量、点测量任务配置文件
     */
    private static JsonStringArray readListStringJson(File prjFolder, String fileName) {
        if (!new File(prjFolder, fileName + "." + FileNameConstants.JSON_SUFFIX).exists()) {//应用自有目录下文件是否存在
            return null;
        } else {
            try {
                return JsonStringArray.read(prjFolder, fileName, FileNameConstants.JSON_SUFFIX);
            } catch (Exception ex) {
                File target = new File(prjFolder, fileName + FileNameConstants.JSON_SUFFIX);
                BCLog.log(log -> log.e("Read Config", "read " + target.getAbsolutePath() + " failure!" + "\n" + ex.toString()));
            }
        }

        return null;
    }

    /**
     * json转换成列表
     */
    //Json数组字符串转换为对应的列表
    private static <T> List<T> getDataList(Context context, String tag, Class<T> cls) {
        List<T> list = new ArrayList<>();
        SharedPreferences preferences = context.getSharedPreferences("bingce_celiangyuan", Context.MODE_PRIVATE);
        String json = preferences.getString(tag, "");
        if (!TextUtils.isEmpty(json)) {
            Gson gson = new Gson();
            JsonArray array = new JsonParser().parse(json).getAsJsonArray();
            for (JsonElement elem : array) {
                try {
                    list.add(gson.fromJson(elem, cls));
                } catch (Exception ignore) {
                }
            }
        }
        return list;
    }

    @WorkerThread
    public static String createDefaultJob(Context context, String projectId, long time) {
        JobRecord jobRecord = new JobRecord("默认作业", projectId, time);
        JobDb.getInstance().save(jobRecord);

        KeyValueRecord defaultJobConfig = new KeyValueRecord(KeyValueConstants.keyOfProjectCurrentJob(projectId), jobRecord.id);
        KeyValueDb.getInstance().save(defaultJobConfig);
        BCLog.Launcher.log(() -> "为项目[" + projectId + "]创建默认作业[" + jobRecord.id + "]");

        return jobRecord.id;
    }
}
