package com.project.survey.ui.instrument.setupstation.util;

import android.content.Context;

import androidx.annotation.NonNull;
import androidx.annotation.WorkerThread;

import com.bingce.TimeCount;
import com.bingce.coordlib.model.CoordinateSystem;
import com.bingce.data.database.KeyValueDb;
import com.bingce.data.database.ProjectDb;
import com.bingce.data.surveyor.designdata.kv.KeyValueConstants;
import com.bingce.data.surveyor.designdata.kv.KeyValueRecord;
import com.bingce.data.surveyor.designdata.project.ProjectRecord;
import com.bingce.utils.StringUtil;

import java.io.File;

public class ProjectDbUtils {
    @WorkerThread
    static boolean collect(@NonNull Context context, @NonNull File projectFolder, CollectLocalDataInAppStartFirstTimeUtil.ILog log, TimeCount.ITimeCounter timeCounter) {
        if (!projectFolder.exists() || projectFolder.isFile()) {
            return false;
        }
        log.log("读取[" + projectFolder.getName() + "]项目数据");
        //收集项目坐标系
        CoordinateSystem coordinateSystem = _3CoordinatorCollectUtils.collectCoordinatorOfProject(projectFolder);
        //创建项目记录
        ProjectRecord projectRecord = new ProjectRecord(projectFolder.getName(), coordinateSystem, timeCounter.time());
        String projectId = projectRecord.id;
        //收集道路信息
        File[] folders = projectFolder.listFiles();
        if (folders != null) {
            for (File folder : folders) {
                String id = RoadDbUtils.collectRoad(projectId, folder, log, timeCounter.time());
                if (StringUtil.isEmpty(id)) {
                    continue;
                }
                projectRecord.roadSequence.add(id);
            }
        }
        //检测是否有线路
        if (projectRecord.roadSequence.isEmpty()) {
            String defaultRoadId = RoadDbUtils.createDefaultRoad(context, projectId);
            projectRecord.roadSequence.add(defaultRoadId);
        }
        KeyValueRecord defaultRoadConfig = new KeyValueRecord(KeyValueConstants.keyOfProjectCurrentRoad(projectId), projectRecord.roadSequence.get(0));
        KeyValueDb.getInstance().save(defaultRoadConfig);
        //需要在更新过线路顺序后再存储project
        ProjectDb.getInstance().save(projectRecord);
        //收集作业
        //创建默认作业，用于存储各种测量信息
        String defaultJobId = JobDbUtils.createDefaultJob(context, projectId, timeCounter.time());
        JobDbUtils.collect(context, projectId, projectFolder, log, timeCounter);
        //收集全局控制点
        _6GlobalControlPointCollectUtils.collect(projectId, defaultJobId, projectFolder, log, timeCounter);
        return true;
    }
}
