package com.project.survey.ui.instrument.setupstation.util;

import android.content.Context;
import android.util.Log;

import androidx.annotation.WorkerThread;

import com.bingce.constants.FileNameConstants;
import com.project.survey.BuildConfig;
import com.bingce.data.database.KeyValueDb;
import com.bingce.data.database.RoadDb;
import com.bingce.data.surveyor.designdata.kv.KeyValueConstants;
import com.bingce.data.surveyor.designdata.kv.KeyValueRecord;
import com.bingce.data.surveyor.designdata.road.RoadRecord;
import com.bingce.log.BCLog;
import com.bingce.road.RoadRecordConvertUtils;
import com.bingce.road.xport.RoadAllUtil;
import com.bingce.utils.IdUtils;

import java.io.File;

import com.project.survey.R;
import cn.liuyanbing.surveyor.model.base.Road;

public class RoadDbUtils {
    /**
     * 收集道路数据
     *
     * @return 成功返回RoadRecord实例（已经被存储到数据库中）的id，失败返回null
     */
    @WorkerThread
    static String collectRoad(String projectId, File roadFolder, CollectLocalDataInAppStartFirstTimeUtil.ILog log, long time) {
        //1.检测是否是道路数据目录
        if (!RoadAllUtil.isRoadFolder(roadFolder)) {
            return null;
        }
        log.log("开始读取道路数据[" + roadFolder.getName() + "]");
        if (BuildConfig.DEBUG) {
            Log.d("__test_develop", "收集Road" + roadFolder.getName() + ":    ");
        }
        //2.优化使用json文件导入道路数据
        File jsonFile = new File(roadFolder, FileNameConstants.DEFAULT_ROAD_DATA_FILE_NAME);
        Road road = new Road(roadFolder.getName());
        if (!jsonFile.exists() || !road.read(jsonFile.getAbsolutePath())) {
            if (BuildConfig.DEBUG) {
                Log.d("__test_develop", "收集Road" + roadFolder.getName() + ":    读取road.json失败");
            }
            //如果json文件导入失败，尝试读取excel或txt
            RoadAllUtil.importRoadArgs(roadFolder, road, false);
        }
        RoadRecord roadRecord = RoadRecordConvertUtils.convert2Record(road, projectId, time);
        //检测id是否是uuid
        //检测UUID是否合法----之前使用id是根据时间创建的，现在需要将其转换为uuid
        int deprecatedIdSize = "20220707092909073544".length();
        if (roadRecord.id.length() <= deprecatedIdSize) {
            roadRecord.id = IdUtils.getUUID();
        }
        RoadDb.getInstance().save(roadRecord);
        return roadRecord.id;
    }

    @WorkerThread
    public static String createDefaultRoad(Context context, String projectId) {
        //设置默认线路
        RoadRecord roadRecord = RoadDb.getInstance().createNewInstance(projectId, context.getString(R.string.default_road));
        KeyValueRecord defaultRoadConfig = new KeyValueRecord(KeyValueConstants.keyOfProjectCurrentRoad(projectId), roadRecord.id);
        KeyValueDb.getInstance().save(defaultRoadConfig);
        BCLog.Launcher.log(() ->"为项目[" + projectId + "]创建默认线路[" + roadRecord.id + "]");
        return roadRecord.id;
    }
}
