package com.project.survey.ui.instrument.setupstation.util;

import static com.bingce.data.database.DBQueryConstant.NOT_DELETED;
import static com.bingce.data.database.DBQueryConstant.WHERE;

import android.text.TextUtils;

import androidx.annotation.NonNull;
import androidx.sqlite.db.SimpleSQLiteQuery;
import androidx.sqlite.db.SupportSQLiteQuery;

import com.bingce.data.BingCeDbConstant;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.utils.RoomFilterBuilder;
import com.bingce.utils.StringUtil;
import com.bingce.viewmodel.AbstractCoordinatePointsLibraryViewModelKt.FilterParameter;


public class StakingPointsLibraryActivitySQLiteUtils {
    @NonNull
    static SimpleSQLiteQuery getSimpleSQLiteQuery(FilterParameter parameter, StringBuilder stringBuilder, String jobOrProjectId, int pageIndex, int pageSize, boolean stakingFlag, String nameOrCodeStr) {
        if (parameter == null) {
            return new SimpleSQLiteQuery(stringBuilder.toString());
        }
        stringBuilder.append(WHERE).append(NOT_DELETED);
        boolean filtered = stakingFlag ?
                RoomFilterBuilder.appendEqualsFilter(true, stringBuilder, BingCeDbConstant.DB_KEY_JOB_ID, jobOrProjectId):
                RoomFilterBuilder.appendEqualsFilter(true,stringBuilder,BingCeDbConstant.DB_KEY_PROJECT_ID,jobOrProjectId);
        filtered = stakingFlag? PointConstants.knownPoint(filtered, stringBuilder) : PointConstants.globalControlPoint(filtered,stringBuilder);

        if (!StringUtil.isEmpty(parameter.getNameKey())) {
            filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, PointConstants.COL_NAME, parameter.getNameKey());
        }
        if (!StringUtil.isEmpty(parameter.getCodeKey())) {
            filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, PointConstants.COL_CODE, parameter.getCodeKey());
        }
        if (!StringUtil.isEmpty(parameter.getRemarkKey())) {
            filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, BingCeDbConstant.DB_KEY_REMARKS, parameter.getRemarkKey());
        }
        if (!TextUtils.isEmpty(nameOrCodeStr)){
            stringBuilder.append(" AND ");
            stringBuilder.append(" (");
            stringBuilder.append(RoomFilterBuilder.filterLike(PointConstants.COL_NAME, nameOrCodeStr));
            RoomFilterBuilder.appendOrLikesFilter(filtered,stringBuilder,PointConstants.COL_CODE,nameOrCodeStr);
            stringBuilder.append(") ");
        }
        if (parameter.getDateStart() != null && parameter.getDateEnd() != null) {
            filtered = RoomFilterBuilder.appendBetweenDate(filtered, stringBuilder, BingCeDbConstant.DB_KEY_CREATED_DATE,
                    parameter.getDateStart(), parameter.getDateEnd());
        } else if (parameter.getDateStart() != null) {
            filtered = RoomFilterBuilder.appendDateFilter(filtered, stringBuilder, parameter.getDateStart(), BingCeDbConstant.DB_KEY_CREATED_DATE, true);
        } else if (parameter.getDateEnd() != null) {
            filtered = RoomFilterBuilder.appendDateFilter(filtered, stringBuilder, parameter.getDateEnd(), BingCeDbConstant.DB_KEY_CREATED_DATE, false);
        }

        RoomFilterBuilder.appendDateOrder(stringBuilder, BingCeDbConstant.DB_KEY_CREATED_DATE, false);
        return new SimpleSQLiteQuery(stringBuilder.toString());
    }


    public static SupportSQLiteQuery countSQLit(String jobId, FilterParameter parameter, boolean stakingFlag) {
        StringBuilder stringBuilder = new StringBuilder(DBQueryConstant.COUNT_FROM)
                .append(PointConstants.DB_NAME);
        return getSimpleSQLiteQuery(parameter, stringBuilder, jobId, -1, -1,stakingFlag,null);
    }

    public static SupportSQLiteQuery searchSQLit(String jobId, FilterParameter parameter, int pageIndex, int pageSize, boolean stakingFlag) {
        StringBuilder stringBuilder = new StringBuilder(DBQueryConstant.SELECT_FROM)
                .append(PointConstants.DB_NAME);
        return getSimpleSQLiteQuery(parameter, stringBuilder, jobId, pageIndex, pageSize,stakingFlag,null);
    }

    public static SupportSQLiteQuery searchSQLit(String jobOrProjectId, FilterParameter parameter, boolean stakingFlag, String nameOrCode){
        StringBuilder stringBuilder = new StringBuilder(DBQueryConstant.SELECT_FROM)
                .append(PointConstants.DB_NAME);
        return getSimpleSQLiteQuery(parameter, stringBuilder, jobOrProjectId, -1, -1,stakingFlag,nameOrCode);
    }
}
