package com.project.survey.ui.instrument.setupstation.util;


import androidx.annotation.WorkerThread;

import com.bingce.TimeCount;
import com.bingce.constants.FileNameConstants;
import com.bingce.data.database.PointDb;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.poi.excel.ExcelReadHelper;
import com.bingce.utils.AppUtil;
import com.project.survey.R;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import blankj.utilcode.util.StringUtils;
import blankj.utilcode.util.ToastUtils;
import cn.liuyanbing.surveyor.model.base.Point;
import cn.liuyanbing.surveyor.model.base.PointArray;

public class _6GlobalControlPointCollectUtils {
    static void collect(String projectId, String defaultJobId, File projectFolder, CollectLocalDataInAppStartFirstTimeUtil.ILog log, TimeCount.ITimeCounter timeCounter) {
        File globalFile = new File(projectFolder, FileNameConstants.CONFIG_FILE_GLOBAL_CONTROL_POINT);
        if (!globalFile.exists()) {
            globalFile = new File(projectFolder, "全局控制点.xls");
        }

        if (!globalFile.exists()) {
            return;
        }
        log.log("读取全局控制点数据");
        importGlobalControlPointExcel(projectId, defaultJobId, globalFile, timeCounter);
//        PointArray pointArray = new PointArray();
//        String error = importGlobalControlPointExcel(globalFile, pointArray);
//        if (!StringUtil.isEmpty(error)) {
//            return;
//        }
//        int size = pointArray.size();
//        List<PointRecord> pointRecords = new ArrayList<>();
//        for (int index = 0; index < size; index++) {
//            Point point = pointArray.get(index);
//            PointRecord pointRecord = new PointRecord(projectId, defaultJobId, "",
//                    PointConstants.POINT_TYPE_GLOBAL_CONTROL_POINT,
//                    point.getName(), point.getCode(), point.getRemarks(),
//                    point.getX(), point.getY(), point.getZ(),
//                    0, 0, 0,
//                    PointConstants.POINT_FORMAT_XYZ, null);
//            pointRecord.createDate = new Date(timeCounter.time());
//            pointRecords.add(pointRecord);
//        }
//        PointDb.getInstance().save(pointRecords);
    }

    public static void importGlobalControlPointExcel(String projectId, String defaultJobId, File globalFile, TimeCount.ITimeCounter timeCounter) {
        PointArray pointArray = new PointArray();
        Result result = importGlobalControlPointExcel(globalFile, pointArray);
        if (!pointArray.isEmpty()) {
            int size = pointArray.size();
            List<PointRecord> pointRecords = new ArrayList<>();
            for (int index = 0; index < size; index++) {
                Point point = pointArray.get(index);
                PointRecord pointRecord = new PointRecord(projectId, defaultJobId, "",
                        PointConstants.POINT_TYPE_GLOBAL_CONTROL_POINT,
                        point.getName(), point.getCode(), point.getRemarks(),
                        point.getX(), point.getY(), point.getZ(),
                        0, 0, 0,
                        PointConstants.POINT_FORMAT_XYZ, null);
                pointRecord.createDate = new Date(timeCounter.time());
                pointRecords.add(pointRecord);
            }
            PointDb.getInstance().save(pointRecords);
        }
        if (!result.isSuccess()) {
            ToastUtils.showShort(result.describe);
        }
    }

    public static Result importGlobalControlPointExcel(File file, PointArray pointArray) {
        int line = 0;
        try {
            List<List<String>> importResult = ExcelReadHelper.excelRead(file, ExcelReadHelper.FIRST_ROW_NOT_IGNORE);
            if (importResult == null || importResult.isEmpty()) {
                if (AppUtil.isZhCN()) {
                    return Result.failure(file.getAbsolutePath() + "，导入失败，原因：导入excel为空");
                } else {
                    return Result.failure(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed));
                }
            }
            for (int i = 0; i < importResult.size(); i++) {
                List<String> row = importResult.get(i);

                line = i + 1;
                if (row.isEmpty() || ExcelReadHelper.isRowEmpty(row) || row.size() < 3) {
                    continue;
                }
                if (row.size() == 3) {
                    pointArray.add(Point.CONTROL, "", Double.parseDouble(row.get(0))
                            , Double.parseDouble(row.get(1)), Double.parseDouble(row.get(2)), "");
                } else if (row.size() == 4) {
                    pointArray.add(Point.CONTROL, row.get(0), Double.parseDouble(row.get(1))
                            , Double.parseDouble(row.get(2)), Double.parseDouble(row.get(3)), "");
                } else if (row.size() == 5) {
                    pointArray.add(Point.CONTROL, row.get(0), Double.parseDouble(row.get(1))
                            , Double.parseDouble(row.get(2)), Double.parseDouble(row.get(3)), row.get(4));
                } else if (row.size() == 6) {
                    pointArray.add(Point.CONTROL, row.get(0), row.get(1), Double.parseDouble(row.get(2))
                            , Double.parseDouble(row.get(3)), Double.parseDouble(row.get(4)), row.get(5));
                } else {
                    pointArray.add(Point.CONTROL, row.get(1), row.get(2), Double.parseDouble(row.get(3))
                            , Double.parseDouble(row.get(4)), Double.parseDouble(row.get(5)), row.get(6), row.get(7));
                }
            }
        } catch (NumberFormatException ex) {
            pointArray.clear();
            if (AppUtil.isZhCN()) {
                return Result.failure(file.getAbsolutePath() + "，导入失败，原因：第" + line + "行，发现非数字字符串");
            } else {
                return Result.failure(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed));
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            if (AppUtil.isZhCN()) {
                return Result.failure(file.getAbsolutePath() + "，导入失败，原因：第" + line + "行:" + ex.getMessage());
            } else {
                return Result.failure(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed));
            }
        }
        return Result.success(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_success));
    }

    public static Result importStakingJobPointTxt(File file, PointArray pointArray) {
        String result = "";
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        BufferedReader buff = null;
        int line = 0;

        try {
            fis = new FileInputStream(file);
            bis = new BufferedInputStream(fis);
            buff = new BufferedReader(new InputStreamReader(bis, "utf-8"));
            String a = null;
            while ((a = buff.readLine()) != null)// 检查数据
            {
                line++;
                if ("".equals(a.trim()) || a.trim().startsWith("#")) {
                    continue;
                }
                a = a.replaceAll("(^[\u0020\u0009\u3000]*)|([\u0020\u0009\u3000]*$)", "");// 删除首尾空格、制表及全角空格
                String[] arr = a.split(",|，", -2);
                if (arr.length < 4) {
                    if (AppUtil.isZhCN()) {
                        return Result.failure(file.getAbsolutePath() + "，导入失败，原因：第" + line + "行，元素个数不能少于4个");
                    } else {
                        return Result.failure(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed));
                    }
                }
                if (arr.length == 4) {
                    /*job.getPointList()*/
                    pointArray.add(Point.STAKEOUT, arr[0], Double.parseDouble(arr[1])
                            , Double.parseDouble(arr[2]), Double.parseDouble(arr[3]), "");
                } else if (arr.length == 5) {
                    /*job.getPointList()*/
                    pointArray.add(Point.STAKEOUT, arr[0], Double.parseDouble(arr[1])
                            , Double.parseDouble(arr[2]), Double.parseDouble(arr[3]), arr[4]);
                } else if (arr.length == 6) {
                    /*job.getPointList()*/
                    pointArray.add(Point.STAKEOUT, arr[0], arr[1], Double.parseDouble(arr[2])
                            , Double.parseDouble(arr[3]), Double.parseDouble(arr[4]), arr[5]);
                }
            }
//            if ("".equals(result)) {
//                result = file.getAbsolutePath() + "，导入成功";
//            }
        } catch (NumberFormatException ex) {
            if (AppUtil.isZhCN()) {
                return Result.failure(file.getAbsolutePath() + "，导入失败，原因：第" + line + "行，发现非数字字符串");
            } else {
                return Result.failure(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed));
            }
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (buff != null) {
                try {
                    buff.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        //执行一次性操作
        //更新数据
        //PointDb.getInstance().replaceKnownPoints(jobId, knownPointRecordsFrom(projectId, jobId, pointArray), isOverride);
        return Result.success(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_success));
    }


    @WorkerThread
    public static void importStakingJobPointCassDat(File file, String projectId, String jobId, TimeCount.ITimeCounter timeCounter, boolean cassXy) {
        PointArray pointArray = new PointArray();
        Result result = importStakingJobPointCassDat(file, pointArray, cassXy);
        if (!pointArray.isEmpty()) {
            int size = pointArray.size();
            List<PointRecord> pointRecords = new ArrayList<>();
            for (int index = 0; index < size; index++) {
                Point point = pointArray.get(index);
                PointRecord pointRecord = new PointRecord(projectId, jobId, "",
                        PointConstants.POINT_TYPE_GLOBAL_CONTROL_POINT,
                        point.getName(), point.getCode(), point.getRemarks(),
                        point.getX(), point.getY(), point.getZ(),
                        0, 0, 0,
                        PointConstants.POINT_FORMAT_XYZ, null);
                pointRecord.createDate = new Date(timeCounter.time());
                pointRecords.add(pointRecord);
            }
            PointDb.getInstance().save(pointRecords);
        }
        if (!result.isSuccess()) {
            ToastUtils.showShort(result.describe);
        }
    }

    /**
     * 导入点数据
     *
     * @param file
     * @return
     */
    @WorkerThread
    public static Result importStakingJobPointCassDat(File file, PointArray pointArray, boolean cassXy) {
        String result = "";
        FileInputStream fis = null;
        BufferedInputStream bis = null;
        BufferedReader buff = null;
        int line = 0;
        try {
            fis = new FileInputStream(file);
            bis = new BufferedInputStream(fis);
            buff = new BufferedReader(new InputStreamReader(bis, "utf-8"));
            String a = null;
            while ((a = buff.readLine()) != null)// 检查数据
            {
                line++;
                if ("".equals(a.trim()) || a.trim().startsWith("#")) {
                    continue;
                }
                a = a.replaceAll("(^[\u0020\u0009\u3000]*)|([\u0020\u0009\u3000]*$)", "");// 删除首尾空格、制表及全角空格
                String[] arr = a.split(",|，", -2);
                if (arr.length != 5) {
                    if (AppUtil.isZhCN()) {
                        return Result.failure(file.getAbsolutePath() + "，导入失败，原因：第" + line + "行，元素个数不等于5个");
                    } else {
                        return Result.failure(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed));
                    }
                }
                if (cassXy) {
                    pointArray.add(Point.STAKEOUT, arr[0], arr[1], Double.parseDouble(arr[2]), Double.parseDouble(arr[3]), Double.parseDouble(arr[4]), "");
                } else {
                    pointArray.add(Point.STAKEOUT, arr[0], arr[1], Double.parseDouble(arr[3]), Double.parseDouble(arr[2]), Double.parseDouble(arr[4]), "");
                }
            }
        } catch (NumberFormatException ex) {
            if (AppUtil.isZhCN()) {
                return Result.failure(file.getAbsolutePath() + "，导入失败，原因：第\" + line + \"行，发现非数字字符串");
            } else {
                return Result.failure(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_failed));
            }
        } catch (IOException ex) {
            ex.printStackTrace();
        } finally {
            if (fis != null) {
                try {
                    fis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (bis != null) {
                try {
                    bis.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            if (buff != null) {
                try {
                    buff.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        //执行一次性操作
        //更新数据
        //            PointDb.getInstance().replaceKnownPoints(jobId, knownPointRecordsFrom(projectId, jobId, pointArray), isOverride);
        return Result.success(file.getAbsolutePath() + "," + StringUtils.getString(R.string.import_success));
    }

    /**
     * 导入点数据
     *
     * @param file
     * @param jobId
     * @return
     */
    @WorkerThread
    public static void importStakingJobPointTxt(File file, String projectId, String jobId, TimeCount.ITimeCounter timeCounter) {
        PointArray pointArray = new PointArray();
        Result result = importStakingJobPointTxt(file, pointArray);
        if (!pointArray.isEmpty()) {
            int size = pointArray.size();
            List<PointRecord> pointRecords = new ArrayList<>();
            for (int index = 0; index < size; index++) {
                Point point = pointArray.get(index);
                PointRecord pointRecord = new PointRecord(projectId, jobId, "",
                        PointConstants.POINT_TYPE_GLOBAL_CONTROL_POINT,
                        point.getName(), point.getCode(), point.getRemarks(),
                        point.getX(), point.getY(), point.getZ(),
                        0, 0, 0,
                        PointConstants.POINT_FORMAT_XYZ, null);
                pointRecord.createDate = new Date(timeCounter.time());
                pointRecords.add(pointRecord);
            }
            PointDb.getInstance().save(pointRecords);
        }
        if (!result.isSuccess()) {
            ToastUtils.showShort(result.describe);
        }
    }


    public static class Result {
        private final int code;
        public final String describe;

        public Result(int code, String describe) {
            this.code = code;
            this.describe = describe;
        }

        public boolean isSuccess() {
            return SUCCESS == code;
        }

        private final static int SUCCESS = 0;

        public static Result success(String str) {
            return new Result(SUCCESS, str);
        }

        public static Result failure(String str) {
            return new Result(-1, str);
        }
    }
}
