package com.project.survey.ui.lofting

import androidx.activity.viewModels
import com.project.survey.R
import com.project.survey.databinding.ActivityLoftingListBinding
import com.project.survey.extend.toast
import com.project.survey.logic.viewmodel.ProjectViewModel
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.ui.lofting.adapter.LoftingListAdapter
import com.project.survey.ui.lofting.pointlofting.PointStakingActivity
import com.project.survey.util.ActivityNavUtil

class LoftingListActivity : BaseBindingActivity<ActivityLoftingListBinding>() {


    companion object {
        fun start() {
            ActivityNavUtil.startActivity<LoftingListActivity> { }
        }
    }

    private val viewModel: ProjectViewModel by viewModels()
    private val adapter by lazy { LoftingListAdapter() }
    private var searchContent = "" // 搜索框里的内容


    override fun getBinding(): ActivityLoftingListBinding =
        ActivityLoftingListBinding.inflate(layoutInflater)

    override fun initView() {
        immersionToolbar(mBinding.toolbar, R.string.lofting)

//        mBinding.recyclerView.addItemDecoration(TransparentDividerDecoration(this))
        mBinding.recyclerView.setAdapter(adapter)
        adapter.isStateViewEnable = true
    }

    override fun initData() {
        viewModel.errorResponse.observe(this) {
            toast(it)
        }
        viewModel.loftingBeanListResponse.observe(this) {
            adapter.submitList(it.rows)
        }

        fetchData()
    }

    override fun initListener() {
        // 搜索
        mBinding.searchWidget.setOnEditorSearchListener {
            searchContent = it
            fetchData()
        }

        // 放样
        adapter.addOnItemChildClickListener(R.id.tvLofting) { _, _, position ->
            adapter.getItem(position)?.let { bean ->
//                PointStakingActivity.start(this, bean.BD_ZYLB_ID)
                PointStakingActivity.start(this, bean)
//                viewModel.fetchLoftingPointList(bean.BD_ZYLB_ID)
            }
        }
    }

    private fun fetchData() {
        viewModel.fetchLoftingList(searchContent)
    }
}