package com.project.survey.ui.lofting.pointlofting;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.graphics.drawable.GradientDrawable;
import android.view.View;

import androidx.annotation.ColorInt;

import com.bingce.device.Device;
import com.bingce.device.enums.DeviceTypeEnum;
import com.bingce.surveyor.util.ConstUtils;
import com.bingce.surveyor.util.PreferencesCadUtil;
import com.bingce.surveyor.util.PreferencesUtil;
import com.bingce.surveyor.util.dialog.CustomRecycleDialog;
import com.bingce.utils.FileUtil;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.Util;
import com.project.survey.R;
import com.project.survey.databinding.ActivityPointStakingSettingBinding;
import com.project.survey.dialog.CustomRecycleStyleDialog;
import com.project.survey.model.TextStyleBean;
import com.project.survey.ui.base.BaseSurveyNewActivity;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

public class PointStakingSettingActivity extends BaseSurveyNewActivity {

    private ActivityPointStakingSettingBinding binding;
    private int limitHrmsIndex = -1;
    private int limitVrmsIndex = -1;
    private int limitPdopIndex = -1;
    private int limitAgeIndex = -1;
    private int limitTipsRangeIndex = -1;
    private int limitStakingIndex = -1;
    private int limitSmoothPointNumberIndex = -1;

    @Override
    public View getContentView() {
        binding = ActivityPointStakingSettingBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        if (IntentUtil.boolExtra(getIntent(), KEY_IS_CAD_SETTING) && !Device.getInstance().isDeviceConnected()) {
            setTitle(getString(R.string.cad_setting));
            binding.rlStatusLimit.setVisibility(View.GONE);
            binding.llHvpd.setVisibility(View.GONE);
            binding.llRangeLimit.setVisibility(View.GONE);
            binding.llCadSetting.setVisibility(View.VISIBLE);
        } else {
            if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
                setTitle(getString(R.string.ts_staking_setting));
                binding.rlStatusLimit.setVisibility(View.GONE);
                binding.llHvpd.setVisibility(View.GONE);
                binding.rlSmoothPointNumberLimit.setVisibility(View.GONE);
            } else {
                setTitle(getString(R.string.rtk_staking_setting));
                binding.rlStatusLimit.setVisibility(View.VISIBLE);
                binding.llHvpd.setVisibility(View.VISIBLE);
                binding.rlSmoothPointNumberLimit.setVisibility(View.VISIBLE);
            }
            binding.llRangeLimit.setVisibility(View.VISIBLE);
            if (IntentUtil.boolExtra(getIntent(), KEY_IS_CAD_SETTING)) {
                binding.llCadSetting.setVisibility(View.VISIBLE);
            } else {
                binding.llCadSetting.setVisibility(View.GONE);
            }
        }
    }

    @Override
    protected void initData() {
        limitStatus();
        limitHrms();
        limitVrms();
        limitPdop();
        limitAge();
        limitTipsRange();
        limitStaking();
        cadUcsWcsStatus();
        cadUnitLength();
        cadBackground();
        cadCleanCache();
        limitSmoothPointNumber();
    }

    /**
     * 状态解
     */
    private void limitStatus() {
        List<String> limitStatusList = new ArrayList<>();
        limitStatusList.add(getString(R.string.rtk_state_fixed));
        limitStatusList.add(getString(R.string.rtk_state_diff_3D));
        limitStatusList.add(getString(R.string.rtk_state_float));
        limitStatusList.add(getString(R.string.rtk_state_single));
        binding.tvStatusLimit.setText(limitStatusList.get(PreferencesUtil.getPointStakingStatusLimitIndex()));
        binding.rlStatusLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.solution_status_limit, limitStatusList, PreferencesUtil.getPointStakingStatusLimitIndex(), false, (index, itemString) -> {
            binding.tvStatusLimit.setText(limitStatusList.get(index));
            PreferencesUtil.putPreference(ConstUtils.preferConst.pointStakingStatusLimit, index);
        }));
    }

    /**
     * Hrms
     */
    private void limitHrms() {
        List<String> limitHrmsList = new ArrayList<>();
        limitHrmsList.add("0.05");
        limitHrmsList.add("0.10");
        limitHrmsList.add("0.20");
        limitHrmsList.add("0.30");
        for (int i = 0; i < limitHrmsList.size(); i++) {
            if (Double.parseDouble(PreferencesUtil.getPointStakingHrmsLimit()) == Double.parseDouble(limitHrmsList.get(i))) {
                limitHrmsIndex = i;
            }
        }

        binding.tvHrmsLimit.setText(PreferencesUtil.getPointStakingHrmsLimit() + getString(R.string.meter));
        binding.rlHrmsLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.hrms_limit, limitHrmsList, limitHrmsIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL, true, PreferencesUtil.getPointStakingHrmsLimit(), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitHrmsIndex = index;
                if (index != -1) {
                    binding.tvHrmsLimit.setText(limitHrmsList.get(index) + getString(R.string.meter));
                } else {
                    binding.tvHrmsLimit.setText(itemString + getString(R.string.meter));
                }
                PreferencesUtil.putPreference(ConstUtils.preferConst.pointStakingHrmsLimit, itemString);
            }
        }));
    }

    /**
     * Vrms
     */
    private void limitVrms() {
        List<String> limitVrmsList = new ArrayList<>();
        limitVrmsList.add("0.05");
        limitVrmsList.add("0.10");
        limitVrmsList.add("0.20");
        limitVrmsList.add("0.30");
        for (int i = 0; i < limitVrmsList.size(); i++) {
            if (Double.parseDouble(PreferencesUtil.getPointStakingVrmsLimit()) == Double.parseDouble(limitVrmsList.get(i))) {
                limitVrmsIndex = i;
            }
        }

        binding.tvVrmsLimit.setText(PreferencesUtil.getPointStakingVrmsLimit() + getString(R.string.meter));
        binding.rlVrmsLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.vrms_limit, limitVrmsList, limitVrmsIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL, true, PreferencesUtil.getPointStakingVrmsLimit(), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitVrmsIndex = index;
                if (index != -1) {
                    binding.tvVrmsLimit.setText(limitVrmsList.get(index) + getString(R.string.meter));
                } else {
                    binding.tvVrmsLimit.setText(itemString + getString(R.string.meter));
                }
                PreferencesUtil.putPreference(ConstUtils.preferConst.pointStakingVrmsLimit, itemString);
            }
        }));
    }

    /**
     * Pdop
     */
    private void limitPdop() {
        List<String> limitPdopList = new ArrayList<>();
        limitPdopList.add("5.00");
        limitPdopList.add("20.00");
        limitPdopList.add("50.00");
        limitPdopList.add("100.00");
        for (int i = 0; i < limitPdopList.size(); i++) {
            if (Double.parseDouble(PreferencesUtil.getPointStakingPdopLimit()) == Double.parseDouble(limitPdopList.get(i))) {
                limitPdopIndex = i;
            }
        }

        binding.tvPdopLimit.setText(PreferencesUtil.getPointStakingPdopLimit());
        binding.rlPdopLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.pdop_limit, limitPdopList, limitPdopIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL, true, PreferencesUtil.getPointStakingPdopLimit(), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitPdopIndex = index;
                if (index != -1) {
                    binding.tvPdopLimit.setText(limitPdopList.get(index));
                } else {
                    binding.tvPdopLimit.setText(itemString);
                }
                PreferencesUtil.putPreference(ConstUtils.preferConst.pointStakingPdopLimit, itemString);
            }
        }));
    }

    private void limitAge() {
        List<String> limitAgeList = new ArrayList<>();
        limitAgeList.add("5");
        limitAgeList.add("10");
        limitAgeList.add("15");
        limitAgeList.add("25");
        for (int i = 0; i < limitAgeList.size(); i++) {
            if (Double.parseDouble(PreferencesUtil.getPointStakingAgeLimit()) == Double.parseDouble(limitAgeList.get(i))) {
                limitAgeIndex = i;
            }
        }

        binding.tvAgeLimit.setText(PreferencesUtil.getPointStakingAgeLimit() + getString(R.string.sec));
        binding.rlAgeLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.diff_age_limit, limitAgeList, limitAgeIndex, CustomRecycleDialog.Builder.INPUT_TYPE_INTEGER, true, PreferencesUtil.getPointStakingAgeLimit(), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitAgeIndex = index;
                if (index != -1) {
                    binding.tvAgeLimit.setText(limitAgeList.get(index) + getString(R.string.sec));
                } else {
                    binding.tvAgeLimit.setText(itemString + getString(R.string.sec));
                }
                PreferencesUtil.putPreference(ConstUtils.preferConst.pointStakingAgeLimit, itemString);
            }
        }));
    }

    public void limitTipsRange() {
        List<String> limitTipsRangeList = new ArrayList<>();
        limitTipsRangeList.add("0.20");
        limitTipsRangeList.add("0.50");
        limitTipsRangeList.add("1.00");
        limitTipsRangeList.add("2.00");
        for (int i = 0; i < limitTipsRangeList.size(); i++) {
            if (Double.parseDouble(PreferencesUtil.getPointStakingTipsRangeLimit()) == Double.parseDouble(limitTipsRangeList.get(i))) {
                limitTipsRangeIndex = i;
            }
        }

        binding.tvTipsRangeLimit.setText(PreferencesUtil.getPointStakingTipsRangeLimit() + getString(R.string.meter));
        binding.rlTipsRangeLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.tips_range, limitTipsRangeList, limitTipsRangeIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL, true, PreferencesUtil.getPointStakingTipsRangeLimit(), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitTipsRangeIndex = index;
                if (index != -1) {
                    binding.tvTipsRangeLimit.setText(limitTipsRangeList.get(index) + getString(R.string.meter));
                } else {
                    binding.tvTipsRangeLimit.setText(itemString + getString(R.string.meter));
                }
                PreferencesUtil.putPreference(ConstUtils.preferConst.pointStakingTipsRangeLimit, itemString);
            }
        }));
    }

    private void limitSmoothPointNumber() {
        List<String> limitSmoothPointNumberList = new ArrayList<>();
        limitSmoothPointNumberList.add("1");
        limitSmoothPointNumberList.add("2");
        limitSmoothPointNumberList.add("5");
        limitSmoothPointNumberList.add("10");
        for (int i = 0; i < limitSmoothPointNumberList.size(); i++) {
            if (PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStakingSmoothPointsNumber, 1) == Integer.parseInt(limitSmoothPointNumberList.get(i))) {
                limitSmoothPointNumberIndex = i;
            }
        }
        binding.tvSmoothPointNumberLimit.setText(String.valueOf(PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStakingSmoothPointsNumber, 1)));
        binding.rlSmoothPointNumberLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.smooth_points_number, limitSmoothPointNumberList, limitSmoothPointNumberIndex, CustomRecycleDialog.Builder.INPUT_TYPE_INTEGER, true, Util.formatDouble2StringDotAuto(PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStakingSmoothPointsNumber, 1)), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitSmoothPointNumberIndex = index;
                if (index != -1) {
                    binding.tvSmoothPointNumberLimit.setText(limitSmoothPointNumberList.get(index));
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkStakingSmoothPointsNumber, Integer.parseInt(limitSmoothPointNumberList.get(index)));
                } else {
                    binding.tvSmoothPointNumberLimit.setText(itemString);
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkStakingSmoothPointsNumber, Integer.parseInt(itemString));
                }
            }
        }));
    }

    private void limitStaking() {
        List<String> limitStakingList = new ArrayList<>();
        limitStakingList.add("0.01");
        limitStakingList.add("0.02");
        limitStakingList.add("0.05");
        limitStakingList.add("0.10");
        for (int i = 0; i < limitStakingList.size(); i++) {
            if (Double.parseDouble(PreferencesUtil.getPointStakingLimit()) == Double.parseDouble(limitStakingList.get(i))) {
                limitStakingIndex = i;
            }
        }

        binding.tvStakingLimit.setText(Double.parseDouble(PreferencesUtil.getPointStakingLimit())/1000 + getString(R.string.meter));
        binding.tvStakingElevationLimit.setText(Double.parseDouble(PreferencesUtil.getShsxPointStakingElevationLimit())/1000 + getString(R.string.meter));
//        binding.rlStakingLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.stakout_difference, limitStakingList, limitStakingIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL, true, PreferencesUtil.getPointStakingLimit(), new CustomRecycleDialog.IClickCallback() {
//            @Override
//            public void onItemSelected(int index, String itemString) {
//                limitStakingIndex = index;
//                if (index != -1) {
//                    binding.tvStakingLimit.setText(limitStakingList.get(index) + getString(R.string.meter));
//                } else {
//                    binding.tvStakingLimit.setText(itemString + getString(R.string.meter));
//                }
//                PreferencesUtil.putPreference(ConstUtils.preferConst.pointStakingLimit, itemString);
//            }
//        }));
    }

    private void cadUcsWcsStatus() {
        List<String> ucsWcsStatusList = new ArrayList<>();
        ucsWcsStatusList.add(getString(R.string.user_coordinate_system));
        ucsWcsStatusList.add(getString(R.string.world_coordinate_system));
        binding.tvWcsUcsStatus.setText(ucsWcsStatusList.get(PreferencesCadUtil.getCadUserSetUcsWcsStatus()));
        binding.rlSelectCoordinateSystem.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.coordinate_system, ucsWcsStatusList, PreferencesCadUtil.getCadUserSetUcsWcsStatus(), false, (index, itemString) -> {
            binding.tvWcsUcsStatus.setText(ucsWcsStatusList.get(index));
            PreferencesCadUtil.putPreference(ConstUtils.preferConst.cadUseSetUcsWcsStatus, index);
        }));
    }

    private void cadUnitLength() {
        List<String> unitLengthList = new ArrayList<>();
        unitLengthList.add(getString(R.string.unit_cad_m));
        unitLengthList.add(getString(R.string.unit_cad_cm));
        unitLengthList.add(getString(R.string.unit_cad_mm));
        binding.tvUnitLength.setText(unitLengthList.get(PreferencesCadUtil.getCadDrawingUnitLength()));
        binding.rlSelectUnitLength.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointStakingSettingActivity.this, R.string.unit_length, unitLengthList, PreferencesCadUtil.getCadDrawingUnitLength(), false, (index, itemString) -> {
            binding.tvUnitLength.setText(unitLengthList.get(index));
//            PreferencesCadUtil.putPreference(ConstUtils.preferConst.cadDrawingUnitLength, index);
        }));
    }

    private int cadBackgroundColorIndex = 0;
    private GradientDrawable mGroupDrawable;

    /**
     * cad背景
     */
    private void cadBackground() {
        mGroupDrawable = (GradientDrawable) binding.tvSelectColor.getBackground();
        setGroupDrawable(PreferencesCadUtil.getCadBackgroundColorValue());
        List<TextStyleBean> list = new ArrayList<>();
        list.add(new TextStyleBean(Color.BLACK));
        list.add(new TextStyleBean(Color.WHITE));
        if (PreferencesCadUtil.getCadBackgroundColorValue() == Color.BLACK) {
            cadBackgroundColorIndex = 0;
        } else {
            cadBackgroundColorIndex = 1;
        }
        binding.rlCadBackground.setOnClickListener(v -> {
            new CustomRecycleStyleDialog.Builder(this).setTitle(R.string.background_color).setListData(list, "", cadBackgroundColorIndex)
                    .setItemClickListener((preferenceKey, position, textStyleBean) -> {
                        cadBackgroundColorIndex = position;
                        setGroupDrawable(textStyleBean.getBackground());
                        PreferencesCadUtil.putPreference(ConstUtils.preferConst.cadBackgroundColor, textStyleBean.getBackground());
                    }).create().show();
        });
    }

    private void setGroupDrawable(@ColorInt int color) {
        if (color == Color.WHITE) {
            //判断为白色是，添加一个边框便于颜色识别（白色和弹框背景色重合了）
            mGroupDrawable.setStroke(2, getColor(R.color.color_8a8a8a));
            mGroupDrawable.setColor(color);
        } else {
            mGroupDrawable.setStroke(1, color);
            mGroupDrawable.setColor(color);
        }
    }

    /**
     * cad文件缓存
     */
    private void cadCleanCache() {
        File file = new File(FileUtil.getSDPath() + "/SurveyorCAD");
        binding.tvCleanCache.setText(formatSize(file.length()));
        binding.rlCleanCache.setOnClickListener(v -> {
            if (file.exists()) {
                boolean result = FileUtil.deleteDir(file);
                if (result) {
                    ToastUtils.showShort(R.string.clean_over_then_restart_activity);
                    binding.tvCleanCache.setText("0.0KB");
                }
            } else {
                ToastUtils.showShort(R.string.clean_over_then_restart_activity);
                binding.tvCleanCache.setText("0.0KB");
            }
        });
    }

    public static String formatSize(long size) {
        String suffix = "KB";
        double fSize = size / 1024;
        if (fSize > 1024) {
            suffix = "MB";
            fSize /= 1024;
        }
        if (fSize > 1024) {
            suffix = "GB";
            fSize /= 1024;
        }
        return fSize + suffix;
    }

    private static String KEY_IS_CAD_SETTING = "isCADSetting";

    public static void start(Context context) {
        start(context, false);
    }

    public static void start(Context context, boolean isCADSetting) {
        Intent intent = new Intent(context, PointStakingSettingActivity.class);
        intent.putExtra(KEY_IS_CAD_SETTING, isCADSetting);
        context.startActivity(intent);
    }
}
