package com.project.survey.ui.lofting.pointlofting.record;

import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;

import androidx.annotation.Nullable;

import com.bingce.activity.AutoLandscapeBingCeActivity;
import com.bingce.data.cache.CachedCurrentJob;
import com.bingce.data.cache.CachedCurrentProject;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.project.survey.R;
import com.project.survey.databinding.ActivityRecordDetailBinding;
import com.project.survey.ui.lofting.pointlofting.record.util.IRecordsUtils;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordDetailActivityRemarksUtils;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordDetailData;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordsUtilsFactory;

import java.util.Date;
import java.util.Map;

import blankj.utilcode.util.ToastUtils;

public class RecordDetailActivity extends AutoLandscapeBingCeActivity {
    private RecordDetailData mCurrentData = null;
    private ActivityRecordDetailBinding binding;

    private final Map<Integer, IRecordsUtils> recordsUtilsMap = RecordsUtilsFactory.buildUtils(this, null);

    @Override
    protected void onBingCeCreate(@Nullable Bundle savedInstanceState) {
        binding = ActivityRecordDetailBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());

        setSupportActionBar(binding.toolbar);
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);

        int mType = IntentUtil.intExtra(getIntent(), KEY_TYPE);
        String recordId = IntentUtil.stringExtra(getIntent(), KEY_ID);
        if (StringUtil.isEmpty(recordId)) {
            ToastUtils.showShort(R.string.no_record);
            finish();
            return;
        }
        IRecordsUtils recordsUtils = recordsUtilsMap.get(mType);

        ThreadPoolUtil.execute(() -> {
            if (recordsUtils != null) {
                RecordDetailData detailData = recordsUtils.findRecordDetailInWorkerThread(recordId);
                setRecord(detailData);
            }
        });

        //设置标题
        if (recordsUtils != null) {
            setTitle(getString(R.string.record_detail) + "-" + recordsUtils.name());
        }
    }

    protected void setRecord(RecordDetailData results) {
        if (results != null) {
            mCurrentData = results;
        } else {
            ToastUtils.showShort(R.string.no_record);
            return;
        }
        StringBuilder stringBuilder = new StringBuilder();
        for (Map.Entry<String, String> entry : results.keyValueMap.entrySet()) {
            stringBuilder.append(entry.getKey()).append("：").append(entry.getValue()).append("\n");
        }
        runOnUiThread(() -> {
            if (results.deviceInfoData != null && results.deviceInfoData.tsStatusData != null) {
                stringBuilder.append(getString(R.string.hr)).append(":").append(Util.formatDouble2StringDotAuto(results.deviceInfoData.tsStatusData.hr));
            }
            binding.activityRecordDetailTextview.setText(stringBuilder.toString());
        });
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_record_detail, menu);
        return true;
    }

    protected void nextRecord(int mType, Date date) {
        if (mCurrentData == null) {
            ToastUtils.showShort(R.string.no_record);
            return;
        }
        ThreadPoolUtil.execute(() -> {
            String projectId = CachedCurrentProject.currentProjectId();
            String jobId = CachedCurrentJob.currentJobId(projectId);
            IRecordsUtils recordsUtils = recordsUtilsMap.get(mType);
            if (recordsUtils != null) {
                RecordDetailData recordDetailData = recordsUtils.findOtherRecordDetailInWorkerThread(
                        projectId, jobId, date, true);
                setRecord(recordDetailData);
            }
        });
    }

    protected void lastRecord(int mType, Date date) {
        if (mCurrentData == null) {
            ToastUtils.showShort(R.string.no_record);
            return;
        }
        ThreadPoolUtil.execute(() -> {
            String projectId = CachedCurrentProject.currentProjectId();
            String jobId = CachedCurrentJob.currentJobId(projectId);
            IRecordsUtils recordsUtils = recordsUtilsMap.get(mType);
            if (recordsUtils != null) {
                RecordDetailData recordDetailData = recordsUtils.findOtherRecordDetailInWorkerThread(
                        projectId, jobId, date, false);
                setRecord(recordDetailData);
            }
        });
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        int mType = IntentUtil.intExtra(getIntent(), KEY_TYPE);
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                break;
            case R.id.menu_record_detail_page_down: {
                Date date = mCurrentData != null ? mCurrentData.date : null;
                nextRecord(mType, date);
            }
            break;
            case R.id.menu_record_detail_page_up: {
                Date date = mCurrentData != null ? mCurrentData.date : null;
                lastRecord(mType, date);
            }
            break;
            case R.id.menu_record_detail_edit_remarks: {
                if (mCurrentData != null) {
                    RecordDetailActivityRemarksUtils.changeRemarks(this, mCurrentData.remarks, remarks -> {
                        ThreadPoolUtil.execute(() -> {
                            IRecordsUtils recordsUtils = recordsUtilsMap.get(mType);
                            if (recordsUtils != null) {
                                mCurrentData = recordsUtils.updateRemarksInWorkerThread(mCurrentData.recordId, remarks);
                                //更新界面--主要是remarks
                                setRecord(mCurrentData);
                            }
                        });
                    });
                }
            }
            break;
        }
        return super.onOptionsItemSelected(item);
    }

    protected static final String KEY_TYPE = "TYPE";
    protected static final String KEY_ID = "recordId";

    public static void start(Context context, int type, String recordId) {
        Intent intent = startActivityIntent(context, RecordDetailActivity.class);
        intent.putExtra(KEY_TYPE, type);
        intent.putExtra(KEY_ID, recordId);
        context.startActivity(intent);
    }
}