package com.project.survey.ui.lofting.pointlofting.record;


import android.annotation.SuppressLint;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ActivityInfo;
import android.graphics.Color;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.TextView;

import androidx.annotation.Nullable;
import androidx.preference.PreferenceManager;

import com.bingce.AppChannel;
import com.bingce.data.base.user.UserConfig;
import com.bingce.data.sync.utils.LoadCurrentJobIdKt;
import com.bingce.data.sync.utils.LoadCurrentProjectIdKt;
import com.bingce.data.sync.utils.LoadCurrentRoadIdKt;
import com.bingce.data.sync.utils.LoadCurrentWaterPipeIdKt;
import com.bingce.data.sync.utils.LoadUserConfigKt;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.StringUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.hjq.permissions.Permission;
import com.hjq.permissions.XXPermissions;
import com.hjq.shape.view.ShapeTextView;
import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.constants.SPConstants;
import com.project.survey.databinding.ActivityRecordsBinding;
import com.project.survey.model.ProjectBean;
import com.project.survey.ui.lofting.pointlofting.record.util.IRecordsUtils;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordTypeConstants;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordsActivityClearUtils;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordsActivityExportUtils;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordsActivityFilterUtils;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordsActivityHeaders;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordsUtilsFactory;
import com.project.survey.ui.lofting.pointlofting.record.util.RecordsViewModelKt;
import com.project.survey.util.SPUtils;

import org.polaric.colorful.ColorfulActivity;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;


import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;


public class RecordsActivity extends ColorfulActivity {
    private static final String TAG = "RecordsActivity";
    private final static String KEY_TYPE = "type";

    private boolean isSortByTimeAcs = false;//是否把结果按照时间升序排列
    private TextView tv_top_date_filter;
    private ShapeTextView tv_filter_other;
    private ShapeTextView tv_filter_time;
    private ShapeTextView tv_filter_dcs_type;
    private ActivityRecordsBinding binding;

    private int currentTypeFromSpinner = RecordTypeConstants.TYPE_POINT_STAKE;
    private final RecordsActivityFilterUtils filterUtils = new RecordsActivityFilterUtils(() -> this,
            () -> currentTypeFromSpinner, () -> tv_filter_time, () -> tv_filter_dcs_type);
    private final RecordsActivityExportUtils exportUtils = new RecordsActivityExportUtils(() -> this, () -> currentTypeFromSpinner);
    private final RecordsActivityClearUtils clearUtils = new RecordsActivityClearUtils(() -> this, () -> currentTypeFromSpinner);

    private final Map<Integer, IRecordsUtils> recordsUtilsMap = RecordsUtilsFactory.buildUtils(this);

    private final RecordsActivityHeaders headers = new RecordsActivityHeaders(() -> this);
    private final RecordsViewModelKt.HolderInActivity holderInActivity =
            new RecordsViewModelKt.HolderInActivity(this);
    private String workId = "";

    public static void start(Context context, int type) {
        Intent intent = new Intent(context, RecordsActivity.class);
        intent.putExtra(KEY_TYPE, type);
        context.startActivity(intent);
    }

    public static void start(Context context) {
        Intent intent = new Intent(context, RecordsActivity.class);
        context.startActivity(intent);
    }

    private static final String KEY_WORK_ID = "keyWorkId";

    public static void start(Context context, String workId) {
        Intent intent = new Intent(context, RecordsActivity.class);
        intent.putExtra(KEY_WORK_ID, workId);
        context.startActivity(intent);
    }

    @SuppressLint("SourceLockedOrientationActivity")
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        if (App.instance.isLandscape()) {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_SENSOR_LANDSCAPE);
            getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
                    WindowManager.LayoutParams.FLAG_FULLSCREEN);
        } else {
            setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
            getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
        }
        super.onCreate(savedInstanceState);
        binding = ActivityRecordsBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        setSupportActionBar(binding.toolbar);
        if (getSupportActionBar() != null)
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
        getSupportActionBar().setTitle("采集记录");

        workId = IntentUtil.stringExtra(this, KEY_WORK_ID);

        tv_top_date_filter = findViewById(R.id.activity_records_top_date_filter);
        tv_filter_time = findViewById(R.id.tv_filter_time);
        tv_filter_dcs_type = findViewById(R.id.tv_filter_dcs_type);
        tv_filter_other = findViewById(R.id.tv_filter_other);
        tv_top_date_filter.setText(R.string.record_default_display_one_week);

        if (App.instance.isThemeDark()) {
            tv_filter_time.getShapeDrawableBuilder().setSolidColor(Color.GRAY).intoBackground();
            tv_filter_dcs_type.getShapeDrawableBuilder().setSolidColor(Color.GRAY).intoBackground();
            tv_filter_other.getShapeDrawableBuilder().setSolidColor(Color.GRAY).intoBackground();
        } else {
            tv_filter_time.getShapeDrawableBuilder().setSolidColor(Color.WHITE).intoBackground();
            tv_filter_dcs_type.getShapeDrawableBuilder().setSolidColor(Color.WHITE).intoBackground();
            tv_filter_other.getShapeDrawableBuilder().setSolidColor(Color.WHITE).intoBackground();
        }
//        initSpinnerType();
        updateRecyclerView();
        headers.initHeaders();
        binding.ivOrder.setOnClickListener(view -> {
            isSortByTimeAcs = !isSortByTimeAcs;
            binding.ivOrder.setImageResource(isSortByTimeAcs ? R.drawable.icon_sort_ascending : R.drawable.icon_sort_descending);
            updateRecyclerView();
        });
        binding.tvFilterTime.setOnClickListener(v -> filterUtils.showTimeFilterDialog(this::updateRecyclerView));
        if (currentTypeFromSpinner != RecordTypeConstants.TYPE_TUNNEL_BACK_BREAK && currentTypeFromSpinner != RecordTypeConstants.TYPE_WATER_PIPE_BACK_BREAK) {
            filterUtils.resetTcsType();
        }
        binding.tvFilterDcsType.setOnClickListener(v -> filterUtils.showTcsTypeDialog(this::updateRecyclerView));


        tv_filter_dcs_type.setVisibility((currentTypeFromSpinner != RecordTypeConstants.TYPE_TUNNEL_BACK_BREAK && currentTypeFromSpinner != RecordTypeConstants.TYPE_WATER_PIPE_BACK_BREAK) ? View.GONE : View.VISIBLE);
        tv_filter_other.setOnClickListener(v -> {
            filterUtils.showFilterDialog(this::updateRecyclerView);
        });
    }

    /**
     * 切换里程坐标等记录类型
     */
//    private void initSpinnerType() {
//        binding.spinner1.setOnItemSelectedListener(spinnerListener);
//
//        List<String> spinnerTitlesList = new ArrayList<>();
//        for (Map.Entry<Integer, IRecordsUtils> entry : recordsUtilsMap.entrySet()) {
//            if (entry == null) {
//                continue;
//            }
//            IRecordsUtils recordsUtils = entry.getValue();
//            if (recordsUtils == null) {
//                continue;
//            }
//            int type = entry.getKey();
//            String name = recordsUtils.name();
//            switch (type) {
//                case RecordTypeConstants.TYPE_CUSTOM_ZHONG_JIAO_ONE:
//                    if (AppChannel.customChannel == AppChannel.CUSTOM_ZJYGJ) {
//                        spinnerTitlesList.add(name);
//                    }
//                    break;
//                case RecordTypeConstants.TYPE_CUSTOM_TIAN_JING_ONE:
//                    if (AppChannel.customChannel == AppChannel.CUSTOM_CR18SZ) {
//                        spinnerTitlesList.add(name);
//                    }
//                    break;
//                case RecordTypeConstants.TYPE_CUSTOM_MDM:
//                    if (AppChannel.customChannel == AppChannel.CUSTOM_MONITOR) {
//                        spinnerTitlesList.add(name);
//                    }
//                    break;
////                case RecordTypeConstants.TYPE_ROTARY_BRIDGE_6COM:
////                    if (AppChannel.customChannel == AppChannel.CUSTOM_6_COM) {
////                        spinnerTitlesList.add(name);
////                    }
////                    break;
//                default:
//                    spinnerTitlesList.add(name);
//                    break;
//            }
//        }
//
//        ArrayAdapter<String> mSpinnerAdapter = new ArrayAdapter<>(this,
//                R.layout.spinner_dropdown_item_my_with_background, android.R.id.text1,
//                spinnerTitlesList);
//        binding.spinner1.setAdapter(mSpinnerAdapter);
//
//        //设置默认记录类型
//        int type = App.instance.getLastRecordBrowseType();
//        Intent intent = getIntent();
//        if (intent != null && intent.hasExtra(KEY_TYPE)) {
//            type = intent.getIntExtra(KEY_TYPE, 0);
//        }
//        int lastType = Math.max(type, 0);
//
//        IRecordsUtils utils = recordsUtilsMap.get(lastType);
//        int index = 0;
//        if (utils != null) {
//            String name = utils.name();
//            boolean found = false;
//            for (index = 0; index < spinnerTitlesList.size(); index++) {
//                if (spinnerTitlesList.get(index).equals(name)) {
//                    found = true;
//                    break;
//                }
//            }
//            if (!found) {
//                index = 0;
//            }
//        }
//        binding.spinner1.setSelection(index);
//    }

//    //里程计算坐标等类型切换监听器
//    AdapterView.OnItemSelectedListener spinnerListener = new AdapterView.OnItemSelectedListener() {
//        @Override
//        public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
//            String typeString = binding.spinner1.getSelectedItem().toString();
//            Set<Map.Entry<Integer, IRecordsUtils>> entries = recordsUtilsMap.entrySet();
//            for (Map.Entry<Integer, IRecordsUtils> entry : entries) {
//                if (entry.getValue().name().equals(typeString)) {
//                    currentTypeFromSpinner = entry.getKey();
//                    if (currentTypeFromSpinner != RecordTypeConstants.TYPE_TUNNEL_BACK_BREAK && currentTypeFromSpinner != RecordTypeConstants.TYPE_WATER_PIPE_BACK_BREAK) {
//                        filterUtils.resetTcsType();
//                        tv_filter_dcs_type.setVisibility(View.GONE);
//                    } else {
//                        tv_filter_dcs_type.setVisibility(View.VISIBLE);
//                    }
//                    break;
//                }
//            }
//
//            updateRecyclerView();
//            App.instance.setLastRecordBrowseType(currentTypeFromSpinner);
//            SharedPreferences prefs =
//                    PreferenceManager.getDefaultSharedPreferences(RecordsActivity.this);
//            SharedPreferences.Editor editor = prefs.edit();
//            editor.putInt("lastRecordBrowseType", App.instance.getLastRecordBrowseType());
//            editor.apply();
//        }
//
//        @Override
//        public void onNothingSelected(AdapterView<?> parent) {
//        }
//    };

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.menu_activity_records, menu);
        return super.onCreateOptionsMenu(menu);
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                break;
            case R.id.menu_record_sort:
                isSortByTimeAcs = !isSortByTimeAcs;
                updateRecyclerView();
                break;
            case R.id.menu_record_export:
                exportUtils.requestStoragePermission(this, exportSuccess());
                break;
//            case R.id.menu_record_filter:
//                binding.spinner1.getSelectedItem().toString();
//                filterUtils.showFilterDialog(this::updateRecyclerView);
//                break;
//            case R.id.menu_record_clear:
//                clearUtils.showClearDialog(() -> {
//                    ProgressDialog mProgressDialog = new ProgressDialog(this);
//                    mProgressDialog.setMessage(getString(R.string.deleting));
//                    mProgressDialog.show();
//                    ThreadPoolUtil.execute(() -> {
//                        IRecordsUtils recordsUtils = recordsUtilsMap.get(currentTypeFromSpinner);
//                        if (recordsUtils != null) {
//                            recordsUtils.clearInWorkerThread(filterUtils.filterParameter(), () -> {
//                                runOnUiThread(mProgressDialog::dismiss);
//                            });
//                        }
//                        ToastUtils.showShort(R.string.clean_over);
//                    });
//                });
//                break;
        }
        return super.onOptionsItemSelected(item);
    }

    private void updateRecyclerView() {
        ThreadPoolUtil.execute(() -> {
//            UserConfig userConfig = LoadUserConfigKt.loadUserConfig();
//            String projectId = LoadCurrentProjectIdKt.loadCurrentProjectId(true, userConfig);
//            if (StringUtil.isEmpty(projectId)) {
//                return;
//            }
//            String jobId = LoadCurrentJobIdKt.loadCurrentJobId(projectId, true, userConfig);
//            if (StringUtil.isEmpty(jobId)) {
//                return;
//            }
//            if (currentTypeFromSpinner == RecordTypeConstants.TYPE_WATER_PIPE_BACK_BREAK){
//                String waterPipeId = LoadCurrentWaterPipeIdKt.loadCurrentWaterPipeId(projectId, true, userConfig);
//                if (StringUtil.isEmpty(waterPipeId)){
//                    return;
//                }
//                runOnUiThread(() -> {
//                    IRecordsUtils recordsUtils = recordsUtilsMap.get(currentTypeFromSpinner);
//                    if (recordsUtils != null) {
//                        recordsUtils.update(binding.recyclerviewRecords, isSortByTimeAcs,
//                                userConfig,
//                                projectId, jobId, waterPipeId,
//                                filterUtils.filterParameter());
//
//                    }
//                });
//            }else {
//                String roadId = LoadCurrentRoadIdKt.loadCurrentRoadId(projectId, true, userConfig);
//                if (StringUtil.isEmpty(roadId)) {
//                    return;
//                }
                runOnUiThread(() -> {
                    IRecordsUtils recordsUtils = recordsUtilsMap.get(currentTypeFromSpinner);
                    if (recordsUtils != null) {
                        recordsUtils.update(binding.recyclerviewRecords, isSortByTimeAcs,
                                SPUtils.INSTANCE.getString(SPConstants.PROJECT_ID, ""), workId,
                                filterUtils.filterParameter());

//                        if (RecordTypeConstants.TYPE_MEASURING_TAPE == currentTypeFromSpinner) {
//                            //如果选中的是进尺，尝试加载数据
//                            holderInActivity.getViewModel().loadMeasuringTapeRecord(userConfig, roadId);
//                        }
                    }
                });
//            }


        });
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if (requestCode == XXPermissions.REQUEST_CODE) {
            if (XXPermissions.isGranted(this, Permission.MANAGE_EXTERNAL_STORAGE)) {
                exportUtils.requestStoragePermission(this, exportSuccess());
            } else {
                ToastUtils.showLong(getString(R.string.no_storage_permission_hint));
            }
        }
    }

    private RecordsActivityExportUtils.IExportCallback exportSuccess() {
        return (name, isExportRtkStatus, isExportHr, exportSuccess) -> {
            IRecordsUtils recordsUtils = recordsUtilsMap.get(currentTypeFromSpinner);
            if (recordsUtils != null) {
                recordsUtils.exportRecord2ExcelAsync(name, isSortByTimeAcs, filterUtils.filterParameter(), headers, isExportRtkStatus, isExportHr,
                        exportSuccess);
            }
        };
    }

    public interface IExportSuccess {
        void showExportSuccessDialog(File file);
    }

}