package com.project.survey.ui.lofting.pointlofting.record.util;

import android.app.ProgressDialog;
import android.content.Context;

import androidx.annotation.UiThread;

import com.bingce.data.DeviceInfoDataUtils;
import com.bingce.data.cache.CachedCurrentJob;
import com.bingce.data.cache.CachedCurrentProject;
import com.bingce.data.database.PointStakeDb;
import com.bingce.data.surveyor.surveydata.pointstake.PointStakeConstants;
import com.bingce.data.surveyor.surveydata.pointstake.PointStakeRecord;
import com.bingce.poi.excel.ExcelExportHelper;
import com.bingce.rtk.model.GnssPosition;
import com.bingce.utils.DateUtils;
import com.bingce.utils.FileUtil;
import com.bingce.utils.Util;
import com.project.survey.R;
import com.project.survey.util.ExportPathUtils;
import com.project.survey.util.Utils;
import com.techyourchance.threadposter.BackgroundThreadPoster;
import com.techyourchance.threadposter.UiThreadPoster;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;

public class KnownPointRecordExporter {
    private final BackgroundThreadPoster mBackgroundThreadPoster = new BackgroundThreadPoster();
    private final UiThreadPoster mUiThreadPoster = new UiThreadPoster();
    private ProgressDialog mProgressDialog;

    public KnownPointRecordExporter() {
    }

    @UiThread
    public void exportKnownPointSetting2ExcelAsync(
            final String name, boolean ascOrder,
            String[] mHeaders61,
            String[] mHeaders62,
            String[] mHeaders63,
            boolean isExportRtkStatus,
            boolean isExportHr,
            Context context,
            RecordsActivityFilterParameter parameter,
            ICallback callback) {
        mProgressDialog = new ProgressDialog(context);
        mProgressDialog.setMessage(context.getString(R.string.export_records));
        mProgressDialog.show();

        mBackgroundThreadPoster.post(() -> {
            String projectId = CachedCurrentProject.currentProjectId();
            String jobId = CachedCurrentJob.currentJobId(projectId);
            List<PointStakeRecord> results = PointStakeDb.getInstance().rawQueryListData(
                    Utils.findRecordsListByRoadNameKRemarksDate(
                            PointStakeConstants.DB_NAME, projectId, jobId, ascOrder, parameter.road, parameter.k, parameter.remarks, parameter.start, parameter.end));
            if (results == null || results.isEmpty()) {
                ToastUtils.showShort(R.string.export_result_empty);
                hideLoading(context, null, null);
                return;
            }

            List<List<String>> exportResult = new ArrayList<>();
            String[] headers = mHeaders61;
            if (isExportRtkStatus) {
                String[] temp = new String[headers.length + mHeaders62.length];
                System.arraycopy(headers, 0, temp, 0, headers.length);
                System.arraycopy(mHeaders62, 0, temp, headers.length, mHeaders62.length);
                headers = temp;
            }
            if (isExportHr) {
                String[] temp = new String[headers.length + mHeaders63.length];
                System.arraycopy(headers, 0, temp, 0, headers.length);
                System.arraycopy(mHeaders63, 0, temp, headers.length, mHeaders63.length);
                headers = temp;
            }

            for (int i = 0; i < results.size(); i++) {
                ArrayList<String> rowResult = new ArrayList<>();
                rowResult.add(results.get(i).roadName);
                rowResult.add(results.get(i).pointName);
                rowResult.add(Util.formatDouble2StringDotAuto(results.get(i).designX));
                rowResult.add(Util.formatDouble2StringDotAuto(results.get(i).designY));
                rowResult.add(Util.formatDouble2StringDotAuto(results.get(i).designZ));
                rowResult.add(Util.formatDouble2StringDotAuto(results.get(i).measuredX));
                rowResult.add(Util.formatDouble2StringDotAuto(results.get(i).measuredY));
                rowResult.add(Util.formatDouble2StringDotAuto(results.get(i).measuredZ));
                rowResult.add(results.get(i).distance);
                rowResult.add(results.get(i).moveUpDown);
                rowResult.add(results.get(i).moveK);
                rowResult.add(results.get(i).moveD);
                rowResult.add(results.get(i).moveWestEast);
                rowResult.add(results.get(i).moveNorthSouth);
                rowResult.add(results.get(i).moveForeBack);
                rowResult.add(results.get(i).moveLeftRight);
                rowResult.add(results.get(i).remarks);
                rowResult.add(DateUtils.toFull(results.get(i).createDate));
                if (isExportRtkStatus) {
                    rowResult.add(DeviceInfoDataUtils.posTypeString(results.get(i).deviceInfoData, GnssPosition::posTypeString));
                    rowResult.add(DeviceInfoDataUtils.stationTypeString(results.get(i).deviceInfoData));
                    rowResult.add(DeviceInfoDataUtils.satelliteString(context, results.get(i).deviceInfoData));
                    rowResult.add(DeviceInfoDataUtils.hrmsString(results.get(i).deviceInfoData));
                    rowResult.add(DeviceInfoDataUtils.vrmsString(results.get(i).deviceInfoData));
                    rowResult.add(DeviceInfoDataUtils.diffAgeString(context, results.get(i).deviceInfoData));
                    rowResult.add(DeviceInfoDataUtils.electricityString(context, results.get(i).deviceInfoData));
                }
                if (isExportHr) {
                    rowResult.add(DeviceInfoDataUtils.hrString(results.get(i).deviceInfoData));
                }
                exportResult.add(rowResult);
            }
            if (!exportResult.isEmpty()) {
                File exportFolder = new File(FileUtil.getSDPath(), "工程测量导出文件"
                        + "/" + context.getString(R.string.record) + "/" + context.getString(R.string.known_point_staking));
                //过滤重名文件，如果有重名，添加后缀
                String exportFileName = ExportPathUtils.filterDuplicateFileName(exportFolder, name, "xls");

                new ExcelExportHelper().exportExcelForBigDataAndSave(headers, exportResult,
                        context.getString(R.string.known_point_staking),
                        ExcelExportHelper.MORE_SHEET_FLAG,
                        FileUtil.getSDPath() + "/" + "工程测量导出文件"
                                + "/" + context.getString(R.string.record) + "/" + context.getString(R.string.known_point_staking),
                        exportFileName);
                hideLoading(context, exportFileName, callback);
            } else {
                ToastUtils.showShort(R.string.export_result_empty);
            }
        });
    }

    private void hideLoading(Context activity, String name, ICallback callback) {
        mUiThreadPoster.post(() -> {
            mProgressDialog.dismiss();
            File path =
                    new File(FileUtil.getSDPath() + "/" + "工程测量导出文件"
                            + "/" + activity.getString(R.string.record) + "/" + activity.getString(R.string.known_point_staking) + "/" + name + ".xls");
            if (path.exists()) {
                callback.onSuccess(path);
            }
        });
    }

    public interface ICallback {
        void onSuccess(File file);
    }
}
