package com.project.survey.ui.lofting.pointlofting.record.util;

import android.view.ViewGroup;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AppCompatActivity;
import androidx.recyclerview.widget.DiffUtil;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.data.base.user.UserConfig;
import com.bingce.data.cache.CachedCurrentJob;
import com.bingce.data.cache.CachedCurrentProject;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.PointStakeDb;
import com.bingce.data.surveyor.surveydata.pointstake.PointStakeConstants;
import com.bingce.data.surveyor.surveydata.pointstake.PointStakeRecord;
import com.bingce.list.SwipeDeletePagingListLayoutHelper;
import com.bingce.utils.DateUtils;
import com.bingce.utils.FileUtil;
import com.bingce.utils.IProvider;
import com.bingce.utils.StringUtil;
import com.bingce.utils.Util;
import com.project.survey.R;
import com.project.survey.databinding.RecyclerviewItemRecordsBinding;
import com.project.survey.ui.lofting.pointlofting.record.RecordDetailActivity;
import com.project.survey.ui.lofting.pointlofting.record.RecordsActivity;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;

import com.project.survey.util.StoragePermissionUtils;
import com.project.survey.util.Utils;


class PointStakeListUtils extends AbstractListUtils {
    private final DiffUtil.ItemCallback<PointStakeRecord> callback = new Utils.ItemCallback<>();

    PointStakeListUtils(IProvider<AppCompatActivity> activityIProvider) {
        super(activityIProvider);
    }

    @Override
    public void update(RecyclerView recyclerView, boolean ascOrder,//asc升序 des，降序
                       UserConfig userConfig,
                       String currentProjectId, String currentJobId, String currentRoadId,
                       RecordsActivityFilterParameter parameter) {
        String road = parameter.road;
        String k = parameter.k;
        String remarks = parameter.remarks;
        Date start = parameter.start;
        Date end = parameter.end;
        SwipeDeletePagingListLayoutHelper.setup(activity(), activity(), recyclerView, callback,
                () -> PointStakeDb.getInstance().rawQueryPagingSource(
                        Utils.findRecordsListByRoadNameKRemarksDate(
                                PointStakeConstants.DB_NAME, currentProjectId, currentJobId, ascOrder, road, k, remarks, start, end)),
                new SwipeDeletePagingListLayoutHelper.IViewHolder0<PointStakeRecord, RecordItemVH>() {
                    @Override
                    public RecordItemVH newItemContentVH(@NonNull ViewGroup parent, int viewType) {
                        return new RecordItemVH(RecyclerviewItemRecordsBinding.inflate(getLayoutInflater()));
                    }

                    @Override
                    public void doBindContent(RecordItemVH viewHolder, PointStakeRecord record, int position) {
                        if (record == null) {
                            return;
                        }
                        String remarks = StringUtil.isEmpty(record.remarks) ? "" : record.remarks;
                        RecyclerViewItemData data = new RecyclerViewItemData(record.id,
                                record.pointName,
                                record.roadName,
                                Util.formatDouble2StringDotAuto(record.designX),
                                Util.formatDouble2StringDotAuto(record.designY),
                                Util.formatDouble2StringDotAuto(record.designZ),
                                getString(R.string.remarks) + ":" + (remarks.length() > 8 ? remarks.substring(0, 8) + "..." : remarks),
                                DateUtils.toFull(record.createDate));
                        viewHolder.doBinder(data);
                    }

                    @Override
                    public void onItemClicked(int index, PointStakeRecord instance) {
                        RecordDetailActivity.start(activity(), RecordTypeConstants.TYPE_POINT_STAKE, instance.id);
                    }

                    @Override
                    public void onDelete(int index, PointStakeRecord instance, SwipeDeletePagingListLayoutHelper.IOnDeleteListener listener) {
                        requestDelete(activity(), index, instance.id, listener);
                    }
                });
    }

    @Override
    public void update(RecyclerView recyclerView, boolean ascOrder, String currentProjectId, String currentJobId, RecordsActivityFilterParameter parameter) {
        String pointName = parameter.road;
        String k = parameter.k;
        String remarks = parameter.remarks;
        Date start = parameter.start;
        Date end = parameter.end;
        SwipeDeletePagingListLayoutHelper.setup(activity(), activity(), recyclerView, callback,
                () -> PointStakeDb.getInstance().rawQueryPagingSource(
                        Utils.findRecordsListByPointNameRemarksDate(
                                PointStakeConstants.DB_NAME, currentProjectId, currentJobId, ascOrder, pointName, remarks, start, end)),
                new SwipeDeletePagingListLayoutHelper.IViewHolder<PointStakeRecord, RecordItemVH>() {
                    @Override
                    public int itemViewType(int position) {
                        return 0;
                    }

                    @Override
                    public void onMenuOpen() {

                    }

                    @Override
                    public void onMenuClosed() {

                    }

                    @Override
                    public boolean isSwipeAble(PointStakeRecord instance) {
                        return false;
                    }

                    @Override
                    public RecordItemVH newItemContentVH(@NonNull ViewGroup parent, int viewType) {
                        return new RecordItemVH(RecyclerviewItemRecordsBinding.inflate(getLayoutInflater()));
                    }

                    @Override
                    public void doBindContent(RecordItemVH viewHolder, PointStakeRecord record, int position) {
                        if (record == null) {
                            return;
                        }
                        String remarks = StringUtil.isEmpty(record.remarks) ? "" : record.remarks;
                        RecyclerViewItemData data = new RecyclerViewItemData(record.id,
                                record.pointName,
                                record.roadName,
                                Util.formatDouble2StringDotAuto(record.designX),
                                Util.formatDouble2StringDotAuto(record.designY),
                                Util.formatDouble2StringDotAuto(record.designZ),
                                getString(R.string.remarks) + ":" + (remarks.length() > 8 ? remarks.substring(0, 8) + "..." : remarks),
                                DateUtils.toFull(record.createDate));
                        viewHolder.doBinder(data);
                    }

                    @Override
                    public void onItemClicked(int index, PointStakeRecord instance) {
                        RecordDetailActivity.start(activity(), RecordTypeConstants.TYPE_POINT_STAKE, instance.id);
                    }

                    @Override
                    public void onDelete(int index, PointStakeRecord instance, SwipeDeletePagingListLayoutHelper.IOnDeleteListener listener) {
                        requestDelete(activity(), index, instance.id, listener);
                    }
                });
    }

    @Override
    public void clearInWorkerThread(RecordsActivityFilterParameter parameter, Runnable completeCallback) {
        String projectId = CachedCurrentProject.currentProjectId();
        String jobId = CachedCurrentJob.currentJobId(projectId);
        List<PointStakeRecord> records = PointStakeDb.getInstance().rawQueryListData(
                Utils.findRecordsListByPointNameRemarksDate(
                        PointStakeConstants.DB_NAME, projectId, jobId, false, parameter.road, parameter.remarks, parameter.start, parameter.end));
        PointStakeDb.getInstance().delete(records);
//        activity().runOnUiThread(mProgressDialog::dismiss);
        if (completeCallback != null) {
            completeCallback.run();
        }
    }

    @Override
    public void exportRecord2ExcelAsync(
            String name, boolean ascOrder,
            RecordsActivityFilterParameter filterParameter, RecordsActivityHeaders headers, boolean isExportRtkStatus, boolean isExportHr, RecordsActivity.IExportSuccess exportSuccess) {
        StoragePermissionUtils.checkStoragePermission(context(), () -> {
            KnownPointRecordExporter exporter = new KnownPointRecordExporter();
            exporter.exportKnownPointSetting2ExcelAsync(name, ascOrder,
                    headers.mHeaders61,
                    headers.mHeaders62,
                    headers.mHeaders63,
                    isExportRtkStatus,
                    isExportHr,
                    context(),
                    filterParameter,
                    file -> {
                        FileUtil.scanFile(context(), file);
//                        showExportSuccessDialog(file);
                        exportSuccess.showExportSuccessDialog(file);
                    });
        });
    }

    @Override
    public void deleteSingleRecordInWorkerThread(String recordId) {
        PointStakeDb.getInstance().deleteById(recordId);
    }

    @Override
    public String name() {
        return getString(R.string.known_point_staking);
    }

    @Override
    public RecordDetailData findRecordDetailInWorkerThread(String recordId) {
        PointStakeRecord results = PointStakeDb.getInstance()
                .rawQueryData(DBQueryConstant.findById(PointStakeConstants.DB_NAME, recordId));
        return convert(results);
    }

    @Override
    public RecordDetailData findOtherRecordDetailInWorkerThread(String projectId, String jobId, Date date, boolean nextOrLast) {
        PointStakeRecord results = PointStakeDb.getInstance()
                .rawQueryData(Utils.findOtherRecord(PointStakeConstants.DB_NAME, projectId, jobId, date, nextOrLast));
        return convert(results);
    }

    @Override
    public RecordDetailData updateRemarksInWorkerThread(String recordId, String remarks) {
        PointStakeRecord results = PointStakeDb.getInstance()
                .rawQueryData(DBQueryConstant.findById(PointStakeConstants.DB_NAME, recordId));
        if (results != null) {
            results.remarks = remarks;
            PointStakeDb.getInstance().save(results);
        }
        return convert(results);
    }

    private RecordDetailData convert(PointStakeRecord results) {
        if (results == null) {
            return null;
        }
        LinkedHashMap<String, String> map = new LinkedHashMap<>();
        map.put(getString(R.string.road_name), results.roadName);
        map.put(getString(R.string.point_name), results.pointName);
        map.put(getString(R.string.design_x), Util.formatDouble2StringDotAuto(results.designX));
        map.put(getString(R.string.design_y), Util.formatDouble2StringDotAuto(results.designY));
        map.put(getString(R.string.design_z), Util.formatDouble2StringDotAuto(results.designZ));
        map.put(getString(R.string.measured_x), Util.formatDouble2StringDotAuto(results.measuredX));
        map.put(getString(R.string.measured_y), Util.formatDouble2StringDotAuto(results.measuredY));
        map.put(getString(R.string.measured_z), Util.formatDouble2StringDotAuto(results.measuredZ));
        map.put(getString(R.string.distance), results.distance);
        map.put(getString(R.string.move_up_down), results.moveUpDown);
        if (!StringUtil.isEmpty(results.moveK) || !StringUtil.isEmpty(results.moveD)) {
            map.put(getString(R.string.move_k), results.moveK);
            map.put(getString(R.string.move_d), results.moveD);
        }
        if (!StringUtil.isEmpty(results.moveWestEast) || !StringUtil.isEmpty(results.moveNorthSouth)) {
            map.put(getString(R.string.move_west_east), results.moveWestEast);
            map.put(getString(R.string.move_north_south), results.moveNorthSouth);
        }
        if (!StringUtil.isEmpty(results.moveForeBack) || !StringUtil.isEmpty(results.moveLeftRight)) {
            map.put(getString(R.string.move_fore_back), results.moveForeBack);
            map.put(getString(R.string.move_left_right), results.moveLeftRight);
        }
        map.put(getString(R.string.remarks), results.remarks);
        map.put(getString(R.string.record_time), DateUtils.toFull(results.createDate));
        return new RecordDetailData(results.id, map, results.deviceInfoData, results.createDate, results.remarks);
    }
}
