package com.project.survey.ui.lofting.pointlofting.record.util;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;
import android.widget.CheckBox;

import androidx.annotation.NonNull;

import com.afollestad.materialdialogs.DialogAction;
import com.afollestad.materialdialogs.MaterialDialog;
import com.afollestad.materialdialogs.internal.MDButton;
import com.bingce.file.FileOperator;
import com.bingce.utils.ContextProviderUtils;
import com.bingce.utils.FileUtil;
import com.bingce.utils.IProvider;
import com.hjq.permissions.OnPermissionCallback;
import com.hjq.permissions.Permission;
import com.hjq.permissions.XXPermissions;
import com.project.survey.App;
import com.project.survey.BuildConfig;
import com.project.survey.R;
import com.project.survey.ui.lofting.pointlofting.record.RecordsActivity;
import com.rengwuxian.materialedittext.MaterialEditText;

import java.io.File;
import java.util.List;

import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;


public class RecordsActivityExportUtils extends ContextProviderUtils {
    private MDButton mPositiveAction;
    private MaterialEditText met_export_name;
    private CheckBox checkBoxRtkStatus, checkBoxHr;
    private boolean isExportRtkStatus = true, isExportHr = true;

    private final IProvider<Integer> currentTypeFromSpinner;

    public RecordsActivityExportUtils(IProvider<Context> contextIProvider, IProvider<Integer> currentTypeFromSpinner) {
        super(contextIProvider);
        this.currentTypeFromSpinner = currentTypeFromSpinner;
    }

    void showExportDialog(IExportCallback exportCallback) {
        MaterialDialog dialog = new MaterialDialog.Builder(context())
                .title(R.string.export_excel)
                .customView(R.layout.dialog_export_record_config, true)
                .positiveText(R.string.export_records)
                .negativeText(R.string.cancel)
                .onPositive(new MaterialDialog.SingleButtonCallback() {
                    @Override
                    public void onClick(@NonNull MaterialDialog dialog,
                                        @NonNull DialogAction which) {
                        String name;
                        if ("".equals(met_export_name.getText().toString().trim())) {
                            ToastUtils.showShort(R.string.file_name_can_not_empty);
                            return;
                        } else {
                            name = met_export_name.getText().toString().trim();
                        }

                        if (isIgnoreDeviceExport()) {
                            isExportRtkStatus = true;
                            isExportHr = true;
                        } else {
                            isExportRtkStatus = checkBoxRtkStatus.isChecked();
                            isExportHr = checkBoxHr.isChecked();
                        }

//                        IRecordsUtils recordsUtils = recordsUtilsMap.get(currentTypeFromSpinner);
//                        if (recordsUtils != null) {
//                            recordsUtils.exportRecord2ExcelAsync(RecordsActivity.this, name, filterParameter(), headers, isExportRtkStatus, isExportHr,
//                                    file -> showExportSuccessDialog(file));
//                        }
                        if (exportCallback != null) {
                            exportCallback.onExport(name, isExportRtkStatus, isExportHr, file -> showExportSuccessDialog(file));
                        }
                    }
                }).build();

        met_export_name =
                dialog.getCustomView().findViewById(R.id.dialog_export_record_config_name);
        checkBoxRtkStatus =
                dialog.getCustomView().findViewById(R.id.dialog_export_record_config_check_rtk_status);
        checkBoxHr = dialog.getCustomView().findViewById(R.id.dialog_export_record_config_check_hr);
        mPositiveAction = dialog.getActionButton(DialogAction.POSITIVE);

        if (isIgnoreDeviceExport()) {
            checkBoxRtkStatus.setVisibility(View.GONE);
            checkBoxHr.setVisibility(View.GONE);
        } else {
            checkBoxRtkStatus.setVisibility(View.VISIBLE);
            checkBoxHr.setVisibility(View.VISIBLE);
            checkBoxRtkStatus.setChecked(isExportRtkStatus);
            checkBoxHr.setChecked(isExportHr);
        }

        initMaterialEditTextColor();

        met_export_name.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                if (!"".equals(met_export_name.getText().toString().trim())) {
                    mPositiveAction.setEnabled(true);
                } else {
                    mPositiveAction.setEnabled(false);
                }
            }
        });
        dialog.show();
        mPositiveAction.setEnabled(false);
    }

    void showExportSuccessDialog(final File file) {
        new MaterialDialog.Builder(context()).title(R.string.export_successful)
                .content("导出文件保存在手机根目录的[工程测量导出文件]文件夹内")
                .positiveText(R.string.send_to)
                .negativeText(R.string.open)
                .neutralText(R.string.close)
                .onPositive((dialog, which) -> FileOperator.shareFile(context(), file, BuildConfig.APPLICATION_ID))
                .onNegative((dialog, which) -> FileUtil.openFile(context(), file)).show();
    }

    private void initMaterialEditTextColor() {
        if (App.instance.isThemeDark()) {
            met_export_name.setMetTextColor(Color.WHITE);
            met_export_name.setPrimaryColor(Color.LTGRAY);
            met_export_name.setFocusFraction(1.0f);
            met_export_name.setMetHintTextColor(Color.GRAY);
            met_export_name.setUnderlineColor(Color.GRAY);
        }
    }


    /**
     * 是否忽略导出（包含RTK状态，包含杆高/镜高）
     *
     * @return true 忽略
     */
    private boolean isIgnoreDeviceExport() {
        return currentTypeFromSpinner.value() == RecordTypeConstants.TYPE_LEVEL
                || currentTypeFromSpinner.value() == RecordTypeConstants.TYPE_SUBWAY_SEGMENT;
    }

    public void requestStoragePermission(Activity activity, IExportCallback exportCallback) {
        XXPermissions.with(activity)
                .permission(Permission.MANAGE_EXTERNAL_STORAGE)
                .request(new OnPermissionCallback() {
                    @Override
                    public void onGranted(List<String> permissions, boolean all) {
                        showExportDialog(exportCallback);
                    }

                    @Override
                    public void onDenied(List<String> permissions, boolean never) {
                        if (never) {
                            ToastUtils.showShort(getString(R.string.manually_grant_read_and_write_permissions_to_files));
                            // 如果是被永久拒绝就跳转到应用权限系统设置页面
                            XXPermissions.startPermissionActivity(activity, permissions);
                        } else {
                            requestStoragePermission(activity, exportCallback);
                        }
                    }
                });
    }

    public interface IExportCallback {
        void onExport(String name, boolean isExportRtkStatus, boolean isExportHr, RecordsActivity.IExportSuccess exportSuccess);
    }
}
