package com.project.survey.ui.lofting.pointlofting.record.util

import androidx.activity.viewModels
import androidx.annotation.WorkerThread
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.bingce.data.base.user.UserConfig
import com.bingce.data.database.DBQueryConstant
import com.bingce.data.database.RoadDb
import com.bingce.data.surveyor.designdata.road.RoadConstants
import com.bingce.data.surveyor.designdata.road.RoadRecord
import com.bingce.http.measuringtape.tryLoadMeasuringTapeRecord
import com.bingce.data.sync.utils.loadCurrentJobId
import com.bingce.data.sync.utils.loadCurrentProjectId
import com.bingce.data.sync.utils.loadCurrentRoadId
import com.bingce.data.sync.utils.loadUserConfig
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch

class RecordsViewModelKt : ViewModel() {
    fun loadRecordBaseInfo(callback: ILoadProjectJobRoadCallback) {
        viewModelScope.launch(Dispatchers.IO) {
            val data = loadRecordBaseData() ?: return@launch
            val roadName = RoadDb.getInstance().rawQueryData(
                DBQueryConstant.findById(RoadConstants.DB_NAME, data.roadId)
            )?.name ?: return@launch

            callback.onLoadedInWorkThread(data.projectId, data.jobId, data.roadId, roadName)
        }
    }

    fun loadRecordBaseData(callback: ILoadProjectIdJobIdRoadRecordCallback) {
        viewModelScope.launch(Dispatchers.IO) {
            val data = loadRecordBaseData() ?: return@launch
            val roadRecord = RoadDb.getInstance().rawQueryData(
                DBQueryConstant.findById(RoadConstants.DB_NAME, data.roadId)
            ) ?: return@launch

            callback.onLoadedInWorkThread(data.projectId, data.jobId, roadRecord)
        }
    }

    @WorkerThread
    fun loadRecordBaseData(): RecordBaseData? {
        val userConfig = loadUserConfig()

        val projectId = loadCurrentProjectId(true, userConfig) ?: return null
        val jobId = loadCurrentJobId(projectId, true, userConfig) ?: return null
        val roadId = loadCurrentRoadId(projectId, true, userConfig) ?: return null
        return RecordBaseData(projectId, jobId, roadId)
    }

    fun loadMeasuringTapeRecord(userConfig: UserConfig?, roadId: String) {
        viewModelScope.launch(Dispatchers.IO) {
            tryLoadMeasuringTapeRecord(userConfig, roadId)
        }
    }

    interface ILoadProjectJobRoadCallback {
        @WorkerThread
        fun onLoadedInWorkThread(
            currentProjectId: String,
            currentJobId: String,
            currentRoadId: String,
            currentRoadName: String
        )
    }

    interface ILoadProjectIdJobIdRoadRecordCallback {
        @WorkerThread
        fun onLoadedInWorkThread(
            currentProjectId: String,
            currentJobId: String,
            currentRoad: RoadRecord
        )
    }

    data class RecordBaseData(
        val projectId: String,
        val jobId: String,
        val roadId: String,
    )

    class HolderInFragment(fragment: Fragment) {
        val viewModel: RecordsViewModelKt by fragment.viewModels()
    }

    class HolderInActivity(activity: AppCompatActivity) {
        val viewModel: RecordsViewModelKt by activity.viewModels()
    }
}