package com.project.survey.ui.login

import androidx.activity.viewModels
import com.gyf.immersionbar.ImmersionBar
import com.project.survey.R
import com.project.survey.databinding.ActivityLoginBinding
import com.project.survey.extend.isVisibleOrGone
import com.project.survey.extend.isVisibleOrInvisible
import com.project.survey.extend.setOnClickNoRepeatListener
import com.project.survey.extend.toast
import com.project.survey.logic.viewmodel.LoginViewModel
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.util.ActivityNavUtil
import com.project.survey.util.LogoutUtil
import com.project.survey.util.Tools

class LoginActivity : BaseBindingActivity<ActivityLoginBinding>() {

    companion object {
        fun start() {
            LogoutUtil.clearSP()

            ActivityNavUtil.finishAllActivities()
            ActivityNavUtil.startActivity<LoginActivity> { }
        }
    }

    override fun getBinding(): ActivityLoginBinding = ActivityLoginBinding.inflate(layoutInflater)

    private val viewModel: LoginViewModel by viewModels()

    private var isOutLogin = false//默认集团登录

    override fun initView() {
        ImmersionBar.with(this).statusBarDarkFont(true).init()
    }

    override fun initData() {
        viewModel.errorResponse.observe(this) {
            mBinding.tvError.text = it
            mBinding.tvError.isVisibleOrInvisible(true)
        }
    }

    override fun initListener() {
        //切换到外部登录
        mBinding.tvShowOutLogin.setOnClickNoRepeatListener {
            isOutLogin = true
            mBinding.tvAccountDesc.text = Tools.getString(R.string.external_account_login)
            mBinding.llShowOutLogin.isVisibleOrGone(false)
            mBinding.llShowInnerLogin.isVisibleOrGone(true)
        }
        //切换到内部集团登录
        mBinding.tvShowInnerLogin.setOnClickNoRepeatListener {
            isOutLogin = false
            mBinding.tvAccountDesc.text = Tools.getString(R.string.group_account_login)
            mBinding.llShowOutLogin.isVisibleOrGone(true)
            mBinding.llShowInnerLogin.isVisibleOrGone(false)
        }

        // 登录
        mBinding.btnLogin.setOnClickNoRepeatListener {
            val account = mBinding.etAccount.text.toString()
            val password = mBinding.passwordView.getText()
            if (account.isBlank() || password.isNullOrBlank()) {
                toast("账号或密码不能为空")
                return@setOnClickNoRepeatListener
            }
            viewModel.externalPersonLogin(isOutLogin, account, password, "123")
//            viewModel.fetchLoginSMSCode(isOutLogin, account, password)
        }

        //忘记密码
        mBinding.tvForgetPwd.setOnClickNoRepeatListener {
            ForgetPwdActivity.start()
        }
    }
}