package com.project.survey.ui.login

import androidx.activity.viewModels
import androidx.lifecycle.lifecycleScope
import com.project.survey.databinding.ActivityLoginInputCodeBinding
import com.project.survey.extend.showSoftInput
import com.project.survey.extend.toast
import com.project.survey.logic.viewmodel.LoginViewModel
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.util.ActivityNavUtil
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch

class LoginInputCodeActivity : BaseBindingActivity<ActivityLoginInputCodeBinding>() {

    companion object {
        const val IS_OUT_LOGIN = "isOutLogin"
        const val ACCOUNT = "account"
        const val PASSWORD = "password"
        const val MOBILE = "mobile"

        fun start(isOutLogin: Boolean,account: String, password: String, mobile: String) {
            ActivityNavUtil.startActivity<LoginInputCodeActivity> {
                putBoolean(IS_OUT_LOGIN, isOutLogin)
                putString(ACCOUNT, account)
                putString(PASSWORD, password)
                putString(MOBILE, mobile)
            }
        }
    }

    override fun getBinding(): ActivityLoginInputCodeBinding =
        ActivityLoginInputCodeBinding.inflate(layoutInflater)

    private val viewModel: LoginViewModel by viewModels()

    private var isOutLogin: Boolean = false
    private var account: String = ""
    private var password: String = ""
    private var mobile: String = ""

    override fun initView() {
        isOutLogin = intent.getBooleanExtra(IS_OUT_LOGIN, false)
        account = intent.getStringExtra(ACCOUNT) ?: ""
        password = intent.getStringExtra(PASSWORD) ?: ""
        mobile = intent.getStringExtra(MOBILE) ?: ""

        immersionToolbar(mBinding.toolbar)

        lifecycleScope.launch {
            delay(500L)
            mBinding.etInputCode.firstEditText?.showSoftInput()
        }

//        if (mobile.length == 11) {
//            mBinding.tvTips.text =
//                String.format(
//                    getString(R.string.the_verification_code_was_sent_for_login),
//                    mobile.replace("(\\d{3})\\d{4}(\\d{4})","$1****$2")
//                )
//        }
    }

    override fun initData() {
        viewModel.errorResponse.observe(this){
            toast(it)
        }
    }

    override fun initListener() {
        mBinding.etInputCode.setOnInputListener {
            viewModel.externalPersonLogin(isOutLogin, account, password, it)
        }
    }

}