package com.project.survey.ui.pointmeasure

import android.net.Uri
import androidx.activity.viewModels
import com.afollestad.materialdialogs.MaterialDialog
import com.bingce.surveyor.agentweb.AgentWebActivity
import com.project.survey.BuildConfig
import com.project.survey.R
import com.project.survey.constants.AdapterConstants
import com.project.survey.constants.Constants
import com.project.survey.constants.SPConstants
import com.project.survey.databinding.ActivityPointMeasureListBinding
import com.project.survey.extend.smartDismiss
import com.project.survey.extend.toast
import com.project.survey.logic.viewmodel.ProjectViewModel
import com.project.survey.ui.MainActivity
import com.project.survey.ui.base.BaseBindingActivity
import com.project.survey.ui.login.LoginActivity
import com.project.survey.ui.pointmeasure.adapter.PointMeasureListAdapter
import com.project.survey.ui.pointmeasure.measure.PointSurveyNewActivity
import com.project.survey.util.ActivityNavUtil
import com.project.survey.util.LogoutUtil
import com.project.survey.util.SPUtils
import com.scwang.smart.refresh.layout.api.RefreshLayout
import com.scwang.smart.refresh.layout.listener.OnRefreshLoadMoreListener

/**
 * 点测量列表
 */
class PointMeasureListActivity : BaseBindingActivity<ActivityPointMeasureListBinding>() {

    companion object {
        const val KEY_TYPE = "KEY_TYPE"

        fun start(type: String) {
            ActivityNavUtil.startActivity<PointMeasureListActivity> {
                putString(KEY_TYPE, type)
            }
        }
    }

    private val viewModel: ProjectViewModel by viewModels()

    private val adapter by lazy { PointMeasureListAdapter(type) }
    private var type = ""
//    private var currentPage = 1

    //    private var start = 0
    private var searchContent = "" // 搜索框里的内容

    override fun getBinding(): ActivityPointMeasureListBinding =
        ActivityPointMeasureListBinding.inflate(layoutInflater)

    override fun initView() {
        type = intent.getStringExtra(KEY_TYPE) ?: ""
        immersionToolbar(
            mBinding.toolbar, when (type) {
                Constants.TYPE_BD_CLJH -> getString(R.string.survey_plan)
                Constants.TYPE_BD_CLGZ_GCJL -> getString(R.string.engineering_metrology)
                else -> ""
            }
        )
        mBinding.smartRefreshLayout.setEnableLoadMore(false)
        mBinding.smartRefreshLayout.setEnableRefresh(false)
//        mBinding.smartRefreshLayout.setOnRefreshLoadMoreListener(this)
        mBinding.recyclerView.setAdapter(adapter)
        adapter.isStateViewEnable = true
    }

    override fun initData() {
        viewModel.errorResponse.observe(this) {
            toast(it)
            mBinding.smartRefreshLayout.smartDismiss()
        }
        // 列表
        viewModel.measureBeanListResponse.observe(this) {
            mBinding.smartRefreshLayout.smartDismiss()
//            if (currentPage == 1) {
                adapter.submitList(it.rows)
                if (it.rows.isEmpty()) {
                    mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
                    adapter.setStateViewLayout(this, R.layout.layout_no_data)
                }
//                else {
//                    currentPage++
//                }
//            } else {
//                if (it.rows.isEmpty()) {
//                    mBinding.smartRefreshLayout.finishLoadMoreWithNoMoreData()
//                } else {
//                    adapter.addAll(it.rows)
//                    currentPage++
//                }
//            }
        }
        // 生成报验单
        viewModel.createInspectionReportResponse.observe(this) {
            navigation2BaoYanDan(it.BD_CLJH_ID)
            val position = adapter.items.indexOfFirst { bean -> bean.BD_CLJH_ID == it.BD_CLJH_ID }
            if (position > -1) {
                adapter.getItem(position)?.let { bean ->
                    bean.CLJH_WORKSTATUS = "报验单已生成"
                    adapter.notifyItemChanged(position, AdapterConstants.WORK_STATUS)
                }
            }
        }
        //
        viewModel.updateMeasureWorkStatusResponse.observe(this) {
//            val position = adapter.items.indexOfFirst { bean -> bean.BD_CLJH_ID == it.id }
//            if (position > -1) {
//                adapter.getItem(position)?.let { bean ->
//                    bean.CLJH_WORKSTATUS = it.status
//                    adapter.notifyItemChanged(position, AdapterConstants.WORK_STATUS)
//                }
//            }
            //刷新数据
            fetchData()
        }

        // 获取列表数据
        fetchData()
    }

    override fun initListener() {
        // 搜索
        mBinding.searchWidget.setOnEditorSearchListener {
            searchContent = it
//            currentPage = 1
            fetchData()
        }
        // 生成报验单
        adapter.addOnItemChildClickListener(R.id.tvGenerateForm) { adapter, _, position ->
            adapter.getItem(position)?.let { bean ->
//                if ("报验单已生成" == bean.CLJH_WORKSTATUS){
//                    navigation2BaoYanDan(bean.BD_CLJH_ID)
//                } else {
                    viewModel.createInspectionReport(bean.BD_CLJH_ID)
//                }
            }
        }
        // 结束测量
        adapter.addOnItemChildClickListener(R.id.tvStopMeasure) { adapter, _, position ->
            adapter.getItem(position)?.let { bean ->
                MaterialDialog.Builder(mContext)
                    .title("确定结束测量吗?")
                    .negativeText("取消")
                    .positiveText("确定")
                    .onPositive { _, _ ->
                        val workId =
                            if (type == Constants.TYPE_BD_CLJH) bean.BD_CLJH_ID else bean.BD_CLGZ_GCJL_ID
                        viewModel.updateMeasureWorkStatus(workId, type, "已结束")
                    }
                    .show()
            }
        }
        // 进入测量
        adapter.addOnItemChildClickListener(R.id.tvEnterMeasure) { adapter, _, position ->
            adapter.getItem(position)?.let { bean ->
                val workId =
                    if (type == Constants.TYPE_BD_CLJH) bean.BD_CLJH_ID else bean.BD_CLGZ_GCJL_ID
                val status =
                    if (type == Constants.TYPE_BD_CLJH) bean.CLJH_WORKSTATUS else bean.GCJL_WORKSTATUS
                if ("待执行" == status || status.isNullOrBlank()) {
                    viewModel.updateMeasureWorkStatus(workId, type, "执行中")
                }
                PointSurveyNewActivity.start(type, workId)
            }
        }
    }

    private fun fetchData() {
        viewModel.fetchMeasureDataList(type, 1, searchContent)
    }

    /**
     * 报验单 Web 页面
     */
    private fun navigation2BaoYanDan(workId: String){
        // http://10.190.183.37/app/micro/func/#/pages/func/form/BD_CLJH/{workId}?authorization={登录token}&appId=2024-1104-1503-4541
        val token = SPUtils.getString(SPConstants.TOKEN)
        val url =
            BuildConfig.BASE_URL + "app/micro/func/#/pages/func/form/BD_CLJH/${workId}?authorization=${token}&appId=2024-1104-1503-4541"
        AgentWebActivity.navigation2(this, Uri.parse(url))
    }


//    override fun onRefresh(refreshLayout: RefreshLayout) {
//        currentPage = 1
//        fetchData()
//    }
//
//    override fun onLoadMore(refreshLayout: RefreshLayout) {
//        fetchData()
//    }


}