package com.project.survey.ui.pointmeasure.measure;



import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_LINE;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_POINT;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_SURFACE;

import android.content.Context;
import android.content.Intent;
import android.graphics.drawable.GradientDrawable;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;

import com.bingce.data.database.CodeDb;
import com.bingce.data.surveyor.surveydata.code.CodeRecord;

import com.bingce.surveyor.util.ConstUtils;

import com.bingce.surveyor.util.PreferencesUtil;
import com.bingce.surveyor.util.dialog.CustomRecycleDialog;
import com.bingce.utils.IdUtils;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.google.gson.Gson;
import com.project.survey.R;
import com.project.survey.databinding.ActivityAddCodeBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.LauncherEvent;
import com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils;
import com.project.survey.util.CommonUtils;
import com.project.survey.util.DrawableUtils;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.ToastUtils;


public class AddCodeActivity extends BaseSurveyNewActivity {

    private ActivityAddCodeBinding binding;
    private List<String> codeTypeList = new ArrayList<>();
    private int codeShapeTypeIndex = 0;//0：点，1：线，2：面
    private int codeShapeColor;//点和线的颜色，面的填充色。
    private int codeShapeSizeIndex = 0;//点、线和面的轮廓线的尺寸大小。
    private int codeStyleIndexPoint = 0;//点的默认选中的下坐标。
    private int codeStyleIndexLine = 0;//线的默认选中的下坐标。
    private int contourColor;//面的轮廓颜色。
    private String codeId;

    @Override
    public View getContentView() {
        binding = ActivityAddCodeBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.add) + getString(R.string.point_code));
    }

    @Override
    protected void initData() {

        String json = IntentUtil.stringExtra(this, CODE_RECORD);

        if (!TextUtils.isEmpty(json)){
            CodeRecord codeRecord = new Gson().fromJson(json, CodeRecord.class);
            codeId = codeRecord.id;
            codeShapeTypeIndex = codeRecord.codeGroupType;
            binding.etCodeName.setText(codeRecord.codeName);
            binding.etCode.setText(codeRecord.code);
            codeShapeSizeIndex = codeRecord.codeSizeShape;
            switch (codeRecord.codeGroupType){
                case SHAPE_TYPE_POINT:
                    codeStyleIndexPoint = codeRecord.codeFeatureShape;
                    codeShapeColor = codeRecord.colorShape;
                    contourColor = codeRecord.colorShape;
                    break;
                case SHAPE_TYPE_LINE:
                    codeStyleIndexLine = codeRecord.codeFeatureShape;
                    codeShapeColor = codeRecord.colorShape;
                    contourColor = codeRecord.colorShape;
                    break;
                case SHAPE_TYPE_SURFACE:
                    codeStyleIndexLine = codeRecord.codeFeatureShape;
                    codeShapeColor = codeRecord.colorShape;
                    contourColor = codeRecord.codeColorContour;
                    break;
            }
        }else {
            codeId = IdUtils.getUUID();
            codeShapeTypeIndex = PreferencesUtil.getAddCodeTypeIndex();
            codeShapeColor = getResources().getColor(R.color.theme_green);
            contourColor = codeShapeColor;
        }

        codeTypeList.add(getString(R.string.point));
        codeTypeList.add(getString(R.string.line));
        codeTypeList.add(getString(R.string.surface));

        setCodeStyle(codeShapeTypeIndex, codeShapeColor);

        binding.ivCodeColorPalette.setImageDrawable(DrawableUtils.tintModifyColorDrawable(getDrawable(R.drawable.icon_code_color_palette),codeShapeColor));

        binding.rlCodeType.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                CustomRecycleDialog.showDialog(AddCodeActivity.this, R.string.type, codeTypeList, codeShapeTypeIndex, false, new CustomRecycleDialog.IClickCallback() {
                    @Override
                    public void onItemSelected(int index, String itemString) {
                        codeShapeTypeIndex = index;
                        setCodeStyle(index, codeShapeColor);
                        PreferencesUtil.putPreference(ConstUtils.preferConst.addCodeTypeIndex, index);
                    }
                });
            }
        });

        binding.rlCodeColor.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                PickerColorActivity.start(launcher, AddCodeActivity.this, codeShapeColor);
            }
        });

        binding.rlCodeStyle.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (codeShapeTypeIndex == SHAPE_TYPE_POINT || codeShapeTypeIndex == SHAPE_TYPE_LINE){
                    CodeStylePointLineActivity.start(launcher, AddCodeActivity.this, codeShapeTypeIndex, codeShapeColor, codeShapeSizeIndex, codeStyleIndexPoint,codeStyleIndexLine);
                }else if (codeShapeTypeIndex == SHAPE_TYPE_SURFACE){
                    CodeStyleSurfaceActivity.start(launcher,AddCodeActivity.this,codeShapeColor,contourColor,codeShapeSizeIndex,codeStyleIndexLine);
                }
            }
        });

        binding.addCodeBtnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                String codeName = binding.etCodeName.getText().toString();
                String code = binding.etCode.getText().toString();
                if (TextUtils.isEmpty(codeName)){
                    ToastUtils.showShort(getString(R.string.enter_code_name));
                    return;
                }
                if (TextUtils.isEmpty(code)){
                    ToastUtils.showShort(getString(R.string.enter_code));
                    return;
                }
                ThreadPoolUtil.execute(new Runnable() {
                    @Override
                    public void run() {

                        CodeRecord codeRecord = new CodeRecord();
                        codeRecord.id = codeId;
                        codeRecord.codeGroupType = codeShapeTypeIndex;
                        codeRecord.codeName = codeName;
                        codeRecord.code = code;
                        codeRecord.colorShape = codeShapeColor;
                        codeRecord.codeSizeShape = codeShapeSizeIndex;
                        switch (codeShapeTypeIndex){
                            case SHAPE_TYPE_POINT:
                                codeRecord.codeFeatureShape = codeStyleIndexPoint;
                                codeRecord.codeColorContour = codeShapeColor;//点线时候轮廓线默认为colorShape
                                break;
                            case SHAPE_TYPE_LINE:
                                codeRecord.codeFeatureShape = codeStyleIndexLine;
                                codeRecord.codeColorContour = codeShapeColor;//点线时候轮廓线默认为colorShape
                                break;
                            case SHAPE_TYPE_SURFACE:
                                codeRecord.codeFeatureShape = codeStyleIndexLine;
                                codeRecord.codeColorContour = contourColor;//面的时候使用
                                break;
                        }

                        CodeDb.getInstance().save(codeRecord);
                        ToastUtils.showShort(getString(R.string.add_success));
                        Intent intent = new Intent();
                        intent.putExtra(ConstUtils.intentConst.keyUpdateCodeRecord, new Gson().toJson(codeRecord));
                        setResult(LauncherEvent.launcher_code_library_update,intent);
                        finish();
                    }
                });
            }
        });
    }

    /**
     * 样式
     * @param index
     */
    private void setCodeStyle(int index,int color) {
        binding.tvCodeType.setText(codeTypeList.get(index));
        switch (index){
            case SHAPE_TYPE_POINT://点
                binding.poinView.setVisibility(View.VISIBLE);
                binding.lineView.setVisibility(View.GONE);
                binding.faceView.setVisibility(View.GONE);
                binding.rlCodeColor.setVisibility(View.VISIBLE);

                ViewGroup.LayoutParams params = binding.poinView.getLayoutParams();
                switch (codeShapeSizeIndex){
                    case 0://小尺寸
                        params.width = CommonUtils.dip2px(5);
                        params.height = CommonUtils.dip2px(5);
                        break;
                    case 1://中尺寸
                        params.width = CommonUtils.dip2px(10);
                        params.height = CommonUtils.dip2px(10);
                        break;
                    case 2://大尺寸
                        params.width = CommonUtils.dip2px(15);
                        params.height = CommonUtils.dip2px(15);
                        break;
                }
                binding.poinView.setLayoutParams(params);

                binding.poinView.setImageDrawable(DrawableUtils.tintModifyColorDrawable(getDrawable(CodeConsUtils.showPointIcon(codeStyleIndexPoint)),color));
                break;

            case SHAPE_TYPE_LINE://线
                binding.poinView.setVisibility(View.GONE);
                binding.lineView.setVisibility(View.VISIBLE);
                binding.faceView.setVisibility(View.GONE);
                binding.rlCodeColor.setVisibility(View.VISIBLE);

                GradientDrawable gdLine = (GradientDrawable)binding.lineView.getBackground();
                int lineWidth = 0;
                switch (codeShapeSizeIndex){
                    case 0:
                        lineWidth = CommonUtils.dip2px(0.8f);
                        break;
                    case 1:
                        lineWidth = CommonUtils.dip2px(1.3f);
                        break;
                    case 2:
                        lineWidth = CommonUtils.dip2px(1.8f);
                        break;
                }
                gdLine.setStroke(lineWidth,color,CommonUtils.dip2px(codeStyleIndexLine + codeStyleIndexLine * 2),CommonUtils.dip2px(codeStyleIndexLine + codeStyleIndexLine * 2));
                break;

            case SHAPE_TYPE_SURFACE://面
                binding.poinView.setVisibility(View.GONE);
                binding.lineView.setVisibility(View.GONE);
                binding.faceView.setVisibility(View.VISIBLE);
                binding.rlCodeColor.setVisibility(View.GONE);

                GradientDrawable gradientDrawable = (GradientDrawable)binding.faceView.getBackground();
                int conturLineWidth = 0;
                switch (codeShapeSizeIndex){
                    case 0:
                        conturLineWidth = CommonUtils.dip2px(0.8f);
                        break;
                    case 1:
                        conturLineWidth = CommonUtils.dip2px(1.3f);
                        break;
                    case 2:
                        conturLineWidth = CommonUtils.dip2px(1.8f);
                        break;
                }
                gradientDrawable.setStroke(conturLineWidth, contourColor,CommonUtils.dip2px(codeStyleIndexLine + codeStyleIndexLine * 2),CommonUtils.dip2px(codeStyleIndexLine + codeStyleIndexLine * 2));
                gradientDrawable.setColor(color);
                
                break;
        }
    }

    private static String CODE_RECORD = "codeRecord";

    public static void start(ActivityResultLauncher<Intent> launcher,Context context,String codeRecordJson){
        Intent intent = new Intent(context, AddCodeActivity.class);
        intent.putExtra(CODE_RECORD,codeRecordJson);
        launcher.launch(intent);
    }

    public final ActivityResultLauncher<Intent> launcher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result != null) {
            switch (result.getResultCode()){
                case LauncherEvent.launcher_add_code_color:
                    int color = result.getData().getIntExtra(ConstUtils.intentConst.keyAddCodeColor, -1);
                    codeShapeColor = color;
                    setCodeStyle(codeShapeTypeIndex,color);
                    binding.ivCodeColorPalette.setImageDrawable(DrawableUtils.tintModifyColorDrawable(getDrawable(R.drawable.icon_code_color_palette),color));
                    break;
                case LauncherEvent.launcher_code_style:
                    codeStyleIndexPoint = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeStyleIndexPoint,0);
                    codeStyleIndexLine = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeStyleIndexLine,0);
                    codeShapeSizeIndex = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeShapeSize,0);
                    setCodeStyle(codeShapeTypeIndex, codeShapeColor);
                    break;
                case LauncherEvent.launcher_code_shape_surface:
                    codeStyleIndexLine = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeStyleIndexLine,0);
                    codeShapeSizeIndex = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeShapeSize,0);
                    contourColor = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeShapeContourColor, -1);
                    int fillColor = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeShapeFillColor, -1);
                    codeShapeColor = fillColor;
                    setCodeStyle(codeShapeTypeIndex, codeShapeColor);
                    binding.ivCodeColorPalette.setImageDrawable(DrawableUtils.tintModifyColorDrawable(getDrawable(R.drawable.icon_code_color_palette),codeShapeColor));
                    break;
            }
        }
    });
}
