package com.project.survey.ui.pointmeasure.measure;

import static androidx.recyclerview.widget.LinearLayoutManager.VERTICAL;
import static com.bingce.data.database.DBQueryConstant.AND;
import static com.bingce.data.database.DBQueryConstant.FROM_BIG_2_SMALL;
import static com.bingce.data.database.DBQueryConstant.NOT_DELETED;
import static com.bingce.data.database.DBQueryConstant.OR;
import static com.bingce.data.database.DBQueryConstant.ORDER_BY;
import static com.bingce.data.database.DBQueryConstant.SELECT_FROM;
import static com.bingce.data.database.DBQueryConstant.WHERE;

import android.content.Context;
import android.content.Intent;
import android.text.Editable;
import android.text.TextWatcher;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.lifecycle.Observer;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.sqlite.db.SimpleSQLiteQuery;

import com.bingce.data.BingCeDbConstant;
import com.bingce.data.database.CodeDb;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.surveyor.surveydata.code.CodeConstants;
import com.bingce.data.surveyor.surveydata.code.CodeRecord;

import com.bingce.surveyor.util.ConstUtils;
import com.bingce.surveyor.util.dialog.CustomDialog;
import com.bingce.utils.ThreadPoolUtil;
import com.google.gson.Gson;
import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.databinding.ActivityCodeLibraryBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.LauncherEvent;
import com.project.survey.ui.pointmeasure.measure.adapter.CodeLibraryAdapter;

import java.util.List;

import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;

public class CodeLibraryActivity extends BaseSurveyNewActivity {

    private ActivityCodeLibraryBinding binding;
    private CodeLibraryAdapter adapter;
    private int selectPosition;
    private List<CodeRecord> codeRecordList;

    @Override
    public View getContentView() {
        binding = ActivityCodeLibraryBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.code_library), getString(R.string.clear));

        if (App.instance.isThemeDark()) {
            binding.topLayout.setBackgroundColor(getColor(R.color.theme_dark_black));
            binding.tvNumber.setTextColor(getColor(R.color.white));
            binding.tvImport.setTextColor(getColor(R.color.white));
        }

        selectPosition = getIntent().getIntExtra(CODE_INDEX, -1);
        binding.codeLibraryBtnNew.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AddCodeActivity.start(launcher, CodeLibraryActivity.this, "");
            }
        });
        binding.codeLibraryBtnEdit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (codeRecordList == null || selectPosition == -1) {
                    ToastUtils.showShort(getString(R.string.select_code_edit));
                    return;
                }
                AddCodeActivity.start(launcher, CodeLibraryActivity.this, new Gson().toJson(codeRecordList.get(selectPosition)));
            }
        });
        binding.codeLibraryBtnDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (codeRecordList == null || selectPosition == -1) {
                    ToastUtils.showShort(getString(R.string.select_code_delete));
                    return;
                }
                new CustomDialog.Builder(CodeLibraryActivity.this).setContent("是否要删除编码" + "“" + codeRecordList.get(selectPosition).codeName + "[" + codeRecordList.get(selectPosition).code + "]”?").setButtonConfirm(new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        ThreadPoolUtil.execute(new Runnable() {
                            @Override
                            public void run() {
                                CodeDb.getInstance().delete(codeRecordList.get(selectPosition));
                                runOnUiThread(new Runnable() {
                                    @Override
                                    public void run() {
                                        codeRecordList.remove(selectPosition);
                                        adapter.notifyItemRemoved(selectPosition);
                                        selectPosition = -1;
                                    }
                                });
                                ToastUtils.showShort(getString(R.string.delete_success));
                            }
                        });
                    }
                }).create().show();
            }
        });
        binding.codeLibraryBtnApply.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (codeRecordList == null || selectPosition == -1) {
                    ToastUtils.showShort(getString(R.string.select_code_apply));
                    return;
                }
                Intent intent = new Intent();
                intent.putExtra(ConstUtils.intentConst.keyApplyIndexCodeRecord, codeRecordList.get(selectPosition).id);
                setResult(LauncherEvent.launcher_code_apply_id, intent);
                finish();
            }
        });
        binding.llImport.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CodeModelLibraryActivity.start(launcher, CodeLibraryActivity.this);
            }
        });

        binding.etEnterNameCode.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence s, int start, int count, int after) {

            }

            @Override
            public void onTextChanged(CharSequence s, int start, int before, int count) {

            }

            @Override
            public void afterTextChanged(Editable s) {
                selectPosition = -1;
                String s1 = SELECT_FROM + CodeConstants.DB_NAME + WHERE + NOT_DELETED + AND + "(code like '%" + s.toString() + "%'" + OR + "codeName like '%" + s.toString() + "%')" + ORDER_BY + BingCeDbConstant.DB_KEY_CREATED_DATE + FROM_BIG_2_SMALL;
                ThreadPoolUtil.execute(new Runnable() {
                    @Override
                    public void run() {
                        codeRecordList = CodeDb.getInstance()
                                .rawQueryListData(new SimpleSQLiteQuery(s1));
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                updateUI();
                            }
                        });
                    }
                });
            }
        });
    }

    @Override
    protected void initData() {
        CodeDb.getInstance()
                .rawQueryListLiveData(DBQueryConstant.findListByJob(CodeConstants.DB_NAME, ""))
                .observe(this, new Observer<List<CodeRecord>>() {
                    @Override
                    public void onChanged(List<CodeRecord> codeList) {
                        codeRecordList = codeList;
                        updateUI();
                    }
                });
    }

    private void updateUI() {
        binding.tvCodeNumber.setText(String.valueOf(codeRecordList.size()));
        adapter = new CodeLibraryAdapter(CodeLibraryActivity.this, codeRecordList, selectPosition);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(CodeLibraryActivity.this);
        linearLayoutManager.setOrientation(VERTICAL);
        binding.recyclerView.setLayoutManager(linearLayoutManager);
        binding.recyclerView.setAdapter(adapter);

        adapter.setOnItemClickListener(new CodeLibraryAdapter.OnItemClickListener() {
            @Override
            public void onItemClickListener(int position) {
                if (codeRecordList != null) {
                    selectPosition = position;
                    adapter.setSelectedPosition(position);
                }
            }
        });
    }

    @Override
    protected void onCommonRightClicked(View view) {
        if (codeRecordList != null && codeRecordList.size() > 0) {
            new CustomDialog.Builder(this).setContent(getString(R.string.want_to_empty_code_library)).setButtonConfirm(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    ThreadPoolUtil.execute(new Runnable() {
                        @Override
                        public void run() {
                            CodeDb.getInstance().delete(codeRecordList);
                            runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    codeRecordList.clear();
                                    adapter.setData(codeRecordList);
                                    setResult(LauncherEvent.launcher_delete_point_lib);
                                }
                            });
                        }
                    });
                }
            }).create().show();
        } else {
            ToastUtils.showShort(getString(R.string.no_data_clear));
        }
    }

    private static String CODE_INDEX = "codeIndex";

    public static void start(ActivityResultLauncher<Intent> launcher, Context context, int codeIndex) {
        Intent intent = new Intent(context, CodeLibraryActivity.class);
        intent.putExtra(CODE_INDEX, codeIndex);
        launcher.launch(intent);
    }

    public static void start(Context context) {
        Intent intent = new Intent(context, CodeLibraryActivity.class);
        context.startActivity(intent);
    }

    public final ActivityResultLauncher<Intent> launcher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result != null) {
            switch (result.getResultCode()) {
                case LauncherEvent.launcher_code_library_update:
                    CodeRecord codeRecord = new Gson().fromJson(result.getData().getStringExtra(ConstUtils.intentConst.keyUpdateCodeRecord), CodeRecord.class);
                    if (codeRecordList != null && selectPosition != -1) {
                        codeRecordList.set(selectPosition, codeRecord);
                        adapter.setSelectedPosition(selectPosition);
                    }
                    break;
            }
        }
    });
}