package com.project.survey.ui.pointmeasure.measure;


import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_LINE;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_POINT;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_SURFACE;

import android.content.Context;
import android.content.Intent;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.lifecycle.Observer;

import com.bingce.data.database.CodeDb;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.surveyor.surveydata.code.CodeConstants;
import com.bingce.data.surveyor.surveydata.code.CodeRecord;
import com.bingce.utils.IdUtils;
import com.bingce.utils.ThreadPoolUtil;
import com.project.survey.R;
import com.project.survey.databinding.ActivityCodeModelLibraryBinding;
import com.project.survey.model.CodeBean;
import com.project.survey.ui.base.BaseSurveyNewActivity;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import blankj.utilcode.util.ToastUtils;

public class CodeModelLibraryActivity extends BaseSurveyNewActivity {

    private ActivityCodeModelLibraryBinding binding;
    private Map mapDb;
    private List<CodeBean> codeBeans;

    @Override
    public View getContentView() {
        binding = ActivityCodeModelLibraryBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.code_model_library));
        CodeDb.getInstance()
                .rawQueryListLiveData(DBQueryConstant.findListByJob(CodeConstants.DB_NAME, ""))
                .observe(this, new Observer<List<CodeRecord>>() {
                    @Override
                    public void onChanged(List<CodeRecord> codeList) {
                        mapDb = new HashMap();
                        for (int i = codeList.size() - 1; i >= 0; i--) {
                            mapDb.put(codeList.get(i).codeName, codeList.get(i).code);
                        }
                    }
                });
    }

    @Override
    protected void initData() {

        binding.codeMoldLibBtnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    codeBeans = dom2xml(getAssets().open("CASS.xml"));
                    ThreadPoolUtil.execute(new Runnable() {
                        @Override
                        public void run() {
                            for (int i = codeBeans.size() - 1; i >= 0; i--) {
                                if (!mapDb.containsKey(codeBeans.get(i).getCodeName()) || !mapDb.containsValue(codeBeans.get(i).getCode())) {
                                    CodeRecord codeRecord = new CodeRecord();
                                    codeRecord.id = IdUtils.getUUID();
                                    switch (codeBeans.get(i).getGroup()) {
                                        case "point":
                                            codeRecord.codeGroupType = 0;
                                            break;
                                        case "line":
                                            codeRecord.codeGroupType = 1;
                                            break;
                                        case "surface":
                                            codeRecord.codeGroupType = 2;
                                            break;
                                    }
                                    codeRecord.codeName = codeBeans.get(i).getCodeName();
                                    codeRecord.code = codeBeans.get(i).getCode();
                                    codeRecord.colorShape = getResources().getColor(R.color.theme_green);
                                    codeRecord.codeSizeShape = 0;
                                    switch (codeRecord.codeGroupType) {
                                        case SHAPE_TYPE_POINT:
                                            codeRecord.codeFeatureShape = 0;
                                            codeRecord.codeColorContour = getResources().getColor(R.color.theme_green);//点线时候轮廓线默认为colorShape
                                            break;
                                        case SHAPE_TYPE_LINE:
                                            codeRecord.codeFeatureShape = 0;
                                            codeRecord.codeColorContour = getResources().getColor(R.color.theme_green);//点线时候轮廓线默认为colorShape
                                            break;
                                        case SHAPE_TYPE_SURFACE:
                                            codeRecord.codeFeatureShape = 0;
                                            codeRecord.codeColorContour = getResources().getColor(R.color.theme_green);//面的时候使用
                                            break;
                                    }
                                    CodeDb.getInstance().save(codeRecord);
                                }
                            }
                            runOnUiThread(new Runnable() {
                                @Override
                                public void run() {
                                    ToastUtils.showShort(getString(R.string.import_successful) + "!");
                                    finish();
                                }
                            });
                        }
                    });
                } catch (Exception e) {
                    ToastUtils.showShort(e.getMessage().toString());
                    e.printStackTrace();
                }
            }
        });
    }

    public List<CodeBean> dom2xml(InputStream is) throws Exception {
        //一系列的初始化
        List<CodeBean> list = new ArrayList<>();
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        //获得Document对象
        Document document = builder.parse(is);
        //获得code的List
        NodeList codeList = document.getElementsByTagName("CodeItem");
        //遍历code标签
        for (int i = 0; i < codeList.getLength(); i++) {
            //获得code标签
            Node node_code = codeList.item(i);
            //获得code标签里面的标签
            NodeList codeNodes = node_code.getChildNodes();
            //新建code对象
            CodeBean codeBean = new CodeBean();
            //遍历code标签里面的标签
            for (int j = 0; j < codeNodes.getLength(); j++) {
                //获得标签
                Node codeNode = codeNodes.item(j);
                //判断是标签
                if ("Name".equals(codeNode.getNodeName())) {
                    String name = codeNode.getTextContent();
                    codeBean.setCodeName(name);
                } else if ("Code".equals(codeNode.getNodeName())) {
                    String code = codeNode.getTextContent();
                    codeBean.setCode(code);
                } else if ("Group".equals(codeNode.getNodeName())) {
                    String group = codeNode.getTextContent();
                    codeBean.setGroup(group);
                } else if ("Style".equals(codeNode.getNodeName())) {
                    String style = codeNode.getTextContent();
                    codeBean.setStyle(style);
                }
            }
            //加到List中
            list.add(codeBean);
        }
        return list;
    }

    public static void start(ActivityResultLauncher<Intent> launcher, Context context) {
        launcher.launch(new Intent(context, CodeModelLibraryActivity.class));
    }
}