package com.project.survey.ui.pointmeasure.measure;



import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_CENTER_CIRCLE;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_CIRCLE_EMPTY;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_CIRCLE_FULL;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_RHOMBUS_EMPTY;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_RHOMBUS_FULL;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_ROUNDED_FIVE_STAR;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_SQUARE_EMPTY;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_SQUARE_FULL;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_STRAIGHT_FIVE_STAR;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TRIANGLE_DOWN_EMPTY;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TRIANGLE_DOWN_FULL;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TRIANGLE_UP_EMPTY;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TRIANGLE_UP_FULL;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_LINE;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_POINT;

import android.content.Context;
import android.content.Intent;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.recyclerview.widget.LinearLayoutManager;


import com.bingce.surveyor.util.ConstUtils;

import com.bingce.surveyor.util.dialog.CustomRecycleDialog;
import com.bingce.utils.IntentUtil;
import com.project.survey.R;
import com.project.survey.databinding.ActivityCodeStylePointLineBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.LauncherEvent;
import com.project.survey.ui.pointmeasure.measure.adapter.CodeStyleAdapter;

import java.util.ArrayList;
import java.util.List;


public class CodeStylePointLineActivity extends BaseSurveyNewActivity {

    private ActivityCodeStylePointLineBinding binding;
    private CodeStyleAdapter adapter;
    private int codeShapeType = 0;//点、线、面形状类型
    private int codeShapeSizeIndex = 0;//尺寸大小index
    private int codeShapeColor = 0;//颜色

    private int codeStyleIndex = 0;//临时的点线面index
    private int codeStyleIndexPoint = 0;//点列表index
    private int codeStyleIndexLine = 0;//线列表index

    @Override
    public View getContentView() {
        binding = ActivityCodeStylePointLineBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.style));
    }

    @Override
    protected void initData() {

        codeShapeType = IntentUtil.intExtra(getIntent(), CODE_SHAPE_TYPE);
        codeShapeColor = IntentUtil.intExtra(getIntent(), CODE_SHAPE_COLOR);
        codeShapeSizeIndex = IntentUtil.intExtra(getIntent(), CODE_SHAPE_SIZE);

        codeStyleIndexPoint = IntentUtil.intExtra(getIntent(), CODE_STYLE_INDEX_POINT);
        codeStyleIndexLine = IntentUtil.intExtra(getIntent(), CODE_STYLE_INDEX_LINE);

        List<Integer> list = new ArrayList<>();

        List<String> sizeList = new ArrayList<>();

        switch (codeShapeType){
            case SHAPE_TYPE_POINT:

                sizeList.add(getString(R.string.small_size));
                sizeList.add(getString(R.string.medium_size));
                sizeList.add(getString(R.string.big_size));

                list.add(SHAPE_CIRCLE_FULL);
                list.add(SHAPE_CIRCLE_EMPTY);
                list.add(SHAPE_SQUARE_FULL);
                list.add(SHAPE_SQUARE_EMPTY);
                list.add(SHAPE_TRIANGLE_UP_FULL);
                list.add(SHAPE_TRIANGLE_UP_EMPTY);
                list.add(SHAPE_TRIANGLE_DOWN_FULL);
                list.add(SHAPE_TRIANGLE_DOWN_EMPTY);
                list.add(SHAPE_RHOMBUS_FULL);
                list.add(SHAPE_RHOMBUS_EMPTY);
                list.add(SHAPE_ROUNDED_FIVE_STAR);
                list.add(SHAPE_STRAIGHT_FIVE_STAR);
                list.add(SHAPE_CENTER_CIRCLE);

                codeStyleIndex = codeStyleIndexPoint;

                break;

            case SHAPE_TYPE_LINE:

                sizeList.add(getString(R.string.thin_size));
                sizeList.add(getString(R.string.medium_size));
                sizeList.add(getString(R.string.coarse_size));

                list.add(0);
                list.add(1);
                list.add(2);
                list.add(3);

                codeStyleIndex = codeStyleIndexLine;

                break;
        }

        adapter = new CodeStyleAdapter(CodeStylePointLineActivity.this, codeStyleIndex, list, codeShapeType,codeShapeColor, codeShapeSizeIndex);
        LinearLayoutManager linearLayoutManager = new LinearLayoutManager(CodeStylePointLineActivity.this);
        linearLayoutManager.setOrientation(LinearLayoutManager.VERTICAL);
        binding.recyclerView.setLayoutManager(linearLayoutManager);
        binding.recyclerView.setAdapter(adapter);
        adapter.setOnItemClickListener(new CodeStyleAdapter.OnItemClickListener() {
            @Override
            public void onItemClickListener(int position) {
                switch (codeShapeType){
                    case SHAPE_TYPE_POINT:
                        codeStyleIndexPoint = position;
                        break;
                    case SHAPE_TYPE_LINE:
                        codeStyleIndexLine = position;
                        break;
                }
                adapter.setSelectedPosition(position);
            }
        });

        binding.tvSize.setText(sizeList.get(codeShapeSizeIndex));

        binding.rlSize.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CustomRecycleDialog.showDialog(CodeStylePointLineActivity.this, R.string.choose_instrument_type, sizeList, codeShapeSizeIndex, false, new CustomRecycleDialog.IClickCallback() {
                    @Override
                    public void onItemSelected(int index, String itemString) {
                        codeShapeSizeIndex = index;
                        binding.tvSize.setText(sizeList.get(index));
                        adapter.setShapeSize(list,index);
                    }
                });
            }
        });

        binding.styleCodeBtnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent();
                intent.putExtra(ConstUtils.intentConst.keyCodeStyleIndexPoint, codeStyleIndexPoint);
                intent.putExtra(ConstUtils.intentConst.keyCodeStyleIndexLine, codeStyleIndexLine);
                intent.putExtra(ConstUtils.intentConst.keyCodeShapeSize, codeShapeSizeIndex);
                setResult(LauncherEvent.launcher_code_style,intent);
                finish();
            }
        });
    }

    private static String CODE_SHAPE_TYPE = "codeShapeType";
    private static String CODE_SHAPE_COLOR = "codeShapeColor";
    private static String CODE_SHAPE_SIZE = "codeShapeSize";
    private static String CODE_STYLE_INDEX_POINT = "codeStyleIndexPoint";
    private static String CODE_STYLE_INDEX_LINE = "codeStyleIndexLine";

    public static void start(ActivityResultLauncher<Intent> launcher, Context context,int codeShapeType, int codeShapeColor,int codeShapeSize,int codeStyleIndexPoint,int codeStyleIndexLine){
        Intent intent = new Intent(context, CodeStylePointLineActivity.class);
        intent.putExtra(CODE_SHAPE_TYPE,codeShapeType);
        intent.putExtra(CODE_SHAPE_COLOR,codeShapeColor);
        intent.putExtra(CODE_SHAPE_SIZE,codeShapeSize);
        intent.putExtra(CODE_STYLE_INDEX_POINT,codeStyleIndexPoint);
        intent.putExtra(CODE_STYLE_INDEX_LINE,codeStyleIndexLine);
        launcher.launch(intent);
    }
}
