package com.project.survey.ui.pointmeasure.measure;


import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_LINE;

import android.content.Context;
import android.content.Intent;
import android.graphics.drawable.GradientDrawable;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;


import com.bingce.surveyor.util.ConstUtils;

import com.bingce.utils.IntentUtil;
import com.project.survey.R;
import com.project.survey.databinding.ActivityCodeStyleSurfaceBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.LauncherEvent;
import com.project.survey.util.CommonUtils;
import com.project.survey.util.DrawableUtils;


public class CodeStyleSurfaceActivity extends BaseSurveyNewActivity {

    private ActivityCodeStyleSurfaceBinding binding;
    private int contourColor,fillColor;
    private int codeShapeSizeIndex = 0;
    private int codeStyleIndexLine = 0;
    private boolean isEditFillColor;

    @Override
    public View getContentView() {
        binding = ActivityCodeStyleSurfaceBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.style));
    }

    @Override
    protected void initData() {

        contourColor = IntentUtil.intExtra(getIntent(),KEY_CONTOUR_COLOR);

        fillColor = IntentUtil.intExtra(getIntent(),KEY_FILL_COLOR);

        codeShapeSizeIndex = IntentUtil.intExtra(getIntent(),KEY_CODE_SHAPE_SIZE_INDEX);

        codeStyleIndexLine = IntentUtil.intExtra(getIntent(),KEY_CODE_STYLE_INDEX_LINE);

        setCodeStyle();

        binding.ivContourColorPalette.setImageDrawable(DrawableUtils.tintModifyColorDrawable(getDrawable(R.drawable.icon_code_color_palette), contourColor));

        binding.ivFillColorPalette.setBackgroundColor(fillColor);

        previewStyle();

        binding.rlContourStyle.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                CodeStylePointLineActivity.start(launcher, CodeStyleSurfaceActivity.this, SHAPE_TYPE_LINE, contourColor, codeShapeSizeIndex, 0,codeStyleIndexLine);
            }
        });

        binding.rlContourColor.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                isEditFillColor = false;
                PickerColorActivity.start(launcher, CodeStyleSurfaceActivity.this, contourColor);
            }
        });

        binding.rlFillColor.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                isEditFillColor = true;
                PickerColorActivity.start(launcher, CodeStyleSurfaceActivity.this, fillColor);
            }
        });

        binding.styleCodeBtnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent intent = new Intent();
                intent.putExtra(ConstUtils.intentConst.keyCodeStyleIndexLine, codeStyleIndexLine);
                intent.putExtra(ConstUtils.intentConst.keyCodeShapeSize, codeShapeSizeIndex);
                intent.putExtra(ConstUtils.intentConst.keyCodeShapeContourColor, contourColor);
                intent.putExtra(ConstUtils.intentConst.keyCodeShapeFillColor, fillColor);
                setResult(LauncherEvent.launcher_code_shape_surface,intent);
                finish();
            }
        });
    }

    /**
     * 样式
     */
    private void setCodeStyle() {
        GradientDrawable gradientDrawable = (GradientDrawable)binding.contourStyle.getBackground();
        int lineWidth = 0;
        switch (codeShapeSizeIndex){
            case 0:
                lineWidth = CommonUtils.dip2px(0.8f);
                break;
            case 1:
                lineWidth = CommonUtils.dip2px(1.3f);
                break;
            case 2:
                lineWidth = CommonUtils.dip2px(1.8f);
                break;
        }
        gradientDrawable.setStroke(lineWidth, contourColor,CommonUtils.dip2px(codeStyleIndexLine + codeStyleIndexLine * 2),CommonUtils.dip2px(codeStyleIndexLine + codeStyleIndexLine * 2));
        gradientDrawable.setColor(getColor(R.color.white));
    }
    private static String KEY_FILL_COLOR = "keyFillColor";
    private static String KEY_CONTOUR_COLOR = "keyContourColor";
    private static String KEY_CODE_SHAPE_SIZE_INDEX = "keyCodeShapeSizeIndex";
    private static String KEY_CODE_STYLE_INDEX_LINE = "keyCodeStyleIndexLine";

    public static void start(ActivityResultLauncher<Intent> launcher,Context context,int codeShapeColor,int contourColor,int codeShapeSizeIndex,int codeStyleIndexLine){
        Intent intent = new Intent(context, CodeStyleSurfaceActivity.class);
        intent.putExtra(KEY_FILL_COLOR,codeShapeColor);
        intent.putExtra(KEY_CONTOUR_COLOR,contourColor);
        intent.putExtra(KEY_CODE_SHAPE_SIZE_INDEX,codeShapeSizeIndex);
        intent.putExtra(KEY_CODE_STYLE_INDEX_LINE,codeStyleIndexLine);
        launcher.launch(intent);
    }

    public final ActivityResultLauncher<Intent> launcher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result != null) {
            switch (result.getResultCode()){
                case LauncherEvent.launcher_add_code_color:
                    int color = result.getData().getIntExtra(ConstUtils.intentConst.keyAddCodeColor, -1);
                    if (!isEditFillColor){
                        contourColor = color;
                        setCodeStyle();
                        binding.ivContourColorPalette.setImageDrawable(DrawableUtils.tintModifyColorDrawable(getDrawable(R.drawable.icon_code_color_palette),color));
                    }else {
                        fillColor = color;
                        binding.ivFillColorPalette.setBackgroundColor(color);
                    }
                    previewStyle();
                    break;
                case LauncherEvent.launcher_code_style:
                    codeStyleIndexLine = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeStyleIndexLine,0);
                    codeShapeSizeIndex = result.getData().getIntExtra(ConstUtils.intentConst.keyCodeShapeSize,0);
                    setCodeStyle();
                    previewStyle();
                    break;
            }
        }
    });

    private void previewStyle(){
        GradientDrawable gradientDrawable = (GradientDrawable)binding.tvPreviewStyle.getBackground();
        int lineWidth = 0;
        switch (codeShapeSizeIndex){
            case 0:
                lineWidth = CommonUtils.dip2px(0.8f);
                break;
            case 1:
                lineWidth = CommonUtils.dip2px(1.3f);
                break;
            case 2:
                lineWidth = CommonUtils.dip2px(1.8f);
                break;
        }
        gradientDrawable.setStroke(lineWidth, contourColor,CommonUtils.dip2px(codeStyleIndexLine + codeStyleIndexLine * 2),CommonUtils.dip2px(codeStyleIndexLine + codeStyleIndexLine * 2));
        gradientDrawable.setColor(fillColor);
    }
}
