package com.project.survey.ui.pointmeasure.measure;

import android.content.Context;
import android.content.Intent;
import android.graphics.Color;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;

import com.bingce.surveyor.util.ConstUtils;

import com.bingce.utils.IntentUtil;
import com.project.survey.R;
import com.project.survey.databinding.ActivityPickerColorBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.LauncherEvent;
import com.project.survey.util.DrawableUtils;
import com.shixia.colorpickerview.OnColorChangeListener;


public class PickerColorActivity extends BaseSurveyNewActivity {

    private ActivityPickerColorBinding binding;
    private int selectColor;

    @Override
    public View getContentView() {
        binding = ActivityPickerColorBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        setTitle(getString(R.string.picker_color));

        binding.pickerCodeBtnConfirm.setOnClickListener(v -> {
            Intent intent = new Intent();
            intent.putExtra(ConstUtils.intentConst.keyAddCodeColor, selectColor);
            setResult(LauncherEvent.launcher_add_code_color,intent);
            finish();
        });

        int defultColor = IntentUtil.intExtra(getIntent(), DEFULT_COLOR);
        selectColor = defultColor;
        binding.ivCodeColorPalette.setImageDrawable(DrawableUtils.tintModifyColorDrawable(getDrawable(R.drawable.icon_code_color_palette),defultColor));

        binding.cpvColor.setOnColorChangeListener(new OnColorChangeListener() {
            @Override
            public void colorChanged(int color) {
                selectColor = color;
                binding.ivCodeColorPalette.setImageDrawable(DrawableUtils.tintModifyColorDrawable(getDrawable(R.drawable.icon_code_color_palette),color));

                binding.tvColorRgb.setText("R:" + Color.red(color) + "   G:" + Color.green(color) + "   B:" + Color.blue(color));
                binding.tvColorXml.setText(toHexEncoding(color,false));
                binding.tvColorXml16.setText(toHexEncoding(color,true));

                binding.tvColorRgb.setTextColor(color);
                binding.tvColorXml.setTextColor(color);
                binding.tvColorXml16.setTextColor(color);
            }
        });
    }

    /**
     * 获取十进制和十六进制色值
     * @param color
     * @param isXml16 是否获取16进制
     * @return
     */
    public static String toHexEncoding(int color,boolean isXml16) {
        String A, R, G, B;
        StringBuilder sb = new StringBuilder();
        A = Integer.toHexString(Color.alpha(color));
        R = Integer.toHexString(Color.red(color));
        G = Integer.toHexString(Color.green(color));
        B = Integer.toHexString(Color.blue(color));
        //判断获取到的A,R,G,B值的长度 如果长度等于1 给A,R,G,B值的前边添0
        A = A.length() == 1 ? "0" + A : A;
        R = R.length() == 1 ? "0" + R : R;
        G = G.length() == 1 ? "0" + G : G;
        B = B.length() == 1 ? "0" + B : B;
        sb.append("#");
        if (isXml16){
            sb.append(A);
        }
        sb.append(R);
        sb.append(G);
        sb.append(B);
        return sb.toString();
    }

    @Override
    protected void initData() {

    }

    private static String DEFULT_COLOR = "defultColor";

    public static void start(ActivityResultLauncher<Intent> launcher,Context context,int defultColor){
        Intent intent = new Intent(context, PickerColorActivity.class);
        intent.putExtra(DEFULT_COLOR,defultColor);
        launcher.launch(intent);
    }
}
