package com.project.survey.ui.pointmeasure.measure;

import static com.bingce.rtk.model.GnssPosition.POS_TYPE_DIF3D;
import static com.bingce.rtk.model.GnssPosition.POS_TYPE_FIXED;
import static com.bingce.rtk.model.GnssPosition.POS_TYPE_FLOAT;
import static com.bingce.surveyor.util.ConstUtils.pointConst.POINT_TYPE_DX;
import static com.bingce.surveyor.util.ConstUtils.pointConst.POINT_TYPE_KS;
import static com.bingce.surveyor.util.ConstUtils.pointConst.POINT_TYPE_KZ;
import static com.bingce.surveyor.util.ConstUtils.pointConst.POINT_TYPE_LX;
import static com.project.survey.util.DrawableUtils.RADIUS_TOP;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.SystemClock;
import android.text.TextUtils;
import android.util.DisplayMetrics;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.animation.AnimationSet;
import android.view.animation.AnimationUtils;
import android.widget.ArrayAdapter;
import android.widget.CompoundButton;
import android.widget.RadioButton;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.lifecycle.Observer;
import androidx.lifecycle.ViewModelProvider;
import androidx.preference.PreferenceManager;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.chart.ChartViewUtils;
import com.bingce.coordlib.model.Blh;
import com.bingce.coordlib.model.Coordinate;
import com.bingce.coordlib.util.CoordUtil;
import com.bingce.data.DeviceInfoData;
import com.bingce.data.database.CodeDb;
import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.database.PointDb;
import com.bingce.data.surveyor.surveydata.code.CodeConstants;
import com.bingce.data.surveyor.surveydata.code.CodeRecord;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointConstants;
import com.bingce.data.surveyor.surveydata.pointsurvey.PointRecord;
import com.bingce.device.Device;
import com.bingce.device.enums.DeviceTypeEnum;
import com.bingce.device.util.DeviceUtils;
import com.bingce.rtk.gnss.Gnss;
import com.bingce.rtk.model.GnssPosition;
import com.bingce.surveyor.agentweb.HelpWebActivity;
import com.bingce.surveyor.util.ConstUtils;
import com.bingce.surveyor.util.PreferencesUtil;
import com.bingce.surveyor.util.SurveyRemarksUtils;
import com.bingce.surveyor.util.dialog.CustomDialog;
import com.bingce.surveyor.util.dialog.CustomRecycleDialog;
import com.bingce.totalstation.TsConfig;
import com.bingce.utils.ColorUtil;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.PointUtils;
import com.bingce.utils.SoundPoolUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.bingce.utils.Util;
import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.constants.SPConstants;
import com.project.survey.databinding.ActivityPointSurveyNewBinding;
import com.project.survey.dialog.CustomLineChartDialog;
import com.project.survey.logic.viewmodel.ProjectViewModel;
import com.project.survey.model.ControlRecord;
import com.project.survey.model.PointEntry;
import com.project.survey.ui.base.BaseSurveyNewActivity;
import com.project.survey.ui.instrument.setupstation.CoordinatePointsLibraryActivity;
import com.project.survey.ui.instrument.setupstation.LauncherEvent;
import com.project.survey.ui.pointmeasure.measure.util.SFUtils;
import com.project.survey.ui.pointmeasure.measure.util.SurveyToolBarClickListenerUtils;
import com.project.survey.util.CommonUtils;
import com.project.survey.util.DeviceConnectUtil;
import com.project.survey.util.DrawableUtils;
import com.project.survey.util.SPUtils;
import com.project.survey.util.SurveyUIUtils;
import com.project.survey.widget.bingce.dragdrop.DefaultItemCallback;
import com.project.survey.widget.bingce.dragdrop.DefaultItemTouchHelper;
import com.project.survey.widget.bingce.dragdrop.FunctionAdapter;
import com.project.survey.widget.bingce.dragdrop.FunctionBlockAdapter;
import com.project.survey.widget.bingce.dragdrop.FunctionItem;
import com.project.survey.widget.bingce.dragdrop.FunctionItemID;
import com.project.survey.widget.bingce.dragdrop.SpaceItemDecoration;
import com.project.survey.widget.bingce.gaode.GaoDeBottomSheetBehavior;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import blankj.utilcode.util.ActivityUtils;
import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;
import cn.liuyanbing.surveyor.model.base.BasePoint;
import lecho.hellocharts.model.Line;
import lecho.hellocharts.model.LineChartData;
import lecho.hellocharts.model.PointValue;
import lecho.hellocharts.renderer.LineChartRenderer;

/**
 * 点测量
 */
public class PointSurveyNewActivity extends BaseSurveyNewActivity implements View.OnClickListener {

    private static final String KEY_WORK_TYPE = "workType";
    private final static String KEY_JOB_ID = "jobId";
    private final static String KEY_PROJECT_ID = "projectId";
    private final static String KEY_ROAD_ID = "roadId";
    private ProjectViewModel viewModel;

    private ActivityPointSurveyNewBinding binding;
    private List<PointRecord> surveyBaseRecordPointList;
    private final Line knowRecordPointLine = new Line();
    private final Line currentPointLine = new Line();
    private final Line addRecordPointLine = new Line();
    private Coordinate currentCoordinate;
    private Blh currentBlh;
    private boolean isMeasureAndRecord;
    private final List<String> mItemsArrayList = new ArrayList<>();
    private int pointTypeIndex = POINT_TYPE_KS;
    private final SurveyRemarksUtils surveyRemarksUtils = new SurveyRemarksUtils();
    private boolean isContinuousSurvey, isControlSurvey;
    private final int RECORD_BY_TIME = 0;
    private final int RECORD_BY_DISTANCE = RECORD_BY_TIME + 1;
    private int recordByWayIndex = RECORD_BY_TIME;
    public long preBackPressTime = 0;
    private CustomLineChartDialog.Builder lineChartViewBuilder;
    private CustomLineChartDialog customDialog;
    private int codeIndex = 0;
    private String recordRemarks;
    private GaoDeBottomSheetBehavior behavior;
    private long lastTimeStamp = 0;
    private BasePoint lastRTKBackSurveyPoint;
    private double controlX, controlY, controlZ;
    private double controlLat, controlLon, controlAlt;
    private final List<String> stringSurveyPointTypeList = new ArrayList<>();
    private final List<CodeRecord> codeRecordsList = new ArrayList<>();
    private String code;
    private SharedPreferences preferences;
    private boolean isFirstUpdateChartView;

    @Override
    public View getContentView() {
        binding = ActivityPointSurveyNewBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @SuppressLint("SetTextI18n")
    @Override
    protected void initView() {
        if (App.instance.isThemeDark()) {
            binding.topLayout.setBackgroundColor(getResources().getColor(R.color.theme_dark_black));
            binding.layoutPoleHrHeight.tvPoleHrHeight.setTextColor(getColor(R.color.white));
            binding.tvCode.setTextColor(getColor(R.color.white));
            binding.sherlock.setTextColor(getColor(R.color.white));
            binding.tvPointName.setTextColor(getColor(R.color.white));
            binding.tvRemark.setTextColor(getColor(R.color.white));
            binding.tvScale.setTextColor(getColor(R.color.white));
            binding.tvLeft01.setTextColor(getColor(R.color.white));
            binding.tvLeft02.setTextColor(getColor(R.color.white));
            binding.tvLeft03.setTextColor(getColor(R.color.white));
            binding.tvRight01.setTextColor(getColor(R.color.white));
            binding.tvRight02.setTextColor(getColor(R.color.white));
            binding.tvRight03.setTextColor(getColor(R.color.white));
            binding.tvShowInformation.setTextColor(getColor(R.color.white));

            DrawableUtils.tintModifyColorVector(binding.ivRemark, R.drawable.icon_remark_gray, R.color.white);
            DrawableUtils.tintModifyColorVector(binding.ivScale, R.drawable.ic_scale, R.color.white);
            DrawableUtils.setClickBackgroundEffect(binding.bottomSheet, R.color.theme_dark_black, R.color.theme_dark_black, 10, RADIUS_TOP);
        }
        viewModel = new ViewModelProvider(this).get(ProjectViewModel.class);
        viewModel.getUploadPointEntryResponse().observe(this, ToastUtils::showShort);

        preferences = PreferenceManager.getDefaultSharedPreferences(this);
        //比例尺
        SurveyToolBarClickListenerUtils.setScaleViewOnClickListener(binding.lineChartView, binding.tvScale, binding.ivScale);
        //工具栏功能（左上角）
        SurveyToolBarClickListenerUtils.setOnClickListener(this, true, false,
                baseSurveyActivityUtils.activityBaseSurveyBinding,
                binding.baseSurveyStakingSetting,
                new SurveyToolBarClickListenerUtils.IToolBarClickListener() {
                    @Override
                    public void onSettingClickListener() {
                        //设置
                        PointSurveySettingActivity.start(PointSurveyNewActivity.this, pointTypeIndex);
                    }

                    @Override
                    public void onPositionCenterClickListener() {
                        //居中
                        if (currentCoordinate != null) {
                            binding.lineChartView.setZoomLevel(currentCoordinate.getY(), currentCoordinate.getX(), binding.lineChartView.getZoomLevel());
                            binding.lineChartView.moveToWithAnimation(currentCoordinate.getY(), currentCoordinate.getX());
                        }
                    }

                    @Override
                    public void onFullScreenClickListener() {
                        //全屏
                        ChartViewUtils.centerInScreen(binding.lineChartView);
                    }

                    @Override
                    public void onAutoPrimsClickListener() {
                        //自动全站仪寻点

                    }
                });

        //编码库(编码库列表，目前版本不支持编码库展示隐藏)
        getCodeDb();

        binding.sherlock.setOnItemClickListener((parent, view, position, id) -> {
            codeIndex = position;
            code = codeRecordsList.get(codeIndex).code;
        });

        binding.ivCodeLibrary.setOnClickListener(v -> CodeLibraryActivity.start(launcher, this, codeIndex - 1));

        //测量点库
        binding.pointLibrary.setOnClickListener(v -> CoordinatePointsLibraryActivity.start4Result(launcher, this, IntentUtil.stringExtra(this, KEY_JOB_ID)));

        //编辑备注
        binding.rlRemark.setOnClickListener(v -> editRecordRemark(recordRemarks));

        //长按编辑备注
        binding.layoutDragButton.drWatchSurveyRecord.setTouchOnLongClickListener(() -> editRecordRemark(recordRemarks));

        //测量点类型
        binding.tvSelectPointType.setOnClickListener(v -> {
            if (stringSurveyPointTypeList == null) {
                return;
            }
            stringSurveyPointTypeList.clear();
            stringSurveyPointTypeList.add(getString(R.string.quick_point));
            if (Device.getInstance().isDeviceConnected() && Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) {
                stringSurveyPointTypeList.add(0, getString(R.string.topographic_point));
                stringSurveyPointTypeList.add(getString(R.string.control_point));
                stringSurveyPointTypeList.add(getString(R.string.continuity_point));
            }
            if (DeviceConnectUtil.isDeviceConnectSelectDialog(this, baseSurveyActivityUtils.activityBaseSurveyBinding)) {
                CustomRecycleDialog.showDialog(this, R.string.select_survey_point_type, stringSurveyPointTypeList, pointTypeIndex, false, (index, itemString) -> {
                    pointTypeIndex = index;
                    binding.tvSelectPointType.setText(itemString);
                    if (index == POINT_TYPE_LX) {
                        binding.layoutDragButton.drWatchSurveyRecord.setVisibility(View.GONE);
                        binding.rlPalyPaused.setVisibility(View.VISIBLE);
                        binding.tvContinuousPointWay.setVisibility(View.VISIBLE);
                    } else {
                        binding.layoutDragButton.drWatchSurveyRecord.setVisibility(View.VISIBLE);
                        binding.rlPalyPaused.setVisibility(View.GONE);
                        binding.tvContinuousPointWay.setVisibility(View.GONE);
                    }
                    if (index == POINT_TYPE_KZ) {
                        binding.etPointName.setText("");
                    } else {
                        queryLastSurveyPointName();
                    }
                    binding.ivPalyPaused.setImageResource(R.drawable.icon_survey_play);
                    binding.tvPalyPaused.setText(getString(R.string.start));
                });
            }
        });

        binding.rlPalyPaused.setOnClickListener(v -> {
            if (!isContinuousSurvey) {
                binding.ivPalyPaused.setImageResource(R.drawable.icon_survey_paused);
                binding.tvPalyPaused.setText(getString(R.string.pause));
                binding.etPointName.setEnabled(false);
            } else {
                binding.ivPalyPaused.setImageResource(R.drawable.icon_survey_play);
                binding.tvPalyPaused.setText(getString(R.string.start));
                binding.etPointName.setEnabled(true);
            }
            isContinuousSurvey = !isContinuousSurvey;
        });

        List<String> stringListRecordWay = new ArrayList<>();
        stringListRecordWay.add(getString(R.string.continuous_same_time));
        stringListRecordWay.add(getString(R.string.continuous_same_distance));
        binding.tvContinuousPointWay.setOnClickListener(v -> CustomRecycleDialog.showDialog(this, R.string.continuous_point_measurement_method, stringListRecordWay, recordByWayIndex, false, new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                recordByWayIndex = index;
                binding.tvContinuousPointWay.setText(stringListRecordWay.get(index));
            }
        }));

        //仅观测
        binding.layoutDragButton.drWatchSurvey.setOnClickListener(v -> watchSurvey());

        //观测并记录
        binding.layoutDragButton.drWatchSurveyRecord.setOnClickListener(v -> watchSurveyRecord());

        //底部信息栏
        ViewGroup bottom_sheet = findViewById(R.id.bottom_sheet);
        behavior = GaoDeBottomSheetBehavior.from(bottom_sheet);
        behavior.setState(6);
        binding.tvBottomInfoPrompt.setText(getString(R.string.tip) + getString(R.string.colon) + getString(R.string.pull_up_edit_information_bar_displayed));
        behavior.setBottomSheetCallback(new GaoDeBottomSheetBehavior.BottomSheetCallback() {

            @SuppressLint({"WrongConstant", "SetTextI18n"})
            @Override
            public void onStateChanged(@NonNull View bottomSheet, int newState) {
                switch (newState) {
                    case 1:
                        //过渡状态此时用户正在向上或者向下拖动bottom sheet
                        break;
                    case 2:
                        // 视图从脱离手指自由滑动到最终停下的这一小段时间
                        break;
                    case 3:
                        //处于完全展开的状态
                        binding.tvBottomInfoPrompt.setText(getString(R.string.tip) + getString(R.string.colon) + getString(R.string.press_hold_drag_adjust_order));
                        break;
                    case 4:
                        //默认的折叠状态
                        binding.ivShowClose.setImageResource(R.mipmap.icon_new_up_gray_arrow);
                        break;
                    case 5:
                        //下滑动完全隐藏 bottom sheet
                        break;
                    case 6:
                        //下滑动完全隐藏 bottom sheet
                        binding.ivShowClose.setImageResource(R.mipmap.icon_new_down_gray_arrow);
                        binding.tvBottomInfoPrompt.setText(getString(R.string.tip) + getString(R.string.colon) + getString(R.string.pull_up_edit_information_bar_displayed));
                        break;
                }
            }

            @Override
            public void onSlide(@NonNull View bottomSheet, float slideOffset) {

            }
        });

        binding.layoutBottomSheet.setOnClickListener(v -> {
            if (behavior.getState() == 6) {
                behavior.setState(4);
                binding.ivShowClose.setImageResource(R.mipmap.icon_new_up_gray_arrow);
            } else {
                behavior.setState(6);
                binding.ivShowClose.setImageResource(R.mipmap.icon_new_down_gray_arrow);
            }
        });

        binding.ivShowClose.startAnimation((AnimationSet) AnimationUtils.loadAnimation(this, R.anim.down_anim));

        init();
        addListener();
    }

    private void editRecordRemark(String content) {
        SurveyRemarksUtils.createRecordRemarks(this, content, remarks -> {
            recordRemarks = remarks;
            if (!TextUtils.isEmpty(recordRemarks)) {
                binding.ivRemark.setImageResource(R.drawable.icon_remark_green);
                binding.tvRemark.setText(getString(R.string.have_remark));
                binding.tvRemark.setTextColor(ColorUtil.getDarkColorPrimary(this));
            } else {
                DrawableUtils.tintModifyColorVector(binding.ivRemark, R.drawable.icon_remark_gray, App.instance.isThemeDark() ? R.color.white : R.color.color_8a8a8a);
                binding.tvRemark.setText(getString(R.string.no_remark));
                binding.tvRemark.setTextColor(getColor(App.instance.isThemeDark() ? R.color.white : R.color.color_cccccc));
            }
        });
    }

    private void getCodeDb() {
        getCodeDb("");
    }

    /**
     * 获取code编码列表
     *
     * @param applyCodeId 编码库应用返回的编码id
     */
    private void getCodeDb(String applyCodeId) {
        CodeDb.getInstance()
                .rawQueryListLiveData(DBQueryConstant.findListByJob(CodeConstants.DB_NAME, ""))
                .observe(this, codeList -> {
                    mItemsArrayList.clear();
                    codeRecordsList.clear();

                    if (!TextUtils.isEmpty(applyCodeId)) {
                        for (int i = 0; i < codeList.size(); i++) {
                            if (applyCodeId.equals(codeList.get(i).id)) {
                                codeIndex = i + 1;
                            }
                        }
                    }

                    mItemsArrayList.add(getString(R.string.without));
                    codeRecordsList.add(new CodeRecord(getString(R.string.without), ""));
                    for (int i = 0; i < codeList.size(); i++) {
                        mItemsArrayList.add(codeList.get(i).codeName);
                        codeRecordsList.add(codeList.get(i));
                    }
                    ArrayAdapter<String> mAdapterSherlockSpinner = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, mItemsArrayList);
                    binding.sherlock.setAdapter(mAdapterSherlockSpinner);
                    if (codeIndex >= mItemsArrayList.size()) {
                        codeIndex = mItemsArrayList.size() - 1;
                    }
                    binding.sherlock.setText(mItemsArrayList.get(codeIndex));
                    code = codeRecordsList.get(codeIndex).code;
                });
    }

    /**
     * 只观测
     */
    private void watchSurvey() {
        if (DeviceConnectUtil.isDeviceConnectSelectDialog(this, baseSurveyActivityUtils.activityBaseSurveyBinding)) {
            if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
                isMeasureAndRecord = false;
                tsMeasure();
            } else {
                ToastUtils.showShort(getString(R.string.not_support_function_observation_only));
            }
        }
    }

    /**
     * 观测并记录
     */
    private void watchSurveyRecord() {
        if (DeviceConnectUtil.isDeviceConnectSelectDialog(this, baseSurveyActivityUtils.activityBaseSurveyBinding)) {
            if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_RTK) {
                if (pointTypeIndex == POINT_TYPE_KS) {
                    checkRtkStringLimit();
                } else if (pointTypeIndex == POINT_TYPE_KZ || pointTypeIndex == POINT_TYPE_DX) {
                    if (TextUtils.isEmpty(binding.etPointName.getText().toString())) {
                        if (pointTypeIndex == POINT_TYPE_KZ) {
                            ToastUtils.showShort(R.string.please_input_control_point_name);
                        } else if (pointTypeIndex == POINT_TYPE_DX) {
                            ToastUtils.showShort(R.string.point_name_can_not_be_empty);
                        }
                        return;
                    }
                    lineChartViewBuilder = new CustomLineChartDialog.Builder(this).setTitle(pointTypeIndex == POINT_TYPE_KZ ? getString(R.string.control_points_being_collected) : getString(R.string.topographic_points_being_collected));
                    lineChartViewBuilder.setButtonConfirmTextColor(R.color.color_E2E1DD);
                    lineChartViewBuilder.setButtonConfirm(v -> checkRtkStringLimit());
                    customDialog = lineChartViewBuilder.create();
                    customDialog.show();
                    isControlSurvey = true;
                }
            } else {
                isMeasureAndRecord = true;
                tsMeasure();
            }
        }
    }

    @Override
    protected void onResume() {
        super.onResume();
        //左侧工具栏的状态显示
        SurveyUIUtils.updatePoleHrHeightDraggingToolBarLayout(
                this, true,
                binding.layoutPoleHrHeight, binding.layoutDragButton, binding.baseSurveyStakingSetting,
                true, false, null);
    }

    @Override
    protected void performMeasure() {
        watchSurvey();
    }

    /**
     * 音量快捷键
     */
    @Override
    protected void performMeasureRecord() {
        watchSurveyRecord();
    }

    @Override
    protected void onHelpClicked() {
        HelpWebActivity.navigation2PointSurveyHelp(this);
    }

    private void checkRtkStringLimit() {
        int indexNum = 0;
        int prePointTypeIndex = 0;
        float rtkHrmsLimitValue = 0;
        float rtkVrmsLimitValue = 0;
        float rtkPdopLimitValue = 0;
        float rtkDiffAgeLimitValue = 0;
        StringBuilder stringBuilder = new StringBuilder();
        GnssPosition gnssPosition = GnssPosition.getInstance();
        if (pointTypeIndex == POINT_TYPE_DX) {
            prePointTypeIndex = PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStatusLimitTopographicPointIndex, 0);
            rtkHrmsLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkHrmsLimitTopographicPointValue, 0.05f);
            rtkVrmsLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkVrmsLimitTopographicPointValue, 0.1f);
            rtkPdopLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkPdopLimitTopographicPointValue, 5);
            rtkDiffAgeLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkDiffAgeLimitTopographicPointValue, 5);
        } else if (pointTypeIndex == POINT_TYPE_KS) {
            prePointTypeIndex = PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStatusLimitQuickPointIndex, 0);
            rtkHrmsLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkHrmsLimitQuickPointValue, 0.2f);
            rtkVrmsLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkVrmsLimitQuickPointValue, 0.3f);
            rtkPdopLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkPdopLimitQuickPointValue, 5);
            rtkDiffAgeLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkDiffAgeLimitQuickPointValue, 10);
        } else if (pointTypeIndex == POINT_TYPE_KZ) {
            prePointTypeIndex = PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStatusLimitControlPointIndex, 0);
            rtkHrmsLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkHrmsLimitControlPointValue, 0.05f);
            rtkVrmsLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkVrmsLimitControlPointValue, 0.1f);
            rtkPdopLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkPdopLimitControlPointValue, 5);
            rtkDiffAgeLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkDiffAgeLimitControlPointValue, 5);
        } else if (pointTypeIndex == POINT_TYPE_LX) {
            prePointTypeIndex = PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStatusLimitContinuityPointIndex, 0);
            rtkHrmsLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkHrmsLimitContinuityPointIValue, 0.05f);
            rtkVrmsLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkVrmsLimitContinuityPointIValue, 0.1f);
            rtkPdopLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkPdopLimitContinuityPointIValue, 5);
            rtkDiffAgeLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkDiffAgeLimitContinuityPointIValue, 5);
        }
        switch (prePointTypeIndex) {
            case 0:
                if (gnssPosition.getPosType() != POS_TYPE_FIXED) {
                    indexNum = indexNum + 1;
                    stringBuilder.append("\n").append(indexNum).append(".").append(getString(R.string.limiting_conditions)).append(getString(R.string.colon)).append(getString(R.string.rtk_state_fixed)).append("，").append(getString(R.string.current)).append(getString(R.string.colon)).append(gnssPosition.getSolutionState());
                }
                break;
            case 1:
                if (gnssPosition.getPosType() != POS_TYPE_FIXED && gnssPosition.getPosType() != POS_TYPE_DIF3D) {
                    indexNum = indexNum + 1;
                    stringBuilder.append("\n").append(indexNum).append(".").append(getString(R.string.limiting_conditions)).append(getString(R.string.colon)).append(getString(R.string.rtk_state_diff)).append("，").append(getString(R.string.current)).append(getString(R.string.colon)).append(gnssPosition.getSolutionState());
                }
                break;
            case 2:
                if (gnssPosition.getPosType() != POS_TYPE_FIXED && gnssPosition.getPosType() != POS_TYPE_DIF3D && gnssPosition.getPosType() != POS_TYPE_FLOAT) {
                    indexNum = indexNum + 1;
                    stringBuilder.append("\n").append(indexNum).append(".").append(getString(R.string.limiting_conditions)).append(getString(R.string.colon)).append(getString(R.string.rtk_state_float)).append("，").append(getString(R.string.current)).append(getString(R.string.colon)).append(gnssPosition.getSolutionState());
                }
                break;
        }

        //formatDouble2Double防止四舍五入误差
        if (Util.formatDouble2Double(gnssPosition.getHrms(), 6) > rtkHrmsLimitValue) {
            indexNum = indexNum + 1;
            stringBuilder.append("\n").append(indexNum).append(".").append(getString(R.string.limiting_conditions)).append(getString(R.string.colon)).append("HRMS ≤ ").append(rtkHrmsLimitValue).append("，").append(getString(R.string.current)).append(getString(R.string.colon)).append(Util.formatDouble2Double(gnssPosition.getHrms(), 6));
        }
        if (Util.formatDouble2Double(gnssPosition.getVrms(), 6) > rtkVrmsLimitValue) {
            indexNum = indexNum + 1;
            stringBuilder.append("\n").append(indexNum).append(".").append(getString(R.string.limiting_conditions)).append(getString(R.string.colon)).append("VRMS ≤ ").append(rtkVrmsLimitValue).append("，").append(getString(R.string.current)).append(getString(R.string.colon)).append(Util.formatDouble2Double(gnssPosition.getVrms(), 6));
        }
        if (Util.formatDouble2Double(gnssPosition.getPdop(), 6) > rtkPdopLimitValue) {
            indexNum = indexNum + 1;
            stringBuilder.append("\n").append(indexNum).append(".").append(getString(R.string.limiting_conditions)).append(getString(R.string.colon)).append("PDOP ≤ ").append(rtkPdopLimitValue).append("，").append(getString(R.string.current)).append(getString(R.string.colon)).append(Util.formatDouble2Double(gnssPosition.getPdop(), 6));
        }
        if (gnssPosition.getDiffAge() > rtkDiffAgeLimitValue) {
            indexNum = indexNum + 1;
            stringBuilder.append("\n").append(indexNum).append(".").append(getString(R.string.limiting_conditions)).append(getString(R.string.colon)).append(getString(R.string.diff_age)).append(" ≤ ").append(rtkDiffAgeLimitValue).append("，").append(getString(R.string.current)).append(getString(R.string.colon)).append(gnssPosition.getDiffAge());
        }
        if (Gnss.getInstance().getDevice().isTiltOpen() && !GnssPosition.getInstance().isTiltEnable()) {
            indexNum = indexNum + 1;
            stringBuilder.append("\n").append(indexNum).append(".").append(getString(R.string.limiting_conditions)).append(getString(R.string.colon)).append(getString(R.string.tilt_status_need_to_shake_the_pole));
        }

        if (!TextUtils.isEmpty(stringBuilder.toString())) {
            CustomDialog.Builder builder = new CustomDialog.Builder(this);
            builder.setTitle(getString(R.string.prompt));
            builder.setContent(getString(R.string.follow_restrictions_are_not_met) + getString(R.string.colon) + stringBuilder.toString() + "\n" + getString(R.string.still_want_save_point_library));
            builder.setButtonCancel(getString(R.string.cancel), v -> {
            });
            builder.setButtonConfirm(getString(R.string.confirm), v -> {
                dialogSaveSurveyPointRecords();
            });

            CustomDialog customDialog = builder.create();
            customDialog.show();
            binding.ivPalyPaused.setImageResource(R.drawable.icon_survey_play);
            binding.tvPalyPaused.setText(getString(R.string.start));
            binding.etPointName.setEnabled(true);
            isContinuousSurvey = !isContinuousSurvey;
        } else {
            dialogSaveSurveyPointRecords();
        }
    }

    private void dialogSaveSurveyPointRecords() {
        if (pointTypeIndex == POINT_TYPE_KS || pointTypeIndex == POINT_TYPE_LX) {
            saveSurveyPointRecords();
        } else if (pointTypeIndex == POINT_TYPE_KZ) {
            saveSurveyPointRecords(PointConstants.POINT_TYPE_SURVEY_CONTROL_POINT, controlX, controlY, controlZ, controlLat, controlLon, controlAlt);
        } else {
            saveSurveyPointRecords(PointConstants.POINT_TYPE_SURVEY_DX, controlX, controlY, controlZ, controlLat, controlLon, controlAlt);
        }
    }

    /**
     * 保存测量点
     */

    private void saveSurveyPointRecords() {
        if (currentCoordinate != null && currentBlh != null)
            saveSurveyPointRecords(PointConstants.POINT_TYPE_SURVEY_KS, currentCoordinate.getX(), currentCoordinate.getY(), currentCoordinate.getZ(),
                    currentBlh.getLatitude(), currentBlh.getLongitude(), currentBlh.getAltitude());
    }

    private void saveSurveyPointRecords(int pointType, double x, double y, double z, double latitude, double longitude, double altitude) {
        String pointName = binding.etPointName.getText().toString();
        if (TextUtils.isEmpty(pointName)) {
            ToastUtils.showShort(R.string.point_name_cannot_be_empty);
            return;
        }
        String workType = IntentUtil.stringExtra(this, KEY_WORK_TYPE);
        String jobId = IntentUtil.stringExtra(this, KEY_JOB_ID);
//        String projectId = IntentUtil.stringExtra(this, KEY_PROJECT_ID);
        String projectId = SPUtils.INSTANCE.getString(SPConstants.PROJECT_ID, "");
//        String roadId = IntentUtil.stringExtra(this, KEY_ROAD_ID);
        DeviceInfoData deviceInfoData = DeviceUtils.collectDeviceInfo();

        PointRecord record = new PointRecord(projectId, jobId, ""
                , pointType, pointName, code
                , recordRemarks
                , x, y, z
                , latitude, longitude, altitude
                , PointConstants.POINT_FORMAT_LATITUDE, deviceInfoData);
        record.workType = workType;

        ThreadPoolUtil.execute(() -> {
            if (!Device.getInstance().rtkAllowRepeatName) {
                long count = PointDb.getInstance().countPointSurveyorWithJobAndName(jobId, record.name);
                if (count > 0) {
                    ToastUtils.showShort(R.string.record_failed_exists_same_name_point);
                    return;
                }
            }
            runOnUiThread(() -> {
                addDrawSurveyPoint(record);
                recordRemarks = "";
                DrawableUtils.tintModifyColorVector(binding.ivRemark, R.drawable.icon_remark_gray, App.instance.isThemeDark() ? R.color.white : R.color.color_8a8a8a);
                binding.tvRemark.setTextColor(getColor(App.instance.isThemeDark() ? R.color.white : R.color.color_cccccc));
                binding.tvRemark.setText(getString(R.string.no_remark));
                binding.etPointName.setText(Util.lastOfStringPlusStep(pointName, Device.getInstance().rtkPointNameAddStep));
            });
            PointDb.getInstance().save(record);
            // 上传测点信息到服务器
            viewModel.uploadPointEntry(record);
        });
    }

    @Override
    protected void initData() {
        ThreadPoolUtil.execute(() -> {
            surveyBaseRecordPointList = PointDb.getInstance().rawQueryListData(PointConstants.findPointSurveyByJob(IntentUtil.stringExtra(getIntent(), KEY_JOB_ID)));
            runOnUiThread(() -> {
                drawChartView();
                //设置画图chartView的属性
                binding.lineChartView.setMaxZoom(1000);
                binding.lineChartView.setViewportCalculationEnabled(false);
                binding.lineChartView.setCurrentViewport(binding.lineChartView.getMaximumViewport());
                drawCurrentSurveyLine(null, null);
            });
        });

        queryLastSurveyPointName();
    }

    /**
     * 查询最近的一个测量点，根据其名称设置以后测量的点名
     */
    @SuppressLint("SetTextI18n")
    private void queryLastSurveyPointName() {
        ThreadPoolUtil.execute(() -> {
            PointRecord pointRecord = PointDb.getInstance().rawQueryData(PointConstants.findLatestPointSurveyRecord(IntentUtil.stringExtra(getIntent(), KEY_JOB_ID)));
            runOnUiThread(() -> {
                if (pointRecord != null) {
                    binding.etPointName.setText(Util.lastOfStringPlusStep(pointRecord.name, Device.getInstance().rtkPointNameAddStep));
                } else {
                    binding.etPointName.setText("Pt1");
                }
            });
        });
    }

    private void drawChartView() {

        List<LineChartData> lineChartDataList = new ArrayList<>();
        LineChartData lineBasePointChartData = new LineChartData();
        int blackPointColor = App.instance.isThemeDark() ? getColor(R.color.color_E2E1DD) : getColor(R.color.color_575757);
        int redPointColor = getColor(R.color.color_FF1515);
        knowRecordPointLine.setCubic(false).setHasLabels(true).setHasPoints(true).setStrokeWidth(0).setHasLines(false).setPointRadius(3).
                setPointColor(blackPointColor).setShowLabelBackground(false).setLabelTextColor(blackPointColor);
        currentPointLine.setCubic(false).setHasLabels(true).setHasPoints(true).setStrokeWidth(0).setHasLines(false).setPointRadius(3).
                setPointColor(redPointColor).setShowLabelBackground(false).setLabelTextColor(redPointColor);
        addRecordPointLine.setCubic(false).setHasLabels(true).setHasPoints(true).setStrokeWidth(0).setHasLines(false).setPointRadius(3).
                setPointColor(blackPointColor).setShowLabelBackground(false).setLabelTextColor(blackPointColor);

        if (surveyBaseRecordPointList != null && surveyBaseRecordPointList.size() > 0) {
            for (int i = 0; i < surveyBaseRecordPointList.size(); i++) {
                PointRecord pointRecord = surveyBaseRecordPointList.get(i);
                knowRecordPointLine.getValues().add(new PointValue(pointRecord.y, pointRecord.x).setLabel(pointRecord.name));
            }
        }

        lineBasePointChartData.getLines().add(knowRecordPointLine);
        lineBasePointChartData.getLines().add(addRecordPointLine);
        lineBasePointChartData.getLines().add(currentPointLine);
        lineChartDataList.add(lineBasePointChartData);

        CommonUtils.mChartViewSetLineChartDatas(binding.lineChartView, lineChartDataList, false, false);
        if (binding.lineChartView.getChartRenderer() instanceof LineChartRenderer) {
            ((LineChartRenderer) binding.lineChartView.getChartRenderer()).calculateMaxViewport(1.5f);
        }
    }

    /**
     * 执行全站仪测量命令
     */
    public void tsMeasure() {
        if (!Device.getInstance().isDeviceConnected()) {
            ToastUtils.showShort(R.string.no_instrument_connected);
        }
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
            tsMeasCoordinateManager.measureCoordinate();
        }
    }

    @Override
    public void onRtkPositionReceived(double antennaHeight, Blh blh, Blh originalBlh, Coordinate coordinate, double hdop, double vdop, double pdop) {
        super.onRtkPositionReceived(antennaHeight, blh, originalBlh, coordinate, hdop, vdop, pdop);
        currentCoordinate = coordinate;
        currentBlh = blh;

        binding.layoutPoleHrHeight.tvPoleHrHeight.setText(Util.formatDouble2String(Gnss.getInstance().getDevice().getPoleHeight(), 3) + "m");

        drawCurrentSurveyLine(coordinate, blh);

        if (pointTypeIndex == POINT_TYPE_LX) {
            if (isContinuousSurvey) {
                if (recordByWayIndex == RECORD_BY_TIME) {
                    long cBackPressTime = SystemClock.uptimeMillis();
                    if (cBackPressTime - preBackPressTime >= Device.getInstance().rtkContinueTimeInterval * 800L) {
                        preBackPressTime = cBackPressTime;
                        checkRtkStringLimit();
                    }
                } else if (recordByWayIndex == RECORD_BY_DISTANCE) {
                    if (surveyBaseRecordPointList != null && surveyBaseRecordPointList.size() > 0) {
                        PointRecord pointRecord = surveyBaseRecordPointList.get(surveyBaseRecordPointList.size() - 1);
                        double distance2D = PointUtils.getDistance2D(new BasePoint(pointRecord.x, pointRecord.y), new BasePoint(coordinate.getX(), coordinate.getY()));
                        if (distance2D >= Device.getInstance().rtkContinueDistanceInterval) {
                            checkRtkStringLimit();
                        }
                    } else {
                        checkRtkStringLimit();
                    }
                }
            }
        } else if (pointTypeIndex == POINT_TYPE_KZ || pointTypeIndex == POINT_TYPE_DX) {
            if (isControlSurvey) {
                if (lineChartViewBuilder.pointList.size() < (pointTypeIndex == POINT_TYPE_KZ ? Device.getInstance().rtkSmoothPointsNumber : PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkSmoothPointsNumberTopographicPoint, 5))) {
                    lineChartViewBuilder.addDrawBlackSurveyPoint(coordinate, blh);
                } else {
                    double control_all_x = 0;
                    double control_all_y = 0;
                    double control_all_z = 0;
                    double control_all_lat = 0;
                    double control_all_lon = 0;
                    double control_all_alt = 0;
                    for (int i = 0; i < lineChartViewBuilder.pointList.size(); i++) {
                        control_all_x += lineChartViewBuilder.pointList.get(i).getX();
                        control_all_y += lineChartViewBuilder.pointList.get(i).getY();
                        control_all_z += lineChartViewBuilder.pointList.get(i).getZ();
                        control_all_lat += lineChartViewBuilder.pointList.get(i).getLatitude();
                        control_all_lon += lineChartViewBuilder.pointList.get(i).getLongitude();
                        control_all_alt += lineChartViewBuilder.pointList.get(i).getAltitude();
                    }

                    int sNum = pointTypeIndex == POINT_TYPE_KZ ? Device.getInstance().rtkSmoothPointsNumber : PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkSmoothPointsNumberTopographicPoint, 5);

                    controlX = control_all_x / sNum;
                    controlY = control_all_y / sNum;
                    controlZ = control_all_z / sNum;
                    controlLat = control_all_lat / sNum;
                    controlLon = control_all_lon / sNum;
                    controlAlt = control_all_alt / sNum;

                    lineChartViewBuilder.addDrawRedSurvryPoint(new ControlRecord(controlX, controlY, controlZ, controlLat, controlLon, controlAlt, pointTypeIndex == POINT_TYPE_KS ? getString(R.string.control_point) : getString(R.string.topographic_point)));
                    lineChartViewBuilder.pointList.add(0, new ControlRecord(controlX, controlY, controlZ, controlLat, controlLon, controlAlt, pointTypeIndex == POINT_TYPE_KS ? getString(R.string.control_point) : getString(R.string.topographic_point)));

                    isControlSurvey = false;

                    lineChartViewBuilder.setButtonConfirmTextColor(R.color.color_575757);
                }
            }
        }
    }

    @Override
    public void onMeasCoordinateSuccess(Coordinate coordinate) {
        super.onMeasCoordinateSuccess(coordinate);
        currentCoordinate = coordinate;
        currentBlh = new Blh(0.0, 0.0, 0.0);

        binding.layoutPoleHrHeight.tvPoleHrHeight.setText(Util.formatDouble2String(TsConfig.getInstance().getHr(), 3) + "m");

        binding.lineChartView.setZoomLevel(coordinate.getY(), coordinate.getX(), binding.lineChartView.getZoomLevel());
        binding.lineChartView.moveToWithAnimation(coordinate.getY(), coordinate.getX());
        drawCurrentSurveyLine(currentCoordinate, null);

        if (isMeasureAndRecord) {
            saveSurveyPointRecords();
        }
    }

    @Override
    public void onDeviceConnectLost() {
        super.onDeviceConnectLost();
        currentCoordinate = null;
        currentBlh = null;
        drawCurrentSurveyLine(currentCoordinate, currentBlh);
    }

    @Override
    public boolean isShowTopBarHrPoleLayout() {
        return false;
    }

    @Override
    protected void performRecord() {
        binding.layoutDragButton.drWatchSurveyRecord.performClick();
    }

    /**
     * 绘制实测点
     */
    @SuppressLint("SetTextI18n")
    private void drawCurrentSurveyLine(Coordinate currentCoordinate, Blh blh) {

        List<FunctionItem> selectFunctionItem = new SFUtils(this, "survey_bottom_all_information", "allData", "selData", ConstUtils.consJsonVariate.pointSurveyDefultJsonValue).getSelectFunctionItem();

        for (int i = 0; i < selectFunctionItem.size(); i++) {
            switch (i) {
                case 0:
                    binding.tvLeft01.setText(SFUtils.getFunctionJsonNameLanguage(this, selectFunctionItem.get(i).name) + ":" + getInformation(selectFunctionItem.get(i).id, currentCoordinate, blh));
                    break;
                case 1:
                    binding.tvLeft02.setText(SFUtils.getFunctionJsonNameLanguage(this, selectFunctionItem.get(i).name) + ":" + getInformation(selectFunctionItem.get(i).id, currentCoordinate, blh));
                    break;
                case 2:
                    binding.tvLeft03.setText(SFUtils.getFunctionJsonNameLanguage(this, selectFunctionItem.get(i).name) + ":" + getInformation(selectFunctionItem.get(i).id, currentCoordinate, blh));
                    break;
                case 3:
                    binding.tvRight01.setText(SFUtils.getFunctionJsonNameLanguage(this, selectFunctionItem.get(i).name) + ":" + getInformation(selectFunctionItem.get(i).id, currentCoordinate, blh));
                    break;
                case 4:
                    binding.tvRight02.setText(SFUtils.getFunctionJsonNameLanguage(this, selectFunctionItem.get(i).name) + ":" + getInformation(selectFunctionItem.get(i).id, currentCoordinate, blh));
                    break;
                case 5:
                    binding.tvRight03.setText(SFUtils.getFunctionJsonNameLanguage(this, selectFunctionItem.get(i).name) + ":" + getInformation(selectFunctionItem.get(i).id, currentCoordinate, blh));
                    break;
            }
        }

        binding.tvLeft01.setVisibility(selectFunctionItem.size() > 0 ? View.VISIBLE : View.GONE);
        binding.tvLeft02.setVisibility(selectFunctionItem.size() > 1 ? View.VISIBLE : View.GONE);
        binding.tvLeft03.setVisibility(selectFunctionItem.size() > 2 ? View.VISIBLE : View.GONE);
        binding.tvRight01.setVisibility(selectFunctionItem.size() > 3 ? View.VISIBLE : View.GONE);
        binding.tvRight02.setVisibility(selectFunctionItem.size() > 4 ? View.VISIBLE : View.GONE);
        binding.tvRight03.setVisibility(selectFunctionItem.size() > 5 ? View.VISIBLE : View.GONE);

        if (currentPointLine.getValues().isEmpty() && knowRecordPointLine.getValues().isEmpty()
                && addRecordPointLine.getValues().isEmpty()) {
            isFirstUpdateChartView = !isFirstUpdateChartView;
        }

        if (currentPointLine != null) {
            currentPointLine.getValues().clear();
        }

        if (currentCoordinate != null) {
            lastTimeStamp = System.currentTimeMillis();
            lastRTKBackSurveyPoint = new BasePoint(currentCoordinate.getX(), currentCoordinate.getY(), currentCoordinate.getZ());
            if (currentPointLine != null) {
                currentPointLine.getValues().add(new PointValue(currentCoordinate.getY(), currentCoordinate.getX()).setLabel(getString(R.string.measured_point)));
            }
        }

        if (isFirstUpdateChartView) {
            ChartViewUtils.centerInScreen(binding.lineChartView);
            isFirstUpdateChartView = !isFirstUpdateChartView;
        }

        ((LineChartRenderer) binding.lineChartView.getChartRenderer()).calculateMaxViewport(1.5f);
        binding.lineChartView.onChartDataChange();
    }

    private String getInformation(String id, Coordinate coordinate, Blh blh) {
        String info = "*";
        switch (Integer.parseInt(id)) {
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_X_ID:
                if (coordinate != null) {
                    info = Util.formatDouble2String(coordinate.getX(), App.notNumber);
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_Y_ID:
                if (coordinate != null) {
                    info = Util.formatDouble2String(coordinate.getY(), App.notNumber);
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_Z_ID:
                if (coordinate != null) {
                    info = Util.formatDouble2String(coordinate.getZ(), App.notNumber);
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_LAT_ID:
                if (blh != null) {
                    info = Util.radianToDmsDoubleString(Math.toRadians(blh.getLatitude()), 4, false);
                } else {
                    info = "*";
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_LON_ID:
                if (blh != null) {
                    info = Util.radianToDmsDoubleString(Math.toRadians(blh.getLongitude()), 4, false);
                } else {
                    info = "*";
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_ALT_ID:
                if (blh != null) {
                    info = Util.formatDouble2String(blh.getAltitude(), App.notNumber) + "m";
                } else {
                    info = "*";
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_SIT_ID_ID:
                if (!TextUtils.isEmpty(GnssPosition.getInstance().getBaseInfo().getBaseID())) {
                    info = GnssPosition.getInstance().getBaseInfo().getBaseID();
                } else {
                    info = "*";
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_SIT_DISTANCE_ID:
                if (GnssPosition.getInstance().getBaseInfo().isReceivedBase()) {
                    info = CoordUtil.formatDouble2String(GnssPosition.getInstance().baseInfo.getDistanceToBase(), App.notNumber) + "m";
                } else {
                    info = getString(R.string.base_info_not_received);
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_SPEED_ID:
                if (lastRTKBackSurveyPoint == null) {
                    info = "*";
                } else {
                    if (coordinate != null) {
                        double distance = PointUtils.getDistance2D(new BasePoint(coordinate.getX(), coordinate.getY()), new BasePoint(lastRTKBackSurveyPoint.getX(), lastRTKBackSurveyPoint.getY()));
                        double currdTime = System.currentTimeMillis();
                        double time = (currdTime - lastTimeStamp) / 1000;
                        info = Util.formatDouble2String(distance / time, App.notNumber);
                    }
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_ANGLE_ID:
                if (lastRTKBackSurveyPoint == null) {
                    info = "*";
                } else {
                    if (coordinate != null) {
                        float mTargetAzimuth = (float) Math.toDegrees(PointUtils.getDirectionDegrees(new BasePoint(coordinate.getX(), coordinate.getY()), new BasePoint(lastRTKBackSurveyPoint.getX(), lastRTKBackSurveyPoint.getY())));
                        info = Util.radianToDmsString(Math.toRadians(mTargetAzimuth), 6, true);
                    }
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_LAST_PROJECTION_ID:
                if (surveyBaseRecordPointList != null && surveyBaseRecordPointList.size() > 0) {
                    if (coordinate != null) {
                        PointRecord pointSurveyData = surveyBaseRecordPointList.get(surveyBaseRecordPointList.size() - 1);
                        double distance2D = PointUtils.getDistance2D(new BasePoint(pointSurveyData.x, pointSurveyData.y), new BasePoint(coordinate.getX(), coordinate.getY()));
                        info = Util.formatDouble2String(distance2D, App.notNumber) + "m";
                    }
                } else {
                    info = "*";
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_LAST_SPACE_ID:
                if (surveyBaseRecordPointList != null && surveyBaseRecordPointList.size() > 0) {
                    if (coordinate != null) {
                        PointRecord pointSurveyData = surveyBaseRecordPointList.get(surveyBaseRecordPointList.size() - 1);
                        double distance3D = PointUtils.getDistance3D(new BasePoint(pointSurveyData.x, pointSurveyData.y, pointSurveyData.h), new BasePoint(coordinate.getX(), coordinate.getY(), coordinate.getZ()));
                        info = Util.formatDouble2String(distance3D, App.notNumber) + "m";
                    }
                } else {
                    info = "*";
                }
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_ACCUMULAT_DISTANCE_2D:
                info = getAccumulat2D() + "m";
                break;
            case FunctionItemID.POINT_SURVEY_ID.SURVEY_ACCUMULAT_DISTANCE_3D:
                info = getAccumulat3D() + "m";
                break;
        }
        return info;
    }

    private double getAccumulat2D() {
        double total_distance_2d = 0;
        if (surveyBaseRecordPointList != null && surveyBaseRecordPointList.size() > 1) {
            for (int i = 0; i < surveyBaseRecordPointList.size(); i++) {
                if (i <= surveyBaseRecordPointList.size() - 2) {
                    double dis2D = PointUtils.getDistance2D(new BasePoint(surveyBaseRecordPointList.get(i).x, surveyBaseRecordPointList.get(i).y), new BasePoint(surveyBaseRecordPointList.get(i + 1).x, surveyBaseRecordPointList.get(i + 1).y));
                    total_distance_2d += dis2D;
                }
            }
        }
        return Util.formatDouble2Double(total_distance_2d, App.notNumber);
    }

    private double getAccumulat3D() {
        double total_distance_3d = 0;
        if (surveyBaseRecordPointList != null && surveyBaseRecordPointList.size() > 1) {
            for (int i = 0; i < surveyBaseRecordPointList.size(); i++) {
                if (i <= surveyBaseRecordPointList.size() - 2) {
                    double dis3D = PointUtils.getDistance3D(new BasePoint(surveyBaseRecordPointList.get(i).x, surveyBaseRecordPointList.get(i).y, surveyBaseRecordPointList.get(i).h), new BasePoint(surveyBaseRecordPointList.get(i + 1).x, surveyBaseRecordPointList.get(i + 1).y, surveyBaseRecordPointList.get(i + 1).h));
                    total_distance_3d += dis3D;
                }
            }
        }
        return Util.formatDouble2Double(total_distance_3d, App.notNumber);
    }

    /**
     * 动态添加测量点
     */
    private void addDrawSurveyPoint(PointRecord record) {
        addRecordPointLine.getValues().add(new PointValue(record.y, record.x).setLabel(record.name));
        surveyBaseRecordPointList.add(record);
        ((LineChartRenderer) binding.lineChartView.getChartRenderer()).calculateMaxViewport(1.5f);
        binding.lineChartView.onChartDataChange();

        boolean isTTSOpen = preferences.getBoolean("key_is_tts_open", true);
        if (isTTSOpen) {
            SoundPoolUtil.getInstanceUsageMedia().singlePlayStopRecord();
        }
    }

    public final ActivityResultLauncher<Intent> launcher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result != null) {
            switch (result.getResultCode()) {
                case LauncherEvent.launcher_code_apply_id:
                    if (result.getData() == null) {
                        return;
                    }
                    String applyCodeId = result.getData().getStringExtra(ConstUtils.intentConst.keyApplyIndexCodeRecord);
                    getCodeDb(applyCodeId);
                    break;
                case LauncherEvent.point_survey_list_change_number:
                    clearUpdateData();
                    break;
                case LauncherEvent.launcher_delete_point_lib:
                    clearUpdateData();
                    break;
            }
        }
    });

    private void clearUpdateData() {
        surveyBaseRecordPointList.clear();
        knowRecordPointLine.getValues().clear();
        addRecordPointLine.getValues().clear();
        initData();
    }

    public static void start(Context context) {
        ThreadPoolUtil.execute(() -> {
//            String projectId = CachedProject.currentProjectId();
//            String roadId = CachedCurrentRoad.currentRoadId(projectId);
//            String jobId = CachedCurrentJob.currentJobId(projectId);
//            JobRecord jobRecord = JobDb.getInstance()
//                    .rawQueryData(DBQueryConstant.findById(JobConstants.DB_NAME, jobId));
//            if (jobRecord == null) {
//                BingCeErrorCode.toast(BingCeErrorCode.NO_CURRENT_JOB);
//                return;
//            }

            // TODO: 2024/8/16 隐藏
            Intent intent = new Intent(context, PointSurveyNewActivity.class);
//            intent.putExtra(KEY_JOB_NAME, jobRecord.name);
//            intent.putExtra(KEY_JOB_ID, jobId);
            intent.putExtra(KEY_PROJECT_ID, "12121");
//            intent.putExtra(KEY_ROAD_ID, roadId);
            context.startActivity(intent);
        });
    }

    private FunctionBlockAdapter blockAdapter;
    private FunctionAdapter functionAdapter;
    private GridLayoutManager gridManager;

    private final List<String> scrollTab = new ArrayList<>();

    private int itemWidth = 0;
    private int lastRow = 0;
    private boolean isMove = false;//滑动状态
    private int scrollPosition = 0;
    private String currentTab;//当前的标签
    private int tabWidth = 0;//标签宽度

    private List<FunctionItem> allData;
    private List<FunctionItem> selData;
    private SFUtils sfUtils;
    private static final int MAX_COUNT = 6;
    private boolean isDrag = false;

    public void init() {

        sfUtils = new SFUtils(this, "survey_bottom_all_information", "allData", "selData", ConstUtils.consJsonVariate.pointSurveyDefultJsonValue);
        allData = sfUtils.getAllFunctionWithState();
        selData = sfUtils.getSelectFunctionItem();

        blockAdapter = new FunctionBlockAdapter(this, selData);
        binding.recyclerViewExist.setLayoutManager(new GridLayoutManager(this, 4));
        binding.recyclerViewExist.setAdapter(blockAdapter);
        binding.recyclerViewExist.addItemDecoration(new SpaceItemDecoration(4, CommonUtils.dip2px(10)));

        DefaultItemCallback callback = new DefaultItemCallback(blockAdapter);
        DefaultItemTouchHelper helper = new DefaultItemTouchHelper(callback);
        helper.attachToRecyclerView(binding.recyclerViewExist);
        blockAdapter.addDragChangedListener(data -> {
            sfUtils.saveSelectFunctionItem(selData);
            drawCurrentSurveyLine(currentCoordinate, currentBlh);
        });

        gridManager = new GridLayoutManager(this, 4);
        gridManager.setSpanSizeLookup(new GridLayoutManager.SpanSizeLookup() {
            @Override
            public int getSpanSize(int position) {
                FunctionItem fi = allData.get(position);
                return fi.isTitle ? 4 : 1;
            }
        });

        functionAdapter = new FunctionAdapter(this, allData);
        binding.recyclerViewAll.setLayoutManager(gridManager);
        binding.recyclerViewAll.setAdapter(functionAdapter);
        SpaceItemDecoration spaceDecoration = new SpaceItemDecoration(4, CommonUtils.dip2px(10));
        binding.recyclerViewAll.addItemDecoration(spaceDecoration);

        itemWidth = getAtyWidth(this) / 4 + CommonUtils.dip2px(2);

        resetEditHeight(selData.size());

        initTab();
    }

    public int getAtyWidth(Context context) {
        try {
            DisplayMetrics mDm = new DisplayMetrics();
            ((Activity) context).getWindowManager().getDefaultDisplay()
                    .getMetrics(mDm);
            return mDm.widthPixels;
        } catch (Exception e) {
            return 0;
        }
    }

    @SuppressLint("NotifyDataSetChanged")
    public void addListener() {
        functionAdapter.setOnItemAddListener(item -> {
            if (selData != null && selData.size() < MAX_COUNT) {   // 更新选择列表，所有列表已在内部进行更新
                try {
                    selData.add(item);
                    resetEditHeight(selData.size());
                    item.isSelect = true;

                    saveBottomInformation();
                    blockAdapter.notifyDataSetChanged();
                    return true;
                } catch (Exception e) {
                    e.printStackTrace();
                }
                return false;
            } else {
                ToastUtils.showShort(getString(R.string.selected_display_information_not_exceed) + MAX_COUNT + getString(R.string.ge));
                return false;
            }
        });

        blockAdapter.setOnItemRemoveListener(item -> {
            // 更新所有列表，选择列表已在内部进行更新
            try {
                if (item != null && item.name != null) {
                    for (int i = 0; i < allData.size(); i++) {
                        FunctionItem data = allData.get(i);
                        if (data != null && data.name != null) {
                            if (item.name.equals(data.name)) {
                                data.isSelect = false;
                                break;
                            }
                        }
                    }

                    saveBottomInformation();

                    functionAdapter.notifyDataSetChanged();
                }
                resetEditHeight(selData.size());
            } catch (Exception e) {
                e.printStackTrace();
            }

        });

        binding.recyclerViewAll.addOnScrollListener(onScrollListener);
    }

    /**
     * 保存底部信息栏的信息数据
     */
    private void saveBottomInformation() {
        sfUtils.saveSelectFunctionItem(selData);
        sfUtils.saveAllFunctionWithState(allData);
        behavior.setState(3);
        binding.ivShowClose.setImageResource(R.mipmap.icon_new_down_gray_arrow);

        if (!Device.getInstance().isDeviceConnected()) {
            drawCurrentSurveyLine(null, null);
        }
    }

    @Override
    public void onClick(View v) {
    }

    @SuppressLint("UseCompatLoadingForDrawables")
    private void initTab() {
        try {
            List<FunctionItem> tabs = sfUtils.getTabNames();

            if (tabs != null && tabs.size() > 0) {
                currentTab = tabs.get(0).name;
                int padding = CommonUtils.dip2px(10);
                int size = tabs.size();
                for (int i = 0; i < size; i++) {
                    FunctionItem item = tabs.get(i);
                    if (item.isTitle) {
                        scrollTab.add(item.name);
                        RadioButton rb = new RadioButton(this);
                        rb.setPadding(padding, 0, padding, 0);
                        rb.setButtonDrawable(null);
                        rb.setGravity(Gravity.CENTER);
                        rb.setText(item.name);
                        rb.setTag(item.subItemCount);
                        rb.setTextSize(TypedValue.COMPLEX_UNIT_SP, 14);
                        try {
                            rb.setTextColor(getColorStateList(R.color.color_777777));
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                        rb.setCompoundDrawablesWithIntrinsicBounds(null, null, null, getDrawable(R.drawable.bg_block_tab));
                        rb.setOnCheckedChangeListener(onCheckedChangeListener);
                        binding.rgTab.addView(rb);
                    }
                }
                ((RadioButton) binding.rgTab.getChildAt(0)).setChecked(true);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private final CompoundButton.OnCheckedChangeListener onCheckedChangeListener = new CompoundButton.OnCheckedChangeListener() {
        @Override
        public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
            try {
                int position = (int) buttonView.getTag();
                String text = buttonView.getText().toString();
                if (!currentTab.equals(text) && isChecked) {
                    currentTab = text;
                    moveToPosition(position);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

        }
    };

    private void resetEditHeight(int size) {
        try {
            if (size == 0) {
                size = 1;
            }
            int row = size / 4 + (size % 4 > 0 ? 1 : 0);
            if (row <= 0)
                row = 1;
            if (lastRow != row) {
                lastRow = row;
                ViewGroup.LayoutParams params = binding.recyclerViewExist.getLayoutParams();
                params.height = itemWidth * row;
                binding.recyclerViewExist.setLayoutParams(params);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private void moveToPosition(int position) {
        int first = gridManager.findFirstVisibleItemPosition();
        int end = gridManager.findLastVisibleItemPosition();
        if (first == -1 || end == -1)
            return;
        if (position <= first) {      //移动到前面
            gridManager.scrollToPosition(position);
        } else if (position >= end) {      //移动到后面
            isMove = true;
            scrollPosition = position;
            gridManager.smoothScrollToPosition(binding.recyclerViewAll, null, position);
        } else {//中间部分
            int n = position - gridManager.findFirstVisibleItemPosition();
            if (n > 0 && n < allData.size()) {
                int top = Objects.requireNonNull(gridManager.findViewByPosition(position)).getTop();
                binding.recyclerViewAll.scrollBy(0, top);
            }
        }
    }

    private final RecyclerView.OnScrollListener onScrollListener = new RecyclerView.OnScrollListener() {
        @Override
        public void onScrollStateChanged(@NonNull RecyclerView recyclerView, int newState) {
            super.onScrollStateChanged(recyclerView, newState);
            try {
                if (isMove && newState == RecyclerView.SCROLL_STATE_IDLE) {
                    isMove = false;
                    View view = gridManager.findViewByPosition(scrollPosition);
                    if (view != null) {
                        int top = (int) view.getTop();
                        recyclerView.scrollBy(0, top);
                    }
                }
                isDrag = newState == RecyclerView.SCROLL_STATE_DRAGGING;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        @Override
        public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
            super.onScrolled(recyclerView, dx, dy);
            if (isDrag) {  //拖动过程中
                int position = gridManager.findFirstVisibleItemPosition();
                if (position > 0) {
                    for (int i = 0; i < position + 1; i++) {
                        if (allData.get(i).isTitle) {
                            currentTab = allData.get(i).name;
                        }
                    }
                    scrollTab(currentTab);
                }
            }
        }
    };

    private void scrollTab(String newTab) {
        try {
            int position = scrollTab.indexOf(currentTab);
            int targetPosition = scrollTab.indexOf(newTab);
            currentTab = newTab;
            if (targetPosition != -1) {
                int x = (targetPosition - position) * getTabWidth();
                RadioButton radioButton = ((RadioButton) binding.rgTab.getChildAt(targetPosition));
                radioButton.setOnCheckedChangeListener(null);
                radioButton.setChecked(true);
                radioButton.setOnCheckedChangeListener(onCheckedChangeListener);
                binding.horizonLScrollView.scrollBy(x, 0);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private int getTabWidth() {
        if (tabWidth == 0) {
            if (binding.rgTab.getChildCount() != 0) {
                tabWidth = binding.rgTab.getWidth() / binding.rgTab.getChildCount();
            }
        }
        return tabWidth;
    }

    @Override
    public void onRtkTiltSwitched(boolean onOff) {
        super.onRtkTiltSwitched(onOff);
        if (onOff) {
            binding.baseSurveyStakingSetting.ivTiltPrimAuto.setImageResource(R.drawable.icon_survey_tilt_black);
            binding.baseSurveyStakingSetting.tvTiltPrimAuto.setTextColor(getColor(R.color.black));
        } else {
            binding.baseSurveyStakingSetting.ivTiltPrimAuto.setImageResource(R.drawable.icon_survey_tilt_gray);
            binding.baseSurveyStakingSetting.tvTiltPrimAuto.setTextColor(getColor(R.color.color_BCBCBC));
        }
    }

    public static void start(String workType, String jobId) {
        Bundle bundle = new Bundle();
        bundle.putString(KEY_WORK_TYPE, workType);
        bundle.putString(KEY_JOB_ID, jobId);
        ActivityUtils.startActivity(bundle, PointSurveyNewActivity.class);
    }
}