package com.project.survey.ui.pointmeasure.measure;

import static com.bingce.surveyor.util.ConstUtils.pointConst.POINT_TYPE_DX;
import static com.bingce.surveyor.util.ConstUtils.pointConst.POINT_TYPE_KS;
import static com.bingce.surveyor.util.ConstUtils.pointConst.POINT_TYPE_KZ;
import static com.bingce.surveyor.util.ConstUtils.pointConst.POINT_TYPE_LX;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.Intent;
import android.view.View;
import android.widget.CompoundButton;

import com.bingce.device.Device;
import com.bingce.device.enums.DeviceTypeEnum;
import com.bingce.surveyor.util.ConstUtils;
import com.bingce.surveyor.util.PreferencesUtil;
import com.bingce.surveyor.util.dialog.CustomRecycleDialog;
import com.bingce.utils.IntentUtil;
import com.bingce.utils.Util;
import com.project.survey.R;
import com.project.survey.databinding.ActivityPointSurveySettingBinding;
import com.project.survey.ui.base.BaseSurveyNewActivity;

import java.util.ArrayList;
import java.util.List;

public class PointSurveySettingActivity extends BaseSurveyNewActivity {
    private ActivityPointSurveySettingBinding binding;
    private int limitHrmsIndex  = -1;
    private int limitVrmsIndex  = -1;
    private int limitPdopIndex = -1;
    private int limitAgeIndex = -1;
    private int limitAddStepNameIndex = -1;
    private int limitSmoothPointNumberIndex = -1;
    private int limitSmoothPointNumberIndexDx = -1;
    private int limitContinueTimeIntervalIndex = -1;
    private int limitContinueDistanceIntervalIndex = -1;
    private int pointType;
    private String pointTypeName;
    private int limitStatusIndex;
    private float limitHrmsValue,limitVrmsValue,pdopLimitValue,ageLimitValue;

    @Override
    public View getContentView() {
        binding = ActivityPointSurveySettingBinding.inflate(getLayoutInflater());
        return binding.getRoot();
    }

    @Override
    protected void initView() {
        pointType = IntentUtil.intExtra(getIntent(),POINT_TYPE);
    }

    @Override
    protected void initData() {

        boolean isOnlyCommLimit = IntentUtil.boolExtra(getIntent(), IS_ONLY_COMM_LIMIT);

        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
            setTitle(getString(R.string.ts_survey_setting));
            if (isOnlyCommLimit) {
                binding.llLayoutSetting.setVisibility(View.GONE);
            } else {
                binding.rlStatusLimit.setVisibility(View.GONE);
                binding.llHvpd.setVisibility(View.GONE);
                binding.rlSmoothPointNumberLimit.setVisibility(View.GONE);
                binding.rlContinueSurveyTimeInterval.setVisibility(View.GONE);
                binding.rlContinueSurveyDistanceInterval.setVisibility(View.GONE);
                binding.lineSmoothPointNumber.setVisibility(View.GONE);
                binding.lineContinueSurveyTimeInterval.setVisibility(View.GONE);
                binding.lineContinueSurveyDistanceInterval.setVisibility(View.GONE);
            }
        } else {
            if (pointType == POINT_TYPE_DX){
                pointTypeName = getString(R.string.topographic_point);
                binding.rlSmoothPointNumberLimit.setVisibility(View.VISIBLE);
                binding.rlContinueSurveyTimeInterval.setVisibility(View.GONE);
                binding.rlContinueSurveyDistanceInterval.setVisibility(View.GONE);
                binding.lineSmoothPointNumber.setVisibility(View.VISIBLE);
                binding.lineContinueSurveyTimeInterval.setVisibility(View.GONE);
                binding.lineContinueSurveyDistanceInterval.setVisibility(View.GONE);
            } else if (pointType == POINT_TYPE_KS) {
                pointTypeName = getString(R.string.quick_point);
                if (isOnlyCommLimit) {
                    binding.llSpecialtySetting.setVisibility(View.GONE);
                } else {
                    binding.rlSmoothPointNumberLimit.setVisibility(View.GONE);
                    binding.rlContinueSurveyTimeInterval.setVisibility(View.GONE);
                    binding.rlContinueSurveyDistanceInterval.setVisibility(View.GONE);
                    binding.lineSmoothPointNumber.setVisibility(View.GONE);
                    binding.lineContinueSurveyTimeInterval.setVisibility(View.GONE);
                    binding.lineContinueSurveyDistanceInterval.setVisibility(View.GONE);
                }
            }else if (pointType == POINT_TYPE_KZ){
                if (isOnlyCommLimit) {
                    binding.llSpecialtySetting.setVisibility(View.GONE);
                } else {
                    pointTypeName = getString(R.string.control_point);
                    binding.rlSmoothPointNumberLimit.setVisibility(View.VISIBLE);
                    binding.rlContinueSurveyTimeInterval.setVisibility(View.GONE);
                    binding.rlContinueSurveyDistanceInterval.setVisibility(View.GONE);
                    binding.lineSmoothPointNumber.setVisibility(View.VISIBLE);
                    binding.lineContinueSurveyTimeInterval.setVisibility(View.GONE);
                    binding.lineContinueSurveyDistanceInterval.setVisibility(View.GONE);
                }
            }else if (pointType == POINT_TYPE_LX){
                if (isOnlyCommLimit) {
                    binding.llSpecialtySetting.setVisibility(View.GONE);
                } else {
                    pointTypeName = getString(R.string.continuity_point);
                    binding.rlSmoothPointNumberLimit.setVisibility(View.GONE);
                    binding.rlContinueSurveyTimeInterval.setVisibility(View.VISIBLE);
                    binding.rlContinueSurveyDistanceInterval.setVisibility(View.VISIBLE);
                    binding.lineSmoothPointNumber.setVisibility(View.GONE);
                    binding.lineContinueSurveyTimeInterval.setVisibility(View.VISIBLE);
                    binding.lineContinueSurveyDistanceInterval.setVisibility(View.VISIBLE);
                }
            }
            setTitle(getString(R.string.rtk_survey) + " - " + pointTypeName + getString(R.string.settings));
        }

        limitStatus();
        limitHrms();
        limitVrms();
        limitPdop();
        limitAge();
        allowRepeatName();
        limitAddStepName();
        if (pointType == POINT_TYPE_KZ){
            limitSmoothPointNumberKZ();
        }else if (pointType == POINT_TYPE_DX){
            limitSmoothPointNumberDX();
        }
        limitContinueTimeInterval();
        limitContinueDistanceInterval();
    }

    /**
     * 状态解
     */
    private void limitStatus() {
        List<String> limitStatusList = new ArrayList<>();
        limitStatusList.add(getString(R.string.rtk_state_fixed));
        limitStatusList.add(getString(R.string.rtk_state_diff_3D));
        limitStatusList.add(getString(R.string.rtk_state_float));
        limitStatusList.add(getString(R.string.rtk_state_single));
        if (pointType == POINT_TYPE_DX){
            limitStatusIndex = PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStatusLimitTopographicPointIndex,0);
        } else if (pointType == POINT_TYPE_KS) {
            limitStatusIndex = PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStatusLimitQuickPointIndex,0);
        }else if (pointType == POINT_TYPE_KZ){
            limitStatusIndex = PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStatusLimitControlPointIndex,0);
        }else if (pointType == POINT_TYPE_LX){
            limitStatusIndex = PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkStatusLimitContinuityPointIndex,0);
        }
        binding.tvStatusLimit.setText(limitStatusList.get(limitStatusIndex));
        binding.rlStatusLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.solution_status_limit, limitStatusList, limitStatusIndex, false, (index, itemString) -> {
            limitStatusIndex = index;
            binding.tvStatusLimit.setText(limitStatusList.get(index));
            if (pointType == POINT_TYPE_DX){
                PreferencesUtil.putPreference(ConstUtils.preferConst.rtkStatusLimitTopographicPointIndex, index);
            } else if (pointType == POINT_TYPE_KS) {
                PreferencesUtil.putPreference(ConstUtils.preferConst.rtkStatusLimitQuickPointIndex, index);
            }else if (pointType == POINT_TYPE_KZ){
                PreferencesUtil.putPreference(ConstUtils.preferConst.rtkStatusLimitControlPointIndex, index);
            }else if (pointType == POINT_TYPE_LX){
                PreferencesUtil.putPreference(ConstUtils.preferConst.rtkStatusLimitContinuityPointIndex, index);
            }
        }));
    }

    /**
     * Hrms
     */
    @SuppressLint("SetTextI18n")
    private void limitHrms() {
        List<String> limitHrmsList = new ArrayList<>();
        limitHrmsList.add("0.05");
        limitHrmsList.add("0.10");
        limitHrmsList.add("0.20");
        limitHrmsList.add("0.30");

        if (pointType == POINT_TYPE_DX){
            limitHrmsValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkHrmsLimitTopographicPointValue,0.05f);
        } else if (pointType == POINT_TYPE_KS) {
            limitHrmsValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkHrmsLimitQuickPointValue,0.20f);
        }else if (pointType == POINT_TYPE_KZ){
            limitHrmsValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkHrmsLimitControlPointValue,0.05f);
        }else if (pointType == POINT_TYPE_LX){
            limitHrmsValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkHrmsLimitContinuityPointIValue,0.05f);
        }
        for (int i = 0; i < limitHrmsList.size(); i++) {
            if (limitHrmsValue == Float.parseFloat(limitHrmsList.get(i))){
                limitHrmsIndex = i;
            }
        }
        binding.tvHrmsLimit.setText(limitHrmsValue + getString(R.string.meter));
        binding.rlHrmsLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.hrms_limit, limitHrmsList, limitHrmsIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL,true, Util.formatDouble2StringDotAuto(limitHrmsValue), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitHrmsIndex = index;
                if (index != -1){
                    binding.tvHrmsLimit.setText(limitHrmsList.get(index) + getString(R.string.meter));
                    limitHrmsValue = Float.parseFloat(limitHrmsList.get(index));
                }else {
                    binding.tvHrmsLimit.setText(itemString + getString(R.string.meter));
                    limitHrmsValue = Float.parseFloat(itemString);
                }
                if (pointType == POINT_TYPE_DX){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkHrmsLimitTopographicPointValue, limitHrmsValue);
                } else if (pointType == POINT_TYPE_KS) {
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkHrmsLimitQuickPointValue, limitHrmsValue);
                }else if (pointType == POINT_TYPE_KZ){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkHrmsLimitControlPointValue, limitHrmsValue);
                }else if (pointType == POINT_TYPE_LX){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkHrmsLimitContinuityPointIValue, limitHrmsValue);
                }
            }
        }));
    }

    /**
     * Vrms
     */
    @SuppressLint("SetTextI18n")
    private void limitVrms() {
        List<String> limitVrmsList = new ArrayList<>();
        limitVrmsList.add("0.05");
        limitVrmsList.add("0.10");
        limitVrmsList.add("0.20");
        limitVrmsList.add("0.30");

        if (pointType == POINT_TYPE_DX){
            limitVrmsValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkVrmsLimitTopographicPointValue,0.10f);
        } else if (pointType == POINT_TYPE_KS) {
            limitVrmsValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkVrmsLimitQuickPointValue,0.30f);
        }else if (pointType == POINT_TYPE_KZ){
            limitVrmsValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkVrmsLimitControlPointValue,0.10f);
        }else if (pointType == POINT_TYPE_LX){
            limitVrmsValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkVrmsLimitContinuityPointIValue,0.10f);
        }
        for (int i = 0; i < limitVrmsList.size(); i++) {
            if (limitVrmsValue == Float.parseFloat(limitVrmsList.get(i))){
                limitVrmsIndex = i;
            }
        }

        binding.tvVrmsLimit.setText(limitVrmsValue + getString(R.string.meter));
        binding.rlVrmsLimit.setOnClickListener(new View.OnClickListener() {
            @SuppressLint("SetTextI18n")
            @Override
            public void onClick(View view) {
                CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.vrms_limit, limitVrmsList, limitVrmsIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL,true,Util.formatDouble2StringDotAuto(limitVrmsValue), (index, itemString) -> {
                    limitVrmsIndex = index;
                    if (index != -1){
                        binding.tvVrmsLimit.setText(limitVrmsList.get(index) + getString(R.string.meter));
                        limitVrmsValue = Float.parseFloat(limitVrmsList.get(index));
                    }else {
                        binding.tvVrmsLimit.setText(itemString + getString(R.string.meter));
                        limitVrmsValue = Float.parseFloat(itemString);
                    }
                    if (pointType == POINT_TYPE_DX){
                        PreferencesUtil.putPreference(ConstUtils.preferConst.rtkVrmsLimitTopographicPointValue, limitVrmsValue);
                    } else if (pointType == POINT_TYPE_KS) {
                        PreferencesUtil.putPreference(ConstUtils.preferConst.rtkVrmsLimitQuickPointValue, limitVrmsValue);
                    }else if (pointType == POINT_TYPE_KZ){
                        PreferencesUtil.putPreference(ConstUtils.preferConst.rtkVrmsLimitControlPointValue, limitVrmsValue);
                    }else if (pointType == POINT_TYPE_LX){
                        PreferencesUtil.putPreference(ConstUtils.preferConst.rtkVrmsLimitContinuityPointIValue, limitVrmsValue);
                    }
                });
            }
        });
    }

    /**
     * Pdop
     */
    private void limitPdop() {
        List<String> limitPdopList = new ArrayList<>();
        limitPdopList.add("5.00");
        limitPdopList.add("20.00");
        limitPdopList.add("50.00");
        limitPdopList.add("100.00");

        if (pointType == POINT_TYPE_DX){
            pdopLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkPdopLimitTopographicPointValue,5f);
        } else if (pointType == POINT_TYPE_KS) {
            pdopLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkPdopLimitQuickPointValue,5f);
        }else if (pointType == POINT_TYPE_KZ){
            pdopLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkPdopLimitControlPointValue,5f);
        }else if (pointType == POINT_TYPE_LX){
            pdopLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkPdopLimitContinuityPointIValue,5f);
        }

        for (int i = 0; i < limitPdopList.size(); i++) {
            if (pdopLimitValue == Float.parseFloat(limitPdopList.get(i))){
                limitPdopIndex = i;
            }
        }

        binding.tvPdopLimit.setText(Util.formatDouble2StringDotAuto(pdopLimitValue));
        binding.rlPdopLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.pdop_limit, limitPdopList, limitPdopIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL,true, Util.formatDouble2StringDotAuto(pdopLimitValue), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitPdopIndex = index;
                if (index != -1){
                    binding.tvPdopLimit.setText(limitPdopList.get(index));
                    pdopLimitValue = Float.parseFloat(limitPdopList.get(index));
                }else {
                    binding.tvPdopLimit.setText(itemString);
                    pdopLimitValue = Float.parseFloat(itemString);
                }
                if (pointType == POINT_TYPE_DX){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkPdopLimitTopographicPointValue, pdopLimitValue);
                } else if (pointType == POINT_TYPE_KS) {
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkPdopLimitQuickPointValue, pdopLimitValue);
                }else if (pointType == POINT_TYPE_KZ){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkPdopLimitControlPointValue, pdopLimitValue);
                }else if (pointType == POINT_TYPE_LX){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkPdopLimitContinuityPointIValue, pdopLimitValue);
                }
            }
        }));
    }

    @SuppressLint("SetTextI18n")
    private void limitAge() {
        List<String> limitAgeList = new ArrayList<>();
        limitAgeList.add("5");
        limitAgeList.add("10");
        limitAgeList.add("15");
        limitAgeList.add("25");

        if (pointType == POINT_TYPE_DX){
            ageLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkDiffAgeLimitTopographicPointValue,5f);
        } else if (pointType == POINT_TYPE_KS) {
            ageLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkDiffAgeLimitQuickPointValue,10f);
        }else if (pointType == POINT_TYPE_KZ){
            ageLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkDiffAgeLimitControlPointValue,5f);
        }else if (pointType == POINT_TYPE_LX){
            ageLimitValue = PreferencesUtil.getPreferenceFloat(ConstUtils.preferConst.rtkDiffAgeLimitContinuityPointIValue,5f);
        }

        for (int i = 0; i < limitAgeList.size(); i++) {
            if (ageLimitValue == Integer.parseInt(limitAgeList.get(i))){
                limitAgeIndex = i;
            }
        }

        binding.tvAgeLimit.setText(Util.formatDouble2StringDotAuto(ageLimitValue) + getString(R.string.sec));
        binding.rlAgeLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.diff_age_limit, limitAgeList, limitAgeIndex, CustomRecycleDialog.Builder.INPUT_TYPE_INTEGER,true,Util.formatDouble2StringDotAuto(ageLimitValue), new CustomRecycleDialog.IClickCallback() {
            @SuppressLint("SetTextI18n")
            @Override
            public void onItemSelected(int index, String itemString) {
                limitAgeIndex = index;
                if (index != -1){
                    binding.tvAgeLimit.setText(limitAgeList.get(index) + getString(R.string.sec));
                    ageLimitValue = Integer.parseInt(limitAgeList.get(index));
                }else {
                    binding.tvAgeLimit.setText(itemString + getString(R.string.sec));
                    ageLimitValue = Integer.parseInt(itemString);
                }
                if (pointType == POINT_TYPE_DX){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkDiffAgeLimitTopographicPointValue, ageLimitValue);
                } else if (pointType == POINT_TYPE_KS) {
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkDiffAgeLimitQuickPointValue, ageLimitValue);
                }else if (pointType == POINT_TYPE_KZ){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkDiffAgeLimitControlPointValue, ageLimitValue);
                }else if (pointType == POINT_TYPE_LX){
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkDiffAgeLimitContinuityPointIValue, ageLimitValue);
                }
            }
        }));
    }

    private void allowRepeatName() {
        binding.switchAllowRepeatName.setChecked(Device.getInstance().rtkAllowRepeatName);
        binding.switchAllowRepeatName.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            @Override
            public void onCheckedChanged(CompoundButton compoundButton, boolean isChecked) {
                Device.getInstance().rtkAllowRepeatName = isChecked;
                Util.putPreference(ConstUtils.preferConst.rtkAllowRepeatName, Device.getInstance().rtkAllowRepeatName);
            }
        });
    }

    public void limitAddStepName(){
        List<String> limitAddStepNameList = new ArrayList<>();
        limitAddStepNameList.add("1");
        limitAddStepNameList.add("2");
        limitAddStepNameList.add("5");
        limitAddStepNameList.add("10");
        for (int i = 0; i < limitAddStepNameList.size(); i++) {
            if (Device.getInstance().rtkPointNameAddStep == Integer.parseInt(limitAddStepNameList.get(i))){
                limitAddStepNameIndex = i;
            }
        }

        binding.tvAddStepNameLimit.setText(String.valueOf(Device.getInstance().rtkPointNameAddStep));
        binding.rlAddStepNameLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.point_name_add_step, limitAddStepNameList, limitAddStepNameIndex, CustomRecycleDialog.Builder.INPUT_TYPE_INTEGER,true,Util.formatDouble2StringDotAuto(Device.getInstance().rtkPointNameAddStep), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                Device.getInstance().rtkPointNameAddStep = Integer.parseInt(itemString);
                limitAddStepNameIndex = index;
                if (index != -1){
                    binding.tvAddStepNameLimit.setText(limitAddStepNameList.get(index));
                }else {
                    binding.tvAddStepNameLimit.setText(itemString);
                }
                Util.putPreference(ConstUtils.preferConst.rtkPointNameAddStep, Device.getInstance().rtkPointNameAddStep);
            }
        }));
    }

    private void limitSmoothPointNumberKZ() {
        List<String> limitSmoothPointNumberList = new ArrayList<>();
        limitSmoothPointNumberList.add("1");
        limitSmoothPointNumberList.add("2");
        limitSmoothPointNumberList.add("5");
        limitSmoothPointNumberList.add("10");
        for (int i = 0; i < limitSmoothPointNumberList.size(); i++) {
            if (Device.getInstance().rtkSmoothPointsNumber == Integer.parseInt(limitSmoothPointNumberList.get(i))){
                limitSmoothPointNumberIndex = i;
            }
        }

        binding.tvSmoothPointNumberLimit.setText(String.valueOf(Device.getInstance().rtkSmoothPointsNumber));
        binding.rlSmoothPointNumberLimit.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.smooth_points_number, limitSmoothPointNumberList, limitSmoothPointNumberIndex, CustomRecycleDialog.Builder.INPUT_TYPE_INTEGER,true,Util.formatDouble2StringDotAuto(Device.getInstance().rtkSmoothPointsNumber), new CustomRecycleDialog.IClickCallback() {
                    @Override
                    public void onItemSelected(int index, String itemString) {
                        Device.getInstance().rtkSmoothPointsNumber = Integer.parseInt(itemString);
                        limitSmoothPointNumberIndex = index;
                        if (index != -1){
                            binding.tvSmoothPointNumberLimit.setText(limitSmoothPointNumberList.get(index));
                        }else {
                            binding.tvSmoothPointNumberLimit.setText(itemString);
                        }
                        Util.putPreference(ConstUtils.preferConst.rtkSmoothPointsNumber, Device.getInstance().rtkSmoothPointsNumber);
                    }
                });
            }
        });
    }

    private void limitSmoothPointNumberDX(){
        List<String> limitSmoothPointNumberList = new ArrayList<>();
        limitSmoothPointNumberList.add("1");
        limitSmoothPointNumberList.add("2");
        limitSmoothPointNumberList.add("5");
        limitSmoothPointNumberList.add("10");
        for (int i = 0; i < limitSmoothPointNumberList.size(); i++) {
            if (PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkSmoothPointsNumberTopographicPoint,5) == Integer.parseInt(limitSmoothPointNumberList.get(i))){
                limitSmoothPointNumberIndexDx = i;
            }
        }
        binding.tvSmoothPointNumberLimit.setText(String.valueOf(PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkSmoothPointsNumberTopographicPoint,5)));
        binding.rlSmoothPointNumberLimit.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.smooth_points_number, limitSmoothPointNumberList, limitSmoothPointNumberIndexDx, CustomRecycleDialog.Builder.INPUT_TYPE_INTEGER,true,Util.formatDouble2StringDotAuto(PreferencesUtil.getPreferenceInt(ConstUtils.preferConst.rtkSmoothPointsNumberTopographicPoint,5)), new CustomRecycleDialog.IClickCallback() {
            @Override
            public void onItemSelected(int index, String itemString) {
                limitSmoothPointNumberIndexDx = index;
                if (index != -1){
                    binding.tvSmoothPointNumberLimit.setText(limitSmoothPointNumberList.get(index));
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkSmoothPointsNumberTopographicPoint, Integer.parseInt(limitSmoothPointNumberList.get(index)));
                }else {
                    binding.tvSmoothPointNumberLimit.setText(itemString);
                    PreferencesUtil.putPreference(ConstUtils.preferConst.rtkSmoothPointsNumberTopographicPoint, Integer.parseInt(itemString));
                }
            }
        }));
    }

    @SuppressLint("SetTextI18n")
    private void limitContinueTimeInterval() {
        List<String> limitContinueTimeList = new ArrayList<>();
        limitContinueTimeList.add("1");
        limitContinueTimeList.add("2");
        limitContinueTimeList.add("5");
        limitContinueTimeList.add("10");
        for (int i = 0; i < limitContinueTimeList.size(); i++) {
            if (Device.getInstance().rtkContinueTimeInterval == Integer.parseInt(limitContinueTimeList.get(i))){
                limitContinueTimeIntervalIndex = i;
            }
        }

        binding.tvContinueSurveyTimeInterval.setText(Device.getInstance().rtkContinueTimeInterval + getString(R.string.sec));
        binding.rlContinueSurveyTimeInterval.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.continuous_survey_time_interval, limitContinueTimeList, limitContinueTimeIntervalIndex, CustomRecycleDialog.Builder.INPUT_TYPE_INTEGER,true,Util.formatDouble2StringDotAuto(Device.getInstance().rtkContinueTimeInterval), new CustomRecycleDialog.IClickCallback() {
            @SuppressLint("SetTextI18n")
            @Override
            public void onItemSelected(int index, String itemString) {
                Device.getInstance().rtkContinueTimeInterval = Integer.parseInt(itemString);
                limitContinueTimeIntervalIndex = index;
                if (index != -1){
                    binding.tvContinueSurveyTimeInterval.setText(limitContinueTimeList.get(index) + getString(R.string.sec));
                }else {
                    binding.tvContinueSurveyTimeInterval.setText(itemString + getString(R.string.sec));
                }
                Util.putPreference(ConstUtils.preferConst.rtkContinueTimeInterval, Device.getInstance().rtkContinueTimeInterval);
            }
        }));
    }

    @SuppressLint("SetTextI18n")
    private void limitContinueDistanceInterval() {
        List<String> limitContinueDistanceList = new ArrayList<>();
        limitContinueDistanceList.add("1.00");
        limitContinueDistanceList.add("2.00");
        limitContinueDistanceList.add("10.00");
        limitContinueDistanceList.add("50.00");
        for (int i = 0; i < limitContinueDistanceList.size(); i++) {
            if (Device.getInstance().rtkContinueDistanceInterval == Double.parseDouble(limitContinueDistanceList.get(i))){
                limitContinueDistanceIntervalIndex = i;
            }
        }

        binding.tvContinueSurveyDistanceInterval.setText(Device.getInstance().rtkContinueDistanceInterval + getString(R.string.meter));
        binding.rlContinueSurveyDistanceInterval.setOnClickListener(view -> CustomRecycleDialog.showDialog(PointSurveySettingActivity.this, R.string.continuous_survey_distance_interval, limitContinueDistanceList, limitContinueDistanceIntervalIndex, CustomRecycleDialog.Builder.INPUT_TYPE_DECIMAL,true,Util.formatDouble2StringDotAuto(Device.getInstance().rtkContinueDistanceInterval), new CustomRecycleDialog.IClickCallback() {
            @SuppressLint("SetTextI18n")
            @Override
            public void onItemSelected(int index, String itemString) {
                Device.getInstance().rtkContinueDistanceInterval = Float.parseFloat(itemString);
                limitContinueDistanceIntervalIndex = index;
                if (index != -1){
                    binding.tvContinueSurveyDistanceInterval.setText(limitContinueDistanceList.get(index) + getString(R.string.meter));
                }else {
                    binding.tvContinueSurveyDistanceInterval.setText(itemString + getString(R.string.meter));
                }
                Util.putPreference(ConstUtils.preferConst.rtkContinueDistanceInterval, Device.getInstance().rtkContinueDistanceInterval);
            }
        }));
    }

    private static final String POINT_TYPE = "pointType";
    private static final String IS_ONLY_COMM_LIMIT = "isOnlyCommLimit";

    public static void start(Context context) {
        start(context, POINT_TYPE_KS, false);
    }
    public static void start(Context context, int pointType) {
        start(context, pointType, false);
    }

    /**
     * @param isOnlyCommLimit 是否只显示常规限制（状态限制、hrms限制、vrms限制、pdop限制、延迟限制） 默认为false
     */
    public static void start(Context context, boolean isOnlyCommLimit){
        start(context, POINT_TYPE_KS, isOnlyCommLimit);
    }
    public static void start(Context context, int pointType, boolean isOnlyCommLimit){
        Intent intent = new Intent(context,PointSurveySettingActivity.class);
        intent.putExtra(POINT_TYPE,pointType);
        intent.putExtra(IS_ONLY_COMM_LIMIT,isOnlyCommLimit);
        context.startActivity(intent);
    }
}
