package com.project.survey.ui.pointmeasure.measure.adapter;

import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_LINE;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_POINT;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_SURFACE;

import android.content.Context;
import android.graphics.drawable.GradientDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;

import com.bingce.data.surveyor.surveydata.code.CodeRecord;
import com.project.survey.R;
import com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils;
import com.project.survey.util.CommonUtils;
import com.project.survey.util.DrawableUtils;

import java.util.List;


public class CodeLibraryAdapter extends RecyclerView.Adapter<CodeLibraryAdapter.ViewHolder> {

    private Context context;
    private List<CodeRecord> codeRecordList;
    private int selectPosition;
    private OnItemClickListener mOnItemClickListener;

    public CodeLibraryAdapter(Context context, List<CodeRecord> codeRecordList, int defaultPosition) {
        this.context = context;
        this.codeRecordList = codeRecordList;
        this.selectPosition = defaultPosition;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        ViewHolder holder = new ViewHolder(LayoutInflater.from(
                context).inflate(R.layout.layout_code_library_item, parent,
                false));
        return holder;
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        holder.tv_code_name.setText(codeRecordList.get(position).codeName);
        holder.tv_code.setText("[" + codeRecordList.get(position).code + "]");
        switch (codeRecordList.get(position).codeGroupType) {
            case SHAPE_TYPE_POINT:
                holder.poinView.setVisibility(View.VISIBLE);
                holder.lineView.setVisibility(View.GONE);
                holder.faceView.setVisibility(View.GONE);
                holder.poinView.setImageDrawable(DrawableUtils.tintModifyColorDrawable(context.getDrawable(CodeConsUtils.showPointIcon(codeRecordList.get(position).codeFeatureShape)), codeRecordList.get(position).colorShape));
                break;
            case SHAPE_TYPE_LINE:
                holder.poinView.setVisibility(View.GONE);
                holder.lineView.setVisibility(View.VISIBLE);
                holder.faceView.setVisibility(View.GONE);
                GradientDrawable gdLine = (GradientDrawable) holder.lineView.getBackground();
                gdLine.setStroke(CommonUtils.dip2px(1f), codeRecordList.get(position).colorShape, CommonUtils.dip2px(codeRecordList.get(position).codeFeatureShape + codeRecordList.get(position).codeFeatureShape * 2), CommonUtils.dip2px(codeRecordList.get(position).codeFeatureShape + codeRecordList.get(position).codeFeatureShape * 2));
                break;
            case SHAPE_TYPE_SURFACE:
                holder.poinView.setVisibility(View.GONE);
                holder.lineView.setVisibility(View.GONE);
                holder.faceView.setVisibility(View.VISIBLE);
                GradientDrawable gdFace = (GradientDrawable) holder.faceView.getBackground();
                gdFace.setStroke(CommonUtils.dip2px(1f), codeRecordList.get(position).codeColorContour);
                gdFace.setColor(codeRecordList.get(position).colorShape);
                break;
        }

        if (selectPosition == position) {
            holder.iv_checkbox.setImageResource(R.mipmap.icon_checkbox_select_ovel);
            holder.tv_code_name.setTextColor(context.getColor(R.color.theme_green));
            holder.tv_code.setTextColor(context.getColor(R.color.theme_green));
        } else {
            holder.tv_code_name.setTextColor(context.getColor(R.color.color_575757));
            holder.tv_code.setTextColor(context.getColor(R.color.color_999999));
            holder.iv_checkbox.setImageResource(R.mipmap.icon_checkbox_unselect_ovel);
        }

        holder.itemView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mOnItemClickListener.onItemClickListener(holder.getAdapterPosition());
            }
        });
    }

    public void setData(List<CodeRecord> data) {
        this.codeRecordList = data;
        notifyDataSetChanged();
    }

    public void setSelectedPosition(int defaultPosition) {
        this.selectPosition = defaultPosition;
        notifyDataSetChanged();
    }

    public void setOnItemClickListener(OnItemClickListener onItemClickListener) {
        this.mOnItemClickListener = onItemClickListener;
    }

    public interface OnItemClickListener {
        void onItemClickListener(int position);
    }

    @Override
    public int getItemCount() {
        return codeRecordList == null ? 0 : codeRecordList.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        ImageView poinView, iv_checkbox;
        View lineView;
        TextView faceView, tv_code_name, tv_code;
        RelativeLayout rl_item_view;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            rl_item_view = itemView.findViewById(R.id.rl_item_view);
            poinView = itemView.findViewById(R.id.poinView);
            lineView = itemView.findViewById(R.id.lineView);
            faceView = itemView.findViewById(R.id.faceView);
            tv_code_name = itemView.findViewById(R.id.tv_code_name);
            tv_code = itemView.findViewById(R.id.tv_code);
            iv_checkbox = itemView.findViewById(R.id.iv_checkbox);
        }
    }
}
