package com.project.survey.ui.pointmeasure.measure.adapter;


import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_LINE;
import static com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils.SHAPE_TYPE_POINT;

import android.content.Context;
import android.graphics.drawable.GradientDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;


import com.project.survey.R;
import com.project.survey.ui.pointmeasure.measure.util.CodeConsUtils;
import com.project.survey.util.CommonUtils;
import com.project.survey.util.DrawableUtils;

import java.util.List;


public class CodeStyleAdapter extends RecyclerView.Adapter<CodeStyleAdapter.SimpleHolder> {

    private Context context;
    private List<Integer> list;
    private int codeShapeType;
    private int codeShapeColor;
    private int codeShapeSize;
    private int selectPosition;
    private OnItemClickListener mOnItemClickListener;

    public CodeStyleAdapter(Context context, int selectPosition, List<Integer> list , int codeShapeType, int codeShapeColor, int codeShapeSize) {
        this.context = context;
        this.selectPosition = selectPosition;
        this.list = list;
        this.codeShapeType = codeShapeType;
        this.codeShapeColor = codeShapeColor;
        this.codeShapeSize = codeShapeSize;
    }

    @NonNull
    @Override
    public SimpleHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        View view = LayoutInflater.from(context).inflate(R.layout.layout_code_style_item, parent, false);
        return new SimpleHolder(view);
    }

    @Override
    public void onBindViewHolder(@NonNull SimpleHolder holder, int position) {

        if (codeShapeType == SHAPE_TYPE_POINT){

            holder.pointView.setVisibility(View.VISIBLE);
            holder.lineView.setVisibility(View.GONE);

            //点列表
            ViewGroup.LayoutParams params = holder.pointView.getLayoutParams();
            switch (codeShapeSize){
                case 0:
                    params.width = CommonUtils.dip2px(5);
                    params.height = CommonUtils.dip2px(5);
                    break;
                case 1:
                    params.width = CommonUtils.dip2px(10);
                    params.height = CommonUtils.dip2px(10);
                    break;
                case 2:
                    params.width = CommonUtils.dip2px(15);
                    params.height = CommonUtils.dip2px(15);
                    break;
            }
            holder.pointView.setLayoutParams(params);

            if (selectPosition == position){
                holder.pointView.setImageDrawable(DrawableUtils.tintModifyColorDrawable(context.getDrawable(CodeConsUtils.showPointIcon(position)),context.getColor(R.color.white)));
                holder.llBg.setBackgroundColor(codeShapeColor);
            }else {
                holder.pointView.setImageDrawable(DrawableUtils.tintModifyColorDrawable(context.getDrawable(CodeConsUtils.showPointIcon(position)), codeShapeColor));
                holder.llBg.setBackgroundColor(context.getColor(R.color.white));
            }

        }else if (codeShapeType == SHAPE_TYPE_LINE){
            //线列表
            holder.pointView.setVisibility(View.GONE);
            holder.lineView.setVisibility(View.VISIBLE);

            GradientDrawable gradientDrawable = (GradientDrawable)holder.lineView.getBackground();

            int lineWidth = 0;
            switch (codeShapeSize){
                case 0:
                    lineWidth = CommonUtils.dip2px(0.8f);
                    break;
                case 1:
                    lineWidth = CommonUtils.dip2px(1.3f);
                    break;
                case 2:
                    lineWidth = CommonUtils.dip2px(1.8f);
                    break;
            }

            if (selectPosition == position){
                gradientDrawable.setStroke(lineWidth,context.getColor(R.color.white),CommonUtils.dip2px(position + position * 2),CommonUtils.dip2px(position + position * 2));
                holder.llBg.setBackgroundColor(codeShapeColor);
            }else {
                gradientDrawable.setStroke(lineWidth,codeShapeColor,CommonUtils.dip2px(position + position * 2),CommonUtils.dip2px(position + position * 2));
                holder.llBg.setBackgroundColor(context.getColor(R.color.white));
            }
        }

        holder.itemView.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mOnItemClickListener.onItemClickListener(holder.getAdapterPosition());
            }
        });
    }

    @Override
    public int getItemCount() {
        return list == null ? 0:list.size();
    }

    public void setSelectedPosition(int selectPosition) {
        this.selectPosition = selectPosition;
        notifyDataSetChanged();
    }

    public void setShapeSize(List<Integer> list,int shapeSize) {
        this.list = list;
        this.codeShapeSize = shapeSize;
        notifyDataSetChanged();
    }

    public void setOnItemClickListener(OnItemClickListener onItemClickListener) {
        this.mOnItemClickListener = onItemClickListener;
    }
    public interface OnItemClickListener {
        void onItemClickListener(int position);
    }

    public class SimpleHolder extends RecyclerView.ViewHolder{
        private ImageView pointView;
        private View lineView;
        private LinearLayout llBg;
        public SimpleHolder(@NonNull View itemView) {
            super(itemView);
            pointView = itemView.findViewById(R.id.pointView);
            lineView = itemView.findViewById(R.id.lineView);
            llBg = itemView.findViewById(R.id.ll_bg);
        }
    }
}
