package com.project.survey.util;

import android.content.Context;

import com.bingce.coordlib.model.CoordinateSystem;
import com.bingce.data.cache.CachedProject;
import com.bingce.data.cache.CachedProjectKt;
import com.bingce.surveyor.util.dialog.CustomInputDialog;
import com.bingce.utils.FileUtil;
import com.bingce.utils.StringUtil;
import com.bingce.utils.Util;
import com.hjq.permissions.Permission;
import com.hjq.permissions.XXPermissions;
import com.project.survey.R;
import com.project.survey.ui.instrument.setupstation.AddResultFormatActivity;
import com.project.survey.ui.instrument.setupstation.db.PointSurveyRecord;
import com.project.survey.ui.instrument.setupstation.db.resultformat.FormatRecord;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;

public class CassCsvTxtExportUtil {

    public static void exportFormat(Context activity, String suffixesUtils, CachedProjectKt currentProject, FormatRecord formatRecord, List<PointSurveyRecord> pointSurveyRecords, CoordinateSystem coordinateSystem) {

        List<String> regexFormatRecordList = new ArrayList<>();
        String[] split = formatRecord.format_content.split(formatRecord.divided_symbols);
        for (int i = 0; i < split.length; i++) {
            regexFormatRecordList.add(split[i]);
        }

        if (XXPermissions.isGranted(activity, Permission.MANAGE_EXTERNAL_STORAGE)) {
            exportCassFile(activity, currentProject, suffixesUtils, formatRecord, regexFormatRecordList, pointSurveyRecords, coordinateSystem);
        } else {
            ToastUtils.showShort(activity.getString(R.string.open_read_write_permissions));
        }
    }

    /**
     * 导出Cass文件
     */
    private static String title;

    private static void exportCassFile(Context context, CachedProjectKt currentProject, String suffixesUtils, FormatRecord formatRecord, List<String> regexList, List<PointSurveyRecord> pointSurveyRecordList, CoordinateSystem coordinateSystem) {
        if (Objects.equals(suffixesUtils, FormatSuffixesUtils.formatDAT)) {
            title = context.getString(R.string.export_cass_file_name);
        } else if (Objects.equals(suffixesUtils, FormatSuffixesUtils.formatTXT)) {
            title = context.getString(R.string.export_txt_file_name);
        } else if (Objects.equals(suffixesUtils, FormatSuffixesUtils.formatCSV)) {
            title = context.getString(R.string.export_csv_file_name);
        }
        new CustomInputDialog.Builder(context).setTitle(title).setButtonConfirm(new CustomInputDialog.setOnConfirmClickListener() {
            @Override
            public void OnConfirmClick(String editText) {
                if (android.text.TextUtils.isEmpty(editText.trim())) {
                    ToastUtils.showShort(context.getString(R.string.enter_file_name));
                    return;
                }
                StringBuilder result = new StringBuilder();
                String divided_symbols = formatRecord.divided_symbols;
                for (int i = 0; i < pointSurveyRecordList.size(); i++) {
                    PointSurveyRecord pointBaseRecord = pointSurveyRecordList.get(i);

                    for (int j = 0; j < regexList.size(); j++) {
                        String s = regexList.get(j);
                        if (s.equals("点名") || s.equals("Point name") || s.equals("№точ.")) {
                            if (j == regexList.size() - 1) {
                                result.append(pointBaseRecord.pointName);
                            } else {
                                result.append(pointBaseRecord.pointName).append(divided_symbols);
                            }
                        } else if (s.equals("编码") || s.equals("Code") || s.equals("Код")) {
                            if (StringUtil.isEmpty(pointBaseRecord.code) || pointBaseRecord.code.equals("空") || pointBaseRecord.code.equals("null") || pointBaseRecord.code.equals("Null") || pointBaseRecord.code.equals("NULL")) {
                                if (j == regexList.size() - 1) {
                                    result.append("");
                                } else {
                                    result.append("").append(divided_symbols);
                                }
                            } else {
                                if (j == regexList.size() - 1) {
                                    result.append(pointBaseRecord.code);
                                } else {
                                    result.append(pointBaseRecord.code).append(divided_symbols);
                                }
                            }
                        } else if (s.equals("纬度") || s.equals("Latitude") || s.equals("Широта")) {
                            if (j == regexList.size() - 1) {
                                result.append(converAngleFormat(formatRecord.angle_format, pointBaseRecord.latitude));
                            } else {
                                result.append(converAngleFormat(formatRecord.angle_format, pointBaseRecord.latitude)).append(divided_symbols);
                            }
                        } else if (s.equals("经度") || s.equals("Longitude") || s.equals("Долгота")) {
                            if (j == regexList.size() - 1) {
                                result.append(converAngleFormat(formatRecord.angle_format, pointBaseRecord.longitude));
                            } else {
                                result.append(converAngleFormat(formatRecord.angle_format, pointBaseRecord.longitude)).append(divided_symbols);
                            }
                        } else if (s.equals("大地高") || s.equals("Geodetic height") || s.equals("Высота")) {
                            if (j == regexList.size() - 1) {
                                result.append(Util.formatDouble2String(pointBaseRecord.altitude, 6));
                            } else {
                                result.append(Util.formatDouble2String(pointBaseRecord.altitude, 6)).append(divided_symbols);
                            }
                        } else if (s.equals("天线类型") || s.equals("Antenna type") || s.equals("Тип антенны")) {
                            if (j == regexList.size() - 1) {
                                result.append(pointBaseRecord.antennaTypeName == null ? "" : pointBaseRecord.antennaTypeName);
                            } else {
                                result.append(pointBaseRecord.antennaTypeName == null ? "" : pointBaseRecord.antennaTypeName).append(divided_symbols);
                            }
                        } else if (s.equals("天线高") || s.equals("Antenna height") || s.equals("Высота антенны")) {
                            if (j == regexList.size() - 1) {
                                if (pointBaseRecord.antennaHeight == -1) {
                                    result.append("null");
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.antennaHeight));
                                }
                            } else {
                                if (pointBaseRecord.antennaHeight == -1) {
                                    result.append("null").append(divided_symbols);
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.antennaHeight)).append(divided_symbols);
                                }
                            }
                        } else if (s.equals("北坐标") || s.equals("Northing") || s.equals("Северные координаты.")) {
                            if (j == regexList.size() - 1) {
                                result.append(Util.formatDouble2String(pointBaseRecord.planeN, 6));
                            } else {
                                result.append(Util.formatDouble2String(pointBaseRecord.planeN, 6)).append(divided_symbols);
                            }
                        } else if (s.equals("东坐标") || s.equals("Easting") || s.equals("Восточные координаты.")) {
                            if (j == regexList.size() - 1) {
                                result.append(Util.formatDouble2String(pointBaseRecord.planeE, 6));
                            } else {
                                result.append(Util.formatDouble2String(pointBaseRecord.planeE, 6)).append(divided_symbols);
                            }
                        } else if (s.equals("高程") || s.equals("Elevation") || s.equals("Восточные координаты.")) {
                            if (j == regexList.size() - 1) {
                                result.append(Util.formatDouble2String(pointBaseRecord.planeH, 6));
                            } else {
                                result.append(Util.formatDouble2String(pointBaseRecord.planeH, 6)).append(divided_symbols);
                            }
                        } else if (s.equals("解状态") || s.equals("Solution state") || s.equals("Режим решения.")) {
                            if (j == regexList.size() - 1) {
                                result.append(pointBaseRecord.solutionState);
                            } else {
                                result.append(pointBaseRecord.solutionState).append(divided_symbols);
                            }
                        } else if (s.equals("解算卫星") || s.equals("Satellite number") || s.equals("Спутник для вычисления")) {
                            if (j == regexList.size() - 1) {
                                if (pointBaseRecord.satelliteNum == -1) {
                                    result.append("null");
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.satelliteNum));
                                }
                            } else {
                                if (pointBaseRecord.satelliteNum == -1) {
                                    result.append("null").append(divided_symbols);
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.satelliteNum)).append(divided_symbols);
                                }
                            }
                        } else if (s.equals("可见卫星") || s.equals("Satellite track num") || s.equals("Видимый спутник")) {
                            if (j == regexList.size() - 1) {
                                if (pointBaseRecord.satelliteTrackNum == -1) {
                                    result.append("null");
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.satelliteTrackNum));
                                }
                            } else {
                                if (pointBaseRecord.satelliteTrackNum == -1) {
                                    result.append("null").append(divided_symbols);
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.satelliteTrackNum)).append(divided_symbols);
                                }
                            }
                        } else if (s.equals(context.getString(R.string.pdop))) {
                            if (j == regexList.size() - 1) {
                                if (pointBaseRecord.pdop == -1) {
                                    result.append("null");
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.pdop));
                                }
                            } else {
                                if (pointBaseRecord.pdop == -1) {
                                    result.append("null").append(divided_symbols);
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.pdop)).append(divided_symbols);
                                }
                            }
                        } else if (s.equals(context.getString(R.string.hrms))) {
                            if (j == regexList.size() - 1) {
                                if (pointBaseRecord.hrms == -1) {
                                    result.append("null");
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.hrms));
                                }
                            } else {
                                if (pointBaseRecord.hrms == -1) {
                                    result.append("null").append(divided_symbols);
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.hrms)).append(divided_symbols);
                                }
                            }
                        } else if (s.equals(context.getString(R.string.vrms))) {
                            if (j == regexList.size() - 1) {
                                if (pointBaseRecord.vrms == -1) {
                                    result.append("null");
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.vrms));
                                }
                            } else {
                                if (pointBaseRecord.vrms == -1) {
                                    result.append("null").append(divided_symbols);
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.vrms)).append(divided_symbols);
                                }
                            }
                        } else if (s.equals("差分延迟") || s.equals("Diff age") || s.equals("Дифференциальная задержка")) {
                            if (j == regexList.size() - 1) {
                                if (pointBaseRecord.diffAge == -1) {
                                    result.append("null");
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.diffAge));
                                }
                            } else {
                                if (pointBaseRecord.diffAge == -1) {
                                    result.append("null").append(divided_symbols);
                                } else {
                                    result.append(Util.formatDouble2StringDotAuto(pointBaseRecord.diffAge)).append(divided_symbols);
                                }
                            }
                        } else if (s.equals("备注") || s.equals("Remark") || s.equals("примечан")) {
                            if (StringUtil.isEmpty(pointBaseRecord.remarks) || pointBaseRecord.remarks.equals("") || pointBaseRecord.remarks.equals("null") || pointBaseRecord.remarks.equals("Null") || pointBaseRecord.remarks.equals("NULL")) {
                                if (j == regexList.size() - 1) {
                                    result.append("");
                                } else {
                                    result.append("").append(divided_symbols);
                                }
                            } else {
                                if (j == regexList.size() - 1) {
                                    result.append(pointBaseRecord.remarks);
                                } else {
                                    result.append(pointBaseRecord.remarks).append(divided_symbols);
                                }
                            }
                        } else if (s.equals("采集时间") || s.equals("Measure time") || s.equals("Время сбора")) {
                            if (j == regexList.size() - 1) {
                                result.append(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(pointBaseRecord.createdAt));
                            } else {
                                result.append(new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(pointBaseRecord.createdAt)).append(divided_symbols);
                            }
                        } else if (s.equals("惯导测量") || s.equals("Tilt survey") || s.equals("Нормативное измерение")) {
                            if (pointBaseRecord.isTiltEnable) {
                                if (j == regexList.size() - 1) {
                                    result.append(context.getString(R.string.yes));
                                } else {
                                    result.append(context.getString(R.string.yes)).append(divided_symbols);
                                }
                            } else {
                                if (j == regexList.size() - 1) {
                                    result.append(context.getString(R.string.no));
                                } else {
                                    result.append(context.getString(R.string.no)).append(divided_symbols);
                                }
                            }
                        }
                    }
                    result.append("\r\n");
                }
                File target = new File(FileUtil.getSDPath() + "/" + context.getString(R.string.surveyor_exported_file) + "/" + context.getString(R.string.point_survey) + "/" + editText + suffixesUtils);
                boolean success = FileUtil.writeString(result.toString(), target, "GBK");
                if (success) {
                    ExcelUtil.showCompleteDialog(context, target);
                } else {
                    ToastUtils.showShort(context.getString(R.string.export_failure));
                }
            }
        }).create().show();
    }

    /**
     * 根据成果导出的格式，经纬度进行转换导出
     *
     * @param angle_format_type
     * @param l
     * @return
     */
    private static String converAngleFormat(int angle_format_type, double l) {
        if (angle_format_type == AddResultFormatActivity.ANGLE_FORMAT_D_MS) {
            //度.分秒
            return Util.radianToDmsDoubleString(Math.toRadians(l), 6, false);
        } else if (angle_format_type == AddResultFormatActivity.ANGLE_FORMAT_D_M_S) {
            //度°分′秒″
            return Util.radianToDmsString(Math.toRadians(l), 6, true);
        } else if (angle_format_type == AddResultFormatActivity.ANGLE_FORMAT_D) {
            //度
            return Util.formatDouble2String(l, 6);
        } else {
            //弧度
            return Util.formatDouble2String(Math.toRadians(l), 6);
        }
    }
}
