package com.project.survey.util;

import static blankj.utilcode.util.ResourceUtils.getDrawable;

import android.annotation.SuppressLint;
import android.content.res.ColorStateList;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.graphics.drawable.StateListDrawable;
import android.view.View;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.graphics.drawable.DrawableCompat;

import com.project.survey.R;

import blankj.utilcode.util.Utils;

public class DrawableUtils {
    /**
     * 对目标Drawable 进行着色
     *
     * @param drawable 目标Drawable
     * @param color    着色的颜色值
     * @return 着色处理后的Drawable
     */
    public static Drawable tintModifyColorDrawable(@NonNull Drawable drawable, int color) {
        Drawable wrappedDrawable = DrawableCompat.wrap(drawable);
        DrawableCompat.setTint(wrappedDrawable, color);
        return wrappedDrawable;
    }

    /**
     * 对目标Drawable 进行着色
     *
     * @param drawable 目标(int) Drawable
     * @param color    着色的颜色值
     * @return 着色处理后的Drawable
     */
    public static Drawable tintModifyColorDrawable(@NonNull int drawable, int color) {
        Drawable wrappedDrawable = DrawableCompat.wrap(getDrawable(drawable));
        DrawableCompat.setTint(wrappedDrawable, (Utils.getApp()).getResources().getColor(color));
        return wrappedDrawable;
    }
    /**
     * 对目标Drawable 进行着色
     *
     * @param drawable 目标Drawable
     * @param colors   着色值
     * @return 着色处理后的Drawable
     */
    public static Drawable tintListDrawable(@NonNull Drawable drawable, ColorStateList colors) {
        Drawable wrappedDrawable = DrawableCompat.wrap(drawable);
        // 进行着色
        DrawableCompat.setTintList(wrappedDrawable, colors);
        return wrappedDrawable;
    }

    /**
     * 修改矢量图标的颜色
     * @param imageView
     * @param icVector
     */
    public static void tintModifyColorVector(ImageView imageView, int icVector){
        tintModifyColorVector(imageView, icVector, R.color.theme_green);
    }

    public static void tintModifyColorVector(ImageView imageView, int icVector, int color){
        imageView.setImageDrawable(DrawableUtils.tintModifyColorDrawable(icVector, color));
    }

    /**
     * 设置按压前后矢量图标的状态
     * @param imageView
     * @param icVector
     */
    public static void setClickVectorEffect(ImageView imageView, int icVector){
        setClickVectorEffect(imageView, icVector, (Utils.getApp()).getResources().getColor(R.color.theme_green_press), (Utils.getApp()).getResources().getColor(R.color.theme_green));
    }

    public static void setClickVectorEffect(ImageView imageView, int icVector, int pressColor, int normalColor){
        //状态
        int[][] states = new int[2][];
        //按下
        states[0] = new int[] {android.R.attr.state_pressed};
        //默认
        states[1] = new int[] {};
        //状态对应颜色值（按下，默认）
        int[] colors = new int[] {pressColor, normalColor};
        ColorStateList colorList = new ColorStateList(states, colors);
        imageView.setImageDrawable(tintListDrawable(getDrawable(icVector),colorList));
    }

    /**
     * 设置按压前后文字的状态
     * @param textView
     * 默认按压色白色，默认色为主题色
     */
    public static void setClickTextEffect(TextView textView){
        setClickTextEffect(textView, R.color.theme_green_press, R.color.theme_green);
    }

    public static void setClickTextEffect(TextView textView, int pressColor, int normalColor){
        //状态
        int[][] states = new int[2][];
        //按下
        states[0] = new int[] {android.R.attr.state_pressed};
        //默认
        states[1] = new int[] {};
        //状态对应颜色值（按下，默认）
        int[] colors = new int[] {(Utils.getApp()).getResources().getColor(pressColor), (Utils.getApp()).getResources().getColor(normalColor)};
        ColorStateList colorList = new ColorStateList(states, colors);
        textView.setTextColor(colorList);
    }

    /**
     * 设置按压前后View的状态
     * @param view
     */

    public static final int RADIUS_ALL = 0;
    public static final int RADIUS_BOTTOM = RADIUS_ALL + 1;
    public static final int RADIUS_LEFT = RADIUS_BOTTOM + 1;
    public static final int RADIUS_RIGHT = RADIUS_LEFT + 1;
    public static final int RADIUS_TOP = RADIUS_RIGHT + 1;

    public static void setClickBackgroundEffect(View view){
        setClickBackgroundEffect(view, R.color.theme_green_press, R.color.theme_green,0, RADIUS_ALL);
    }

    public static void setClickBackgroundEffect(View view, int dpStroke){
        setClickBackgroundEffect(view, R.color.theme_green_press, R.color.theme_green, dpStroke, RADIUS_ALL);
    }

    public static void setClickBackgroundEffect(View view, int dpStroke, int radiusPosition){
        setClickBackgroundEffect(view, R.color.theme_green_press, R.color.theme_green, dpStroke, radiusPosition);
    }

    public static void setClickBackgroundEffect(View view, int pressColor, int normalColor, int dpStroke){
        setClickBackgroundEffect(view, pressColor, normalColor, dpStroke, RADIUS_ALL);
    }

    public static void setClickBackgroundEffect(View view, int pressColor, int normalColor, int dpStroke, int radiusPosition){

        int radiusLeftUpValue = 0;
        int radiusLeftDownValue = 0;
        int radiusRightUpValue = 0;
        int radiusRightDownValue = 0;
        switch (radiusPosition){
            case RADIUS_ALL:
                radiusLeftUpValue = DisplayUtils.dip2px(dpStroke);
                radiusLeftDownValue = DisplayUtils.dip2px(dpStroke);
                radiusRightUpValue = DisplayUtils.dip2px(dpStroke);
                radiusRightDownValue = DisplayUtils.dip2px(dpStroke);
                break;
            case RADIUS_LEFT:
                radiusLeftUpValue = DisplayUtils.dip2px(dpStroke);
                radiusLeftDownValue = DisplayUtils.dip2px(dpStroke);
                break;
            case RADIUS_RIGHT:
                radiusRightUpValue = DisplayUtils.dip2px(dpStroke);
                radiusRightDownValue = DisplayUtils.dip2px(dpStroke);
                break;
            case RADIUS_TOP:
                radiusLeftUpValue = DisplayUtils.dip2px(dpStroke);
                radiusRightUpValue = DisplayUtils.dip2px(dpStroke);
                break;
            case RADIUS_BOTTOM:
                radiusLeftDownValue = DisplayUtils.dip2px(dpStroke);
                radiusRightDownValue = DisplayUtils.dip2px(dpStroke);
                break;
        }
        Drawable normal,press;
        StateListDrawable drawable = new StateListDrawable();
        press = DrawableUtils.setShapeColor((Utils.getApp()).getResources().getColor(pressColor),new float[]{radiusLeftUpValue,radiusLeftUpValue,radiusRightUpValue,radiusRightUpValue,radiusRightDownValue,radiusRightDownValue,radiusLeftDownValue,radiusLeftDownValue});
        normal = DrawableUtils.setShapeColor((Utils.getApp()).getResources().getColor(normalColor),new float[]{radiusLeftUpValue,radiusLeftUpValue,radiusRightUpValue,radiusRightUpValue,radiusRightDownValue,radiusRightDownValue,radiusLeftDownValue,radiusLeftDownValue});
        drawable.addState(new int[]{android.R.attr.state_pressed},press);
        drawable.addState(new int[]{-android.R.attr.state_pressed},normal);
        view.setBackgroundDrawable(drawable);
    }

    /**
     * 设置圆角背景
     *
     * @param
     * @param radii       new float[] { r0, r0, r1, r1, r2, r2, r3,r3 }
     *                    设置图片四个角圆形半径：1、2两个参数表示左上角，3、4表示右上角，5、6表示右下角，7、8表示左下角
     * @return
     */
    @SuppressLint("WrongConstant")
    public static GradientDrawable setShapeColor(int color, float[] radii) {
        GradientDrawable drawable = new GradientDrawable();
        drawable.setShape(GradientDrawable.RECTANGLE);
        drawable.setGradientType(GradientDrawable.RECTANGLE);
        drawable.setCornerRadii(radii);
        drawable.setColor(color);
        return drawable;
    }
}
