package com.project.survey.util;

import android.content.Context;
import android.text.TextUtils;

import androidx.annotation.MainThread;

import com.bingce.data.cache.CachedProject;
import com.bingce.data.cache.CachedProjectKt;
import com.bingce.file.FileOperator;
import com.bingce.surveyor.util.dialog.CustomDialog;
import com.bingce.surveyor.util.dialog.CustomInputDialog;
import com.bingce.utils.FileUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.hjq.permissions.Permission;
import com.hjq.permissions.XXPermissions;
import com.project.survey.BuildConfig;
import com.project.survey.R;
import com.project.survey.dialog.CustomStyleDialog;
import com.project.survey.ui.instrument.setupstation.db.PointSurveyRecord;

import java.io.File;
import java.io.IOException;
import java.util.List;

import blankj.utilcode.util.ToastUtils;
import cn.liuyanbing.javadxf.AciColor;
import cn.liuyanbing.javadxf.DxfDocument;
import cn.liuyanbing.javadxf.Entities.Point;
import cn.liuyanbing.javadxf.Entities.Text;
import cn.liuyanbing.javadxf.Exception.DxfException;
import cn.liuyanbing.javadxf.Header.DxfVersion;
import cn.liuyanbing.javadxf.Tables.Layer;
import cn.liuyanbing.javadxf.Vector3;

public class DxfUtil {
    /**
     * 导出dxf文件调用此方法
     *
     * @param context
     */
    private static List<PointSurveyRecord> pointSurveyRecordLists;

    public static void exportFormat(Context context, CachedProjectKt currentProject) {
        exportFormat(context, currentProject, null);
    }

    public static void exportFormat(Context context, CachedProjectKt currentProject, List<PointSurveyRecord> pointSurveyRecordList) {
        if (pointSurveyRecordList != null) {
            pointSurveyRecordLists = pointSurveyRecordList;
        }
        if (XXPermissions.isGranted(context, Permission.MANAGE_EXTERNAL_STORAGE)) {
            showEditTextDialog(context, FileUtil.getSDPath() + "/" + context.getString(R.string.surveyor_exported_file) + "/" + context.getString(R.string.point_survey) + "/");
        } else {
            ToastUtils.showShort(context.getString(R.string.open_read_write_permissions));
        }
    }

    private static CustomInputDialog.Builder inputBuilder;

    private static void showEditTextDialog(Context context, String filePath) {
        inputBuilder = new CustomInputDialog.Builder(context);
        inputBuilder.setTitle(context.getString(R.string.export_dxf_file_name)).setInputHint(context.getString(R.string.file_name)).setButtonConfirm(editText -> {
            if (TextUtils.isEmpty(editText.trim())) {
                ToastUtils.showShort(context.getString(R.string.enter_file_name));
            } else {
                if (isCheckExistFile(filePath + editText.trim() + ".dxf")) {
                    new CustomDialog.Builder(context).setContent("\"" + filePath + "\""
                            + context.getString(R.string.folder_name_already_exists) + "[" + editText.trim() + ".dxf]"
                            + context.getString(R.string.replace_file)).setButtonConfirm(context.getString(R.string.replace)
                            , v -> writeTxtToFile(context, filePath, editText.trim())).setButtonCancel(context.getString(R.string.rename)
                            , v -> inputBuilder.create().show()).create().show();
                } else {
                    writeTxtToFile(context, filePath, editText.trim());
                }
            }
        }).create().show();
    }

    /**
     * 导出dxf
     */
    public static void writeTxtToFile(Context context, String filePath, String fileName) {
        //导出数据库的点
        DxfDocument dxf = new DxfDocument();
        ThreadPoolUtil.execute(() -> {
            for (int i = 0; i < pointSurveyRecordLists.size(); i++) {
                PointSurveyRecord pointSurveyRecord = pointSurveyRecordLists.get(i);
                Point point1 = new Point(new Vector3(pointSurveyRecord.planeE, pointSurveyRecord.planeN, pointSurveyRecord.planeH));
                point1.setLayer(new Layer("0"));
                point1.setColor(new AciColor(30));
                dxf.AddEntity(point1);

                Text text = new Text(pointSurveyRecordLists.get(i).pointName, new Vector3(pointSurveyRecord.planeE, pointSurveyRecord.planeN, pointSurveyRecord.planeH), 0.4);
                text.setLayer(new Layer("0"));
                text.setColor(new AciColor(30));
                dxf.AddEntity(text);
            }
            ThreadPoolUtil.executeInMain(() -> {
                try {
                    File file = new File(filePath + fileName + ".dxf");
                    if (file.getParentFile() != null && !file.getParentFile().exists()) {
                        file.getParentFile().mkdirs();
                        file.createNewFile();
                    }
                    dxf.Save(file.getPath(), DxfVersion.AutoCad2004);
                    FileUtil.scanFile(context, file);
                    showCompleteDialog(context, file);
                } catch (DxfException | IOException e) {
                    e.printStackTrace();
                }
            });
        });
    }

    @MainThread
    public static void showCompleteDialog(Context context, File file) {
        new CustomStyleDialog.Builder(context)
                .setTitle(context.getString(R.string.export_successful))
                .setContent(file.getPath())
                .setButtonSample(context.getString(R.string.close))
                .setButtonCancel(context.getString(R.string.open), v -> FileUtil.openFile(context, file))
                .setButtonConfirm(context.getString(R.string.send_to), v -> FileOperator.shareFile(context, file, BuildConfig.APPLICATION_ID))
                .create()
                .show();
    }

    /**
     * 判断导出文件是否重复
     *
     * @param filePaths 文件路径
     */
    private static boolean isCheckExistFile(String filePaths) {
        File file = new File(filePaths);
        return file.exists();
    }
}
