package com.project.survey.util;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.pm.ActivityInfo;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiManager;
import android.os.Environment;
import android.os.ResultReceiver;
import android.text.TextUtils;
import android.util.Log;
import android.view.Display;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.InputMethodManager;
import android.widget.LinearLayout;

import androidx.preference.PreferenceManager;

import com.bingce.device.Device;
import com.bingce.device.enums.DeviceTypeEnum;
import com.bingce.log.BCLog;
import com.bingce.totalstation.event.TsUpdateBatteryEvent;
import com.gyf.immersionbar.ImmersionBar;
import com.project.survey.R;

import org.greenrobot.eventbus.EventBus;

import java.io.File;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import blankj.utilcode.util.ActivityUtils;
import blankj.utilcode.util.KeyboardUtils;
import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;

public class SystemUtils {

    /**
     * 适配手机底部返回键（例如：小米底部横线返回）
     */
    public static void SystemBottomBack(Activity activity) {
        //MIUI系统底部返回键适配
        activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
        activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);       //设置沉浸式状态栏，在MIUI系统中，状态栏背景透明。原生系统中，状态栏背景半透明。
        activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_NAVIGATION);   //设置沉浸式虚拟键，在MIUI系统中，虚拟键背景透明。原生系统中，虚拟键背景半透明。
    }

    /**
     * 适配显示出沉浸式状态栏的高度
     */
    public static void SystemTopStatusBar(Activity activity, View view) {
        int hight = ImmersionBar.getStatusBarHeight(activity);
        LinearLayout.LayoutParams searchParams = (LinearLayout.LayoutParams) view.getLayoutParams();
        searchParams.topMargin = hight;
        view.setLayoutParams(searchParams);
    }

    /**
     * 禁止App旋转，只能竖屏
     */
    @SuppressLint("SourceLockedOrientationActivity")
    public static void SystemVerticalScreen(Activity activity) {
        activity.setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
    }

    /**
     * 隐藏软键盘(只适用于Activity，不适用于Fragment)
     */
    public static void hideSoftKeyboard() {
        Activity act = ActivityUtils.getTopActivity();
        if (act != null) {
            if (act.getCurrentFocus() != null) {
                InputMethodManager inputMethodManager = (InputMethodManager) act.getSystemService(Activity.INPUT_METHOD_SERVICE);
                inputMethodManager.hideSoftInputFromWindow(act.getCurrentFocus().getWindowToken(), 0);
            } else {
                if (KeyboardUtils.isSoftInputVisible(act))
                    KeyboardUtils.toggleSoftInput();
            }
        }
    }

    /**
     * 隐藏软键盘(可用于Activity，Fragment)
     *
     * @param viewList 需要放的是当前界面所有触发软键盘弹出的控件
     */
    public static void hideSoftKeyboard(List<View> viewList) {
        if (viewList == null) return;
        InputMethodManager inputMethodManager = (InputMethodManager) (Utils.getApp()).getSystemService(Activity.INPUT_METHOD_SERVICE);
        for (View v : viewList) {
            inputMethodManager.hideSoftInputFromWindow(v.getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);
        }
    }

    /**
     * 判断sd卡是否可用
     */
    public static boolean isExternalStorageAvailable() {
        return Environment.getExternalStorageState().equals(
                Environment.MEDIA_MOUNTED);
    }

    /**
     * 获取当前时间
     *
     * @return
     */
    public static String getSimpleDateFormat() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日 HH:mm:ss");// HH:mm:ss
        //获取当前时间
        Date date = new Date(System.currentTimeMillis());
        return simpleDateFormat.format(date);
    }

    /**
     * 获取当前时间戳
     *
     * @return
     */
    public static long getSimpleDateTimeStamp() {
        SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy年MM月dd日 HH:mm:ss");// HH:mm:ss
        //获取当前时间
        Date date = new Date(System.currentTimeMillis());
        return date.getTime();
    }

    /**
     * 复制
     *
     * @param text
     */
    public static void copy(String text) {
        // 获取系统剪贴板
        ClipboardManager clipboard = (ClipboardManager) (Utils.getApp()).getSystemService(Context.CLIPBOARD_SERVICE);
        // 创建一个剪贴数据集，包含一个普通文本数据条目（需要复制的数据）,其他的还有
        ClipData clipData = ClipData.newPlainText(null, text);
        // 把数据集设置（复制）到剪贴板
        clipboard.setPrimaryClip(clipData);
        if (!TextUtils.isEmpty(text)) {
            ToastUtils.showShort((Utils.getApp()).getResources().getString(R.string.has_copy_to_clipboard));
        }
    }

    /**
     * 获取文件创建时间
     *
     * @param file
     * @return
     */
    private static final String mformatType = "yyyy/MM/dd";

    public static String getFileCreateTime(File file) {
        Calendar cal = Calendar.getInstance();
        long time = file.lastModified();
        SimpleDateFormat formatter = new SimpleDateFormat(mformatType);
        cal.setTimeInMillis(time);

        // 输出：修改时间[2] 2009-08-17 10:32:38
        return formatter.format(cal.getTime());
    }

    /**
     * 生成的UUID
     *
     * @return
     */
    @SuppressLint("MissingPermission")
    public static String getUUID() {
        UUID uuid = UUID.randomUUID();
        String uniqueId = uuid.toString();
        BCLog.log(log -> log.e("debug", "uuid=" + uniqueId));
        return uniqueId;
    }

    /**
     * 获取wifi状态
     *
     * @return
     */
    public static boolean getWifiEnabled() {
        WifiManager wm = (WifiManager) (Utils.getApp()).getSystemService(Context.WIFI_SERVICE);
        return wm.isWifiEnabled();
    }

    /**
     * 开关wifi
     *
     * @param enabled true：打开  false：关闭
     * @return
     */
    public static void setWifiEnabled(boolean enabled) {
        WifiManager wm = (WifiManager) (Utils.getApp()).getSystemService(Context.WIFI_SERVICE);
        wm.setWifiEnabled(enabled);
    }

    /**
     * 获取手机数据流量是否打开
     *
     * @return
     */
    public static boolean getMobileDataEnabled() {
        boolean mobileDataEnabled;
        ConnectivityManager connectivityManager = (ConnectivityManager) (Utils.getApp()).getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
        if (activeNetworkInfo == null) {
            Log.e("mobileData", "没有联网");
            mobileDataEnabled = false;
        } else {
            if (activeNetworkInfo.getType() == ConnectivityManager.TYPE_MOBILE) {
                Log.e("mobileData", "连接的网络是移动数据流量");
                mobileDataEnabled = true;
            } else {
                Log.e("mobileData", "不是移动数据流量");
                mobileDataEnabled = false;
            }
        }
        return mobileDataEnabled;
    }

    /**
     * 移动网络开关
     *
     * @param enabled true：打开  false：关闭
     */
    public static void setMobileDataEnabled(boolean enabled) {
        ConnectivityManager conMgr = (ConnectivityManager) (Utils.getApp()).getSystemService(Context.CONNECTIVITY_SERVICE);
        Class<?> conMgrClass = null;//ConnectivityManager类
        Field iConMgrField = null;//ConnectivityManager类中的字段
        Object iConMgr = null;//IConnectivityManager类的引用
        Class<?> iConMgrClass = null;//IConnectivityManager类
        Method setMobileDataEnabledMethod = null;//setMobileDataEnabled方法
        try {
            //取得ConnectivityManager类
            conMgrClass = Class.forName(conMgr.getClass().getName());
            //取得ConnectivityManager类中的对象mService
            iConMgrField = conMgrClass.getDeclaredField("mService");
            //设置mService可访问
            iConMgrField.setAccessible(true);
            //取得mService的实例化类IConnectivityManager
            iConMgr = iConMgrField.get(conMgr);
            //取得IConnectivityManager类
            iConMgrClass = Class.forName(iConMgr.getClass().getName());
            //取得IConnectivityManager类中的setMobileDataEnabled(boolean)方法
            setMobileDataEnabledMethod = iConMgrClass.getDeclaredMethod("setMobileDataEnabled", Boolean.TYPE);
            //设置setMobileDataEnabled方法可访问
            setMobileDataEnabledMethod.setAccessible(true);
            //调用setMobileDataEnabled方法
            setMobileDataEnabledMethod.invoke(iConMgr, enabled);
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (SecurityException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (IllegalArgumentException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取wifi热点状态
     *
     * @return
     */
    public static boolean getWifiApEnable() {
        WifiManager wifiManager = (WifiManager) (Utils.getApp()).getApplicationContext().getSystemService(Context.WIFI_SERVICE);
        if (wifiManager == null) {
            return false;
        }
        try {
            Method method = wifiManager.getClass().getDeclaredMethod("isWifiApEnabled");
            method.setAccessible(true);
            return (Boolean) method.invoke(wifiManager);
        } catch (NoSuchMethodException | InvocationTargetException | IllegalAccessException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 网络热点是否打开
     *
     * @param enabled
     */
    public static void setWifiApEnabled(boolean enabled) {
        ConnectivityManager connManager = (ConnectivityManager) (Utils.getApp()).getApplicationContext().getSystemService(Context.CONNECTIVITY_SERVICE);
        Field iConnMgrField = null;
        try {
            iConnMgrField = connManager.getClass().getDeclaredField("mService");
            iConnMgrField.setAccessible(true);
            Object iConnMgr = iConnMgrField.get(connManager);
            Class<?> iConnMgrClass = Class.forName(iConnMgr.getClass().getName());
            if (enabled) {
                Method startTethering = iConnMgrClass.getMethod("startTethering", int.class, ResultReceiver.class, boolean.class);
                startTethering.invoke(iConnMgr, 0, null, true);
            } else {
                Method stopTethering = iConnMgrClass.getMethod("stopTethering", int.class);
                stopTethering.invoke(iConnMgr, 0);
            }
        } catch (NoSuchFieldException e) {
            e.printStackTrace();
        } catch (NoSuchMethodException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取当前连接蓝牙的名称（没有蓝牙连接，返回为空）
     *
     * @return
     */
    public static String getConnectedBtDeviceName() {
        //获取蓝牙适配器
        BluetoothAdapter bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
        //得到已匹配的蓝牙设备列表
        Set<BluetoothDevice> bondedDevices = bluetoothAdapter.getBondedDevices();
        if (bondedDevices != null && bondedDevices.size() > 0) {
            for (BluetoothDevice bondedDevice : bondedDevices) {
                try {
                    //使用反射调用被隐藏的方法
                    Method isConnectedMethod = BluetoothDevice.class.getDeclaredMethod("isConnected", (Class[]) null);
                    isConnectedMethod.setAccessible(true);
                    boolean isConnected = (boolean) isConnectedMethod.invoke(bondedDevice, (Object[]) null);
                    if (isConnected) {
                        return bondedDevice.getName();
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                }
            }
        }
        return null;
    }

    public static int[] getScreenWidthHeight(Context context) {
        Activity activity = (Activity) context;
        Display display = activity.getWindowManager().getDefaultDisplay();
        int width = display.getWidth();
        int height = display.getHeight();
        return new int[]{width, height};
    }

    public static String getFileName(String path) {
        int start = path.lastIndexOf("/");
        int end = path.lastIndexOf(".");
        if (start != -1 && end != -1) {
            return path.substring(start + 1, end);
        } else {
            return null;
        }
    }

    public static void setScreenAlwaysLight(Context context) {
        Activity activity = (Activity) context;
        boolean isWakeLock = PreferenceManager.getDefaultSharedPreferences(context).getBoolean("key_is_wake_lock", false);
        if (isWakeLock) {
            activity.getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        } else {
            activity.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        }
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
            EventBus.getDefault().post(new TsUpdateBatteryEvent());
        }
    }
}
