package com.project.survey.util;

import android.app.Activity;
import android.content.Context;
import android.text.TextUtils;

import androidx.annotation.MainThread;
import androidx.annotation.NonNull;
import androidx.core.content.PermissionChecker;

import com.bingce.data.cache.CachedProject;
import com.bingce.data.cache.CachedProjectKt;
import com.bingce.file.FileOperator;
import com.bingce.surveyor.util.dialog.CustomDialog;
import com.bingce.surveyor.util.dialog.CustomInputDialog;
import com.bingce.utils.FileUtil;
import com.bingce.utils.ThreadPoolUtil;
import com.hjq.permissions.Permission;
import com.hjq.permissions.XXPermissions;
import com.project.survey.BuildConfig;
import com.project.survey.R;
import com.project.survey.dialog.CustomStyleDialog;

import java.io.File;
import java.io.RandomAccessFile;

import blankj.utilcode.util.ToastUtils;

public class TxtUtil {

    public static void exportFormat(Context context, String strContent, String format_suffixes, CachedProjectKt currentProject) {
        exportFormat(context, "", strContent, format_suffixes, currentProject, false);
    }

    /**
     * 导出txt文件调用此方法
     *
     * @param context
     * @param strContent      导出内容
     * @param format_suffixes 格式后缀
     * @param isFinish        是否关闭当前页面
     */
    public static void exportFormat(Context context, String fileName, String strContent, String format_suffixes, CachedProjectKt currentProject, boolean isFinish) {
        if (XXPermissions.isGranted(context, Permission.MANAGE_EXTERNAL_STORAGE)) {
            agreePermission(context, fileName, strContent, format_suffixes, currentProject, isFinish);
        } else {
            ToastUtils.showShort(context.getString(R.string.open_read_write_permissions));
        }
    }

    /**
     * 同意权限
     *
     * @param context
     * @param fileName
     * @param strContent
     * @param format_suffixes
     * @param currentProject
     */
    private static void agreePermission(Context context, String fileName, String strContent, String format_suffixes, CachedProjectKt currentProject, boolean isFinish) {
        //单独列出导出Html(当结尾和文件名都传过来时，无需弹出编辑文件名弹框。当没传过来的文件名，和正常的编辑文件名一样弹出编辑弹框)
        if (format_suffixes.equals(FormatSuffixesUtils.formatHTML) && !TextUtils.isEmpty(fileName)) {
            if (isCheckExistFile(FileUtil.getSDPath() + "/" + context.getString(R.string.surveyor_exported_file) + "/" + context.getString(R.string.point_survey) + "/" + format_suffixes)) {
                new CustomDialog.Builder(context).setContent("\"" + FileUtil.getSDPath() + "/" + context.getString(R.string.surveyor_exported_file)
                                + "/" + context.getString(R.string.point_survey) + "/" + "\"" + context.getString(R.string.folder_name_already_exists) + "[" + fileName + format_suffixes + "]"
                                + context.getString(R.string.replace_file)).setButtonConfirm(context.getString(R.string.replace), v -> writeTxtToFile(context, strContent, FileUtil.getSDPath() + "/"
                                + context.getString(R.string.surveyor_exported_file) + "/" + context.getString(R.string.point_survey) + "/", fileName + format_suffixes, isFinish))
                        .setButtonCancel(context.getString(R.string.rename), v -> inputBuilder.create().show()).create().show();
            } else {
                writeTxtToFile(context, strContent, FileUtil.getSDPath() + "/" + context.getString(R.string.surveyor_exported_file) + "/" + context.getString(R.string.point_survey) + "/", fileName + format_suffixes, isFinish);
            }
        } else {
            //传过来的点名为空时，弹出编写文件名称弹框
            showEditTextDialog(context, strContent, FileUtil.getSDPath() + "/" + context.getString(R.string.surveyor_exported_file) + "/" + context.getString(R.string.point_survey) + "/", format_suffixes, isFinish);
        }
    }

    private static CustomInputDialog.Builder inputBuilder;
    private static String dialogTitle;

    private static void showEditTextDialog(Context context, String strContent, String filePath, String format_suffixes, boolean isFinish) {
        inputBuilder = new CustomInputDialog.Builder(context);
        if (format_suffixes.equals(FormatSuffixesUtils.formatHDM)) {
            dialogTitle = context.getString(R.string.export_hdm_file_name);
        } else if (format_suffixes.equals(FormatSuffixesUtils.formatKML)) {
            dialogTitle = context.getString(R.string.export_kml_file_name);
        } else if (format_suffixes.equals(FormatSuffixesUtils.formatTXT)) {
            dialogTitle = context.getString(R.string.export_txt_file_name);
        } else {
            dialogTitle = context.getString(R.string.export_excel_file_name);
        }
        inputBuilder.setTitle(dialogTitle).setInputHint(context.getString(R.string.file_name)).setButtonConfirm(editText -> {
            if (TextUtils.isEmpty(editText.trim())) {
                ToastUtils.showShort(context.getString(R.string.enter_file_name));
            } else {
                if (isCheckExistFile(filePath + editText.trim() + format_suffixes)) {
                    new CustomDialog.Builder(context).setContent("\"" + filePath + "\""
                                    + context.getString(R.string.folder_name_already_exists) + "[" + editText.trim() + format_suffixes + "]"
                                    + context.getString(R.string.replace_file)).setButtonConfirm(context.getString(R.string.replace)
                                    , v -> writeTxtToFile(context, strContent, filePath, editText.trim() + format_suffixes, isFinish))
                            .setButtonCancel(context.getString(R.string.rename), v -> inputBuilder.create().show()).create().show();
                } else {
                    writeTxtToFile(context, strContent, filePath, editText.trim() + format_suffixes, isFinish);
                }
            }
        }).create().show();
    }

    /**
     * 判断权限是否授权
     *
     * @param context
     * @param permissions
     * @return
     */
    public static boolean hasPermission(@NonNull Context context, @NonNull String... permissions) {
        if (permissions.length == 0) {
            return false;
        }
        for (String per : permissions) {
            int result = PermissionChecker.checkSelfPermission(context, per);
            if (result != PermissionChecker.PERMISSION_GRANTED) {
                return false;
            }
        }
        return true;
    }


    // 将字符串写入到文本文件中
    public static void writeTxtToFile(Context context, String content, String filePath, String fileName, boolean isFinish) {
        String strFilePath = filePath + fileName;
        // 每次写入时，都换行写
        String strContent = content + "\r\n";
        try {
            File file = new File(strFilePath);
            if (file.getParentFile() != null && !file.getParentFile().exists()) {
                file.getParentFile().mkdirs();
                file.createNewFile();
            }
            RandomAccessFile raf = new RandomAccessFile(file, "rwd");
            raf.seek(file.length());
            raf.write(strContent.getBytes());
            raf.close();
            ThreadPoolUtil.executeInMain(() -> {
                FileUtil.scanFile(context, file);
                showCompleteDialog(context, file, isFinish);
            });
        } catch (Exception e) {
            ToastUtils.showShort(e.getMessage());
        }
    }

    @MainThread
    public static void showCompleteDialog(Context context, File file, boolean isFinish) {
        new CustomStyleDialog.Builder(context)
                .setTitle(context.getString(R.string.export_successful))
                .setContent(file.getPath())
                .setButtonSample(context.getString(R.string.close))
                .setButtonCancel(context.getString(R.string.open), v -> FileUtil.openFile(context, file))
                .setButtonConfirm(context.getString(R.string.send_to), v -> FileOperator.shareFile(context, file, BuildConfig.APPLICATION_ID))
                .setButtonCancel(v -> {
                    if (isFinish) {
                        ((Activity) context).finish();
                    }
                })
                .create()
                .show();
    }

    /**
     * 判断导出文件是否重复
     *
     * @param filePaths 文件路径
     */
    private static boolean isCheckExistFile(String filePaths) {
        File file = new File(filePaths);
        return file.exists();
    }
}
