package com.project.survey.util;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.DiffUtil;
import androidx.sqlite.db.SimpleSQLiteQuery;

import com.bingce.data.database.DBQueryConstant;
import com.bingce.data.surveyor.BCSurveyWithDeviceInfoRecord;
import com.bingce.utils.RoomFilterBuilder;
import com.project.survey.App;

import java.util.Date;

public class Utils {

    static abstract class AbstractItemCallback<T> extends DiffUtil.ItemCallback<T> {
        protected abstract String idOf(@NonNull T t);

        @Override
        final public boolean areItemsTheSame(@NonNull T oldItem, @NonNull T newItem) {
//            return oldItem.id.equals(newItem.id);
            return idOf(oldItem).equals(idOf(newItem));
        }

        @Override
        final public boolean areContentsTheSame(@NonNull T oldItem, @NonNull T newItem) {
//            return oldItem.id.equals(newItem.id);
            return idOf(oldItem).equals(idOf(newItem));
        }
    }

    public static class ItemCallback<T extends BCSurveyWithDeviceInfoRecord> extends AbstractItemCallback<T> {
        @Override
        protected String idOf(@NonNull T t) {
            return t.id;
        }
    }

    public static SimpleSQLiteQuery findRecordsListByRoadNameKRemarksDate(
            String DB_NAME, String projectId, String jobId, boolean ascOrder,//asc升序 des，降序
            String roadName, String k, String remarks, Date start, Date end) {
        if (App.instance.isShowAllRecordInCurrentProject) {
            return DBQueryConstant.findRecordsListByProjectRoadNameKRemarksDate(DB_NAME, projectId, ascOrder, roadName, k, remarks, start, end);
        } else {
            return DBQueryConstant.findRecordsListByJobRoadNameKRemarksDate(DB_NAME, jobId, ascOrder, roadName, k, remarks, start, end);
        }
    }

    public static SimpleSQLiteQuery findRecordsListByPointNameRemarksDate(
            String DB_NAME, String projectId, String jobId, boolean ascOrder,//asc升序 des，降序
            String pointName, String remarks, Date start, Date end) {

        StringBuilder stringBuilder = (new StringBuilder("SELECT * FROM ")).append(DB_NAME);
        stringBuilder.append(" WHERE ").append(" (BC_DELETE != 1 ) ");
        boolean filtered = RoomFilterBuilder.appendEqualsFilter(true, stringBuilder, "project_id", projectId);
        filtered = RoomFilterBuilder.appendEqualsFilter(filtered, stringBuilder, "work_id", jobId);
        filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, "pointName", pointName);
        filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, "remarks", remarks);
        RoomFilterBuilder.appendBetweenDate(filtered, stringBuilder, "create_date", start, end);
        RoomFilterBuilder.appendDateOrder(stringBuilder, "create_date", ascOrder);
        return new SimpleSQLiteQuery(stringBuilder.toString());

//        StringBuilder stringBuilder = (new StringBuilder("SELECT * FROM ")).append(DB_NAME);
//        stringBuilder.append(" WHERE ").append(" (BC_DELETE != 1 ) ");
//        boolean filtered = RoomFilterBuilder.appendEqualsFilter(true, stringBuilder, "work_id", jobId);
//        filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, "road_name", roadName);
//        filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, "k", k);
//        filtered = RoomFilterBuilder.appendLikesFilter(filtered, stringBuilder, "remarks", remarks);
//        RoomFilterBuilder.appendBetweenDate(filtered, stringBuilder, "create_date", start, end);
//        RoomFilterBuilder.appendDateOrder(stringBuilder, "create_date", ascOrder);
//        return new SimpleSQLiteQuery(stringBuilder.toString());
    }

    public static SimpleSQLiteQuery findRecordsListByRoadNameKRemarksDate(
            String DB_NAME, String projectId, String jobId, boolean ascOrder,//asc升序 des，降序
            String roadName, String k, String remarks, Date start, Date end, String tcsType) {
        if (App.instance.isShowAllRecordInCurrentProject) {
            return DBQueryConstant.findRecordsListByProjectRoadNameKRemarksDate(DB_NAME, projectId, ascOrder, roadName, k, remarks, start, end, tcsType);
        } else {
            return DBQueryConstant.findRecordsListByJobRoadNameKRemarksDate(DB_NAME, jobId, ascOrder, roadName, k, remarks, start, end, tcsType);
        }
    }

    public static SimpleSQLiteQuery findOtherRecord(String DB_NAME, String projectId, String jobId, Date date, boolean nextOrLast) {
        if (App.instance.isShowAllRecordInCurrentProject) {
            return DBQueryConstant.findOtherRecordInProject(DB_NAME, projectId, date, nextOrLast);
        } else {
            return DBQueryConstant.findOtherRecordInJob(DB_NAME, jobId, date, nextOrLast);
        }
    }
}
