package com.project.survey.widget

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.view.inputmethod.EditorInfo
import androidx.appcompat.widget.LinearLayoutCompat
import com.project.survey.R
import com.project.survey.databinding.WidgetSearchBinding
import com.project.survey.databinding.WidgetSectionItemBinding
import com.project.survey.extend.hideSoftInput
import com.project.survey.extend.isVisibleOrGone
import com.project.survey.extend.toast

class SearchWidget @JvmOverloads constructor(
    context: Context,
    val attrs: AttributeSet? = null,
    defStyleAttr: Int = 0
) : LinearLayoutCompat(context, attrs, defStyleAttr) {

    private val binding: WidgetSearchBinding

    init {
        binding = WidgetSearchBinding.inflate(LayoutInflater.from(context), this, true)
        val typeArray = context.obtainStyledAttributes(attrs, R.styleable.SearchWidget)

        binding.etSearch.setHint(typeArray.getString(R.styleable.SearchWidget_sw_hint))

        typeArray.recycle()
    }


    fun setOnEditorSearchListener(onSearch: (String) -> Unit) {
        binding.etSearch.setOnEditorActionListener { textView, i, keyEvent ->
            if (i == EditorInfo.IME_ACTION_SEARCH) {
                onSearch(textView.text.toString())
                binding.etSearch.hideSoftInput()
            }
            return@setOnEditorActionListener true
        }
    }
}