package com.project.survey.widget.bingce.dragdrop;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;


import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.ui.pointmeasure.measure.util.SFUtils;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.Utils;


public class FunctionAdapter extends RecyclerView.Adapter {

    private List<FunctionItem> data = new ArrayList<>();

    private LayoutInflater inflater;
    private Context context;

    public FunctionAdapter(Context context, @NonNull List<FunctionItem> data) {
        this.context = context;
        if (data != null) {
            this.data = data;
        }
        inflater = LayoutInflater.from(context);
    }

    @Override
    public RecyclerView.ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        RecyclerView.ViewHolder holder;
        if (0 == viewType) {
            holder = new TitleViewHolder(inflater.inflate(R.layout.layout_function_text, parent, false));
        } else {
            holder = new FunctionViewHolder(inflater.inflate(R.layout.layout_grid_item, parent, false));
        }
        return holder;
    }

    @Override
    public void onBindViewHolder(RecyclerView.ViewHolder viewHolder, int position) {
        if (0 == getItemViewType(position)) {
            TitleViewHolder holder = (TitleViewHolder) viewHolder;
            holder.title.setText(SFUtils.getFunctionJsonNameLanguage(context, data.get(position).name));
            holder.tip.setText(SFUtils.getFunctionJsonNameLanguage(context, data.get(position).tip));
            if (App.instance.isThemeDark()) {
                holder.title.setTextColor(context.getColor(R.color.white));
                holder.tip.setTextColor(context.getColor(R.color.white));
            }
        } else {
            final int index = position;
            FunctionViewHolder holder = (FunctionViewHolder) viewHolder;
            FunctionItem fi = data.get(position);
//            setImage(fi.imageUrl,holder.iv);
            if (App.instance.isThemeDark()) {
                holder.squareRelativeLayout.setBackgroundColor(context.getColor(R.color.dark));
                holder.text.setTextColor(context.getColor(R.color.white));
            }
            holder.text.setText(SFUtils.getFunctionJsonNameLanguage(context, fi.name));
            holder.iv.setImageResource(fi.isSelect ? R.drawable.icon_location_information_earth_gray : R.drawable.icon_location_information_earth);
            holder.btn.setImageResource(fi.isSelect ? R.drawable.ic_block_selected : R.drawable.ic_block_add);
            holder.btn.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    FunctionItem f = data.get(index);
                    if (!f.isSelect) {
                        if (listener != null) {
                            if (listener.add(f)) {
                                f.isSelect = true;
                                notifyDataSetChanged();
                            }
                        }
                    }
                }
            });
        }
    }

    public void setImage(String url, ImageView iv) {
        try {
            int rid = context.getResources().getIdentifier(url, "drawable", context.getPackageName());
            iv.setImageResource(rid);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public int getItemViewType(int position) {
        return data.get(position).isTitle ? 0 : 1;
    }

    @Override
    public int getItemCount() {
        return data.size();
    }

    private class TitleViewHolder extends RecyclerView.ViewHolder {

        private TextView title, tip;

        public TitleViewHolder(View itemView) {
            super(itemView);
            title = itemView.findViewById(R.id.title);
            tip = itemView.findViewById(R.id.tip);
        }
    }

    private class FunctionViewHolder extends RecyclerView.ViewHolder {

        private ImageView iv, btn;
        private TextView text;
        private SquareRelativeLayout squareRelativeLayout;

        public FunctionViewHolder(View itemView) {
            super(itemView);
            iv = itemView.findViewById(R.id.iv);
            text = itemView.findViewById(R.id.text);
            btn = itemView.findViewById(R.id.btn);
            squareRelativeLayout = itemView.findViewById(R.id.squareRelativeLayout);
        }
    }

    public interface OnItemAddListener {
        boolean add(FunctionItem item);
    }

    private OnItemAddListener listener;

    public void setOnItemAddListener(OnItemAddListener listener) {
        this.listener = listener;
    }

}
