package com.project.survey.widget.bingce.dragdrop;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.recyclerview.widget.RecyclerView;


import com.project.survey.App;
import com.project.survey.R;
import com.project.survey.ui.pointmeasure.measure.util.SFUtils;

import java.util.ArrayList;
import java.util.List;

import blankj.utilcode.util.Utils;

public class FunctionBlockAdapter extends RecyclerView.Adapter<FunctionBlockAdapter.ViewHolder> implements ItemTouchHelperAdapter {
    private List<FunctionItem> data = new ArrayList<>();
    private LayoutInflater inflater;
    private Context context;
    private DragICallback mDragCallback;

    public FunctionBlockAdapter(Context context, @NonNull List<FunctionItem> data) {
        this.context = context;
        inflater = LayoutInflater.from(context);
        if (data != null) {
            this.data = data;
        }
    }
    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        return new ViewHolder(inflater.inflate(R.layout.layout_grid_item, parent, false));
    }
    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        final int index = position;
        FunctionItem fi = data.get(position);
        setImage(fi.imageUrl, holder.iv);
        holder.text.setText(SFUtils.getFunctionJsonNameLanguage(context,fi.name));
        holder.btn.setImageResource(R.drawable.ic_block_delete);
        if (App.instance.isThemeDark()) {
            holder.squareRelativeLayout.setBackgroundColor(context.getColor(R.color.dark));
            holder.text.setTextColor(context.getColor(R.color.white));
        }
        holder.btn.setOnClickListener(v -> {
            FunctionItem fi1 = data.remove(index);
            if (listener != null) {
                listener.remove(fi1);
            }
            notifyDataSetChanged();
        });
    }

    public void updateDataSetChanged(List<FunctionItem> data) {
        this.data = data;
        notifyDataSetChanged();
    }

    public void setImage(String url, ImageView iv) {
        try {
            int rid = context.getResources().getIdentifier(url,"drawable",context.getPackageName());
            iv.setImageResource(rid);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    @Override
    public int getItemCount() {
        return data.size();
    }
    @Override
    public void onItemMove(RecyclerView.ViewHolder holder, int fromPosition, int targetPosition) {
        if (fromPosition < data.size() && targetPosition < data.size()) {
            FunctionItem moveItem = data.get(fromPosition);
            data.remove(fromPosition);
            data.add(targetPosition, moveItem);
            notifyItemMoved(fromPosition, targetPosition);
        }
    }
    @Override
    public void onItemSelect(RecyclerView.ViewHolder holder) {
        holder.itemView.setScaleX(0.8f);
        holder.itemView.setScaleY(0.8f);
    }

    @Override
    public void onItemClear(RecyclerView.ViewHolder holder) {
        holder.itemView.setScaleX(1.0f);
        holder.itemView.setScaleY(1.0f);
        mDragCallback.dragChanged(data);
    }

    @Override
    public void onItemDismiss(RecyclerView.ViewHolder holder) {

    }
    public class ViewHolder extends RecyclerView.ViewHolder {
        private ImageView iv, btn;
        private TextView text;
        private SquareRelativeLayout squareRelativeLayout;
        public ViewHolder(View itemView) {
            super(itemView);
            iv = itemView.findViewById(R.id.iv);
            text = itemView.findViewById(R.id.text);
            btn = itemView.findViewById(R.id.btn);
            squareRelativeLayout = itemView.findViewById(R.id.squareRelativeLayout);
        }
    }

    public interface OnItemRemoveListener {
        void remove(FunctionItem item);
    }

    private OnItemRemoveListener listener;

    public void setOnItemRemoveListener(OnItemRemoveListener listener) {
        this.listener = listener;
    }

    public interface DragICallback {
        void dragChanged(List<FunctionItem> data);
    }

    public void addDragChangedListener(DragICallback callback) {
        mDragCallback = callback;
    }
}
