package org.polaric.colorful;
import android.content.Context;
import android.content.res.TypedArray;
import androidx.preference.Preference;
import androidx.preference.PreferenceViewHolder;
import android.util.AttributeSet;

import color.Rid;
import color.Rlayout;
import color.Rstyleable;

public class ColorPickerPreference extends Preference implements ColorPickerDialog.OnColorSelectedListener {
    private boolean primary;
    private boolean accent;

    public ColorPickerPreference(Context context, AttributeSet attrs) {
        super(context, attrs);
        setWidgetLayoutResource(Rlayout.preference_colorpicker());

        TypedArray ta = context.obtainStyledAttributes(attrs, Rstyleable.colorpicker());
        try {
            primary = ta.getBoolean(Rstyleable.colorpicker_primary_color(),false);
            accent = ta.getBoolean(Rstyleable.colorpicker_accent_color(), false);
        } finally {
            ta.recycle();
        }
    }

    @Override
    public void onColorSelected(Colorful.ThemeColor color) {
        if (primary) {
            Colorful.config(getContext())
                    .primaryColor(color)
                    .apply();
        } else if (accent) {
            Colorful.config(getContext())
                    .accentColor(color)
                    .apply();
        }
        if (getOnPreferenceChangeListener()!=null) {
            getOnPreferenceChangeListener().onPreferenceChange(this, color);
        }
    }

    @Override
    public void onBindViewHolder(PreferenceViewHolder holder) {
        super.onBindViewHolder(holder);
        if (primary) {
            ((CircularView) holder.findViewById(Rid.color_indicator())).setColor(getContext().getResources().getColor(Colorful.getThemeDelegate().getPrimaryColor().getColorRes()));
        } else if (accent) {
            ((CircularView) holder.findViewById(Rid.color_indicator())).setColor(getContext().getResources().getColor(Colorful.getThemeDelegate().getAccentColor().getColorRes()));
        }
    }

    @Override
    protected void onClick() {
        super.onClick();
        ColorPickerDialog dialog = new ColorPickerDialog(getContext());
        dialog.setOnColorSelectedListener(this);
        dialog.show();
    }
}
