package org.polaric.colorful;

import android.app.ActivityManager;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.Drawable;
import android.os.Build;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.WindowManager;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

public abstract class ColorfulActivity extends AppCompatActivity {
    private String themeString;

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        themeString = Colorful.getThemeString();
        setTheme(Colorful.getThemeDelegate().getStyleResBase());
//        getTheme().applyStyle(Colorful.getThemeDelegate().getStyleResPrimary(), true);
//        getTheme().applyStyle(Colorful.getThemeDelegate().getStyleResAccent(), true);
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            if (Colorful.getThemeDelegate().isTranslucent()) {
                getWindow().addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
            }

            ActivityManager.TaskDescription tDesc = new ActivityManager.TaskDescription(null, null, getResources().getColor(Colorful.getThemeDelegate().getPrimaryColor().getColorRes()));
            setTaskDescription(tDesc);
        }
    }

    @Override
    public void setContentView(View view) {
        //判断view的背景色
        autoSwitchRootViewBgColor(view);
        super.setContentView(view);
    }

    @Override
    protected void onResume() {
        super.onResume();
        if (!Colorful.getThemeString().equals(themeString)) {
            Log.d(Util.LOG_TAG, "Theme change detected, restarting activity");
            recreate();
        }
    }

    public static void autoSwitchRootViewBgColor(View contentView) {
        if (contentView == null) {
            return;
        }
        //判断contentView是否设置了颜色，如果是，就跳过
        Drawable bg = contentView.getBackground();
        if (bg != null) {
            if (bg instanceof ColorDrawable) {
                ColorDrawable colorDrawable = (ColorDrawable) bg;
                Log.d("___bg____", "color:" + colorDrawable.getColor());
            } else {
                Log.d("___bg____", bg.toString());
            }
            return;
        }
        int bgColor = useDarkTheme ? R.color.app_background_color_night : R.color.app_background_color_day;
        contentView.setBackgroundColor(contentView.getContext().getResources().getColor(bgColor));
    }

    public static void onThemeChanged(boolean useDarkColor) {
        useDarkTheme = useDarkColor;
    }

    private static boolean useDarkTheme = false;
}
