//
// Source code recreated from a .class file by IntelliJ IDEA
// (powered by FernFlower decompiler)
//

package com.project.survey.util;

import android.text.TextUtils;
import android.widget.ImageView;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.fragment.app.FragmentActivity;
import androidx.lifecycle.Lifecycle;
import androidx.lifecycle.LifecycleEventObserver;
import androidx.lifecycle.LifecycleOwner;

import com.bingce.BaseApp;
import com.bingce.device.Device;
import com.bingce.device.enums.DeviceTypeEnum;
import com.bingce.device.ui.widget.BingCeTextView;
import com.bingce.rtk.command.RTK;
import com.bingce.rtk.gnss.Gnss;
import com.bingce.surveyor.util.dialog.CustomInputDialog;
import com.bingce.surveyor.util.dialog.CustomInputDialog.Builder;
import com.bingce.totalstation.TotalStation;
import com.bingce.totalstation.TsConfig;
import com.bingce.totalstation.TsListener;
import com.bingce.utils.Util;
import com.project.survey.R;
import com.project.survey.databinding.LayoutPoleHrHeightBinding;

import blankj.utilcode.util.ToastUtils;
import blankj.utilcode.util.Utils;

public class SurveyDevicePoleHrHeightUtils {
    public SurveyDevicePoleHrHeightUtils() {
    }

    public static void initDevicePoleHrHigh(FragmentActivity activity,
                                            final LayoutPoleHrHeightBinding layoutPoleHrHeightBinding) {
        activity.getLifecycle().addObserver(new LifecycleEventObserver() {
            public void onStateChanged(@NonNull LifecycleOwner source, @NonNull Lifecycle.Event event) {
                TsListener.ChangeHrListener changeHrListener = new TsListener.ChangeHrListener() {
                    public void onChangeHrSuccess(double hr) {
                        ToastUtils.showShort(R.string.edit_hr_successful);
                        layoutPoleHrHeightBinding.tvPoleHrHeight.setText(Util.formatDouble2String(hr, 3) + "m");
                    }

                    public void onChangeHrFailed(String errorMsg) {
                        ToastUtils.showShort(errorMsg);
                    }
                };
                switch (event) {
                    case ON_CREATE:
                        TsListener.getInstance().addChangeHrRespListener(changeHrListener);
                        break;
                    case ON_DESTROY:
                        TsListener.getInstance().removeChangeHrRespListener(changeHrListener);
                }

            }
        });
        setDevicePoleHrHigh(activity, layoutPoleHrHeightBinding.llPoleHrHeight, layoutPoleHrHeightBinding.ivPoleHrHeight, layoutPoleHrHeightBinding.tvPoleHrHeight, false);
    }

    public static void setDevicePoleHrHigh(FragmentActivity activity, LinearLayout llPoleHrHeight,
                                           ImageView ivPoleHrHeight, BingCeTextView tvPoleHrHeight,
                                           boolean isTitleBar) {
        int notNumber = 3;
        if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
            ivPoleHrHeight.setImageDrawable(Utils.getApp().getDrawable(isTitleBar ? R.drawable.icon_hr_white : (((BaseApp) Utils.getApp()).isThemeDark() ? R.drawable.icon_hr_white : R.drawable.icon_hr_white)));
            tvPoleHrHeight.setText(Util.formatDouble2String(TsConfig.getInstance().getHr(), notNumber) + "m");
        } else {
            ivPoleHrHeight.setImageDrawable(Utils.getApp().getDrawable(isTitleBar ? R.drawable.icon_pole_high_white : (((BaseApp) Utils.getApp()).isThemeDark() ? R.drawable.icon_pole_high_white : R.drawable.icon_pole_high_white)));
            tvPoleHrHeight.setText(Util.formatDouble2String(Gnss.getInstance().getDevice().getPoleHeight(), notNumber) + "m");
        }

        llPoleHrHeight.setOnClickListener((v) -> {
            double inputPH;
            String title;
            if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
                title = Utils.getApp().getString(R.string.edit_ts_hr);
                inputPH = TsConfig.getInstance().getHr();
            } else {
                title = Utils.getApp().getString(R.string.edit_antenna_height);
                inputPH = Gnss.getInstance().getDevice().getPoleHeight();
            }

            (new CustomInputDialog.Builder(activity)).setTitle(title).setInputText(Utils.getApp().getString(R.string.enter_pole_height)).setInputText(Util.formatDouble2String(inputPH, notNumber)).setInputFormat(Builder.INPUT_TYPE_PM_DECIMAL).setButtonConfirm(new CustomInputDialog.setOnConfirmClickListener() {
                public void OnConfirmClick(String editText) {
                    if (TextUtils.isEmpty(editText)) {
                        editText = "0";
                    }

                    if (Device.getInstance().deviceType == DeviceTypeEnum.DEVICE_TYPE_TS) {
                        this.saveHrHigh(editText);
                    } else {
                        this.saveRTKHigh(editText);
                    }

                    tvPoleHrHeight.setText(Util.formatDouble2String(Double.parseDouble(editText), notNumber) + "m");
                }

                private void saveHrHigh(String editText) {
                    TotalStation.getInstance().requestChangeHr(Double.parseDouble(editText));
                }

                private void saveRTKHigh(String editText) {
                    Gnss.getInstance().getDevice().setPoleHeight(Double.parseDouble(editText));
                    Gnss.getInstance().save();
                    RTK.getInstance().sendSetPoleHeightCmd();
                }
            }).create().show();
        });
    }
}
